// #include <bits/stdint-uintn.h>
// #include <stdint-gcc.h>
#include <errno.h> //errno
#include <fcntl.h> //open
#include <math.h>
#include <pthread.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h> //ioctl
#include <sys/mman.h> // mmap head file
#include <sys/types.h>
#include <unistd.h> //read write close

#include <fcntl.h> // open, close
#include <stdlib.h>
#include <time.h>

#define DMA_BUF_SIZE	    0x100000
#define VASTAI_ZONE_MAX_NUM 4
#define CAPACITY_RATIO_BASE 100

// csram ddr zone type
typedef enum {
	zone_stream, // csram, stream_info
	zone_model, // ddr local share, model_info opcode_csr
	zone_operator, // ddr VDSP0 Entry1, code
	zone_share, // ddr share weight, model input/output, video
	zone_count,
} zone_type_t;

struct addr_size {
	uint64_t base; // base address
	uint64_t size; // size(Bytes)
};

// structure of bar
struct __attribute__((packed)) vastai_bar {
	uint32_t id; // bar id
	uint32_t : 32;
	uint64_t ddr_base; // base address of ddr
	uint64_t base; // base address
	uint64_t size; // size(Bytes)
};

struct __attribute__((packed)) vastai_zone {
	uint32_t id;
	uint32_t zone_type;
	uint32_t alig_ddr_start;
	uint32_t entry;
	uint32_t bank_num : 8;
	uint64_t : 56;
	struct addr_size bank[VASTAI_ZONE_MAX_NUM];
};

struct __attribute__((packed)) vastai_vdsp {
	/* data */
	uint32_t id; // vdsp id
	uint32_t : 32;
	uint64_t base; // base address
	uint64_t size; // size(Bytes)
};

struct __attribute__((packed)) vastai_wdbuf {
	uint32_t bank_num; // bank number
	uint32_t : 32;
	uint64_t bank_size; // bank size(Bytes)
	uint64_t base; // base address
	uint64_t size; // size(Bytes)
	// uint64_t total_size;        // wdbuf size(Bytes)
};

struct __attribute__((packed)) vastai_dlc {
	uint32_t id;
	uint32_t sram_flag; // bit 1 use, bit 0 not used
	uint32_t wdma_fw_entry;
	uint32_t : 32;
	uint64_t csr_base; // csr base address
	uint64_t csr_size; // csr size(Bytes)
	struct vastai_wdbuf wdbuf;
};

struct __attribute__((packed)) vastai_ssram {
	uint32_t id;
	uint32_t : 32;
	uint64_t base; // base address
	uint64_t size; // size(Bytes)
};

// structure of odma
struct __attribute__((packed)) vastai_odma {
	uint32_t id; // odma id
	uint32_t : 32;
	uint64_t base; // base address
	uint64_t size; // size(Bytes)
};

// soc equal to a soc in nature
struct __attribute__((packed)) vastai_soc {
	uint32_t type;
	uint16_t vacc_id;
	uint16_t render_id;
	uint8_t bar_num; // bar number
	uint8_t zone_num; // zone number(csram, ddr)
	uint8_t vdsp_num; // vdsp core number
	uint8_t core_num; // dlc core number
	uint8_t ssram_num; // ssram number
	uint8_t odma_num; // odma number
	uint8_t odma_snum; //odma src number
	uint8_t odma_bnum; //odma branch number
	uint8_t ctrans_bnum; // ctrans branch number
	uint8_t : 8;
	// uint16_t alig_oak_operator;	//64k
	uint16_t alig_ssram; // alignment of SRAM
	uint16_t alig_wdbuf_conv; // alignment of WDBUF
	uint16_t alig_wdbuf_fc_gemm; // alignment of WDBUF
	uint32_t wdma_bud; // alignment of WDMA 32M boundry (32M)
	uint32_t odma_bud; // alignment of ODMA 512M boundry (512M)
	uint32_t throttle_max_bound;
	uint32_t capacity_ratio; // capacity ratio * CAPACITY_RATIO_BASE
	uint64_t wdbuf_broadcast_base; // base address of wdbuf for odma
	uint64_t ddr_total_size; // total size of DDR
	uint64_t ddr_2_bit_ecc_start;
	uint64_t phys_addr;
	uint64_t phys_size;
	uint64_t phys_soc;
	uint8_t reserved[76]; // reserved field
};

#define offsetof(TYPE, MEMBER) ((size_t) & ((TYPE *)0)->MEMBER)

#define container_of(ptr, type, member)                                        \
	({                                                                     \
		const typeof(((type *)0)->member) *__mptr = (ptr);             \
		(type *)((char *)__mptr - offsetof(type, member));             \
	})

#define VACC_RESET_DEV	     0x5601
#define VACC_RUN_MODEL	     0x40185602
#define VACC_RUN_STREAM	     0x40185603
#define VACC_LOAD_MODEL	     0x40185604
#define VACC_DESTROY_MODEL   0x40185605
#define VACC_LOAD_OPERATOR   0x40185606
#define VACC_GET_CURRENT_PID 0x80185608
#define VACC_GET_PROCESS     0xc0185609
#define VACC_RELEASE_PROCESS 0xc0185610
#define VACC_GET_OUT_DESC    0xc0185611
#define VACC_ECC2BIT	     0xc0015612
#define VACC_MOVEMENT	     0x80185613
#define VACC_RD_VERSION	     0x80045614
#define VACC_RD_FW_STATUS    0x80045615

#define VACC_RD_SOC   0x80a45621
#define VACC_RD_ZONE  0xc0585622
#define VACC_RD_VDSP  0xc0185623
#define VACC_RD_DLC   0xc0405624
#define VACC_RD_SSRAM 0xc0185625
#define VACC_RD_ODAM  0xc0185626
#define VACC_RD_BAR   0xc0205628

int main()
{
	int fd = 0;
	int ret = 0;
	int data[5][256];
	void *pdata = NULL;
	void *pdata_temp = NULL;
	FILE *fp = 0;
	struct vastai_soc soc;
	memset(&soc, 0, sizeof(struct vastai_soc));
	struct vastai_zone zone;
	// memset(&ddr, 0, sizeof(struct vastai_ddr));
	struct vastai_dlc dlc;
	struct vastai_ssram ssram;
	struct vastai_odma odma;

	// run_load_model_t run_model;
	// run_stream_t run_stream;
	// get_desc_t get_desc;
	// // struct vastai_input vastai_input;
	// cp_die2die_t cp_die2die;

	fd = open("/dev/vacc0", O_RDWR);
	if (fd < 0) {
		printf("/dev/vacc0 open error. fd:%d : %s\n", fd,
		       strerror(errno));
	} else {
		printf("/dev/vacc0 open ok.\n");

		// //get device info
		ret = ioctl(fd, VACC_RD_SOC, &soc);
		if (ret != 0) {
			printf("ioctl: %s\n", strerror(errno));
		} else {
			printf("\nsoc info==============\n");
			printf("soc.type=%d,\n", soc.type);
			printf("soc.core_num=%d,\n", soc.core_num);
			printf("soc.sram_num=%d,\n", soc.ssram_num);
			printf("soc.zone_num=%d,\n", soc.zone_num);
			printf("soc.odma_num=%d,\n", soc.odma_num);
			printf("soc.odma_snum=%d,\n", soc.odma_snum);
			printf("soc.odma_bnum=%d,\n", soc.odma_bnum);
			printf("soc.ctrans_bnum=%d,\n", soc.ctrans_bnum);
			printf("soc.alig_ssram=%d,\n", soc.alig_ssram);
			printf("soc.alig_wdbuf_conv=%d,\n",
			       soc.alig_wdbuf_conv);
			printf("soc.alig_wdbuf_fc_gemm=%d,\n",
			       soc.alig_wdbuf_fc_gemm);
			printf("soc.wdma_bud=0x%x,\n", soc.wdma_bud);
			printf("soc.odma_bud=0x%x,\n", soc.odma_bud);
			printf("soc.throttle_max_bound=%d,\n",
			       soc.throttle_max_bound);
			printf("soc.capacity_ratio=%d,\n", soc.capacity_ratio);
			printf("soc.wdbuf_broadcast_base=0x%lx,\n",
			       soc.wdbuf_broadcast_base);
			printf("soc.ddr_total_size=0x%lx,\n",
			       soc.ddr_total_size);
			printf("soc.ddr_2_bit_ecc_start=0x%lx,\n",
			       soc.ddr_2_bit_ecc_start);
			printf("soc.phys_addr=0x%lx,\n", soc.phys_addr);
			printf("soc.phys_size=0x%lx,\n", soc.phys_size);
			printf("soc.phys_soc=0x%lx,\n", soc.phys_soc);
		}

		printf("\ndlc info==============\n");
		for (int32_t i = 0; i < soc.core_num; i++) {
			memset(&dlc, 0, sizeof(struct vastai_dlc));
			dlc.id = i;
			ret = ioctl(fd, VACC_RD_DLC, &dlc);
			if (ret != 0) {
				printf("ioctl: %s\n", strerror(errno));
			} else {
				printf("dlc.id=%d,\n", dlc.id);
				printf("dlc.sram_flag=0x%x,\n", dlc.sram_flag);
				printf("dlc.wdma_fw_entry=%d,\n",
				       dlc.wdma_fw_entry);
				printf("dlc.csr.base=0x%lx,\n", dlc.csr_base);
				printf("dlc.csr.size=0x%lx,\n", dlc.csr_size);
				printf("dlc.wdbuf.bank_num=%d,\n",
				       dlc.wdbuf.bank_num);
				printf("dlc.wdbuf.bank_size=0x%lx,\n",
				       dlc.wdbuf.bank_size);
				printf("dlc.wdbuf.base=0x%lx,\n",
				       dlc.wdbuf.base);
				printf("dlc.wdbuf.size=0x%lx,\n",
				       dlc.wdbuf.size);
			}
		}

		printf("\nssram info==============\n");
		for (int32_t i = 0; i < soc.ssram_num; i++) {
			memset(&ssram, 0, sizeof(struct vastai_ssram));
			ssram.id = i;
			ret = ioctl(fd, VACC_RD_SSRAM, &ssram);
			if (ret != 0) {
				printf("ioctl: %s\n", strerror(errno));
			} else {
				printf("ssram.id=%d,\n", ssram.id);
				printf("ssram.base=0x%lx,\n", ssram.base);
				printf("ssram.size=0x%lx,\n", ssram.size);
			}
		}

		printf("\nodma info==============\n");
		for (int32_t i = 0; i < soc.odma_num; i++) {
			memset(&odma, 0, sizeof(struct vastai_odma));
			odma.id = i;
			ret = ioctl(fd, VACC_RD_ODAM, &odma);
			if (ret != 0)
				printf("ioctl: %s\n", strerror(errno));
			else {
				printf("odma.id=%d,\n", odma.id);
				printf("odma.base=0x%lx,\n", odma.base);
				printf("odma.size=0x%lx,\n", odma.size);
			}
		}

		close(fd);
	}
}
