// #include <bits/stdint-uintn.h>
// #include <stdint-gcc.h>
#include <stdint.h>
#include <errno.h> //errno
#include <fcntl.h> //open
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h> //ioctl
#include <sys/types.h>
#include <unistd.h> //read write close

#include <fcntl.h> // open, close
#include <stdlib.h>
#include <time.h>

typedef enum {
	zone_model, // 0x08C58000 ~ 0x08CD7800 - 1   model info
	zone_pipeline, // 0x08CD7800 ~ 0x08CF0800 - 1   pipeline
	zone_opcode, // 0x80F600000 ~ 0x80FC00000 - 1 local share, opcode csr
	zone_share, // 0x814000000 weight, model input/output ringbuf
	zone_count,
} zone_type_t;

struct addr_size {
	uint64_t base; // base address
	uint64_t size; // size(Bytes)
};

struct __attribute__((packed)) vastai_zone {
	uint32_t id;
	uint32_t zone_type;
	// uint32_t reserved;
	struct addr_size zone;
};

struct __attribute__((packed)) vastai_ssram {
	uint32_t id;
	uint32_t reserved;
	struct addr_size ssram;
};

struct __attribute__((packed)) vastai_wdbuf {
	uint32_t bank_num; // bank number
	uint32_t reserved;
	uint64_t bank_size; // bank size(Bytes)
	struct addr_size wdbuf;
	// uint64_t total_size;        // wdbuf size(Bytes)
};

struct __attribute__((packed)) vastai_dlc {
	uint32_t id;
	uint32_t sram_flag; // bit 1 use, bit 0 not used
	uint32_t wdma_fw_entry;
	uint32_t reserved;
	struct addr_size csr;
	struct vastai_wdbuf wdbuf;
};

struct __attribute__((packed)) vastai_oak {
	uint32_t type;
	uint32_t core_num; // core number
	uint32_t ssram_num;
	uint32_t zone_num;
	uint32_t odma_num;
	uint32_t odma_snum;
	uint32_t odma_bnum;
	// uint32_t reserved;
	uint32_t alig_ddr; // alignment of DDR
	uint32_t alig_ddr_bug; // alignment of DDR 32M boundry
	uint32_t alig_ssram; // alignment of SRAM
	uint32_t alig_wdbuf_conv; // alignment of WDBUF
	uint32_t alig_wdbuf_fc_gemm; // alignment of WDBUF
	uint64_t wdbuf_broadcast_base; // base address of wdbuf for odma
};

typedef struct __attribute__((packed)) run_model {
	uint32_t dev : 32;
	uint32_t data_id : 32;
	/* dev soc entry addr */
	uint32_t model_addr : 32;
	uint32_t : 32;
	uint64_t : 64;
} run_model_t;

typedef struct __attribute__((packed)) run_stream {
	uint32_t dev : 32;
	uint32_t data_id : 32;
	/* dev soc entry addr */
	uint32_t stream_addr : 32;
	/* dev soc addr */
	uint64_t : 64;
	uint32_t : 32;
} run_stream_t;

typedef struct __attribute__((packed)) get_desc {
	uint32_t dev;
	uint32_t data_id;
	uint32_t context_id;
	/* dev soc entry addr */
	uint32_t stream_addr;
	/* reserved 64bit */
	uint64_t : 64;
} get_desc_t;

typedef struct __attribute__((packed)) set_input {
	uint32_t dev;
	uint32_t data_id;
	/* dev soc entry addr */
	uint32_t stream_addr;
	uint32_t lenth;
	/* dev soc addr */
	uint64_t input_addr;
	/* host user addr */
	void *input;
} set_input_t;

typedef struct __attribute__((packed)) get_output {
	uint32_t dev;
	uint32_t data_id;
	/* dev soc entry addr */
	uint32_t stream_addr;
	uint32_t lenth;
	/* dev soc addr */
	uint64_t output_addr;
	/* host user space addr */
	void *output;
} get_output_t;

struct __attribute__((packed)) vastai_input {
	uint32_t dev;
	uint32_t data_id;
	uint64_t lenth;
	void *input;
	uint64_t input_addr;
	// u64 output_addr;
	uint64_t stream_addr;
};

#define offsetof(TYPE, MEMBER) ((size_t) & ((TYPE *)0)->MEMBER)

#define container_of(ptr, type, member)                                        \
	({                                                                     \
		const typeof(((type *)0)->member) *__mptr = (ptr);             \
		(type *)((char *)__mptr - offsetof(type, member));             \
	})

#define CMD_RESET_DEV	 0x5601
#define CMD_RUN_MODEL	 0x40185602
#define CMD_RUN_STREAM	 0x40185603
#define CMD_GET_OUT_DESC 0x80185611
#define CMD_GET_OUTPUT	 0xC0205612
#define CMD_SET_INPUT	 0x40205613
#define CMD_RD_OAK	 0x80385621
#define CMD_RD_ZONE	 0xC0185622
#define CMD_RD_DLC	 0xC0405623
#define CMD_RD_SSRAM	 0xC0185624

int main(int argc, char *argv[])
{
	int fd = 0;
	int ret = 0;
	int data[5][256];
	FILE *fp = 0;
	struct vastai_oak oak;
	memset(&oak, 0, sizeof(struct vastai_oak));
	struct vastai_zone zone;
	// memset(&ddr, 0, sizeof(struct vastai_ddr));
	struct vastai_dlc dlc;
	// memset(&dlc, 0, sizeof(struct vastai_dlc));
	struct vastai_ssram ssram;
	// memset(&ssram, 0, sizeof(struct vastai_ssram));

	struct vastai_input vastai_input;

	if (argc < 5) {
		printf("arg error number:%d\n", argc);
		return -1;
	}

	fd = open("/dev/vacc0", O_RDWR);
	if (fd < 0) {
		printf("/dev/vacc0 open error. fd:%d : %s\n", fd,
		       strerror(errno));
	} else {
		printf("/dev/vacc0 open ok.\n");
		// ret = ioctl(fd, CMD_RESET_DEV);
		// if (ret != 0) {
		// 	printf("ioctl: %s\n", strerror(errno));
		// }
		// ret = ioctl(fd, CMD_RUN_MODEL);
		// if (ret != 0) {
		// 	printf("ioctl: %s\n", strerror(errno));
		// }
		// ret = ioctl(fd, CMD_RUN_PIPELINE);
		// if (ret != 0) {
		// 	printf("ioctl: %s\n", strerror(errno));
		// }

		// //get device info
		// ret = ioctl(fd, CMD_RD_OAK, &oak);
		// if (ret != 0) {
		// 	printf("ioctl: %s\n", strerror(errno));
		// } else {
		// 	printf("/dev/vacc ioctl val :\n");
		// 	printf("oak.type=%d,\n", oak.type);
		// 	printf("oak.core_num=%d,\n", oak.core_num);
		// 	printf("oak.ssram_num=%d,\n", oak.ssram_num);
		// 	printf("oak.zone_num=%d,\n", oak.zone_num);
		// 	printf("oak.odma_num=%d,\n", oak.odma_num);
		// 	printf("oak.odma_snum=%d,\n", oak.odma_snum);
		// 	printf("oak.odma_bnum=%d,\n", oak.odma_bnum);
		// 	printf("oak.alig_ddr=%d,\n", oak.alig_ddr);
		// 	printf("oak.alig_ddr_bug=%d,\n", oak.alig_ddr_bug);
		// 	printf("oak.alig_ssram=%d,\n", oak.alig_ssram);
		// 	printf("oak.alig_wdbuf_conv=%d,\n", oak.alig_wdbuf_conv);
		// 	printf("oak.alig_wdbuf_fc_gemm=%d,\n", oak.alig_wdbuf_fc_gemm);
		// 	printf("oak.wdbuf_broadcast_base=%ld,\n", oak.wdbuf_broadcast_base);
		// }

		// for (int32_t i = 0; i < oak.zone_num; i++) {
		// 	memset(&zone, 0, sizeof(struct vastai_zone));
		// 	zone.id = i;
		// 	ret = ioctl(fd, CMD_RD_ZONE, &zone);
		// 	if (ret != 0) {
		// 		printf("ioctl: %s\n", strerror(errno));
		// 	} else {
		// 		printf("/dev/vacc ioctl val :\n");
		// 		printf("zone.id=%d,\n", zone.id);
		// 		printf("zone.zone_type=%d,\n", zone.zone_type);
		// 		printf("zone.zone.base=0x%lx,\n", zone.zone.base);
		// 		printf("zone.zone.size=0x%lx,\n", zone.zone.size);
		// 	}
		// }

		// for (int32_t i = 0; i < oak.core_num; i++) {
		// 	memset(&dlc, 0, sizeof(struct vastai_dlc));
		// 	dlc.id = i;
		// 	ret = ioctl(fd, CMD_RD_DLC, &dlc);
		// 	if (ret != 0) {
		// 		printf("ioctl: %s\n", strerror(errno));
		// 	} else {
		// 		printf("/dev/vacc ioctl val :\n");
		// 		printf("dlc.id=%d,\n", dlc.id);
		// 		printf("dlc.sram_flag=0x%x,\n", dlc.sram_flag);
		// 		printf("dlc.wdma_fw_entry=%d,\n", dlc.wdma_fw_entry);
		// 		printf("dlc.csr.base=0x%lx,\n", dlc.csr.base);
		// 		printf("dlc.csr.size=0x%lx,\n", dlc.csr.size);
		// 		printf("dlc.wdbuf.bank_num=%d,\n", dlc.wdbuf.bank_num);
		// 		printf("dlc.wdbuf.bank_size=0x%lx,\n", dlc.wdbuf.bank_size);
		// 		printf("dlc.wdbuf.wdbuf.base=0x%lx,\n", dlc.wdbuf.wdbuf.base);
		// 		printf("dlc.wdbuf.wdbuf.size=0x%lx,\n", dlc.wdbuf.wdbuf.size);
		// 	}
		// }

		// for (int32_t i = 0; i < oak.ssram_num; i++) {
		// 	memset(&ssram, 0, sizeof(struct vastai_ssram));
		// 	ssram.id = i;
		// 	ret = ioctl(fd, CMD_RD_SSRAM, &ssram);
		// 	if (ret != 0) {
		// 		printf("ioctl: %s\n", strerror(errno));
		// 	} else {
		// 		printf("/dev/vacc ioctl val :\n");
		// 		printf("ssram.id=%d,\n", ssram.id);
		// 		printf("ssram.ssram.base=0x%lx,\n", ssram.ssram.base);
		// 		printf("ssram.ssram.size=0x%lx,\n", ssram.ssram.size);
		// 	}
		// }

		// //test ddr udma read and write
		// uint8_t test[10] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
		// ret = pwrite(fd, test, sizeof(test)/sizeof(test[0]), 0x814000000);
		// printf("pwrite num: %d\n", ret);
		// // sleep(1);
		// memset(test, 0, sizeof(test)/sizeof(test[0]));
		// ret = pread(fd, test, sizeof(test)/sizeof(test[0]), 0x814000000);
		// printf("pread num: %d\n", ret);
		// for(uint8_t i = 0; i < sizeof(test); i++)
		// {
		// 	printf("test num: %d\n", test[i]);
		// }

		// //test csram map read and write
		// ret = pwrite(fd, test, sizeof(test)/sizeof(test[0]), 0x08C58000);
		// printf("pwrite num: %d\n", ret);
		// memset(test, 0, sizeof(test)/sizeof(test[0]));
		// ret = pread(fd, test, sizeof(test)/sizeof(test[0]), 0x08C58000);
		// printf("pread num: %d\n", ret);
		// for(uint8_t i = 0; i < sizeof(test); i++)
		// {
		// 	printf("test num: %d\n", test[i]);
		// }

		// fp = fopen("input", "wb+");
		// if (0 == fp) {
		// 	perror("fopen error\n");
		// 	exit(1);
		// }
		// //test ddr udma wait for write
		// srand((unsigned)time(0));
		// for (uint32_t i = 0; i < 3; i++) {
		// 	if (i < 2) {
		// 		for (uint32_t j = 0; j < 256; j++) {
		// 			data[i][j] = rand();
		// 		}
		// 	} else {
		// 		for (uint32_t j = 0; j < 8; j++) {
		// 			data[i][j] = rand();
		// 		}
		// 	}
		// }
		// fwrite(data, 1, sizeof(int) * (2 * 256 + 8), fp);
		// ret = pwrite(fd, data, sizeof(int) * (2 * 256 + 8), 0x814000000);
		// printf("pwrite num: %d\n", ret);
		// for (uint32_t i = 0; i < 5; i++) {
		// 	if (i < 4) {
		// 		for (uint32_t j = 0; j < 256; j++) {
		// 			data[i][j] = rand();
		// 		}
		// 	} else {
		// 		for (uint32_t j = 0; j < 16; j++) {
		// 			data[i][j] = rand();
		// 		}
		// 	}
		// }
		// fwrite(data, 1, sizeof(int) * (4 * 256 + 16), fp);
		// ret = pwrite(fd, data, sizeof(int) * (4 * 256 + 16), 0x814000000 + sizeof(int) * (2 * 256 + 8));
		// printf("pwrite num: %d\n", ret);
		// fclose(fp);

		// //test ddr udma wait for read
		// fp = fopen("output", "wb+");
		// if (0 == fp) {
		// 	perror("fopen error\n");
		// 	exit(1);
		// }
		// memset(data, 0, sizeof(data)/sizeof(data[0][0]));
		// ret = pread(fd, data, sizeof(int) * (2 * 256 + 8), 0x814000000);
		// printf("pread num: %d\n", ret);
		// fwrite(data, 1, sizeof(int) * (2 * 256 + 8), fp);
		// memset(data, 0, sizeof(data)/sizeof(data[0][0]));
		// ret = pread(fd, data, sizeof(int) * (4 * 256 + 16), 0x814000000 + sizeof(int) * (2 * 256 + 8));
		// printf("pread num: %d\n", ret);
		// fwrite(data, 1, sizeof(int) * (4 * 256 + 16), fp);
		// fclose(fp);

		// fp = fopen("image_input", "wb+");
		// if (0 == fp) {
		// 	perror("fopen error\n");
		// 	exit(1);
		// }
		// //test ddr set input
		// srand((unsigned)time(0));
		// memset(data, 0, sizeof(data)/sizeof(data[0][0]));
		// for (uint32_t i = 0; i < 3; i++) {
		// 	if (i < 2) {
		// 		for (uint32_t j = 0; j < 256; j++) {
		// 			data[i][j] = rand();
		// 		}
		// 	} else {
		// 		for (uint32_t j = 0; j < 8; j++) {
		// 			data[i][j] = rand();
		// 		}
		// 	}
		// }
		// fwrite(data, 1, sizeof(int) * (2 * 256 + 8), fp);
		// vastai_input.lenth = sizeof(int) * (2 * 256 + 8);
		// vastai_input.input = data;
		// vastai_input.input_addr = 0x814000000;
		// ret = ioctl(fd, VACC_SET_INPUT, &vastai_input);
		// if (ret != 0) {
		// 	printf("ioctl: %s\n", strerror(errno));
		// }
		// fclose(fp);

		// fp = fopen("image_output", "wb+");
		// if (0 == fp) {
		// 	perror("fopen error\n");
		// 	exit(1);
		// }
		// memset(data, 0, sizeof(data)/sizeof(data[0][0]));
		// ret = pread(fd, data, sizeof(int) * (2 * 256 + 8), 0x814000000);
		// printf("pread num: %d\n", ret);
		// fwrite(data, 1, sizeof(int) * (2 * 256 + 8), fp);
		// fclose(fp);

		if (!strcmp("get", argv[1])) {
			// tools get model info, csr opcode, weight
			printf("get %s\n", argv[2]);
			fp = fopen(argv[2], "wb+");
			if (0 == fp) {
				perror("fopen error\n");
				close(fd);
				exit(1);
			}
			uint64_t addr = 0;
			sscanf(argv[3], "%ld", &addr);
			printf("%s addr:0x%lx\n", argv[2], addr);
			uint64_t size = 0;
			sscanf(argv[4], "%ld", &size);
			printf("%s size:0x%lx\n", argv[2], size);
			void *pdata = malloc(size);

			ret = pread(fd, pdata, size, addr);
			if (ret < 0) {
				printf("ioctl: %s\n", strerror(errno));
				fclose(fp);
				close(fd);
				exit(1);
			}
			printf("pread num: 0x%lx\n", ret);
			fwrite(pdata, 1, size, fp);
			fclose(fp);
		}

		close(fd);
	}
}