
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <errno.h>
#include <string.h>

typedef unsigned int u32;
typedef unsigned long long int u64;

typedef struct __attribute__((packed)) dmabuf_alloc_cmd {
	u32 size;
	int dma_buf_fd;
	u64 dma_addr_t;
} dmabuf_alloc_cmd_t;

typedef struct __attribute__((packed)) dmabuf_start_cmd {
	u32 is_dev_to_host;
	u32 dma_buf_fd;
	u64 axi_addr;
	u32 size;
	u32 die_index;
} dmabuf_start_cmd_t;

/* zero-copy by dma-buf */
#define VACC_DMA_BUF_ALLOC    _IOWR('V', 0xA0, struct dmabuf_alloc_cmd)
#define VACC_DMA_BUF_START    _IOWR('V', 0xA1, struct dmabuf_start_cmd)
#define VACC_DMA_BUF_TRANS    _IOWR('V', 0xA2, union vastai_cmd)
#define VACC_DMA_BUF_TRANS_SG _IOWR('V', 0xA3, union vastai_cmd)

union vastai_cmd {
	struct {
		// 0 idle, user space to kernel space
		// 1 curren process init mmap, kernel space to user space
		// 2 has been initialized, kernel space to user space, or user space to kernel space
		u32 state : 2;
		u32 : 30;
		u32 pid : 32;
		u64 addr : 64;
		u32 size : 32;
		u32 : 32;
		u64 : 64;
	} __attribute__((packed)) mmap_init;
	struct {
		u32 dev : 32;
		// u32 cmd : 32;
		u32 pid : 32;
		u32 model_id : 32;
		u32 data_id : 32;
		/* dev soc entry addr */
		u32 model_addr : 32;
		u32 inout_addr : 32;
		u64 : 64;
	} __attribute__((packed)) run_load_model;
	struct {
		u32 dev : 32;
		// u32 cmd : 32;
		u32 pid : 32;
		u32 : 32;
		u32 : 32;
		/* dev soc entry addr */
		u32 operator_addr : 32;
		u32 operator_size : 32;
		u64 : 64;
	} __attribute__((packed)) run_load_operator;
	struct {
		u32 dev : 32;
		// u32 cmd : 32;
		u32 pid : 32;
		u32 stream_id : 32;
		u32 data_id : 32;
		/* dev soc entry addr */
		u32 stream_addr : 32;
		/* reserved 32bit */
		u32 : 32;
		u64 : 64;
	} __attribute__((packed)) run_stream;
	struct {
		u32 dev;
		u32 pid : 32;
		u32 stream_id : 32;
		u32 data_id;
		/* dev soc entry addr */
		u32 stream_addr;
		u32 context_id;
		u32 error_code;
		u32 : 32;
	} __attribute__((packed)) get_desc;
	struct {
		u32 pid : 32;
		u32 vpid : 32; // VACC_GET_CURRENT_PID
		u64 ns : 64; // VACC_GET_CURRENT_PID
		u32 level : 32; // VACC_GET_CURRENT_PID
		// VACC_GET_PROCESS
		// 0 process exit, 1 process at vacc_dev->process_head
		// VACC_RELEASE_PROCESS
		// 0 process not used, 1 process used not to release
		u32 state : 2;
		// VACC_RELEASE_PROCESS
		// 0 normal release
		// 1 warn release, not all results were taken
		// 2 error release, not getting enough task results
		u32 release : 2;
		u32 : 28;
		u64 : 64;
	} __attribute__((packed)) process;

	struct {
		u32 remote_die;
		u32 len;
		u32 is_local_2_remote : 8;
		/* reserved 8bit */
		u32 : 8;
		u64 local_address : 40;
		u64 remote_address : 40;
		/* reserved 32bit */
		u32 : 32;
		u64 : 64;
	} __attribute__((packed)) dev_2_dev;

	struct {
		u32 size;
		int dma_buf_fd;
		u64 dma_addr_t;
	} __attribute__((packed)) alloc_cmd;

	struct {
		u32 is_dev_to_host;
		u32 dma_buf_fd;
		u64 axi_addr;
		u32 size;
		u32 die_index;
	} __attribute__((packed)) dma_start_cmd;

	struct {
		u32 is_dev_to_host;
		void *vir_addr;
		u64 axi_addr;
		u32 length;
		u32 die_index;
		int pid;
	} __attribute__((packed)) dma_trans_cmd;

	struct {
		u32 is_dev_to_host;
		u32 die_index;
		u64 axi_addr;
		struct vastai_channel_buf *channel_buf;
		int channel_num;
		int pid;
	} __attribute__((packed)) dma_trans_sg_cmd;
};

union die_index_data {
	u32 val;
	struct {
		u32 die_id : 8;
		u32 dev_id : 16;
		u32 RESV : 8;
	};
};

static int open_dev(int dev_id)
{
	char file_path[64] = { 0 };
	int filep;

	snprintf(file_path, sizeof(file_path), "/dev/vacc%d", dev_id);
	filep = open(file_path, O_RDWR);
	if (filep < 0) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!\n",
			 file_path);
		perror(temp_buf);
	}
	return filep;
}

static int vastai_alloc_dma_buf(int size, u32 die_index)
{
	union die_index_data temp = { .val = die_index };
	int kchar_fd = open_dev(temp.die_id);	// use die_id replace dev_id to appoint to /dev/vacc[0...5] 
	// union vastai_cmd kcmd = { .alloc_cmd.size = size,
	// 			  .alloc_cmd.dma_buf_fd = 0 };
	dmabuf_alloc_cmd_t alloc_cmd;
	alloc_cmd.size = size;
	alloc_cmd.dma_addr_t = 0;
	alloc_cmd.dma_addr_t = 0;

	if (kchar_fd < 0) {
		printf("\t\topen kchar is failed %d\n", kchar_fd);
		return kchar_fd;
	}

	int ret = ioctl(kchar_fd, VACC_DMA_BUF_ALLOC, &alloc_cmd);
	if (ret < 0) {
		printf("\t\tioctl is failed %d\n", ret);
	} else {
		ret = alloc_cmd.dma_buf_fd;
	}
	close(kchar_fd);
	return ret;
}

/* is sync */
static int vastai_trigger_dma(int dma_buf, u64 axi_addr, u32 length,
			      u32 is_dev_to_host, u32 die_index)
{
	union die_index_data temp = { .val = die_index };
	int kchar_fd = open_dev(temp.die_id);
	// union vastai_cmd kcmd = { .dma_start_cmd.is_dev_to_host =
	// 				  is_dev_to_host,
	// 			  .dma_start_cmd.dma_buf_fd = dma_buf,
	// 			  .dma_start_cmd.axi_addr = axi_addr,
	// 			  .dma_start_cmd.size = length,
	// 			  .dma_start_cmd.die_index = die_index };
	dmabuf_start_cmd_t dma_start_cmd;
	memset(&dma_start_cmd, 0, sizeof(dma_start_cmd));
	dma_start_cmd.is_dev_to_host = is_dev_to_host;
	dma_start_cmd.dma_buf_fd = dma_buf;
	dma_start_cmd.axi_addr = axi_addr;
	dma_start_cmd.size = length;
	dma_start_cmd.die_index = die_index;

	if (kchar_fd < 0) {
		printf("\t\topen kchar is failed %d\n", kchar_fd);
		return kchar_fd;
	}

	int ret = ioctl(kchar_fd, VACC_DMA_BUF_START, &dma_start_cmd);
	close(kchar_fd);
	return ret;
}

/**
 * @brief this function will set device's memory by pcie dma.
 * 
 * @param addr : device's ddr address.
 * @param val : char val. memset val.
 * @param alloc_size :  buf size alloc of pc.
 * @param size : memset buf size of device ddr.
 * @param die_index
 * @return int : success is 0. other is errno.
 */
int dma_memset(u64 addr, char val, unsigned long alloc_size, unsigned long size,
	       unsigned int die_index)
{
	printf("\t\tstep 1: trigger dma host2dev\n");
	int dma_buf_fd = vastai_alloc_dma_buf(alloc_size, die_index);
	int ret;
	if (dma_buf_fd < 0) {
		printf("\t\tget dma_buf_fd error %d\n", dma_buf_fd);
		return -1;
	}

	void *mem = mmap(NULL, size, PROT_WRITE | PROT_READ, MAP_SHARED,
			 dma_buf_fd, 0);
	if (mem == MAP_FAILED) {
		printf("\t\tmap failed %lld, %s\n", (long long)mem,
		       strerror(errno));
		return -1;
	}

	memset(mem, val, size);

	munmap(mem, size);
	ret = vastai_trigger_dma(dma_buf_fd, addr, size, 0 /* host to dev */,
				 die_index);
	if (ret < 0) {
		printf("\t\ttrigger dma is failed %d\n", ret);
		return -1;
	}
	close(dma_buf_fd);
	return 0;
}

/* read dma buf back and check */
int dma_memory_check(u64 addr, char val, unsigned long size,
		     unsigned int die_index)
{
	printf("\t\tstep 2: trigger dma dev2host\n");
	int dma_buf_fd = vastai_alloc_dma_buf(size, 0);
	if (dma_buf_fd < 0) {
		printf("\t\tget dma_buf_fd error %d\n", dma_buf_fd);
		return -1;
	}

	int ret = vastai_trigger_dma(dma_buf_fd, addr, size,
				     1 /* dev to host */, die_index);
	if (ret < 0) {
		printf("\t\ttrigger dma is failed %d\n", ret);
		return -1;
	}

	void *mem = mmap(NULL, size, PROT_WRITE | PROT_READ, MAP_SHARED,
			 dma_buf_fd, 0);
	if (mem == MAP_FAILED) {
		printf("\t\tmap failed %lld, %s\n", (long long)mem,
		       strerror(errno));
		return -1;
	}

	/* step 3. check data */
	printf("\t\tstep 3: check dma data\n");
	for (unsigned long i = 0; i < size; i++) {
		if (((char *)mem)[i] != val) {
			printf("\t\tdata is not match\n");
			return -1;
		}
	}
	munmap(mem, size);
	close(dma_buf_fd);
	return 0;
}

int dma_buf_basic_test(unsigned int die_index)
{
	int alloc_size = 1024 * 1024;
	printf("test case: dma loopback, size: 0x%x\n", alloc_size);

	/* step 1. host to dev */
	int ret = dma_memset(0x840000000, 0x53, alloc_size, alloc_size, die_index);
	if (ret < 0) {
		printf("\ttest case: dma loopback, size: 0x%x failed: alloc failed\n",
		       alloc_size);
		return ret;
	}
	ret = dma_memory_check(0x840000000, 0x53, alloc_size, die_index);
	if (ret < 0) {
		printf("\ttest case: dma loopback, size: 0x%x failed: check failed\n",
		       alloc_size);
		return ret;
	}

	printf("\ttest success\n");
	return ret;
}

int dma_buf_alloc_mmap_test(unsigned int die_index)
{
	int dma_buf_fd[2];
	void *mem[2];
	unsigned long alloc_size = 0x440000;

	printf("test case: dma buf alloc, mmap\n");
	dma_buf_fd[0] = vastai_alloc_dma_buf(alloc_size, die_index);
	if (dma_buf_fd[0] < 0) {
		printf("\tdma buf alloc test failed : alloc failed\n");
		return -1;
	}

	mem[0] = mmap(NULL, alloc_size, PROT_WRITE | PROT_READ, MAP_SHARED,
		      dma_buf_fd[0], 0);

	dma_buf_fd[1] = vastai_alloc_dma_buf(alloc_size, die_index);
	if (dma_buf_fd[1] < 0) {
		printf("\tdma buf alloc test failed : alloc failed\n");
		return -1;
	}

	mem[1] = mmap(NULL, alloc_size, PROT_WRITE | PROT_READ, MAP_SHARED,
		      dma_buf_fd[1], 0);
	munmap(mem[0], alloc_size);
	if (!(mem[1])) {
		printf("\tdma buf alloc test failed : mmap failed\n");
		return -1;
	}

	mem[0] = mmap(NULL, alloc_size, PROT_WRITE | PROT_READ, MAP_SHARED,
		      dma_buf_fd[0], 0);
	if (!(mem[1])) {
		printf("\tdma buf alloc test failed : mmap failed\n");
		return -1;
	}

	munmap(mem[0], alloc_size);
	munmap(mem[1], alloc_size);
	close(dma_buf_fd[0]);
	close(dma_buf_fd[1]);

	printf("\ttest success\n");
	return 0;
}

int test_multi_write(unsigned int die_index)
{
	int ret;

	printf("test case: #OP2245 alloc bigger size than use\n");

	ret = dma_memset(0x840000000, 0x1, 8 * 1024 * 1024, 5 * 1024 * 1024, die_index);
	if (ret < 0) {
		printf("\ttest case: #OP2245 alloc bigger size than use failed: send failed\n");
		return ret;
	}
	ret = dma_memset(0x840000000 + 5 * 1024 * 1024, 0x2, 8 * 1024 * 1024,
			 5 * 1024 * 1024, die_index);
	if (ret < 0) {
		printf("\ttest case: #OP2245 alloc bigger size than use failed: send failed\n");
		return ret;
	}
	ret = dma_memory_check(0x840000000, 0x1, 5 * 1024 * 1024, die_index);
	if (ret < 0) {
		printf("\ttest case: #OP2245 alloc bigger size than use failed: check failed\n");
		return ret;
	}
	ret = dma_memory_check(0x840000000 + 5 * 1024 * 1024, 0x2,
			       5 * 1024 * 1024, die_index);
	if (ret < 0) {
		printf("\ttest case: #OP2245 alloc bigger size than use failed: check failed\n");
		return ret;
	}
	ret = dma_memset(0x840000000, 0x1, 8 * 1024 * 1024, 5 * 1024 * 1024, die_index);
	if (ret < 0) {
		printf("\ttest case: #OP2245 alloc bigger size than use failed: send failed\n");
		return ret;
	}
	ret = dma_memory_check(0x840000000, 0x1, 5 * 1024 * 1024, die_index);
	if (ret < 0) {
		printf("\ttest case: #OP2245 alloc bigger size than use failed: check failed\n");
		return ret;
	}
	ret = dma_memory_check(0x840000000 + 5 * 1024 * 1024, 0x2,
			       5 * 1024 * 1024, die_index);
	if (ret < 0) {
		printf("\ttest case: #OP2245 alloc bigger size than use failed: check failed\n");
		return ret;
	}
	printf("\ttest success\n");
	return ret;
}

typedef struct _dmabuf_alloc_t {
	void *addr;
	size_t size;
	int fd;
} dmabuf_alloc_t;

int vaccrt_malloc_by_dmabuf(u32 dev_id, dmabuf_alloc_t *dmabuf_alloc)
{
	int error_no = 0;

	if (0 == dmabuf_alloc->size) {
		printf("The param 'size' is 0");
		return -1;
	}

	dmabuf_alloc->fd = vastai_alloc_dma_buf(dmabuf_alloc->size, dev_id);
	if (dmabuf_alloc->fd < 0) {
		printf("[device:%u] alloc_dma_buf error.", dev_id);
	}

	dmabuf_alloc->addr =
		mmap(NULL, dmabuf_alloc->size, PROT_WRITE | PROT_READ,
		     MAP_SHARED, dmabuf_alloc->fd, 0);
	if (dmabuf_alloc->addr == MAP_FAILED) {
		printf("[device:%u] mmap failed.", dev_id);
		return -1;
	}

	return error_no;
}

int vaccrt_free_by_dmabuf(dmabuf_alloc_t *dmabuf_alloc)
{
	int error_no = 0;

	// assert(dmabuf_alloc->addr!=NULL);
	// assert(dmabuf_alloc->size!=0);

	munmap(dmabuf_alloc->addr, dmabuf_alloc->size);
	close(dmabuf_alloc->fd);

	return error_no;
}

int vaccrt_memcpy_by_dmabuf(u32 dev_id, int dma_buf_fd, const void *addr_from,
			    size_t size, void *addr_to, int direction)
{
	int error_no = 0;
	int ret = 0;

	void *addr_device = (direction == 0 ? addr_to : (void *)addr_from);
	u32 is_dev_to_host = (direction == 0 ? 0 : 1);

	ret = vastai_trigger_dma(dma_buf_fd, (u64)addr_device, size,
				 is_dev_to_host, dev_id);
	if (ret < 0) {
		printf("\t\ttrigger dma is failed %d\n", ret);
		return -1;
	}

	return error_no;

#if 0
    // Distinguish Read and Write By Direction
    if (direction == kHost2Ddr) {
        // Write addr_from->Host'DDR address, ad
        ret = dev_info->trigger_dma_buf(dev_info, dma_buf_fd, (uint64_t)addr_to, size,
                    0 /* host to dev */, dev_id);
        if (ret < 0) {
            printf("\t\ttrigger dma is failed %d\n", ret);
            return -1;
        }
    } else {
        // ? addr_from ?device?DDR??, addr_to ?host?DDR??
        ret = dev_info->trigger_dma_buf(dev_info, dma_buf_fd, (uint64_t)addr_from, size,
				     1 /* dev to host */, dev_id);
        if (ret < 0) {
            printf("\t\ttrigger dma is failed %d\n", ret);
            return -1;
        }
    }
#endif
}

int vaccrt_memset_by_dmabuf(u32 dev_id, const void *addr, unsigned char val,
			    size_t size)
{
	/*
		addr is the address of Device'DDR;
	*/
	int error_no = 0;
	int ret;

	int dma_buf_fd = vastai_alloc_dma_buf(size, dev_id);

	if (dma_buf_fd < 0) {
		printf("\t\tget dma_buf_fd error %d\n", dma_buf_fd);
		return -1;
	}

	void *mem = mmap(NULL, size, PROT_WRITE | PROT_READ, MAP_SHARED,
			 dma_buf_fd, 0);
	if (mem == MAP_FAILED) {
		printf("\t\tmap failed %lld, %s\n", (long long)mem,
		       strerror(errno));
		return -1;
	}

	memset(mem, val, size);
	munmap(mem, size);

	ret = vastai_trigger_dma(dma_buf_fd, (u64)addr, size,
				 0 /* host to dev */, dev_id);
	if (ret < 0) {
		printf("\t\ttrigger dma is failed %d\n", ret);
		return -1;
	}
	close(dma_buf_fd);
	return error_no;
}

int main(int argc, char* argv[])
{
#if 1
	int ret;
	unsigned int die_index = 0;

	if (argc == 2) {
		die_index = atoi(argv[1]);
	}

	ret = dma_buf_alloc_mmap_test(die_index);
	if (ret) {
		return ret;
	}

	ret = dma_buf_basic_test(die_index);
	if (ret) {
		return ret;
	}

	ret = test_multi_write(die_index);
	if (ret) {
		return ret;
	}
#else
	int ret = 0, i = 0;
	dmabuf_alloc_t dmabuf_alloc;
	dmabuf_alloc.addr = 0;
	dmabuf_alloc.fd = 0;
	dmabuf_alloc.size = 1024 * 1024;
	unsigned char *p = NULL;

	if (vaccrt_malloc_by_dmabuf(0, &dmabuf_alloc) < 0) {
		printf("vaccrt_malloc_by_dmabuf failed.");
		return -1;
	}

	// clean host buffer
	memset(dmabuf_alloc.addr, 0x00, dmabuf_alloc.size);

	ret = vaccrt_memset_by_dmabuf(0, (const void *)0x840000000, 0x5A,
				      dmabuf_alloc.size);
	if (ret < 0) {
		printf("vaccrt_memset_by_dmabuf failed.\n");
		return -1;
	}

	// From Device to Host
	ret = vaccrt_memcpy_by_dmabuf(0, dmabuf_alloc.fd,
				      (const void *)0x840000000,
				      dmabuf_alloc.size, dmabuf_alloc.addr, 1);
	if (ret < 0) {
		printf("vaccrt_memcpy_by_dmabuf from device to host failed.\n");
		return -1;
	}

	printf("First read from device to host.\n");
	p = (unsigned char *)dmabuf_alloc.addr;
	for (i = 0; i < 256; i++) {
		printf("0x%02X ", *p);
		p++;
		if ((i + 1) % 16 == 0)
			printf("\n");
	}
	printf("\n");

	printf("set host memory as 0x10.\n");
	memset(dmabuf_alloc.addr, 0x10, dmabuf_alloc.size);

	printf("copy host to device format as 0x10.\n");
	// From Host to Device
	ret = vaccrt_memcpy_by_dmabuf(0, dmabuf_alloc.fd,
				      (const void *)dmabuf_alloc.addr,
				      dmabuf_alloc.size, (void *)0x840000000,
				      0);
	if (ret < 0) {
		printf("vaccrt_memcpy_by_dmabuf from device to host failed.\n");
		return -1;
	}

	printf("clean host memory.\n");
	// read from device to host again.
	memset(dmabuf_alloc.addr, 0x00, dmabuf_alloc.size);
	printf("show host memory.\n");
	p = (unsigned char *)dmabuf_alloc.addr;
	for (i = 0; i < 256; i++) {
		printf("0x%02X ", *p);
		p++;
		if ((i + 1) % 16 == 0)
			printf("\n");
	}
	printf("\n");

	printf("do read from device to host.\n");
	// Frome Device to Host
	ret = vaccrt_memcpy_by_dmabuf(0, dmabuf_alloc.fd,
				      (const void *)0x840000000,
				      dmabuf_alloc.size, dmabuf_alloc.addr, 1);
	if (ret < 0) {
		printf("vaccrt_memcpy_by_dmabuf from device to host failed.\n");
		return -1;
	}

	printf("Second read from device to host.\n");
	p = (unsigned char *)dmabuf_alloc.addr;
	for (i = 0; i < 256; i++) {
		printf("0x%02X ", *p);
		p++;
		if ((i + 1) % 16 == 0)
			printf("\n");
	}
	printf("\n");

	vaccrt_free_by_dmabuf(&dmabuf_alloc);

	printf("dma buf test success\n");
	return 0;
#endif
}
