// #include <bits/stdint-uintn.h>
// #include <stdint-gcc.h>
#include <errno.h> //errno
#include <fcntl.h> //open
#include <math.h>
#include <pthread.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h> //ioctl
#include <sys/mman.h>
#include <sys/types.h>
#include <unistd.h> //read write close

#include <fcntl.h> // open, close
#include <stdlib.h>
#include <time.h>

#define VASTAI_ZONE_MAX_NUM 4

// csram ddr zone type
typedef enum {
	zone_stream, // csram, stream_info
	zone_model, // ddr local share, model_info opcode_csr
	zone_operator, // ddr VDSP0 Entry1, code
	zone_share, // ddr share weight, model input/output, video
	zone_count,
} zone_type_t;

struct addr_size {
	uint64_t base; // base address
	uint64_t size; // size(Bytes)
};

// structure of bar
struct __attribute__((packed)) vastai_bar {
	uint32_t id; // bar id
	uint32_t : 32;
	uint64_t ddr_base; // base address of ddr
	uint64_t base; // base address
	uint64_t size; // size(Bytes)
};

struct __attribute__((packed)) vastai_zone {
	uint32_t id;
	uint32_t zone_type;
	uint32_t alig_ddr_start;
	uint32_t entry;
	uint32_t bank_num : 8;
	uint64_t : 56;
	struct addr_size bank[VASTAI_ZONE_MAX_NUM];
};

// soc equal to a soc in nature
struct __attribute__((packed)) vastai_soc {
	uint32_t type;
	uint16_t vacc_id;
	uint16_t render_id;
	uint8_t bar_num; // bar number
	uint8_t zone_num; // zone number(csram, ddr)
	uint8_t vdsp_num; // vdsp core number
	uint8_t core_num; // dlc core number
	uint8_t ssram_num; // ssram number
	uint8_t odma_num; // odma number
	uint8_t odma_snum; //odma src number
	uint8_t odma_bnum; //odma branch number
	uint8_t ctrans_bnum; // ctrans branch number
	uint8_t : 8;
	// uint16_t alig_oak_operator;	//64k
	uint16_t alig_ssram; // alignment of SRAM
	uint16_t alig_wdbuf_conv; // alignment of WDBUF
	uint16_t alig_wdbuf_fc_gemm; // alignment of WDBUF
	uint32_t wdma_bud; // alignment of WDMA 32M boundry (32M)
	uint32_t odma_bud; // alignment of ODMA 512M boundry (512M)
	uint32_t throttle_max_bound;
	uint32_t capacity_ratio; // capacity ratio * CAPACITY_RATIO_BASE
	uint64_t wdbuf_broadcast_base; // base address of wdbuf for odma
	uint64_t ddr_total_size; // total size of DDR
	uint64_t ddr_2_bit_ecc_start;
	uint64_t phys_addr;
	uint64_t phys_size;
	uint64_t phys_soc;
	uint8_t reserved[76]; // reserved field
};

typedef enum {
	mmap_idle,
	mmap_initializing,
	mmap_initialized,
} mmap_state_t;

typedef struct __attribute__((packed)) mmap_init {
	// 0 idle, user space to kernel space
	// 1 curren process init mmap, kernel space to user space
	// 2 has been initialized, kernel space to user space, or user space to kernel
	// space
	uint32_t state : 2;
	uint32_t : 30;
	uint32_t pid : 32;
	uint64_t addr : 64;
	uint32_t size : 32;
	uint32_t : 32;
	uint64_t : 64;
} mmap_init_t;

#define VACC_RESET_DEV	     0x5601
#define VACC_RUN_MODEL	     0x40185602
#define VACC_RUN_STREAM	     0x40185603
#define VACC_LOAD_MODEL	     0x40185604
#define VACC_DESTROY_MODEL   0x40185605
#define VACC_LOAD_OPERATOR   0x40185606
#define VACC_GET_CURRENT_PID 0x80185608
#define VACC_GET_PROCESS     0xc0185609
#define VACC_RELEASE_PROCESS 0xc0185610
#define VACC_GET_OUT_DESC    0xc0185611
#define VACC_ECC2BIT	     0xc0015612
#define VACC_MOVEMENT	     0x80185613
#define VACC_RD_VERSION	     0x80045614
#define VACC_RD_FW_STATUS    0x80045615
#define VACC_RD_SOC	     0x80a45621
#define VACC_RD_ZONE	     0xc0585622
#define VACC_RD_VDSP	     0xc0185623
#define VACC_RD_DLC	     0xc0405624
#define VACC_RD_SSRAM	     0xc0185625
#define VACC_RD_ODAM	     0xc0185626
#define VACC_RD_BAR	     0xc0205628
#define VACC_MMAP_INIT	     0xc0205680

void main()
{
	int fd = 0;
	int ret = 0;
	int data[5][256];
	void *pdata = NULL;
	void *pdata_temp = NULL;
	FILE *fp = 0;

	fd = open("/dev/vacc0", O_RDWR);
	if (fd < 0) {
		printf("/dev/vacc0 open error. fd:%d : %s\n", fd,
		       strerror(errno));
	} else {
		printf("/dev/vacc0 open ok.\n");

		struct vastai_soc soc;
		memset(&soc, 0, sizeof(struct vastai_soc));
		ret = ioctl(fd, VACC_RD_SOC, &soc);
		if (ret != 0) {
			printf("ioctl: %s\n", strerror(errno));
			return;
		}

		struct vastai_bar bar;
		memset(&bar, 0, sizeof(struct vastai_bar));
		bar.id = 1;
		ret = ioctl(fd, VACC_RD_BAR, &bar);
		if (ret != 0) {
			printf("ioctl CMD_RD_BAR: %s\n", strerror(errno));
			return;
		} else {
			printf("--id = %u\n", bar.id);
			printf("  ddr_base = 0x%lx\n", bar.ddr_base);
			printf("  base = 0x%lx\n", bar.base);
			printf("  size = 0x%lx\n", bar.size);
		}

		struct vastai_zone zone;
		memset(&zone, 0, sizeof(struct vastai_zone));
		zone.id = 1;
		ret = ioctl(fd, VACC_RD_ZONE, &zone);
		if (ret != 0) {
			printf("ioctl CMD_RD_ZONE: %s\n", strerror(errno));
			return;
		} else {
			printf("--id = %u\n", zone.id);
			printf("  zone_type = %u\n", zone.zone_type);
			printf("  alig_ddr_start = 0x%x\n",
			       zone.alig_ddr_start);
			printf("  entry = 0x%x\n", zone.entry);
			printf("  bank_num = 0x%x\n", zone.bank_num);
		}

		printf("video share mem map base: 0x%lx, size: 0x%lx\n",
		       soc.phys_addr, soc.phys_size);
		void *mem = mmap(NULL, soc.phys_size, PROT_WRITE | PROT_READ,
				 MAP_SHARED, fd, soc.phys_addr);
		if (mem == MAP_FAILED) {
			printf("video share mem map failed %lld",
			       (long long)mem);
			return;
		}
		memset(mem, 0xAA, 100);
		printf("write video share mem finish\n\n");

		/**************************************************************************************/
		mmap_init_t mmap_init;
		memset(&mmap_init, 0, sizeof(mmap_init));
		mmap_init.pid = (uint32_t)getpid();
		ret = ioctl(fd, VACC_MMAP_INIT, &mmap_init);
		printf("ai share mem address: 0x%lx, size:0x%x, status:%u, pid:%u\n",
		       mmap_init.addr, mmap_init.size, mmap_init.state,
		       mmap_init.pid);
		if (ret < 0) {
			printf("ioctl VACC_MMAP_INIT: fd:%d :%s", fd,
			       strerror(errno));
			return;
		}
		if (mmap_init.state == mmap_idle) {
			printf("ioctl VACC_MMAP_INIT mmap_idle error");
			return;
		}
		void *mem_map =
			mmap(NULL, mmap_init.size, PROT_WRITE | PROT_READ,
			     MAP_SHARED, fd, mmap_init.addr);
		if (mem_map == MAP_FAILED) {
			printf("mmap_init map failed %lld", (long long)mem_map);
			return;
		}
		if (mmap_init.state == mmap_initializing) {
			mmap_init.state = mmap_initialized;
			ret = ioctl(fd, VACC_MMAP_INIT, &mmap_init);
			if (ret < 0) {
				printf("ioctl mmap_initialized error: fd:%d :%s",
				       fd, strerror(errno));
				return;
			}
			printf("ioctl mmap_initialized ok\n");
		}
		if (mmap_init.state == mmap_initialized) {
			char i = 0;
			while (1) {
				*(char *)mem_map = i;
				printf("write ai mem_map:0x%x\n",
				       (uint8_t) * (char *)mem_map);
				i++;
				sleep(2);
			}
		}

		close(fd);
	}
}
