#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/poll.h>
#include <linux/printk.h>
#include <linux/slab.h>
// #include <linux/interrupt.h>
#include <linux/completion.h>
#include <linux/pid_namespace.h>
#include <linux/sched.h>
#include <linux/sched/signal.h>
#include <linux/signal.h>
#include <linux/types.h>
#include <linux/uaccess.h>
#include <linux/dma-buf.h>


#include <linux/pci.h>
#include "vastai_ai.h"
#include "vastai_common.h"
#include "ddr_partition_v05.h"

#ifndef KB
#define KB 1024
#endif

#ifndef MB
#define MB (1024 * 1024)
#endif

#ifndef GB
#define GB (1024UL * 1024 * 1024)
#endif

#ifndef TB
#define TB (1024ULL * 1024 * 1024 * 1024)
#endif

// (@kevinx)emulator size is small
#ifndef MMAP_SIZE
#ifdef CONFIG_VASTAI_AI_EMU
#define MMAP_SIZE (6 * MB)
#else
#define MMAP_SIZE (12 * MB)
#endif // #ifdef CONFIG_VASTAI_AI_EMU
#endif // #ifndef MMAP_SIZE

// base of zone_model
#ifndef MODEL_BASE
// #define MODEL_BASE MSG_TO_AI_CSR_BASE
#define MODEL_BASE MSG_TO_ODSP
#endif

// size of zone_model (13M >> 10M >> 29M)
#ifndef MODEL_SIZE
// #define MODEL_SIZE (LOG_BUF_BASE - MSG_TO_AI_CSR_BASE)
#define MODEL_SIZE (LOG_BUF_BASE - MSG_TO_ODSP)
#endif

// entry of zone_model
#ifndef MODEL_ENTRY
// #define MODEL_ENTRY 0x85600000
#define MODEL_ENTRY 0x84600000
#endif

// base of zone_stream
#ifndef STREAM_BASE
#define STREAM_BASE MODULE_INFO_BASE
#endif

// size of zone_stream (610K)
#ifndef STREAM_SIZE
#define STREAM_SIZE (VDMCU_CSR_BASE - MODULE_INFO_BASE)
#endif

// entry of zone_stream
#ifndef STREAM_ENTRY
#define STREAM_ENTRY MODULE_INFO_BASE
#endif

// base of zone_opcode
#ifndef OPERATOR_BASE
#define OPERATOR_BASE 0x80B600000
#endif

// size of zone_opcode
#ifndef OPERATOR_SIZE
#define OPERATOR_SIZE 0x200000
#endif

// offset of zone_opcode
#ifndef OPERATOR_OFFSET
#define OPERATOR_OFFSET (VDSP1_CODE_BASE - VDSP0_CODE_BASE)
#endif

// entry of zone_operator
#ifndef OPERATOR_ENTRY
#define OPERATOR_ENTRY 0x80600000
#endif

// base of SHARE_TO_HOST (0x814000000)
#ifndef VD_MMAP_BASE
#define VD_MMAP_BASE SHARE_TO_HOST_BASE
#endif

// size of SHARE_TO_HOST (32M)
#ifndef VD_MMAP_SIZE
#define VD_MMAP_SIZE (ENCODER_HEAP_BASE - SHARE_TO_HOST_BASE)
#endif

// base of size of share front (0x844000000)
#ifndef SHARE_FRONT_BASE
#define SHARE_FRONT_BASE DDR_DYNAMIC_ALLOC_FRONT_BASE
#endif

// size of share front
#ifndef SHARE_FRONT_SIZE
#define SHARE_FRONT_SIZE (DDR_BOOT_FW_BASE - DDR_DYNAMIC_ALLOC_FRONT_BASE)
#endif

// base of size of share inter (0x880100000)
#ifndef SHARE_INTER_BASE
#define SHARE_INTER_BASE DDR_DYNAMIC_ALLOC_INTER_BASE
#endif

// size of share inter
#ifndef SHARE_INTER_SIZE
#define SHARE_INTER_SIZE (DDR_RESET_VECTOR_BASE - DDR_DYNAMIC_ALLOC_INTER_BASE)
#endif

// base of size of share back (0x8C1000000)
#ifndef SHARE_BACK_BASE
#define SHARE_BACK_BASE DDR_DYNAMIC_ALLOC_BACK_BASE
#endif

// // size of share back
// #ifndef SHARE_BACK_SIZE
// #define SHARE_BACK_SIZE (0 * GB)
// #endif

// size of WDMA boundary
#ifndef WDMA_BOUND
#define WDMA_BOUND (32 * MB)
#endif

// size of ODMA boundary
#ifndef ODMA_BOUND
#define ODMA_BOUND (512 * MB)
#endif

// alignment of CSRAM start
#ifndef ALIG_CSRAM_START
#define ALIG_CSRAM_START (4)
#endif

// alignment of DDR start
#ifndef ALIG_DDR_START
#define ALIG_DDR_START (4 * KB)
#endif

// alignment of CSRAM operator start
#ifndef ALIG_OPERATOR_START
#define ALIG_OPERATOR_START (64 * KB)
#endif

struct vastai_cdev_data {
	struct vastai_cdev *vacc_dev;
	pid_t pid;
};

static loff_t vastai_cdev_llseek(struct file *filp, loff_t offset, int orig)
{
	u32 i, j;
	// loff_t ret = 0;
	loff_t pos = 0;
	loff_t head = 0;
	struct vastai_cdev_data *vastai_cdev_data = (struct vastai_cdev_data *)filp->private_data;
	struct vastai_cdev *vacc_dev = (struct vastai_cdev *)vastai_cdev_data->vacc_dev;
	struct vastai_soc soc = vacc_dev->soc;
	struct vastai_zone *zone = vacc_dev->zone;

	switch (orig) {
	case 0: // head
		head = 0;
		break;

	case 1: // current
		head = filp->f_pos;
		break;

		// case 2://end
		// 	break;

	default:
		goto out;
		// break;
	}

	pos = head + offset;
	if (pos < 0) {
		goto out;
		// } else if (pos == 0) {
		// 	filp->f_pos = pos;
		// 	return filp->f_pos;
	} else {
		for (i = 0; i < soc.zone_num; i++) {
			for (j = 0; j < zone->bank_num; j++) {
				if (pos >= zone->bank[j].base &&
				    pos < zone->bank[j].base +
						    zone->bank[j].size) {
					filp->f_pos = pos;
					return filp->f_pos;
				}
			}

			zone++;
		}
		for (i = 0; i < soc.vdsp_num; i++) {
			if (pos >= vacc_dev->vdsp[i].base &&
			    pos < vacc_dev->vdsp[i].base +
					    vacc_dev->vdsp[i].size) {
				filp->f_pos = pos;
				return filp->f_pos;
			}
		}
	}

out:
	return -EINVAL;
}

static ssize_t vastai_cdev_read(struct file *filp, char __user *buffer,
				size_t size, loff_t *ppos)
{
	int ret = 0;
	u32 i, j;
	loff_t pos = *ppos;
	loff_t lenth = size;
	loff_t lenth_t = 0;
	// loff_t lenth_queue = 0;
	struct queue_desc *queue_desc = NULL;
	struct vastai_cdev_data *vastai_cdev_data = (struct vastai_cdev_data *)filp->private_data;
	struct vastai_cdev *vacc_dev = (struct vastai_cdev *)vastai_cdev_data->vacc_dev;
	struct vastai_soc soc = vacc_dev->soc;
	struct vastai_zone *zone = vacc_dev->zone;

	DECLARE_WAITQUEUE(wait, current);

	// pr_debug("test module cdev read.\n");
	if (0 == lenth) {
		ret = -EINVAL;
		goto out;
	}

	for (i = 0; i < soc.zone_num; i++) {
		for (j = 0; j < zone->bank_num; j++) {
			if (pos >= zone->bank[j].base &&
			    pos < zone->bank[j].base + zone->bank[j].size) {
				if (lenth > zone->bank[j].base +
						    zone->bank[j].size - pos)
					lenth = zone->bank[j].base +
						zone->bank[j].size - pos;
				if (zone_stream == i)
					goto csram;
				else
					goto ddr;
			}
		}

		zone++;
	}
	for (i = 0; i < soc.vdsp_num; i++) {
		if (pos >= vacc_dev->vdsp[i].base &&
		    pos < vacc_dev->vdsp[i].base + vacc_dev->vdsp[i].size) {
			if (lenth > vacc_dev->vdsp[i].base +
					    vacc_dev->vdsp[i].size - pos)
				lenth = vacc_dev->vdsp[i].base +
					vacc_dev->vdsp[i].size - pos;
			goto ddr;
		}
	}
	for (i = 0; i < soc.core_num; i++) {
		if (pos >= vacc_dev->dlc[i].csr_base &&
		    pos < vacc_dev->dlc[i].csr_base + vacc_dev->dlc[i].csr_size) {
			if (lenth > vacc_dev->dlc[i].csr_base +
					    vacc_dev->dlc[i].csr_size - pos)
				lenth = vacc_dev->dlc[i].csr_base +
					vacc_dev->dlc[i].csr_size - pos;
			// dlc csr
			goto ddr;
		}
	}
	for (i = 0; i < soc.odma_num; i++) {
		if (pos >= vacc_dev->odmas[i].base &&
		    pos < vacc_dev->odmas[i].base + vacc_dev->odmas[i].size) {
			if (lenth > vacc_dev->odmas[i].base +
					    vacc_dev->odmas[i].size - pos)
				lenth = vacc_dev->odmas[i].base +
					vacc_dev->odmas[i].size - pos;
			// odma csr
			goto ddr;
		}
	}
	ret = -EINVAL;
	goto out;

csram:
	// pr_debug("test module cdev read csram.\n");
	ret = vastai_map_read(vacc_dev, buffer, lenth, pos);
	if (!ret) {
		*ppos += lenth;
		ret = lenth;
	}
	goto out;

ddr:
	queue_desc = (struct queue_desc *)kvzalloc(sizeof(struct queue_desc),
						   GFP_KERNEL);
	if (!queue_desc) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "[vaerr] %s: failed to kvzalloc queue_desc\n",
			       __func__);
		ret = -ENOMEM;
		goto out;
	}
	// queue_desc.type = 1;
	queue_desc->pos = pos;
	queue_desc->lenth = lenth;
	queue_desc->buffer = (void *)buffer;

	add_wait_queue(&vacc_dev->r_wait, &wait);

	lenth_t = lenth;
	while (lenth) {
		ret = vastai_add_read_queue(vacc_dev, queue_desc);
		if (ret < 0) {
			goto ddr_out;
		} else if (0 == ret) {
			if (filp->f_flags & O_NONBLOCK) {
				ret = -EAGAIN;
				goto ddr_out;
			}

			set_current_state(TASK_INTERRUPTIBLE);
			schedule();

			if (signal_pending(current)) {
				ret = -ERESTARTSYS;
				goto ddr_out;
			}
		} else {
			if (ret <= lenth)
				lenth -= ret;
			else
				lenth = 0;
		}
	}

ddr_out:
	remove_wait_queue(&vacc_dev->r_wait, &wait);
	set_current_state(TASK_RUNNING);
	if (ret < 0)
		goto out;

	ret = lenth_t;
	*ppos += lenth_t;

out:
	kvfree(queue_desc);
	return ret;
}

static ssize_t vastai_cdev_write(struct file *filp, const char __user *buffer,
				 size_t size, loff_t *ppos)
{
	int ret = 0;
	u32 i, j;
	loff_t pos = *ppos;
	loff_t lenth = size;
	loff_t lenth_t = 0;
	// loff_t lenth_queue = 0;
	struct queue_desc *queue_desc = NULL;
	struct vastai_cdev_data *vastai_cdev_data = (struct vastai_cdev_data *)filp->private_data;
	struct vastai_cdev *vacc_dev = (struct vastai_cdev *)vastai_cdev_data->vacc_dev;
	struct vastai_soc soc = vacc_dev->soc;
	struct vastai_zone *zone = vacc_dev->zone;

	DECLARE_WAITQUEUE(wait, current);

	// pr_debug("test module cdev write.\n");
	if (0 == lenth) {
		ret = -EINVAL;
		goto out;
	}

	for (i = 0; i < soc.zone_num; i++) {
		for (j = 0; j < zone->bank_num; j++) {
			if (pos >= zone->bank[j].base &&
			    pos < zone->bank[j].base + zone->bank[j].size) {
				if (lenth > zone->bank[j].base +
						    zone->bank[j].size - pos)
					lenth = zone->bank[j].base +
						zone->bank[j].size - pos;
				if (zone_stream == i)
					goto csram;
				else
					goto ddr;
			}
		}

		zone++;
	}
	for (i = 0; i < soc.vdsp_num; i++) {
		if (pos >= vacc_dev->vdsp[i].base &&
		    pos < vacc_dev->vdsp[i].base + vacc_dev->vdsp[i].size) {
			if (lenth > vacc_dev->vdsp[i].base +
					    vacc_dev->vdsp[i].size - pos)
				lenth = vacc_dev->vdsp[i].base +
					vacc_dev->vdsp[i].size - pos;
			goto ddr;
		}
	}
	ret = -EINVAL;
	goto out;

csram:
	// pr_debug("test module cdev write csram.\n");
	ret = vastai_map_write(vacc_dev, buffer, lenth, pos);
	if (!ret) {
		*ppos += lenth;
		ret = lenth;
	}
	goto out;

ddr:
	queue_desc = (struct queue_desc *)kvzalloc(sizeof(struct queue_desc),
						   GFP_KERNEL);
	if (!queue_desc) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "[vaerr] %s: failed to kvzalloc queue_desc\n",
			       __func__);
		ret = -ENOMEM;
		goto out;
	}
	// queue_desc.type = 1;
	queue_desc->pos = pos;
	queue_desc->lenth = lenth;
	queue_desc->buffer = (void *)buffer;

	add_wait_queue(&vacc_dev->w_wait, &wait);

	lenth_t = lenth;
	while (lenth) {
		ret = vastai_add_write_queue(vacc_dev, queue_desc);
		if (ret < 0) {
			goto ddr_out;
		} else if (0 == ret) {
			if (filp->f_flags & O_NONBLOCK) {
				ret = -EAGAIN;
				goto ddr_out;
			}

			set_current_state(TASK_INTERRUPTIBLE);
			schedule();

			if (signal_pending(current)) {
				ret = -ERESTARTSYS;
				goto ddr_out;
			}
		} else {
			if (ret <= lenth)
				lenth -= ret;
			else
				lenth = 0;
		}
	}

ddr_out:
	remove_wait_queue(&vacc_dev->w_wait, &wait);
	set_current_state(TASK_RUNNING);
	if (ret < 0)
		goto out;

	ret = lenth_t;
	*ppos += lenth_t;

out:
	kvfree(queue_desc);
	return ret;
}

static unsigned int vastai_cdev_poll(struct file *filp,
				     struct poll_table_struct *wait)
{
	unsigned int mask = 0;
	struct vastai_cdev_data *vastai_cdev_data = (struct vastai_cdev_data *)filp->private_data;
	struct vastai_cdev *vacc_dev = (struct vastai_cdev *)vastai_cdev_data->vacc_dev;

	poll_wait(filp, &vacc_dev->r_wait, wait);
	poll_wait(filp, &vacc_dev->w_wait, wait);

	return mask;
}

static inline int vastai_cmd_mmap_init(struct vastai_cdev *va_dev,
				       union vastai_cmd *mmap)
{
	int ret = 0;
	int is_init = 0;
	pid_t pid = task_tgid_nr(current);

	if (mmap_idle == mmap->mmap_init.state) {
	try_init:
		if (va_dev->mmap.state < mmap_initialized) {
			mutex_lock(&va_dev->mmap.mutex);
			if (mmap_idle == va_dev->mmap.state) {
				va_dev->mmap.state = mmap_initializing;
				va_dev->mmap.pid = pid;
				is_init = 1;
				// pr_debug("test module cdev mmap mmap_idle\n");
			}
			mutex_unlock(&va_dev->mmap.mutex);
			if (is_init || mmap_initialized == va_dev->mmap.state)
				goto out;

			ret = wait_event_interruptible(va_dev->mmap.wait,
						       (va_dev->mmap.state !=
							mmap_initializing));
			if (!ret)
				goto try_init;
		}
	} else if (mmap_initialized == mmap->mmap_init.state) {
		if (va_dev->mmap.pid == pid) {
			mutex_lock(&va_dev->mmap.mutex);
			if (mmap_initializing == va_dev->mmap.state) {
				va_dev->mmap.state = mmap_initialized;
				wake_up_interruptible(&va_dev->mmap.wait);
			}
			else
				ret = -EINVAL;
			mutex_unlock(&va_dev->mmap.mutex);
			// pr_debug("test module cdev mmap mmap_initialized\n");
		} else
			ret = -EINVAL;
	} else
		ret = -EINVAL;

out:
	return ret;
}

typedef int (*set_desc_func)(struct vastai_dmadesc *, struct dma_buf *,
			     union vastai_cmd *, int, u32, u32,
			     struct vastai_pci_info *);

static int _vastai_dma_start_set_desc(struct vastai_dmadesc *desc,
				      struct dma_buf *dmabuf,
				      union vastai_cmd *vcmd, int i,
				      u32 done_size, u32 temp_size,
				      struct vastai_pci_info *pci_info)
{
	int ret = 0;
	union core_bitmap core_id = { .val = 0 };
	struct vastai_dma_buf *dm = get_vastai_dma_buf(dmabuf, i);

	desc->is_host_to_dev = !vcmd->dmabuf_start_cmd.is_dev_to_host;
	desc->is_src_not_user_mem = 1;
	desc->is_src_dma_addr = 1;
	desc->dev_addr = vcmd->dmabuf_start_cmd.axi_addr + done_size;
	desc->dma_lenth = temp_size;
	desc->host_addr.dma_addr = dm->dma_bus_addr;
	ret = vastai_pci_dma_transfer_sync(
		pci_info, vcmd->dmabuf_start_cmd.die_index, core_id, desc, 1);

	return ret;
}

static int _vastai_dma_trans_set_desc(struct vastai_dmadesc *desc,
				      struct dma_buf *dmabuf,
				      union vastai_cmd *vcmd, int i,
				      u32 done_size, u32 temp_size,
				      struct vastai_pci_info *pci_info)
{
	int ret = 0;

	desc[i].is_host_to_dev = !vcmd->dmabuf_trans_cmd.is_dev_to_host;
	desc[i].is_src_not_user_mem = 0;
	desc[i].is_src_dma_addr = 0;
	desc[i].dev_addr = vcmd->dmabuf_trans_cmd.axi_addr + done_size;
	desc[i].dma_lenth = temp_size;
	desc[i].host_addr.vir_addr =
		vcmd->dmabuf_trans_cmd.vir_addr + done_size;

	return ret;
}

static int _vastai_common_cut_dma_buf(u32 size, union vastai_cmd *vcmd,
				      struct vastai_dmadesc *desc,
				      struct vastai_pci_info *pci_info,
				      struct dma_buf *dmabuf,
				      set_desc_func pSetDescFunc)
{
	int ret = 0;
	u32 done_size = 0;
	u32 i = 0;

	while (done_size < size) {
		u32 temp_size = (size - done_size) < VASTAI_MAX_DMA_BUF ?
					(size - done_size) :
					VASTAI_MAX_DMA_BUF;

		ret = pSetDescFunc(desc, dmabuf, vcmd, i, done_size, temp_size,
				   pci_info);
		if (ret)
			return ret;

		i++;
		done_size += temp_size;
	}

	return ret;
}

static long vastai_cdev_ioctl(struct file *filp, unsigned int cmd,
			      unsigned long arg)
{
	int ret;
	int id = 0;
	u64 addr = 0;
	u64 addr_min = 0;
	u64 addr_max = 0;
	union vastai_cmd vastai_cmd;
	// union vastai_cmd_e vastai_cmd_e;
	void __user *argp = (void __user *)arg;
	struct vastai_cdev_data *vastai_cdev_data = (struct vastai_cdev_data *)filp->private_data;
	struct vastai_cdev *vacc_dev = (struct vastai_cdev *)vastai_cdev_data->vacc_dev;
	// zone_model is the first zone
	// struct vastai_zone *zone = vacc_dev->zone;
	// struct process_node *process;
	struct dev_process *dev_process = NULL;
	pid_t pid = task_tgid_nr(current);

	switch (cmd) {
	case VACC_RESET_DEV: {
		// pr_info("test module cdev ioctl VACC_RESET_DEV:0x%x\n", VACC_RESET_DEV);
		/* code */
		break;
	}

	case VACC_LOAD_MODEL: {
		// pr_info("test module cdev ioctl VACC_LOAD_MODEL:0x%lx\n", VACC_LOAD_MODEL);
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		// addr = zone[zone_model].bank[0].base + vastai_cmd.run_load_model.model_addr -
		//        MODEL_ENTRY;
		// addr_min = zone->zone.base;
		// addr_max = zone->zone.base + zone->zone.size;
		addr = vastai_cmd.run_load_model.model_addr;
		addr_min = MODEL_ENTRY;
		addr_max = MODEL_ENTRY + MODEL_SIZE;
		if (addr < addr_min || addr >= addr_max) {
			VASTAI_PCI_ERR(
				NULL, DUMMY_DIE_ID,
				"[vaerr] model_addr:0x%llx, min:0x%llx, max:0x%llx\n",
				addr, addr_min, addr_max);
			return -EINVAL;
		}
		vastai_cmd.run_load_model.pid = pid;
		return vastai_cmd_run(vacc_dev, &vastai_cmd, CMD_LOAD_MODEL);
		break;
	}

	case VACC_DESTROY_MODEL: {
		// pr_info("test module cdev ioctl VACC_DESTROY_MODEL:0x%lx\n", VACC_DESTROY_MODEL);
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		// addr = zone->zone.base + vastai_cmd.run_load_model.model_addr -
		//        MODEL_ENTRY;
		// addr_min = zone->zone.base;
		// addr_max = zone->zone.base + zone->zone.size;
		addr = vastai_cmd.run_load_model.model_addr;
		addr_min = MODEL_ENTRY;
		addr_max = MODEL_ENTRY + MODEL_SIZE;
		if (addr < addr_min || addr >= addr_max) {
			VASTAI_PCI_ERR(
				NULL, DUMMY_DIE_ID,
				"[vaerr] model_addr:0x%llx, min:0x%llx, max:0x%llx\n",
				addr, addr_min, addr_max);
			return -EINVAL;
		}
		vastai_cmd.run_load_model.pid = pid;
		return vastai_cmd_run(vacc_dev, &vastai_cmd, CMD_DESTROY_MODEL);
		break;
	}

	case VACC_RUN_MODEL: {
		// pr_info("test module cdev ioctl VACC_RUN_MODEL:0x%lx\n", VACC_RUN_MODEL);
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		// addr = zone->zone.base + vastai_cmd.run_load_model.model_addr -
		//        MODEL_ENTRY;
		// addr_min = zone->zone.base;
		// addr_max = zone->zone.base + zone->zone.size;
		// pr_debug("model_addr:0x%x, zone min addr:0x%llx, max addr:0x%llx\n", vastai_cmd.run_load_model.model_addr, addr_min, addr_max);
		addr = vastai_cmd.run_load_model.model_addr;
		addr_min = MODEL_ENTRY;
		addr_max = MODEL_ENTRY + MODEL_SIZE;
		if (addr < addr_min || addr >= addr_max) {
			VASTAI_PCI_ERR(
				NULL, DUMMY_DIE_ID,
				"[vaerr] model_addr:0x%llx, min:0x%llx, max:0x%llx\n",
				addr, addr_min, addr_max);
			return -EINVAL;
		}
		vastai_cmd.run_load_model.pid = pid;
		return vastai_cmd_run(vacc_dev, &vastai_cmd, CMD_RUN_MODEL);
		break;
	}

	case VACC_LOAD_OPERATOR: {
		// pr_info("test module cdev ioctl VACC_LOAD_OPERATOR:0x%lx\n", VACC_LOAD_OPERATOR);
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		// addr = (zone + zone_operator)->zone.base +
		//        vastai_cmd.run_load_operator.operator_addr -
		//        OPERATOR_ENTRY;
		// addr_min = (zone + zone_operator)->zone.base;
		// addr_max = (zone + zone_operator)->zone.base +
		// 	   (zone + zone_operator)->zone.size;
		addr = vastai_cmd.run_load_operator.operator_addr;
		addr_min = OPERATOR_ENTRY;
		addr_max = OPERATOR_ENTRY + OPERATOR_SIZE;
		if (addr < addr_min || addr >= addr_max) {
			VASTAI_PCI_ERR(
				NULL, DUMMY_DIE_ID,
				"[vaerr] operator_addr:0x%llx, min:0x%llx, max:0x%llx\n",
				addr, addr_min, addr_max);
			return -EINVAL;
		}
		vastai_cmd.run_load_operator.pid = pid;
		return vastai_cmd_run(vacc_dev, &vastai_cmd, CMD_LOAD_OPERATOR);
		break;
	}

	case VACC_RUN_STREAM: {
		// pr_info("test module cdev ioctl VACC_RUN_STREAM:0x%lx\n", VACC_RUN_STREAM);
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		// addr = (zone + zone_stream)->zone.base +
		//        vastai_cmd.run_stream.stream_addr - STREAM_ENTRY;
		// addr_min = (zone + zone_stream)->zone.base;
		// addr_max = (zone + zone_stream)->zone.base +
		// 	   (zone + zone_stream)->zone.size;
		// pr_debug("stream_addr:0x%x, zone min addr:0x%llx, max addr:0x%llx\n", vastai_cmd.run_stream.stream_addr, addr_min, addr_max);
		addr = vastai_cmd.run_stream.stream_addr;
		addr_min = STREAM_ENTRY;
		addr_max = STREAM_ENTRY + STREAM_SIZE;
		if (addr < addr_min || addr >= addr_max) {
			VASTAI_PCI_ERR(
				NULL, DUMMY_DIE_ID,
				"[vaerr] model_addr:0x%llx, min:0x%llx, max:0x%llx\n",
				addr, addr_min, addr_max);
			return -EINVAL;
		}
		vastai_cmd.run_stream.pid = pid;
		return vastai_cmd_run(vacc_dev, &vastai_cmd, CMD_RUN_STREAM);
		break;
	}

	case VACC_GET_OUT_DESC: {
		// pr_info("test module cdev ioctl VACC_GET_OUT_DESC:0x%lx\n", VACC_GET_OUT_DESC);
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		vastai_cmd.get_desc.pid = pid;
		ret = vastai_cmd_get_desc(vacc_dev, &vastai_cmd);
		if (ret)
			return ret;
		if (copy_to_user(argp, &vastai_cmd, sizeof(union vastai_cmd)))
			return -EFAULT;
		break;
	}

	case VACC_GET_CURRENT_PID: {
		// pr_info("test module cdev ioctl VACC_GET_CURRENT_PID:%ld\n", VACC_GET_CURRENT_PID);
		vastai_cmd.process.pid = pid;
		vastai_cmd.process.vpid = task_tgid_vnr(current);
		vastai_cmd.process.ns = (u64)task_active_pid_ns(current);
		vastai_cmd.process.level =
			(u32)task_active_pid_ns(current)->level;
		if (copy_to_user(argp, &vastai_cmd, sizeof(union vastai_cmd)))
			return -EFAULT;
		break;
	}

	case VACC_GET_PROCESS: {
		// pr_info("test ioctl VACC_GET_PROCESS:0x%lx\n", VACC_GET_PROCESS);
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		// if (vastai_cmd.process.pid == pid)
		// 	return -EINVAL;

		vastai_cmd.process.state = 0;
		mutex_lock(&vacc_dev->dev_trans_ai->mutex);
		list_for_each_entry (dev_process, &vacc_dev->dev_trans_ai->head,
				     node) {
			if (dev_process->pid == vastai_cmd.process.pid && !dev_process->proc_state) {
				vastai_cmd.process.state = 1;
				break;
			}
		}
		mutex_unlock(&vacc_dev->dev_trans_ai->mutex);
		// mutex_lock(&vacc_dev->process_mutex);
		// list_for_each_entry (process, &vacc_dev->process_head, node) {
		// 	if (process->pid == vastai_cmd.process.pid) {
		// 		vastai_cmd.process.state = 1;
		// 		break;
		// 	}
		// }
		// mutex_unlock(&vacc_dev->process_mutex);
		if (copy_to_user(argp, &vastai_cmd, sizeof(union vastai_cmd)))
			return -EFAULT;

		break;
	}

	case VACC_RELEASE_PROCESS: {
		// pr_info("test ioctl VACC_RELEASE_PROCESS:0x%lx\n", VACC_RELEASE_PROCESS);
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		if (vastai_cmd.process.pid == pid)
			return -EINVAL;

		vastai_cmd.process.state = 0;
		mutex_lock(&vacc_dev->dev_trans_ai->mutex);
		list_for_each_entry (dev_process, &vacc_dev->dev_trans_ai->head,
				     node) {
			if (dev_process->pid == vastai_cmd.process.pid) {
				vastai_cmd.process.state = 1;
				break;
			}
		}
		mutex_unlock(&vacc_dev->dev_trans_ai->mutex);
		// vastai_cmd.process.state = vastai_release_dev_process(
		// 	vacc_dev, vastai_cmd.process.pid);
		if (copy_to_user(argp, &vastai_cmd, sizeof(union vastai_cmd)))
			return -EFAULT;
		break;
	}

	case VACC_CP_DIE2DIE: {
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		if (vastai_ai_data_movement(vacc_dev, &vastai_cmd))
			return -EIO;
		break;
	}

	case VACC_DDR_OVERFLOW: {
		// pr_info("test ioctl VACC_DDR_OVERFLOW:0x%lx\n", VACC_DDR_OVERFLOW);
		// if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
		// 	return -EFAULT;
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "[vaerr] %s DDR Memory Overflow!\n", __func__);
		break;
	}

	case VACC_RD_VERSION: {
		if (copy_to_user(argp, &(vacc_dev->ai_version), sizeof(u32)))
			return -EFAULT;
		break;
	}

	case VACC_RD_FW_STATUS: {
		if (copy_to_user(argp, &(vacc_dev->fw_status), sizeof(u32)))
			return -EFAULT;
		break;
	}

	case VACC_RD_SOC: {
		// pr_info("test module cdev ioctl VACC_RD_SOC:%ld\n", VACC_RD_SOC);
		vacc_dev->soc.render_id = vastai_get_render_id(
			vacc_dev->pci_info, vacc_dev->die_index);
		if (copy_to_user(argp, &vacc_dev->soc,
				 sizeof(struct vastai_soc)))
			return -EFAULT;
		//pr_info("render id %d\n", vacc_dev->soc.render_id);
		break;
	}

	case VACC_RD_ZONE: {
		get_user(id, (int *)argp);
		// pr_info("test module cdev ioctl VACC_RD_ZONE:%ld, zone_id:%d\n", VACC_RD_ZONE, id);

		if (id >= vacc_dev->soc.zone_num)
			return -EINVAL;

		if (copy_to_user(argp, vacc_dev->zone + id,
				 sizeof(struct vastai_zone)))
			return -EFAULT;
		break;
	}

	case VACC_RD_DLC: {
		get_user(id, (int *)argp);
		// pr_info("test module cdev ioctl VACC_RD_DLC:%ld, dlc_id:%d\n", VACC_RD_DLC, id);

		if (id >= vacc_dev->soc.core_num)
			return -EINVAL;

		if (copy_to_user(argp, vacc_dev->dlc + id,
				 sizeof(struct vastai_dlc)))
			return -EFAULT;
		break;
	}

	case VACC_RD_SSRAM: {
		get_user(id, (int *)argp);
		// pr_info("test module cdev ioctl VACC_RD_SSRAM:%ld, ssram_id:%d\n", VACC_RD_SSRAM, id);

		if (id >= vacc_dev->soc.ssram_num)
			return -EINVAL;

		if (copy_to_user(argp, vacc_dev->ssram + id,
				 sizeof(struct vastai_ssram)))
			return -EFAULT;
		break;
	}

	case VACC_RD_BAR: {
		get_user(id, (int *)argp);
		// pr_info("test module cdev ioctl VACC_RD_BAR:%ld, bar_id:%d\n", VACC_RD_BAR, id);

		if (id >= vacc_dev->soc.bar_num)
			return -EINVAL;

		if (copy_to_user(argp, vacc_dev->bars + id,
				 sizeof(struct vastai_bar)))
			return -EFAULT;
		break;
	}

	case VACC_RD_ODAM: {
		get_user(id, (int *)argp);
		// pr_info("test module cdev ioctl VACC_RD_ODAM:%ld, odma_id:%d\n", VACC_RD_ODAM, id);

		if (id >= vacc_dev->soc.odma_bnum)
			return -EINVAL;

		if (copy_to_user(argp, vacc_dev->odmas + id,
				 sizeof(struct vastai_odma)))
			return -EFAULT;
		break;
	}

	case VACC_RD_VDSP: {
		get_user(id, (int *)argp);
		// pr_info("test module cdev ioctl VACC_RD_ODAM:%ld, odma_id:%d\n", VACC_RD_ODAM, id);

		if (id >= vacc_dev->soc.vdsp_num)
			return -EINVAL;

		if (copy_to_user(argp, vacc_dev->vdsp + id,
				 sizeof(struct vastai_vdsp)))
			return -EFAULT;
		break;
	}

	case VACC_ECC2BIT: {
		/* TODO: for now, this flag is always be false */
		bool is_power_on_info = 0;
		union die_index_data die_index;

		die_index.val = vacc_dev->die_index;
		/*
		   ret = 0 (get ecc error report)
		   ret = -512 (thread kill by "kill -9 or CTRL-c")
		*/
		ret = vastai_wait_ecc_2_bit(vacc_dev->pci_info, die_index);
		if (ret < 0) {
			VASTAI_PCI_ERR(
				NULL, DUMMY_DIE_ID,
				"[vaerr] %s wait ecc report out with exception %d",
				__func__, ret);
			return ret;
		}

		/* if we get complete we need do this：
			1. read ddr at device: 0x8_1520_0000 ~ 0x8_1530_0000 1Mbyte
			2. there is a fifo head at this buf head. refer: struct vastai_fifo
			3. we can get valid entry number from this format:
				valid_entry_count = (fifo->elem_count - ((int)fifo->wr - (int)fifo-rd)) % fifo->elem_count
		*/

		if (copy_to_user(argp, (void *)(&is_power_on_info),
				 sizeof(is_power_on_info)))
			return -EFAULT;
		break;
	}

	case VACC_MMAP_INIT: {
		// pr_info("test module cdev ioctl VACC_MMAP_INIT:%ld\n", VACC_MMAP_INIT);
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		ret = vastai_cmd_mmap_init(vacc_dev, &vastai_cmd);
		if (ret)
			return ret;
		vastai_cmd.mmap_init.state = vacc_dev->mmap.state;
		vastai_cmd.mmap_init.addr = vacc_dev->mmap.addr;
		vastai_cmd.mmap_init.size = vacc_dev->mmap.size;
		if (copy_to_user(argp, &vastai_cmd, sizeof(union vastai_cmd)))
			return -EFAULT;
		break;
	}

	case VACC_DMA_BUF_ALLOC: {
		struct dma_buf *dmabuf;

		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;

		if (iommu_is_enable(vacc_dev->pci_info))
			dmabuf = vastai_alloc_dma_buf_sg(
				vacc_dev->pci_info,
				vastai_cmd.dmabuf_alloc_cmd.size);
		else
			dmabuf = vastai_alloc_dma_buf(
				vacc_dev->pci_info,
				vastai_cmd.dmabuf_alloc_cmd.size);

		if (dmabuf) {
			int fd = dma_buf_fd(dmabuf, O_CLOEXEC);
			vastai_cmd.dmabuf_alloc_cmd.dma_buf_fd = fd;
		} else {
			vastai_cmd.dmabuf_alloc_cmd.dma_buf_fd = -ENOMEM;
		}

		if (copy_to_user(argp, &vastai_cmd, sizeof(union vastai_cmd)))
			return -EFAULT;

		break;
	}

	case VACC_DMA_BUF_START: {
		struct vastai_dmadesc desc;
		struct dma_buf *dmabuf = NULL;
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		vastai_cmd.dmabuf_start_cmd.die_index = vacc_dev->die_index;
		dmabuf = dma_buf_get(vastai_cmd.dmabuf_start_cmd.dma_buf_fd);
		if (IS_ERR(dmabuf)) {
			VASTAI_PCI_ERR(vacc_dev->pci_info, DUMMY_DIE_ID,
				       "%s dma buf fd is invalid\n", __func__);
			return PTR_ERR(dmabuf);
		}
		if (iommu_is_enable(vacc_dev->pci_info)) {
			union core_bitmap core_id = { .val = 0 };
			struct vastai_dma_buf *dm =
				get_vastai_dma_buf(dmabuf, 0);
			unsigned int is_src_dma_addr = 1;
			unsigned int is_src_not_user_mem = 1;
			if (vastai_cmd.dmabuf_start_cmd.size > dm->size) {
				ret = -EINVAL;
				goto free_buf;
			}
			ret = vastai_pci_dma_transfer_sg(
				vacc_dev->pci_info,
				vastai_cmd.dmabuf_start_cmd.die_index, core_id,
				dm, is_src_dma_addr,
				!vastai_cmd.dmabuf_start_cmd.is_dev_to_host,
				is_src_not_user_mem,
				vastai_cmd.dmabuf_start_cmd.axi_addr,
				vastai_cmd.dmabuf_start_cmd.size);
		} else {
			ret = _vastai_common_cut_dma_buf(
				vastai_cmd.dmabuf_start_cmd.size, &vastai_cmd,
				&desc, vacc_dev->pci_info, dmabuf,
				_vastai_dma_start_set_desc);
		}
	free_buf:
		dma_buf_put(dmabuf);
		if (copy_to_user(argp, &vastai_cmd, sizeof(union vastai_cmd)))
			return -EFAULT;
		break;
	}

	case VACC_DMA_BUF_TRANS: {
		union core_bitmap core_id = { .val = 0 };
		u32 size = 0;
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		vastai_cmd.dmabuf_trans_cmd.die_index = vacc_dev->die_index;
		size = vastai_cmd.dmabuf_trans_cmd.length;
		if (iommu_is_enable(vacc_dev->pci_info) &&
		    ((u64)vastai_cmd.dmabuf_trans_cmd.vir_addr % PAGE_SIZE ==
		     0)) {
			ret = vastai_pci_dma_transfer_by_uaddr(
				vacc_dev->pci_info,
				vastai_cmd.dmabuf_trans_cmd.die_index, core_id,
				!vastai_cmd.dmabuf_trans_cmd.is_dev_to_host,
				vastai_cmd.dmabuf_trans_cmd.axi_addr,
				(u64)vastai_cmd.dmabuf_trans_cmd.vir_addr,
				size);
		} else {
			struct vastai_dmadesc *desc;
			u32 entry_count =
				DIV_ROUND_UP(size, VASTAI_MAX_DMA_BUF);

			desc = kvmalloc(sizeof(struct vastai_dmadesc) *
						entry_count,
					GFP_KERNEL);

			_vastai_common_cut_dma_buf(size, &vastai_cmd, desc,
						   vacc_dev->pci_info, NULL,
						   _vastai_dma_trans_set_desc);

			ret = vastai_pci_dma_transfer_sync_pid(
				vacc_dev->pci_info,
				vastai_cmd.dmabuf_trans_cmd.die_index, core_id,
				desc, entry_count,
				vastai_cmd.dmabuf_trans_cmd.pid);

			kvfree(desc);
		}

		if (copy_to_user(argp, &vastai_cmd, sizeof(union vastai_cmd)))
			return -EFAULT;

		break;
	}

	case VACC_DMA_BUF_TRANS_SG: {
		union core_bitmap core_id = { .val = 0 };
		struct vastai_channel_buf *channel_buf;
		int i, channel_num;
		if (copy_from_user(&vastai_cmd, argp, sizeof(union vastai_cmd)))
			return -EFAULT;
		vastai_cmd.dmabuf_trans_sg_cmd.die_index = vacc_dev->die_index;
		channel_num = vastai_cmd.dmabuf_trans_sg_cmd.channel_num;
		channel_buf = kvmalloc(sizeof(struct vastai_channel_buf) *
					       channel_num,
				       GFP_KERNEL);
		if (!channel_buf) {
			return -ENOMEM;
		}
		ret = copy_from_user(
			channel_buf, vastai_cmd.dmabuf_trans_sg_cmd.channel_buf,
			sizeof(struct vastai_channel_buf) * channel_num);
		if (ret) {
			kvfree(channel_buf);
			return -1;
		}

		for (i = 0; i < channel_num; i++) {
			VASTAI_PCI_DBG(
				vacc_dev->pci_info,
				vastai_cmd.dmabuf_trans_sg_cmd.die_index,
				"channel[%d], w:%d, h:%d, dst_w_off:%d, dst_h_off:%d, src_w_off:%d, src_h_off:%d\n",
				i, channel_buf[i].width, channel_buf[i].high,
				channel_buf[i].dst_width_offset,
				channel_buf[i].dst_high_offset,
				channel_buf[i].src_width_offset,
				channel_buf[i].src_high_offset);
		}

		vastai_dma_video(vacc_dev->pci_info,
				 vastai_cmd.dmabuf_trans_sg_cmd.die_index,
				 core_id, channel_buf, channel_num,
				 vastai_cmd.dmabuf_trans_sg_cmd.axi_addr,
				 vastai_cmd.dmabuf_trans_sg_cmd.pid);

		kvfree(channel_buf);

		if (copy_to_user(argp, &vastai_cmd, sizeof(union vastai_cmd)))
			return -EFAULT;

		break;
	}

	default:
		return -EINVAL;
		break;
	}

	return 0;
}

static void vastai_vma_open(struct vm_area_struct *vma)
{
}

static void vastai_vma_close(struct vm_area_struct *vma)
{
}

static struct vm_operations_struct vastai_mmap_ops = {
	.open = vastai_vma_open,
	.close = vastai_vma_close,
};

static int vastai_cdev_mmap(struct file *filp, struct vm_area_struct *vma)
{
	int ret = 0;
	size_t offset = 0;
	struct mmap_page *mmap_page = NULL;
	struct vastai_cdev_data *vastai_cdev_data = (struct vastai_cdev_data *)filp->private_data;
	struct vastai_cdev *vacc_dev = (struct vastai_cdev *)vastai_cdev_data->vacc_dev;

	// pr_debug("test module mmap vm_pgoff:0x%lx\n", vma->vm_pgoff);
	// pr_debug("test module mmap VD_RESERVE pfn:0x%lx\n", __phys_to_pfn(VD_MMAP_BASE - vacc_dev->bars[1].ddr_base + vacc_dev->bars[1].base));
	// pr_debug("test module mmap MMAP pfn:0x%lx\n", __phys_to_pfn(vacc_dev->mmap.addr));

	if (vma->vm_pgoff == __phys_to_pfn(vacc_dev->mmap.addr)) {
		if (vma->vm_end - vma->vm_start == vacc_dev->mmap.size) {
			list_for_each_entry (mmap_page, &vacc_dev->mmap.head,
					     node) {
				if (remap_pfn_range(
					    vma, vma->vm_start + offset,
					    page_to_pfn(mmap_page->page),
					    PAGE_SIZE * ((size_t)1
							 << (MAX_ORDER - 1)),
					    vma->vm_page_prot)) {
					ret = -EAGAIN;
					goto out;
				}
				offset += PAGE_SIZE *
					  ((size_t)1 << (MAX_ORDER - 1));
			}
			vma->vm_ops = &vastai_mmap_ops;
		} else
			ret = -EINVAL;
	} else if (vma->vm_pgoff ==
		   __phys_to_pfn(vacc_dev->soc.video_phys_addr)) {
		unsigned long size =
			vacc_dev->soc.video_size <=
					vma->vm_end - vma->vm_start ?
				vacc_dev->soc.video_size :
				vma->vm_end - vma->vm_start;
#if (defined(__aarch64__) || defined(__arm__))
		vma->vm_page_prot = phys_mem_access_prot(
			filp, vma->vm_pgoff, size, vma->vm_page_prot);
#endif
		if (remap_pfn_range(vma, vma->vm_start, vma->vm_pgoff, size,
				    vma->vm_page_prot)) {
			ret = -EAGAIN;
			goto out;
		}
		vma->vm_ops = &vastai_mmap_ops;
	} else
		ret = -EINVAL;

out:
	return ret;
}

static int vastai_cdev_fasync(int fd, struct file *filp, int mode)
{
	struct vastai_cdev_data *vastai_cdev_data = (struct vastai_cdev_data *)filp->private_data;
	struct vastai_cdev *vacc_dev = (struct vastai_cdev *)vastai_cdev_data->vacc_dev;

	return fasync_helper(fd, filp, mode, &vacc_dev->fasync);
}

static int vastai_cdev_open(struct inode *inode, struct file *filp)
{
	int ret = 0;
	struct vastai_cdev *vacc_dev = NULL;
	struct vastai_file_info *file_info = NULL;
	struct vastai_cdev_data *vastai_cdev_data = NULL;
	// struct process_node *process = NULL;
    struct vastai_soc *soc = NULL;
    struct vastai_pci_info *priv = NULL;
	pid_t pid = task_tgid_nr(current);

	file_info = to_vastai_file_info(inode->i_cdev);
	vacc_dev = container_of(file_info, struct vastai_cdev, file_info);

        soc = (struct vastai_soc *)&(vacc_dev->soc);
        priv = vacc_dev->pci_info;
        if (soc->type == VASTAI_SV100_HV) {
                soc->harvest_core_flag = vastai_get_dlc_cores_info(priv, vacc_dev->die_index);
        }

	vastai_cdev_data = kzalloc(sizeof(struct vastai_cdev_data), GFP_KERNEL);
	vastai_cdev_data->vacc_dev = vacc_dev;
	vastai_cdev_data->pid = pid;
	filp->private_data = vastai_cdev_data;

	ret = vastai_add_dev_process(vacc_dev, pid);
	// if (ret)
	// 	goto out;

	// process = kzalloc(sizeof(struct process_node), GFP_KERNEL);
	// if (!process) {
	// 	ret = -ENOMEM;
	// 	goto out;
	// }
	// process->pid = pid;
	// mutex_lock(&vacc_dev->process_mutex);
	// list_add_tail(&process->node, &vacc_dev->process_head);
	// mutex_unlock(&vacc_dev->process_mutex);

	// pr_debug("test module cdev open\n");
// out:
	return ret;
}

static int vastai_cdev_release(struct inode *inode, struct file *filp)
{
	// struct vastai_cdev *vacc_dev = NULL;
	// struct vastai_file_info *file_info = NULL;
	// struct process_node *process = NULL;
	// pid_t pid = task_tgid_nr(current);

	// file_info = to_vastai_file_info(inode->i_cdev);
	// vacc_dev = container_of(file_info, struct vastai_cdev, file_info);

	struct vastai_cdev_data *vastai_cdev_data = (struct vastai_cdev_data *)filp->private_data;
	struct vastai_cdev *vacc_dev = (struct vastai_cdev *)vastai_cdev_data->vacc_dev;

	if (vacc_dev->mmap.pid == vastai_cdev_data->pid) {
		mutex_lock(&vacc_dev->mmap.mutex);
		if (mmap_initializing == vacc_dev->mmap.state) {
			vacc_dev->mmap.state = mmap_idle;
			wake_up_interruptible(&vacc_dev->mmap.wait);
		}
		mutex_unlock(&vacc_dev->mmap.mutex);
		// pr_debug("test module cdev mmap reset mmap_idle\n");
	}

	vastai_remove_dev_process(vacc_dev, vastai_cdev_data->pid);

	kfree(filp->private_data);

	// mutex_lock(&vacc_dev->process_mutex);
	// list_for_each_entry (process, &vacc_dev->process_head, node) {
	// 	if (process->pid == pid) {
	// 		list_del(&process->node);
	// 		kfree(process);
	// 		break;
	// 	}
	// }
	// mutex_unlock(&vacc_dev->process_mutex);

	// vastai_cdev_fasync(-1, filp, 0);

	// pr_debug("test module cdev release, pid %d\n", task_tgid_nr(current));
	return 0;
}

static const struct file_operations fop = {
	.owner = THIS_MODULE,
	.llseek = vastai_cdev_llseek,
	.read = vastai_cdev_read,
	.write = vastai_cdev_write,
	.poll = vastai_cdev_poll,
	.unlocked_ioctl = vastai_cdev_ioctl,
	.mmap = vastai_cdev_mmap,
	.open = vastai_cdev_open,
	.release = vastai_cdev_release,
	.fasync = vastai_cdev_fasync,
};

static int vacc_dev_init(struct vastai_cdev *cdev)
{
	u32 i = 0;
        u32 die_id = 0;
	// u64 size_video = VIDEO_SIZE; // real video size
	// u64 size_max = 0; // size of video + share
	struct vastai_cdev *vacc_dev = cdev;
	// struct vastai_soc soc = vacc_dev->soc;
	struct vastai_soc *soc = (struct vastai_soc *)&(cdev->soc);
	struct vastai_zone *zone = NULL;
	struct vastai_dlc *dlc = NULL;
	struct vastai_ssram *ssram = NULL;
	struct vastai_bar *bars = NULL;
	struct vastai_pci_info *priv = NULL;
	struct vastai_odma *odmas = NULL;
	struct vastai_vdsp *vdsp = NULL;

	priv = vacc_dev->pci_info;
	if (!priv)
		goto out;
	vacc_dev->ai_version = AI_DRIVER_VERSION;
        die_id = vastai_pci_get_die_id(priv, cdev->die_index);

	soc->harvest_core_flag =
		vastai_get_dlc_cores_info(priv, cdev->die_index);
	// harvest core info should be used after vastai_get_dlc_cores_info
	if (0 == priv->harvest_info[die_id].type) {
		soc->type = VASTAI_SV100;
		soc->enabled_core_num = 8;
	} else if (1 == priv->harvest_info[die_id].type ||
		   4 == priv->harvest_info[die_id].type) {
		soc->type = VASTAI_SV100_HV;
		soc->enabled_core_num = 4;
	} else {
		soc->type = VASTAI_DEV_COUNT; //invalid device type
		soc->enabled_core_num = 0;
	}

	soc->vacc_id = vastai_get_vacc_id(cdev->pci_info, cdev->die_index);
	soc->bar_num = 2; //bar0 for CSR, bar1 for DDR
	soc->zone_num = zone_count;
	soc->vdsp_num = 4;
	soc->core_num = 8;
	soc->ssram_num = 8;
	soc->odma_num = 2;
	soc->odma_snum = 8;
	soc->odma_bnum = 16;
	soc->ctrans_bnum = 16;
	soc->wdma_bud = WDMA_BOUND; // 32M
	soc->odma_bud = ODMA_BOUND;
	soc->throttle_max_bound = 90;
	soc->capacity_ratio = 115;
	soc->capacity_base = 100;
	soc->alig_ssram = 128;
	soc->alig_wdbuf_conv = 128;
	soc->alig_wdbuf_fc_gemm = 256;
	soc->wdbuf_broadcast_base = 0x40000000;
	soc->ddr_total_size = vastai_pci_info_get_ddr_attr(vacc_dev->pci_info);
	soc->ddr_2_bit_ecc_start = 0x815200000;

	// init zones
	zone = (struct vastai_zone *)kzalloc(
		sizeof(struct vastai_zone) * soc->zone_num, GFP_KERNEL);
	if (!zone)
		goto out;
	vacc_dev->zone = zone;
	zone->id = zone_stream;
	zone->zone_type = zone_stream;
	zone->alig_ddr_start = ALIG_CSRAM_START;
	zone->entry = STREAM_ENTRY;
	zone->bank_num = 1;
	zone->bank[0].base = STREAM_BASE;
	zone->bank[0].size = STREAM_SIZE;
	zone++;
	zone->id = zone_model;
	zone->zone_type = zone_model;
	zone->alig_ddr_start = ALIG_DDR_START;
	zone->entry = MODEL_ENTRY;
	zone->bank_num = 1;
	zone->bank[0].base = MODEL_BASE;
	zone->bank[0].size = MODEL_SIZE;
	zone++;
	zone->id = zone_operator;
	zone->zone_type = zone_operator;
	zone->alig_ddr_start = ALIG_OPERATOR_START;
	zone->entry = OPERATOR_ENTRY;
	zone->bank_num = 1;
	zone->bank[0].base = OPERATOR_BASE;
	zone->bank[0].size = OPERATOR_SIZE;
	zone++;
	zone->id = zone_share;
	zone->zone_type = zone_share;
	zone->alig_ddr_start = ALIG_DDR_START;
	zone->entry = 0;
	zone->bank_num = 3;
	zone->bank[0].base = SHARE_FRONT_BASE;
	zone->bank[0].size = SHARE_FRONT_SIZE;
	zone->bank[1].base = SHARE_INTER_BASE;
	zone->bank[1].size = SHARE_INTER_SIZE;
	zone->bank[2].base = SHARE_BACK_BASE;
	zone->bank[2].size =
		soc->ddr_total_size - (SHARE_BACK_BASE - DDR_BASE_ADDR);
	if ((SHARE_BACK_BASE - DDR_BASE_ADDR) > soc->ddr_total_size)
		goto out;

	// init vdsp
	vdsp = (struct vastai_vdsp *)kzalloc(
		sizeof(struct vastai_vdsp) * soc->vdsp_num, GFP_KERNEL);
	if (!vdsp)
		goto out;
	vacc_dev->vdsp = vdsp;
	for (i = 0; i < soc->vdsp_num; i++) {
		vdsp->id = i;
		vdsp->size = OPERATOR_SIZE;
		vdsp->base = OPERATOR_BASE + i * OPERATOR_OFFSET;
		vdsp++;
	}

	// init dlc
	dlc = (struct vastai_dlc *)kzalloc(
		sizeof(struct vastai_dlc) * soc->core_num, GFP_KERNEL);
	if (!dlc)
		goto out;
	vacc_dev->dlc = dlc;
	for (i = 0; i < soc->core_num; i++) {
		dlc->id = i;
		dlc->wdma_fw_entry = 0x400;
		dlc->csr_base = 0x700000 + 0x100000 * i;
		dlc->csr_size = MB; // 1MB
		dlc->wdbuf.bank_size = 128 * KB;
		dlc->wdbuf.base = 0x40800000 + 0x800000 * i;
		if (i < 4) {
			dlc->core_flag = 1; // big core
			dlc->wdbuf.bank_num = 32;
			dlc->wdbuf.size = 4 * MB; // 4M
		} else {
			dlc->core_flag = 0; // small core
			dlc->wdbuf.bank_num = 16;
			dlc->wdbuf.size = 2 * MB; // 2M
		}
		dlc++;
	}

	// init ssram
	ssram = (struct vastai_ssram *)kzalloc(
		sizeof(struct vastai_ssram) * soc->ssram_num, GFP_KERNEL);
	if (!ssram)
		goto out;
	vacc_dev->ssram = ssram;
	for (i = 0; i < soc->ssram_num; i++) {
		ssram->id = i;
		ssram->base = 0x50000000 + 0x1000000 * i;
		ssram->size = 5 * MB; // 5M
		ssram++;
	}

	// init bars
	bars = (struct vastai_bar *)kzalloc(
		sizeof(struct vastai_bar) * soc->bar_num, GFP_KERNEL);
	if (!bars)
		goto out;
	vacc_dev->bars = bars;
	// classified by die id
	bars[0].id = 0;
	bars[1].id = 1;

	switch (vacc_dev->die_id) {
	case 0:
		// bars[0].ddr_base =
		// 	(u64)(VASTAI_PCIE_BAR0_BASE0_IN & 0xFFFFFFFFF);
		// bars[0].size = (u64)(priv->bar[0].mmio_len);
		// bars[0].base = (u64)(priv->bar[0].mmio_start);
		if (priv->bar[VASTAI_PCI_BAR2].minish_bar_size) {
			bars[1].ddr_base =
				(u64)(VASTAI_PCIE_BAR2_BASE0_IN & 0xFFFFFFFFF);
			bars[1].size = (u64)(priv->bar[2].mmio_len);
			bars[1].base = (u64)(priv->bar[2].mmio_start);
		} else {
			// bars[0].ddr_base =
			// 	(u64)(priv->bar[0].at_addr & 0xFFFFFFFFF);
			// bars[0].size = (u64)(priv->bar[0].mmio_len);
			// bars[0].base = (u64)(priv->bar[0].mmio_start);
			bars[1].ddr_base =
				(u64)(priv->bar[2].at_addr & 0xFFFFFFFFF);
			// bars[1].size = (u64)(priv->bar[2].mmio_len);
			bars[1].size = (u64)(VD_MMAP_SIZE);
			bars[1].base = (u64)(priv->bar[2].mmio_start);
		}
		break;
	case 1:
		// bars[0].ddr_base =
		// 	(u64)(VASTAI_PCIE_BAR1_BASE0_IN & 0xFFFFFFFFF);
		// bars[0].size = (u64)(priv->bar[1].mmio_len / 4);
		// bars[0].base = (u64)(priv->bar[1].mmio_start +
		// 			 (vacc_dev->die_id - 1) * bars[0].size);
		if (priv->bar[VASTAI_PCI_BAR4].minish_bar_size) {
			bars[1].ddr_base =
				(u64)(VASTAI_PCIE_BAR4_BASE0_IN & 0xFFFFFFFFF);
			bars[1].size = (u64)(priv->bar[4].mmio_len / 4);
			bars[1].base =
				(u64)(priv->bar[4].mmio_start +
				      (vacc_dev->die_id - 1) * bars[1].size);
		} else {
			// bars[0].ddr_base =
			// 	(u64)(priv->bar[1].at_addr & 0xFFFFFFFFF);
			// bars[0].size = (u64)(priv->bar[1].mmio_len);
			// bars[0].base = (u64)(priv->bar[1].mmio_start;
			bars[1].ddr_base =
				(u64)(priv->bar[4].at_addr & 0xFFFFFFFFF);
			// bars[1].size = (u64)(priv->bar[4].mmio_len);
			bars[1].size = (u64)(VD_MMAP_SIZE);
			bars[1].base = (u64)(priv->bar[4].mmio_start);
		}
		break;
	case 2:
		// bars[0].ddr_base =
		// 	(u64)(VASTAI_PCIE_BAR1_BASE1_IN & 0xFFFFFFFFF);
		// bars[0].size = (u64)(priv->bar[1].mmio_len / 4);
		// bars[0].base = (u64)(priv->bar[1].mmio_start +
		// 			 (vacc_dev->die_id - 1) * bars[0].size);
		if (priv->bar[VASTAI_PCI_BAR4].minish_bar_size) {
			bars[1].ddr_base =
				(u64)(VASTAI_PCIE_BAR4_BASE1_IN & 0xFFFFFFFFF);
			bars[1].size = (u64)(priv->bar[4].mmio_len / 4);
			bars[1].base =
				(u64)(priv->bar[4].mmio_start +
				      (vacc_dev->die_id - 1) * bars[1].size);
		}
		break;
	case 3:
		// bars[0].ddr_base =
		// 	(u64)(VASTAI_PCIE_BAR1_BASE2_IN & 0xFFFFFFFFF);
		// bars[0].size = (u64)(priv->bar[1].mmio_len / 4);
		// bars[0].base = (u64)(priv->bar[1].mmio_start +
		// 			 (vacc_dev->die_id - 1) * bars[0].size);
		if (priv->bar[VASTAI_PCI_BAR4].minish_bar_size) {
			bars[1].ddr_base =
				(u64)(VASTAI_PCIE_BAR4_BASE2_IN & 0xFFFFFFFFF);
			bars[1].size = (u64)(priv->bar[4].mmio_len / 4);
			bars[1].base =
				(u64)(priv->bar[4].mmio_start +
				      (vacc_dev->die_id - 1) * bars[1].size);
		}
		break;
	default:
		// return -EINVAL;
		break;
	}

	if (VD_MMAP_BASE < vacc_dev->bars[1].ddr_base ||
	    VD_MMAP_SIZE > vacc_dev->bars[1].size)
		goto out;
	soc->video_phys_addr = vacc_dev->bars[1].base + VD_MMAP_BASE -
			       vacc_dev->bars[1].ddr_base;
	soc->video_size = VD_MMAP_SIZE;
	soc->video_soc_addr = VD_MMAP_BASE;

	// init odmas
	odmas = (struct vastai_odma *)kzalloc(
		sizeof(struct vastai_odma) * soc->odma_num, GFP_KERNEL);
	if (!odmas)
		goto out;
	vacc_dev->odmas = odmas;
	for (i = 0; i < soc->odma_num; i++) {
		odmas->id = i;
		odmas->base = 0xF00000 + 0x100000 * i;
		odmas->size = MB; // 1M
		odmas++;
	}

	return 0;

out:
	// kfree(&soc);
	kfree(vacc_dev->zone);
	kfree(vacc_dev->dlc);
	kfree(vacc_dev->ssram);
	kfree(vacc_dev->bars);
	kfree(vacc_dev->odmas);
	kfree(vacc_dev->vdsp);
	return -ENOMEM;
}

int vacc_init(void *pcie_dev, unsigned int die_index)
{
	int ret = 0;
	union die_index_data did = { .val = (u32)die_index };
	struct vastai_cdev *vacc_dev = NULL;
	struct vastai_pci_info *pci_info = NULL;
	size_t mmap_size = 0;
	struct mmap_page *mmap_page = NULL;
	struct mmap_page *mmap_page_temp = NULL;
	if (!pcie_dev) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "[vaerr] %s: pcie_dev is NULL\n", __func__);
		return -EINVAL;
	}

	/* allocate vastai_cdev device data */
	vacc_dev = kzalloc(sizeof(struct vastai_cdev), GFP_KERNEL);
	if (!vacc_dev) {
		VASTAI_PCI_ERR(
			NULL, DUMMY_DIE_ID,
			"[vaerr] %s: failed to kzalloc vastai_cdev device\n",
			__func__);
		return -ENOMEM;
	}

	pci_info = (struct vastai_pci_info *)pcie_dev;
	// vacc_dev->dev_id = pci_info->dev_id;
	vacc_dev->pci_info = pci_info;
	vacc_dev->die_id = did.die_id;
	vacc_dev->dev_id = did.dev_id;
	vacc_dev->die_index = die_index;

	INIT_LIST_HEAD(&vacc_dev->mmap.head);

	ret = vastai_pci_transfer_init(vacc_dev);
	if (ret)
		goto out_free_dev;

	ret = vastai_file_create(&vacc_dev->file_info, pcie_dev, &fop, "vacc%u",
				 did.seq_num);
	if (ret) {
		goto out_free_dev;
	}

	// mutex_init(&vacc_dev->process_mutex);
	// INIT_LIST_HEAD(&vacc_dev->process_head);

	init_waitqueue_head(&vacc_dev->r_wait);
	init_waitqueue_head(&vacc_dev->w_wait);
	init_waitqueue_head(&vacc_dev->wait_runsteam);

	ret = vastai_ai_callback_reg(pcie_dev, vacc_dev);
	if (ret)
		goto out_unreg_class;

	ret = vacc_dev_init(vacc_dev);
	if (ret)
		goto out_unreg_class;

	init_waitqueue_head(&vacc_dev->mmap.wait);
	mutex_init(&vacc_dev->mmap.mutex);
	mmap_size =
		ALIGN(MMAP_SIZE, PAGE_SIZE * ((size_t)1 << (MAX_ORDER - 1)));
	do {
		mmap_page = kzalloc(sizeof(struct mmap_page), GFP_KERNEL);
		if (!mmap_page)
			goto out_unreg_class;
		mmap_page->page =
			alloc_pages(GFP_KERNEL | __GFP_ZERO, (MAX_ORDER - 1));
		if (!mmap_page->page) {
			kfree(mmap_page);
			goto out_unreg_class;
		}
		list_add_tail(&mmap_page->node, &vacc_dev->mmap.head);
		mmap_size -= PAGE_SIZE * ((size_t)1 << (MAX_ORDER - 1));
	} while (mmap_size);
	mmap_page =
		list_first_entry(&vacc_dev->mmap.head, struct mmap_page, node);
	vacc_dev->mmap.addr = (u64)page_to_phys(mmap_page->page);
	vacc_dev->mmap.size =
		ALIGN(MMAP_SIZE, PAGE_SIZE * ((size_t)1 << (MAX_ORDER - 1)));

	// add to list
	INIT_LIST_HEAD(&vacc_dev->vacc_dev_node);
	list_add_tail(&vacc_dev->vacc_dev_node, &pci_info->vacc_dev_head);

	// pr_debug("test module install information:\n");
	// pr_debug("test module install VACC_RUN_MODEL:0x%x\n", VACC_RUN_MODEL);
	// pr_debug("test module install VACC_RUN_STREAM:0x%x\n", VACC_RUN_STREAM);
	// pr_debug("test module install VACC_LOAD_MODEL:0x%x\n", VACC_LOAD_MODEL);
	// pr_debug("test module install VACC_DESTROY_MODEL:0x%x\n", VACC_DESTROY_MODEL);
	// pr_debug("test module install VACC_LOAD_OPERATOR:0x%x\n", VACC_LOAD_OPERATOR);
	// pr_debug("test module install VACC_GET_CURRENT_PID:0x%x\n", VACC_GET_CURRENT_PID);
	// pr_debug("test module install VACC_GET_PROCESS:0x%x\n", VACC_GET_PROCESS);
	// pr_debug("test module install VACC_RELEASE_PROCESS:0x%x\n", VACC_RELEASE_PROCESS);
	// pr_debug("test module install VACC_GET_OUT_DESC:0x%x\n", VACC_GET_OUT_DESC);
	// pr_debug("test module install VACC_CP_DIE2DIE:0x%x\n", VACC_CP_DIE2DIE);
	// pr_debug("test module install VACC_ECC2BIT:0x%x\n", VACC_ECC2BIT);
	// pr_debug("test module install VACC_RD_VERSION:0x%x\n", VACC_RD_VERSION);
	// pr_debug("test module install VACC_RD_FW_STATUS:0x%x\n", VACC_RD_FW_STATUS);
	// pr_debug("test module install VACC_RD_SOC:0x%x\n", VACC_RD_SOC);
	// pr_debug("test module install VACC_RD_ZONE:0x%x\n", VACC_RD_ZONE);
	// pr_debug("test module install VACC_RD_VDSP:0x%x\n", VACC_RD_VDSP);
	// pr_debug("test module install VACC_RD_DLC:0x%x\n", VACC_RD_DLC);
	// pr_debug("test module install VACC_RD_SSRAM:0x%x\n", VACC_RD_SSRAM);
	// pr_debug("test module install VACC_RD_ODAM:0x%x\n", VACC_RD_ODAM);
	// pr_debug("test module install VACC_RD_BAR:0x%x\n", VACC_RD_BAR);
	// pr_debug("test module install VACC_DDR_OVERFLOW:0x%x\n", VACC_DDR_OVERFLOW);
	// pr_debug("test module install VACC_MMAP_INIT:0x%x\n", VACC_MMAP_INIT);
	return 0; // success

out_unreg_class:
	vastai_file_destroy(&vacc_dev->file_info);
out_free_dev:
	list_for_each_entry_safe (mmap_page, mmap_page_temp,
				  &vacc_dev->mmap.head, node) {
		list_del(&mmap_page->node);
		__free_pages(mmap_page->page, (MAX_ORDER - 1));
		kfree(mmap_page);
	}
	kfree(vacc_dev);
	VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
		       "[vaerr] %s: driver initialisation failed\n", __FILE__);
	return ret; // error
}

void vacc_exit(void *pcie_dev)
{
	struct vastai_cdev *vacc_dev = NULL;
	struct vastai_cdev *vacc_dev_temp = NULL;
	// struct process_node *process = NULL;
	// struct process_node *process_temp = NULL;
	struct mmap_page *mmap_page = NULL;
	struct mmap_page *mmap_page_temp = NULL;

	list_for_each_entry_safe (
		vacc_dev, vacc_dev_temp,
		&((struct vastai_pci_info *)pcie_dev)->vacc_dev_head,
		vacc_dev_node) {
		vastai_ai_callback_unreg(pcie_dev, vacc_dev);
		list_del(&vacc_dev->vacc_dev_node);
		vastai_file_destroy(&vacc_dev->file_info);
		// kfree(vacc_dev->soc);
		kfree(vacc_dev->zone);
		kfree(vacc_dev->dlc);
		kfree(vacc_dev->ssram);
		kfree(vacc_dev->bars);
		kfree(vacc_dev->odmas);
		kfree(vacc_dev->vdsp);
		// mutex_lock(&vacc_dev->process_mutex);
		// list_for_each_entry_safe (process, process_temp,
		// 			  &vacc_dev->process_head, node) {
		// 	list_del(&process->node);
		// 	kfree(process);
		// }
		// mutex_unlock(&vacc_dev->process_mutex);
		list_for_each_entry_safe (mmap_page, mmap_page_temp,
					  &vacc_dev->mmap.head, node) {
			list_del(&mmap_page->node);
			__free_pages(mmap_page->page, (MAX_ORDER - 1));
			kfree(mmap_page);
		}
		vastai_pci_transfer_exit(vacc_dev);
		kfree(vacc_dev);
	}
}

// module_init(vacc_init);
// module_exit(vacc_exit);

// MODULE_AUTHOR("Guoliang Li <guoliang.li@vastaitech.com>");
// MODULE_LICENSE("Dual BSD/GPL");
// MODULE_DESCRIPTION("vacc device driver");
