#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/list.h>
#include <linux/printk.h>
#include <linux/slab.h>
#include <linux/completion.h>
#include <linux/sched/signal.h>
#include <linux/signal.h>
#include <linux/uaccess.h>
#include <linux/workqueue.h>

#include "vastai_common.h"

static void vastai_ai_out_work(struct work_struct *work)
{
	int is_find = 0, is_insert = 0;
	struct vastai_cdev *va_dev = NULL;
	struct dev_process *dev_process = NULL;
	struct dev_out_desc *out_desc = NULL;

	out_desc = container_of(work, struct dev_out_desc, work);
	va_dev = (struct vastai_cdev *)out_desc->ctxt;

	if (out_desc->context_id >> 31) {
		wake_up_interruptible(&va_dev->wait_runsteam);
		VASTAI_PCI_DBG(NULL, DUMMY_DIE_ID,
			       "%s: wake up va_dev->wait_runsteam\n", __func__);
	}

	mutex_lock(&va_dev->dev_trans_ai->mutex);
	list_for_each_entry (dev_process, &va_dev->dev_trans_ai->head, node) {
		if (dev_process->pid == out_desc->pid) {
			if (out_desc->context_id >> 31) {
				if (unlikely(
					    atomic_dec_if_positive(
						    &dev_process->run_ref_num) < 0))
					VASTAI_PCI_ERR(
						NULL, DUMMY_DIE_ID,
						"%s: pid:%u failed to run_ref_num--\n",
						__func__, dev_process->pid);
				if (unlikely(
					    atomic_inc_return(
						    &dev_process->out_ref_num) > VASTAI_MAX_OUT_COUNT))
					VASTAI_PCI_ERR(
						NULL, DUMMY_DIE_ID,
						"%s: pid:%u failed to out_ref_num++\n",
						__func__, dev_process->pid);
			}
			// if current proc_state 0 >> 1, ssert delays work queue to execute after this
			if (likely(!dev_process->proc_state)) {
				mutex_lock(&dev_process->mutex);
				list_add_tail(&out_desc->node, &dev_process->head);
				mutex_unlock(&dev_process->mutex);
				wake_up_interruptible(&dev_process->wait);
				is_insert = 1;
			} else {
				if (unlikely(!atomic_read(&dev_process->run_ref_num))) {
					list_del(&dev_process->node);
				}
			}
			is_find = 1;
			break;
		}
	}
	mutex_unlock(&va_dev->dev_trans_ai->mutex);

	if (unlikely(!is_insert)) {
		if (unlikely(!is_find))
			VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
				       "%s: pid:%u failed to get process\n",
				       __func__, out_desc->pid);
		kfree(out_desc);
	}
}

static void vastai_ai_del_work(struct work_struct *work)
{
	struct vastai_cdev *va_dev = NULL;
	struct dev_process *dev_process = NULL;
	struct dev_out_desc *out_desc = NULL, *out_desc_temp = NULL;

	dev_process = container_of((struct delayed_work *)work,
				   struct dev_process, work);
	va_dev = (struct vastai_cdev *)dev_process->ctxt;

	// note: proc_state == 1
	mutex_lock(&va_dev->dev_trans_ai->mutex);
	if ((dev_process->node).next != LIST_POISON1 || (dev_process->node).prev != LIST_POISON2) {
		list_del(&dev_process->node);
	}
	if (list_empty(&va_dev->dev_trans_ai->head)) {
		va_dev->mmap.state = mmap_idle;
	}
	mutex_unlock(&va_dev->dev_trans_ai->mutex);

	if (unlikely(atomic_read(&dev_process->run_ref_num)))
		VASTAI_PCI_INFO(NULL, DUMMY_DIE_ID,
				"%s: pid:%u dev_process run_ref_num > 0\n",
				__func__, dev_process->pid);
	if (atomic_read(&dev_process->out_ref_num)) {
		// VASTAI_PCI_INFO(
		// 	NULL, DUMMY_DIE_ID,
		// 	"%s: pid:%u dev_process out_ref_num > 0\n",
		// 	__func__, dev_process->pid);
		mutex_lock(&dev_process->mutex);
		list_for_each_entry_safe (out_desc, out_desc_temp,
					  &dev_process->head, node) {
			list_del(&out_desc->node);
			kfree(out_desc);
		}
		mutex_unlock(&dev_process->mutex);
	}
	kfree(dev_process);
}

static int vastai_ai_trans_call(void *pci_dev, u32 die_index, int irq,
				void *ctxt, void *out)
{
	struct vastai_cdev *va_dev = (struct vastai_cdev *)ctxt;
	struct dev_out_desc *out_desc =
		kzalloc(sizeof(struct dev_out_desc), GFP_ATOMIC);

	if (!out_desc) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "%s: can not get buf\n",
			       __func__);
		BUG_ON(!out_desc);
	}
	memcpy(out_desc, out, sizeof(struct out_desc));
	// done
	out_desc->ctxt = ctxt;
	INIT_WORK(&out_desc->work, vastai_ai_out_work);
	queue_work(va_dev->dev_trans_ai->workqueue, &out_desc->work);
	// schedule_work(&out_desc->work);

	return 0;
}

// void vastai_pci_resource_recycling(struct work_struct *work)
// {
// 	struct out_desc *out_desc = NULL;
// 	struct vastai_cdev *va_dev = NULL;
// 	out_desc = container_of(work, struct out_desc, dma_work);
// 	va_dev = (struct vastai_cdev *)out_desc->ctxt;
// 	if(va_dev->fasync){
		// kill_fasync(&va_dev->fasync,SIGIO,POLL_HUP);
//     }
// }
int vastai_map_read(struct vastai_cdev *va_dev, char __user *buffer,
		    size_t lenth, loff_t pos)
{
	int ret = 0;
	void *data = NULL;
	if (!va_dev) {
		return -EINVAL;
	}

	data = kvzalloc(lenth, GFP_KERNEL);
	if (!data) {
		VASTAI_PCI_ERR(
			NULL, DUMMY_DIE_ID,
			"[vaerr] %s: failed to kvzalloc dma_data_struct\n",
			__func__);
		ret = -ENOMEM;
		goto out;
	}
	/* TODO : pls tell pcie driver which pcie device and which die */
	ret = vastai_pci_mem_read(va_dev->pci_info, va_dev->die_index, pos,
				  data, lenth);
	if (ret)
		goto out;
	ret = copy_to_user(buffer, data, lenth);
	if (ret) {
		ret = -EFAULT;
		goto out;
	}

out:
	kvfree(data);
	return ret;
}

size_t vastai_add_read_queue(struct vastai_cdev *va_dev,
			     struct queue_desc *desc)
{
	union core_bitmap core_id = { .val = 0x0 };
	int ret = 0;
	int i;
	void *vir_addr = desc->buffer;
	u64 dev_addr = desc->pos;
	size_t dma_lenth = desc->lenth;
	struct vastai_dmadesc *dmadesc;
	int dma_desc_unit = 0;
	pid_t pid = task_tgid_nr(current);

	dma_desc_unit = DIV_ROUND_UP(desc->lenth, VASTAI_MAX_DMA_BUF);

	dmadesc = kvzalloc(sizeof(struct vastai_dmadesc) * dma_desc_unit,
			   GFP_KERNEL);
	if (!dmadesc)
		return -ENOMEM;

     if(iommu_is_enable(va_dev->pci_info) && ((u64)vir_addr%PAGE_SIZE == 0)){
            ret = vastai_pci_dma_transfer_by_uaddr(
            va_dev->pci_info,
            va_dev->die_index,
            core_id,
            0,
            dev_addr,
            (u64)vir_addr,
            dma_lenth);

     } else {

	    for (i = 0; dma_lenth > 0; i++) {
		dmadesc[i].is_src_dma_addr = 0;
		dmadesc[i].is_host_to_dev = 0;
		dmadesc[i].is_src_not_user_mem = 0;
		dmadesc[i].host_addr.vir_addr = vir_addr;
		dmadesc[i].dev_addr = dev_addr;

		if (dma_lenth < VASTAI_MAX_DMA_BUF) {
			dmadesc[i].dma_lenth = dma_lenth;
		} else {
			dmadesc[i].dma_lenth = VASTAI_MAX_DMA_BUF;
		}

		vir_addr += dmadesc[i].dma_lenth;
		dev_addr += dmadesc[i].dma_lenth;
		dma_lenth -= dmadesc[i].dma_lenth;
	    }
	    ret = vastai_pci_dma_transfer_sync_pid(va_dev->pci_info, va_dev->die_index,
					   core_id, dmadesc, i, pid);
     }

	kvfree(dmadesc);
	if (ret < 0) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "[vaerr] %s: dma_transfer_error %d\n",
			       __func__, ret);
		return ret;
	} else {
		return desc->lenth;
	}
}

int vastai_map_write(struct vastai_cdev *va_dev, const char __user *buffer,
		     size_t lenth, loff_t pos)
{
	int ret = 0;
	void *data = NULL;
	if (!va_dev) {
		return -EINVAL;
	}

	data = kvzalloc(lenth, GFP_KERNEL);
	if (!data) {
		VASTAI_PCI_ERR(
			NULL, DUMMY_DIE_ID,
			"[vaerr] %s: failed to kvzalloc dma_data_struct\n",
			__func__);
		ret = -ENOMEM;
		goto out;
	}
	ret = copy_from_user(data, buffer, lenth);
	if (ret) {
		ret = -EFAULT;
		goto out;
	}
	/* TODO : pls tell pcie driver which pcie device and which die */
	ret = vastai_pci_mem_write(va_dev->pci_info, va_dev->die_index, pos,
				   data, lenth);
	if (ret)
		goto out;

out:
	kvfree(data);
	return ret;
}

size_t vastai_add_write_queue(struct vastai_cdev *va_dev,
			      struct queue_desc *desc)
{
	union core_bitmap core_id = { .val = 0x0 };
	int ret = 0;
	int i;
	void *vir_addr = desc->buffer;
	u64 dev_addr = desc->pos;
	size_t dma_lenth = desc->lenth;
	struct vastai_dmadesc *dmadesc;
	int dma_desc_unit = 0;
	pid_t pid = task_tgid_nr(current);

	dma_desc_unit = DIV_ROUND_UP(desc->lenth, VASTAI_MAX_DMA_BUF);

	dmadesc = kvzalloc(sizeof(struct vastai_dmadesc) * dma_desc_unit,
			   GFP_KERNEL);
	if (!dmadesc)
		return -ENOMEM;

    if(iommu_is_enable(va_dev->pci_info) && ((u64)vir_addr%PAGE_SIZE == 0)){
        ret = vastai_pci_dma_transfer_by_uaddr(
            va_dev->pci_info,
            va_dev->die_index,
            core_id,
            1,
            dev_addr,
            (u64)vir_addr,
            dma_lenth);

    } else {
	    for (i = 0; dma_lenth > 0; i++) {

		dmadesc[i].is_src_dma_addr = 0;
		dmadesc[i].is_host_to_dev = 1;
		dmadesc[i].is_src_not_user_mem = 0;
		dmadesc[i].host_addr.vir_addr = vir_addr;
		dmadesc[i].dev_addr = dev_addr;

		if (dma_lenth < VASTAI_MAX_DMA_BUF) {
			dmadesc[i].dma_lenth = dma_lenth;
		} else {
			dmadesc[i].dma_lenth = VASTAI_MAX_DMA_BUF;
		}

		vir_addr += dmadesc[i].dma_lenth;
		dev_addr += dmadesc[i].dma_lenth;
		dma_lenth -= dmadesc[i].dma_lenth;
	    }
	    ret = vastai_pci_dma_transfer_sync_pid(va_dev->pci_info, va_dev->die_index,
					   core_id, dmadesc, i, pid);
    }

	kvfree(dmadesc);
	if (ret < 0) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "[vaerr] %s: dma_transfer_error %d\n",
			       __func__, ret);
		return ret;
	} else {
		return desc->lenth;
	}
}

int vastai_cmd_run(struct vastai_cdev *va_dev, union vastai_cmd *run, u32 cmd)
{
	int ret = 0;
	int is_interrupt_come = 0;
	struct buf_desc cmd_desc;
	struct dev_process *dev_process = NULL, *dev_process_temp = NULL;
	union core_bitmap core_id;
	pid_t pid = task_tgid_nr(current);

	if (!va_dev || !run) {
		ret = -EINVAL;
		goto out;
	}

	switch (cmd) {
	case CMD_LOAD_OPERATOR:
		cmd_desc.data.cmd_model_desc.dev =
			0x0F << (run->run_load_model.dev + 8);
		core_id.val = cmd_desc.data.cmd_model_desc.dev;
		break;
	case CMD_RUN_STREAM:
		cmd_desc.data.cmd_model_desc.dev =
			0x01 << (run->run_load_model.dev + 8);
		core_id.val = cmd_desc.data.cmd_model_desc.dev;
		break;
	case CMD_RUN_MODEL:
		cmd_desc.data.cmd_model_desc.dev = 0x01
						   << run->run_load_model.dev;
		core_id.val = 0x1; /* CMCU only one */
		break;
	case CMD_LOAD_MODEL:
		cmd_desc.data.cmd_model_desc.dev = 0x01
						   << run->run_load_model.dev;
		core_id.val = 0x1; /* CMCU only one */
		goto out;
		break;
	case CMD_DESTROY_MODEL:
		// pr_info("CMD_DESTROY_MODEL is not supported now!\n");
		goto out;
	default:
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "cmd 0x%x is not supported!\n", cmd);
		ret = -EINVAL;
		goto out;
	}
	cmd_desc.data.cmd_model_desc.cmd = cmd;
	cmd_desc.data.cmd_model_desc.pid = run->run_load_model.pid;
	cmd_desc.data.cmd_model_desc.model_id = run->run_load_model.model_id;
	cmd_desc.data.cmd_model_desc.data_id = run->run_load_model.data_id;
	cmd_desc.data.cmd_model_desc.model_addr =
		run->run_load_model.model_addr;
	cmd_desc.data.cmd_model_desc.inout_addr =
		run->run_load_model.inout_addr;

	if (CMD_RUN_STREAM == cmd || CMD_RUN_MODEL == cmd) {
		mutex_lock(&va_dev->dev_trans_ai->mutex);
		list_for_each_entry (dev_process, &va_dev->dev_trans_ai->head,
				     node) {
			if (dev_process->pid == run->run_load_model.pid) {
				dev_process_temp = dev_process;
				break;
			}
		}
		mutex_unlock(&va_dev->dev_trans_ai->mutex);
		if (!dev_process_temp) {
			VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
				       "%s: pid:%u failed to get process\n",
				       __func__, run->run_load_model.pid);
			ret = -EINVAL;
			goto out;
		}
	}

	is_interrupt_come = wait_event_interruptible(
		va_dev->wait_runsteam,
		(ret = vastai_pci_send_msg_pid(va_dev->pci_info, va_dev->die_index,
					   core_id, &cmd_desc,
					   sizeof(cmd_desc), pid)) != -ENOMEM);
	if (is_interrupt_come) {
		// VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "%s: recv interrupte %d\n",
		// 	       __func__, is_interrupt_come);
		goto out;
	}
	if (ret) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "%s: send msg failed %d\n",
			       __func__, ret);
		goto out;
	}
	if (dev_process_temp) {
		if (unlikely(atomic_inc_return(&dev_process_temp->run_ref_num) >
			     VASTAI_MAX_OUT_COUNT))
			VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
				       "%s: pid:%u failed to run_ref_num++\n",
				       __func__, dev_process_temp->pid);
	}

out:
	return ret;
}

static struct dev_out_desc *vastai_get_out_desc(struct dev_process *dev_process)
{
	struct dev_out_desc *out_desc = NULL;

	mutex_lock(&dev_process->mutex);

	out_desc = list_first_entry_or_null(&dev_process->head,
					    struct dev_out_desc, node);
	if (out_desc)
		list_del(&out_desc->node);

	mutex_unlock(&dev_process->mutex);

	return out_desc;
}

int vastai_cmd_get_desc(struct vastai_cdev *va_dev, union vastai_cmd *get_desc)
{
	int ret = 0;
	struct dev_out_desc *out_desc = NULL;
	struct dev_process *dev_process = NULL, *dev_process_temp = NULL;

	if (!get_desc->get_desc.pid) {
		ret = -EINVAL;
		goto out;
	}

	mutex_lock(&va_dev->dev_trans_ai->mutex);
	list_for_each_entry (dev_process, &va_dev->dev_trans_ai->head, node) {
		if (dev_process->pid == get_desc->get_desc.pid) {
			dev_process_temp = dev_process;
			break;
		}
	}
	mutex_unlock(&va_dev->dev_trans_ai->mutex);
	if (unlikely(!dev_process_temp)) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "%s: pid:%u failed to get process\n", __func__,
			       get_desc->get_desc.pid);
		ret = -EINVAL;
		goto out;
	}
	if (unlikely(dev_process_temp->proc_state)) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "%s: pid:%u dev_process state is %d\n", __func__,
			       dev_process_temp->pid,
			       dev_process_temp->proc_state);
		ret = -EINVAL;
		goto out;
	}

	ret = wait_event_interruptible_timeout(
		dev_process_temp->wait,
		(out_desc = vastai_get_out_desc(dev_process_temp)), HZ / 100);
	if (-ERESTARTSYS == ret) {
		// VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "%s: recv interrupte %d\n", __func__, ret);
		goto out;
	}

	if (out_desc) {
		if (out_desc->context_id >> 31) {
			if (unlikely(atomic_dec_if_positive(
					     &dev_process_temp->out_ref_num) <
				     0))
				VASTAI_PCI_ERR(
					NULL, DUMMY_DIE_ID,
					"%s: pid:%u failed to out_ref_num--\n",
					__func__, out_desc->pid);
		}
		get_desc->get_desc.dev = out_desc->dev;
		get_desc->get_desc.pid = out_desc->pid;
		get_desc->get_desc.stream_id = (u32)out_desc->stream_id;
		get_desc->get_desc.data_id = (u32)out_desc->data_id;
		get_desc->get_desc.stream_addr = (u32)out_desc->stream_addr;
		get_desc->get_desc.context_id = (u32)out_desc->context_id;
		get_desc->get_desc.error_code = (u32)out_desc->error_code;
		kfree(out_desc);
		ret = 0;
	} else
		ret = -EAGAIN;

out:
	return ret;
}

int vastai_add_dev_process(struct vastai_cdev *va_dev, pid_t pid)
{
	int ret = 0;
	struct dev_process *dev_process = NULL, *dev_process_temp = NULL;

	mutex_lock(&va_dev->dev_trans_ai->mutex);
	if (list_empty(&va_dev->dev_trans_ai->head)) {
		va_dev->mmap.state = mmap_idle;
	}
	list_for_each_entry (dev_process, &va_dev->dev_trans_ai->head, node) {
		if (dev_process->pid == pid) {
			dev_process_temp = dev_process;
			break;
		}
	}

	if (unlikely(dev_process_temp)) {
		VASTAI_PCI_INFO(NULL, DUMMY_DIE_ID,
			       "%s: repeat add process, pid:%u, state:%d\n",
			       __func__, pid, dev_process_temp->proc_state);
		// ret = -EINVAL;
		atomic_inc_return(&dev_process_temp->ref_num);
		goto out;
	}

	dev_process_temp = kzalloc(sizeof(struct dev_process), GFP_KERNEL);
	if (unlikely(!dev_process_temp)) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "%s: pid:%u failed to kzalloc dev_process\n",
			       __func__, pid);
		ret = -ENOMEM;
		goto out;
	}

	dev_process_temp->ctxt = va_dev;
	dev_process_temp->pid = pid;
	atomic_set(&dev_process_temp->ref_num, 1);
	atomic_set(&dev_process_temp->run_ref_num, 0);
	atomic_set(&dev_process_temp->out_ref_num, 0);
	mutex_init(&dev_process_temp->mutex);
	init_waitqueue_head(&dev_process_temp->wait);
	INIT_LIST_HEAD(&dev_process_temp->head);
	INIT_DELAYED_WORK(&dev_process_temp->work, vastai_ai_del_work);
	list_add_tail(&dev_process_temp->node, &va_dev->dev_trans_ai->head);

out:
	mutex_unlock(&va_dev->dev_trans_ai->mutex);
	return ret;
}

void vastai_remove_dev_process(struct vastai_cdev *va_dev, pid_t pid)
{
	// int ret;
	struct dev_process *dev_process = NULL, *dev_process_temp = NULL;
	struct dev_out_desc *out_desc = NULL, *out_desc_temp = NULL;

	mutex_lock(&va_dev->dev_trans_ai->mutex);
	list_for_each_entry (dev_process, &va_dev->dev_trans_ai->head, node) {
		if (dev_process->pid == pid) {
			dev_process_temp = dev_process;
			break;
		}
	}

	// TODO video delay add workqueue
	if (unlikely(!dev_process_temp)) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
			       "%s: pid:%u failed to remove process\n",
			       __func__, pid);
		goto out;
	}

	atomic_dec_if_positive(&dev_process_temp->ref_num);
	if(unlikely(atomic_read(&dev_process_temp->ref_num)))
		goto out;

	if (unlikely(atomic_read(&dev_process_temp->run_ref_num))) {
		VASTAI_PCI_INFO(NULL, DUMMY_DIE_ID,
				"%s: pid:%u dev_process run_ref_num > 0\n",
				__func__, dev_process_temp->pid);
		dev_process_temp->proc_state = 1;
		if (unlikely(!queue_delayed_work(
			    va_dev->dev_trans_ai->workqueue,
			    &dev_process_temp->work,
			    120 * HZ))) { // debug info and release resource if join failed.
			VASTAI_PCI_INFO(
				NULL, DUMMY_DIE_ID,
				"%s: pid:%u queue_delayed_work failed.\n",
				__func__, dev_process_temp->pid);
		} else
			goto out;
	}
	if (unlikely(atomic_read(&dev_process_temp->out_ref_num))) {
		// VASTAI_PCI_INFO(NULL, DUMMY_DIE_ID,
		// 		"%s: pid:%u dev_process out_ref_num > 0\n",
		// 		__func__, dev_process_temp->pid);
		mutex_lock(&dev_process_temp->mutex);
		list_for_each_entry_safe (out_desc, out_desc_temp,
					  &dev_process_temp->head, node) {
			list_del(&out_desc->node);
			kfree(out_desc);
		}
		mutex_unlock(&dev_process_temp->mutex);
	}
	list_del(&dev_process_temp->node);
	kfree(dev_process_temp);

out:
	if (list_empty(&va_dev->dev_trans_ai->head)) {
		va_dev->mmap.state = mmap_idle;
	}
	mutex_unlock(&va_dev->dev_trans_ai->mutex);

	// return ret;
}

// int vastai_release_dev_process(struct vastai_cdev *va_dev, pid_t pid)
// {
// 	int ret = 0;
// 	struct dev_process *dev_process = NULL;

// 	mutex_lock(&va_dev->dev_trans_ai->mutex);
// 	list_for_each_entry (dev_process, &va_dev->dev_trans_ai->head, node) {
// 		if (dev_process->pid == pid) {
// 			ret = 1;
// 			break;
// 		}
// 	}
// 	mutex_unlock(&va_dev->dev_trans_ai->mutex);

// 	return ret;
// }

int vastai_ai_data_movement(struct vastai_cdev *va_dev,
			    union vastai_cmd *vastai_cmd)
{
	u32 local_die_id =
		vastai_pci_get_die_id(va_dev->pci_info, va_dev->die_index);
	u32 remote_die_id = UINT_MAX;
	u64 remote_address = 0x0;
	u64 local_address = vastai_cmd->dev_2_dev.local_address;
	int ret;
	u32 len = vastai_cmd->dev_2_dev.len;
	int i;

	for (i = 0; i < va_dev->pci_info->die_num; i++) {
		if (vastai_cmd->dev_2_dev.remote_die ==
			vastai_get_vacc_id(va_dev->pci_info, va_dev->pci_info->dies[i].die_index))
			remote_die_id = vastai_pci_get_die_id(va_dev->pci_info, va_dev->pci_info->dies[i].die_index);

	}
	if (remote_die_id == UINT_MAX) {
		VASTAI_PCI_ERR(va_dev->pci_info, vastai_pci_get_die_id(va_dev->pci_info, va_dev->die_index),
			       "[vaerr] %s: can't found remote_die_seq_id:%d\n",
			       __func__, vastai_cmd->dev_2_dev.remote_die);
		return -EINVAL ;
	}

	VASTAI_PCI_INFO(
		NULL, DUMMY_DIE_ID,
		"local_address 0x%llx, remote_die_index 0x%x, remote_address 0x%llx, len 0x%x\n",
		(u64)vastai_cmd->dev_2_dev.local_address,
		vastai_cmd->dev_2_dev.remote_die,
		(u64)vastai_cmd->dev_2_dev.remote_address,
		vastai_cmd->dev_2_dev.len);

	if (local_die_id + 1 > remote_die_id) {
		VASTAI_PCI_ERR(
			NULL, DUMMY_DIE_ID,
			"[vaerr] %s: die_id is error. local[%d] remote[%d]\n",
			__func__, local_die_id, remote_die_id);
		return -EIO;
	} else {
		remote_address =
			0x1000000000 * (remote_die_id - local_die_id - 1) +
			vastai_cmd->dev_2_dev.remote_address;
	}

	while (len) {
		u32 current_len =
			len <= VASTAI_MAX_DMA_BUF ? len : VASTAI_MAX_DMA_BUF;

		ret = vastai_pci_data_movement(
			va_dev->pci_info, va_dev->die_index, local_address,
			remote_address, current_len,
			vastai_cmd->dev_2_dev.is_local_2_remote);
		if (ret)
			return ret;
		remote_address += current_len;
		local_address += current_len;
		len -= current_len;
	}
	return ret;
}

int vastai_ai_callback_reg(void *pcie_dev, struct vastai_cdev *va_dev)
{
	int ret = 0;
	int vastai_sv_irq[] = AI_IRQ_NUM_ARRAY;
	u32 i = 0;

	for (i = 0; i < sizeof(vastai_sv_irq) / sizeof(vastai_sv_irq[0]); i++) {
		ret = vastai_irq_register((struct vastai_pci_info *)pcie_dev,
					  va_dev->die_index, vastai_sv_irq[i],
					  vastai_ai_trans_call, va_dev);
		if (ret)
			goto out;
	}

out:
	return ret;
}

int vastai_ai_callback_unreg(void *pcie_dev, struct vastai_cdev *va_dev)
{
	int ret = 0;
	int vastai_sv_irq[] = AI_IRQ_NUM_ARRAY;
	u32 i = 0;

	for (i = 0; i < sizeof(vastai_sv_irq) / sizeof(vastai_sv_irq[0]); i++) {
		ret = vastai_irq_unregister((struct vastai_pci_info *)pcie_dev,
					    va_dev->die_index,
					    vastai_sv_irq[i]);
		if (ret)
			goto out;
	}

out:
	return ret;
}

int vastai_pci_transfer_init(struct vastai_cdev *va_dev)
{
	int ret;
	if (!va_dev) {
		return -EINVAL;
	}

	// ai dev send to host
	va_dev->dev_trans_ai = (struct dev_trans *)kzalloc(
		sizeof(struct dev_trans), GFP_KERNEL);
	if (!va_dev->dev_trans_ai) {
		VASTAI_PCI_ERR(
			NULL, DUMMY_DIE_ID,
			"[vaerr] %s: failed to kzalloc va_dev->dev_trans_ai\n",
			__func__);
		ret = -ENOMEM;
		goto out;
	}

	mutex_init(&va_dev->dev_trans_ai->mutex);
	INIT_LIST_HEAD(&va_dev->dev_trans_ai->head);
	va_dev->dev_trans_ai->workqueue =
		alloc_ordered_workqueue("vastai_ai", 0);
	if (!va_dev->dev_trans_ai->workqueue) {
		VASTAI_PCI_ERR(
			NULL, DUMMY_DIE_ID,
			"[vaerr] %s: failed to alloc va_dev->dev_trans_ai->workqueue\n",
			__func__);
		ret = -EAGAIN;
		goto out;
	}

	return 0;

out:
	if (va_dev->dev_trans_ai) {
		kfree(va_dev->dev_trans_ai);
	}
	VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID,
		       "[vaerr] %s: vastai_pci_transfer_init failed\n",
		       __FILE__);
	return ret;
}

void vastai_pci_transfer_exit(struct vastai_cdev *va_dev)
{
	struct dev_out_desc *out_desc, *out_desc_temp;
	struct dev_process *dev_process, *dev_process_temp;

	if (!va_dev) {
		return;
	}

	if (va_dev->dev_trans_ai) {
		// disable insert work_queue, flash work_queue
		mutex_lock(&va_dev->dev_trans_ai->mutex);
		list_for_each_entry_safe (dev_process, dev_process_temp,
					  &va_dev->dev_trans_ai->head, node) {
			if (dev_process->proc_state)
				cancel_delayed_work_sync(&dev_process->work);
		}
		mutex_unlock(&va_dev->dev_trans_ai->mutex);

		flush_workqueue(va_dev->dev_trans_ai->workqueue);
		destroy_workqueue(va_dev->dev_trans_ai->workqueue);

		mutex_lock(&va_dev->dev_trans_ai->mutex);
		list_for_each_entry_safe (dev_process, dev_process_temp,
					  &va_dev->dev_trans_ai->head, node) {
			mutex_lock(&dev_process->mutex);
			list_for_each_entry_safe (out_desc, out_desc_temp,
						  &dev_process->head, node) {
				list_del(&out_desc->node);
				kfree(out_desc);
			}
			list_del(&dev_process->node);
			mutex_unlock(&dev_process->mutex);
			kfree(dev_process);
		}
		mutex_unlock(&va_dev->dev_trans_ai->mutex);
		kfree(va_dev->dev_trans_ai);
	}

	VASTAI_PCI_INFO(NULL, DUMMY_DIE_ID, "%s: uninstall\n", __func__);
}
