#!/bin/bash

module_version=""
cur_version=""
noClean="False"
multiHwType="False"
hwType="0" #default type
target=$(arch)

CUR_PATH=$(readlink -f "$(dirname "$0")")
SRC_ROOT="${CUR_PATH}/.."
OUT_DIR="${SRC_ROOT}"

#DEBUG_EN="-v"

# 0. change to source tree and make clean

cd .. && make 

if [ $(id -u) != 0 ]; then
	echo "ERROR: you must be root to use the shell script!"
	exit 1
fi

while getopts Ch:mv: opt
do
	case "$opt" in
	v) module_version=$OPTARG ;;
	C) noClean="True";;
	m) multiHwType="True";;
	h) hwType=$OPTARG;;
	*) echo "ERROR: Unknown option!"; exit 2 ;;
	esac
done

cd $SRC_ROOT

if [ $noClean == 'True' ]; then
	echo "not make clean"
else
	make clean
fi

if [ $multiHwType == 'True' ]; then
	cd $CUR_PATH
	#make default
	./$0 -C -h 0
	#make hwType == 1
	./$0 -C -h 1
	#make hwType == 2
	./$0 -C -h 2
	#make hwType == 3
	./$0 -C -h 3
fi

#we create dkms.conf auto, avoid git status is dirty after we package
echo 'PACKAGE_NAME="vastai-pci-pcie-dev"' > dkms.conf
echo 'PACKAGE_VERSION="000"' >> dkms.conf
echo 'CLEAN="make clean"' >> dkms.conf
echo "MAKE[0]=\"'make' CONFIG_VASTAI_HW_TYPE=${hwType} KVERSION=\${kernelver}\"" >> dkms.conf
echo 'BUILT_MODULE_NAME[0]="vastai_pci"' >> dkms.conf
echo 'DEST_MODULE_LOCATION[0]="/extra"' >> dkms.conf
echo 'AUTOINSTALL="yes"' >> dkms.conf
echo 'NO_WEAK_MODULES="yes"' >> dkms.conf
# dkms matches the kernel version(≥4.x.x), otherwise skip
echo 'BUILD_EXCLUSIVE_KERNEL="^[4-9].*"' >> dkms.conf

if type lsb_release >/dev/null 2>&1; then
	DISTRIB_ID=$(lsb_release -i -s)
else
	echo "ERROR: lsb_release don't support, please install redhat-lsb-core"
	exit 2
fi

PACKAGE_TYPE=""
case $DISTRIB_ID in
	CentOS)
		PACKAGE_TYPE="rpm"
		# depend tools: rpm-build
		;;
	Ubuntu)
		PACKAGE_TYPE="deb"
		;;
	RedHatEnterprise)
		PACKAGE_TYPE="rpm"
		# depend tools: rpm-build
		;;
	*)
		echo "ERROR: running OS($DISTRIB_ID) don't support make package"
		exit 2
		;;
esac

if [ $module_version ]; then
	VASTAI_DRV_VERSION=$module_version
else
	#default version:  compiled_ver increase 1
	cur_version=$(cat include/vastai_version.h | grep "VASTAI_PCI_DRIVER_VERSION" \
		| awk '{print $3}' | sed 's/[()"]//g')
	cur_major_ver=$(echo ${cur_version} | awk -F '.' '{print $1}')
	cur_minor_ver=$(echo ${cur_version} | awk -F '.' '{print $2}')
	cur_revision_ver=$(echo ${cur_version} | awk -F '.' '{print $3}')
	cur_compiled_ver=$(echo ${cur_version} | awk -F '.' '{print $4}')
	cur_compiled_ver=$(expr $cur_compiled_ver + 1)

	if [ $cur_compiled_ver -ge "100" ]; then
		echo "ERROR: compiled_version is greater than 99, pls modify versoin manually"
		exit 3
	fi

	VASTAI_DRV_VERSION=$(printf "%02d.%02d.%02d.%02d" \
		$cur_major_ver  $cur_minor_ver $cur_revision_ver $cur_compiled_ver)
	echo "vastai pcie driver version change $VASTAI_DRV_VERSION"
fi

#keep branch name is lowercase
GIT_BRANCH_NAME=$(echo $(git rev-parse --abbrev-ref HEAD) | sed 's/[A-Z]/\L&/g')
GIT_COMMIT_HASH=$(git log -1 --format="%h")
GIT_COMMIT_MSG=$(git log -1 --format="%s")
GIT_COMMIT_MSG_FORMAT=$(echo "$GIT_COMMIT_MSG" | sed 's/["]//g')
MKPACKAGE_TIME=$(date +"%Y-%m-%d %H:%M:%S")

PACKAGE_NAME_PREFIX="vastai_pci"
PACKAGE_NAME=$(echo "${PACKAGE_NAME_PREFIX}_${GIT_BRANCH_NAME}_hwtype_${hwType}" | sed 's/_/-/g')
sed -i "s/BuildArch:.*/BuildArch:\t${target}/" ${CUR_PATH}/${PACKAGE_NAME_PREFIX}-dkms-mkrpm.spec


DKMS_SRC_DIR=/usr/src/${PACKAGE_NAME}-${VASTAI_DRV_VERSION}
DKMS_TREE_DIR=/var/lib/dkms/${PACKAGE_NAME}/${VASTAI_DRV_VERSION}
DKMS_SOURCES="dkms.conf
	Makefile
	README.md
	ai_driver
	boot
	file_node
	ep_driver
	fw
	include
	logsys
	tools_driver
	dkms
	video_driver"

echo ""
echo "------MAKE PACKAGE: ${PACKAGE_NAME}/${VASTAI_DRV_VERSION}------"
echo ""
# 0. check dkms version
echo 0.check dkms version
dkms --version > /home/version.txt

while read rows
do
	echo "current ${rows}, this Script need dkms version(>=1.95, <=2.8)"
done < /home/version.txt
rm -f /home/version.txt

# 1. remove old from dkms tree
echo 1.remove old from dkms tree
#added_versions=$(dkms status | grep "${PACKAGE_NAME}," | awk '{print $2}' | \
#	sed  's/,//g' | xargs);
#for ver in $added_versions; do
#	dkms remove ${PACKAGE_NAME}/${ver} --all
#	rm -rf $DEBUG_EN /usr/src/${PACKAGE_NAME}-${ver}
#done
added_versions=$(dkms status -m ${PACKAGE_NAME} -v ${VASTAI_DRV_VERSION})
if [ "$added_versions" ]; then
	dkms remove ${PACKAGE_NAME}/${VASTAI_DRV_VERSION} --all
	rm -rf $DEBUG_EN /usr/src/${PACKAGE_NAME}-${VASTAI_DRV_VERSION}
fi

# 2.copy source code to /usr/src
echo 2.dkms_src_install: cp source_code to $DKMS_SRC_DIR

echo "modify package name of dkms.conf to $PACKAGE_NAME"
sed -i 's/PACKAGE_NAME=.*$/PACKAGE_NAME='"\"${PACKAGE_NAME}\""'/g' \
	dkms.conf
echo "modify version of dkms.conf to $VASTAI_DRV_VERSION"
sed -i 's/PACKAGE_VERSION=.*$/PACKAGE_VERSION='"\"${VASTAI_DRV_VERSION}\""'/g' \
	dkms.conf
echo "modify version of include/vastai_version.h to $VASTAI_DRV_VERSION"
sed -i 's/^.*VASTAI_PCI_DRIVER_VERSION.*$/#define VASTAI_PCI_DRIVER_VERSION '"\(\"${VASTAI_DRV_VERSION}\"\)"'/g' \
	include/vastai_version.h

if [ ! -d $DKMS_SRC_DIR ]; then
	mkdir $DEBUG_EN $DKMS_SRC_DIR
fi
for dkms_src in $DKMS_SOURCES; do
	cp -a $DEBUG_EN $dkms_src ${DKMS_SRC_DIR}/${dkms_src}
done
if [ $PACKAGE_TYPE = "rpm" ]; then
	cp ${CUR_PATH}/${PACKAGE_NAME_PREFIX}-dkms-mkrpm.spec ${DKMS_SRC_DIR}/${PACKAGE_NAME}-dkms-mkrpm.spec
fi
if [ $PACKAGE_TYPE = "deb" ]; then
	cp ${CUR_PATH}/${PACKAGE_NAME_PREFIX}-dkms-mkdeb ${DKMS_SRC_DIR}/${PACKAGE_NAME}-dkms-mkdeb -r
fi
cp ${CUR_PATH}/vastai.rules ${DKMS_SRC_DIR}/

# put git info to package
sed -i 's/^.*VASTAI_PCI_GIT_BRANCH_NAME.*$/#define VASTAI_PCI_GIT_BRANCH_NAME '"\(\"${GIT_BRANCH_NAME}\"\)"'/g' \
	${DKMS_SRC_DIR}/include/vastai_version.h
sed -i 's/^.*VASTAI_PCI_GIT_COMMIT_HASH.*$/#define VASTAI_PCI_GIT_COMMIT_HASH '"\(\"${GIT_COMMIT_HASH}\"\)"'/g' \
	${DKMS_SRC_DIR}/include/vastai_version.h
#sed -i 's/^.*VASTAI_PCI_GIT_COMMIT_MSG.*$/#define VASTAI_PCI_GIT_COMMIT_MSG '"\(\"${GIT_COMMIT_MSG_FORMAT}\"\)"'/g' \
#	${DKMS_SRC_DIR}/include/vastai_version.h
sed -i 's/^.*VASTAI_PCI_MKPACKAGE_TIME.*$/#define VASTAI_PCI_MKPACKAGE_TIME '"\(\"${MKPACKAGE_TIME}\"\)"'/g' \
	${DKMS_SRC_DIR}/include/vastai_version.h

# 3. install new to dkms tree
echo 3.add new to dkms tree
dkms add ${PACKAGE_NAME}/${VASTAI_DRV_VERSION}
dkms build ${PACKAGE_NAME}/${VASTAI_DRV_VERSION}
if [ $? != 0 ]; then
	echo "ERROR: dkms build failed!"
	exit 2
fi

# 4. make package
echo 4.make package
echo "make ${PACKAGE_TYPE} package for $DISTRIB_ID"

echo ""
echo "Version: change from $cur_version to $VASTAI_DRV_VERSION"
echo ""

if [ $PACKAGE_TYPE = "rpm" ]; then
	dkms mk${PACKAGE_TYPE} ${PACKAGE_NAME}/${VASTAI_DRV_VERSION} --source-only
elif [ $PACKAGE_TYPE = "deb" ]; then
	dkms mk${PACKAGE_TYPE} ${PACKAGE_NAME}/${VASTAI_DRV_VERSION} --source-only
fi

# 5. move created package to destination directory
echo 5.move created package to $(readlink -f $OUT_DIR)
if [ ! -d $OUT_DIR ]; then
	mkdir $DEBUG_EN $OUT_DIR
fi

if [ $PACKAGE_TYPE = "rpm" ]; then
	mv -f $DEBUG_EN ${DKMS_TREE_DIR}/rpm/*${target}* $OUT_DIR/${PACKAGE_NAME}_${VASTAI_DRV_VERSION}_${target}.rpm
elif [ $PACKAGE_TYPE = "deb" ]; then
	mv -f $DEBUG_EN ${DKMS_TREE_DIR}/deb/* $OUT_DIR/${PACKAGE_NAME}_${VASTAI_DRV_VERSION}_${target}.deb
fi

if [ $? != 0 ]; then
	echo "Warning:please check dkms version(>=1.95, <=2.8)"
	exit 2
fi

# 6. remove new installed modules from dkms tree
echo 6.remove new installed modules from dkms tree
dkms remove ${PACKAGE_NAME}/${VASTAI_DRV_VERSION} --all
if [ $? != 0 ]; then
	echo "ERROR: dkms remove failed!"
	exit 2
fi
rm -rf $DEBUG_EN /usr/src/${PACKAGE_NAME}-${VASTAI_DRV_VERSION}
