#include "vastai_die.h"
#include "vastai_pci_test.h"
#include "vastai_fifo.h"
#include "vastai_udma_engine.h"
#include "vastai_pci_dbg.h"

#include "vastai_pci_boot.h" //vastai_die_get_pcie_atter need this header
#include "vastai_pci_vf.h"

#define VASTAI_DIE_ERR(die, fmt, args...) \
	do { \
		VASTAI_PCI_ERR(die->pci_info, \
			((union die_index_data)die->die_index).die_id, \
			fmt, ##args); \
	} while (0)

#define VASTAI_DIE_INFO(die, fmt, args...) \
	do { \
		VASTAI_PCI_INFO(die->pci_info, \
			((union die_index_data)die->die_index).die_id, \
			fmt, ##args); \
	} while (0)

#define VASTAI_DIE_DBG(die, fmt, args...) \
	do { \
		VASTAI_PCI_DBG(die->pci_info, \
			((union die_index_data)die->die_index).die_id, \
			fmt, ##args); \
	} while (0)

#define vastai_die_mem_read(die, addr, buf, len) \
	vastai_pci_mem_read(die->pci_info, die->die_index, addr, buf, len)

#define vastai_die_mem_write(die, addr, buf, len) \
	vastai_pci_mem_write(die->pci_info, die->die_index, addr, buf, len)

#ifdef CONFIG_VASTAI_PCI_SET_BAR
static const int vastai_pci_set_bar = 1;
#else
static const int vastai_pci_set_bar = 0;
#endif

int vastai_die_get_pcie_attr(struct vastai_sv100_die *die)
{
	int ret;
	union link_control_status_reg reg;
	u64 reg_addr = VASTAI_PCI_LINK_CONTROL_STATUS_REG;
	int die_id = vastai_pci_get_die_id(die->pci_info, die->die_index);

	if (die_id > 0 && !vastai_pci_set_bar) {
		return 0;
	}
	ret = vastai_pci_mem_read_direct(die->pci_info, die->die_index,
					reg_addr, &(reg.val), 4);
	if (ret < 0)
		VASTAI_DIE_ERR(die, "%s [read] 0x%llx, len=4, error:%d\n",
				__func__, reg_addr, ret);
	die->pcie_nls = reg.bit.negotiated_link_speed;
	die->pcie_nlw = reg.bit.negotiated_link_width;

//	if (die_id > 0 && (die->pci_info->minish_bar_size == 1))
//		vastai_pci_reset_bar_at(die->pci_info, die->die_index);

	return ret;
}

int vastai_die_init(struct vastai_sv100_die *die,
		    struct vastai_pci_info *pci_info, u32 die_id)
{
	int i = 0;
	struct vastai_fifo fifo = { 0 };
	u32 fifo_size;

	die->pci_info = pci_info;
	die->die_index = vastai_pci_gen_die_index(pci_info, die_id);

	if (!die->pci_info || die->die_index == UINT_MAX) {
		return -EIO;
	}

	if(vastai_get_die_boot_mode(die)) {
		return -EIO;
	}

	/* init smcu dma desc fifo */
	/* init dma chn fifo */
	/* TODO: move into udma_init */
	for (i = 0; i < VASTAI_DMA_CHN; i++) {
		u64 dma_desc_fifo_addr = vastai_udma_config[i].dma_desc_fifo_addr;
		if (die->boot_mode == BOOT_VF_SRIOV) {
			dma_desc_fifo_addr += VASTAI_DMA_DESC_VF_OFFSET;
			fifo_size = VASTAI_DMA_DESC_FIFO_SIZE - VASTAI_DMA_DESC_VF_OFFSET;
		} else if (die->boot_mode == BOOT_PF_SRIOV){
			fifo_size = VASTAI_DMA_DESC_FIFO_SIZE;
		} else {
			fifo_size = VASTAI_DMA_DESC_FIFO_SIZE;
		}
		vastai_fifo_init_arg(
			&fifo, ALIGN(sizeof(PCIE_xd_desc), 4), fifo_size);
		vastai_pci_mem_write_direct(pci_info, die->die_index,
				     dma_desc_fifo_addr,
				     &fifo, sizeof(fifo));
	}

	spin_lock_init(&(die->msg_ring_buf_lock));
	vastai_pci_set_vdsp_depth(pci_info, die_id,
				  VASTAI_PCIE_2_VDSP_MSG_DEPTH, 0);
#ifdef CONFIG_VASTAI_CMCU_RINGBUF
	vastai_pci_set_cmcu_depth(pci_info, die_id,
				  VASTAI_PCIE_2_CMCU_MSG_DEPTH);
#endif
	vastai_die_msgq_init(die);

#ifdef CONFIG_VASTAI_TOOLS_SUITE
	/* vatools smi block config */
	vatools_fw_config_smi(pci_info, die->die_index);
#endif
	if (die->boot_mode == BOOT_VF_SRIOV) {
		fifo_size = VASTAI_DMA_TRANS_INFO_FIFO_SIZE - VASTAI_DMA_TRANS_INFO_VF_OFFSET;
	} else if (die->boot_mode == BOOT_PF_SRIOV) {
		fifo_size = VASTAI_DMA_TRANS_INFO_FIFO_SIZE;
	} else {
		fifo_size = VASTAI_DMA_TRANS_INFO_FIFO_SIZE;
	}
	vastai_fifo_init_arg(
		&fifo, ALIGN(sizeof(struct pcie_transfer_info), 4),
		fifo_size);
	for (i = 0; i < VASTAI_DMA_CHN; i++) {
		u64 trans_info_addr = vastai_udma_config[i].trans_info_fifo_addr;
		if (die->boot_mode == BOOT_VF_SRIOV)
			trans_info_addr += VASTAI_DMA_TRANS_INFO_VF_OFFSET;
		vastai_pci_mem_write_direct(pci_info, die->die_index,
					trans_info_addr, &fifo, sizeof(fifo));
	}

	if(die->boot_mode != BOOT_PF_NORMAL) vastai_vf_os_fifo_init(die);

	vastai_fifo_init_arg(
		&fifo, sizeof(struct vastai_cedar_img_info),
		1024);
	vastai_pci_mem_write_direct(pci_info, die->die_index,
			     CEDAR_SVC_INFO, &fifo, sizeof(fifo));

	mutex_init(&die->cedar_lock);

#ifdef CONFIG_TOOLS_V2
	mutex_init(&die->tools_die_mutex);
#endif

	die->dev_freq = 100;
	atomic_set(&die->ctrl_cmd_list.ctrl_seq_id, 0);
	INIT_LIST_HEAD(&die->ctrl_cmd_list.ctrl_cmd_head.list);
	mutex_init(&(die->get_dlc_mutex));
	init_completion(&(die->get_dlc_core_done));

	return vastai_udma_init(die);
}

void vastai_die_deinit(struct vastai_sv100_die *die)
{
	vastai_udma_deinit(die);
}

