#include <linux/dmi.h>
#include "vastai_dmi_table.h"

#define DMI_STATE 0x8BC0748

struct dmi_info host_dmi_info;

struct dmi_info dmi_table[] = {
	{
		.sys_vendor = "Inspur",
		.product_name = "SA5212M4",
		.id = 0x010101,
	}, {
		.sys_vendor = "Inspur",
		.product_name = "NF5280M5",
		.id = 0x010201,
	}, {
		.sys_vendor = "Inspur",
		.product_name = "NF5280M6",
		.id = 0x010202,
	}, {
		.sys_vendor = "Inspur",
		.product_name = "NF5468M5",
		.id = 0x010301,
	}, {
		.sys_vendor = "Inspur",
		.product_name = "NF5468M6",
		.id = 0x010302,
	}, {
		.sys_vendor = "Inspur",
		.product_name = "NE5260M5",
		.id = 0x010401,
	},{
		.sys_vendor = "LTHPC",
		.product_name = "SR222-Server-System",
		.id = 0x020101,
	}, {
		.sys_vendor = "HUAQIN",
		.product_name = "P6410",
		.id = 0x020101,
	}, {
		.sys_vendor = "Supermicro",
		.product_name = "AS -4124GS-TNR",
		.id = 0x040101,
	}, {
		.sys_vendor = "Supermicro",
		.product_name = "SYS-6949GP-TRT",
		.id = 0x040101,
	}, {
		.sys_vendor = "XFUSION",
		.product_name = "2288H V5",
		.id = 0x030101,
	}, {
		.sys_vendor = "XFUSION",
		.product_name = "2288H V6",
		.id = 0x030102,
	}, {
		.sys_vendor = "Huawei",
		.product_name = "TaiShan 200 (Model 2280)",
		.id = 0x050101,
	}, {
		.sys_vendor = "Lenovo",
		.product_name = "ThinkSystem SR665",
		.id = 0x060101,
	}, {
		.sys_vendor = "New H3C Technologies",
		.product_name = "H3C UniServer R4900 G5",
		.id = 0x040101,
	}, {
		.sys_vendor = "New H3C Technologies",
		.product_name = "H3C UniServer R5300 G5",
		.id = 0x070201,
	}, {
		.sys_vendor = "H3C",
		.product_name = "R4900 G3",
		.id = 0x070301,
	}, {
		.sys_vendor = "System manufacturer",
		.product_name = "System Product Name",
		.id = 0x0f0101,
	}, {
		.sys_vendor = "Default string",
		.product_name = "Default string",
		.id = 0x0f0201,
	}
};

static inline int dmicmp(const struct dmi_info *dmi1,
			  const struct dmi_info *dmi2)
{
	int ret;

	ret = strncmp(dmi1->sys_vendor,
		      dmi2->sys_vendor, strlen(dmi1->sys_vendor));
	if (ret)
		return ret;

	ret = strncmp(dmi1->product_name,
		      dmi2->product_name, strlen(dmi1->product_name));
	return ret;
}

static u32 dmi_info_get_id(struct dmi_info *dmi_info)
{
	u32 id = UINT_MAX;
	int i = 0;

	for (i = 0; i < ARRAY_SIZE(dmi_table); i++) {
		if (!dmicmp(&dmi_table[i], dmi_info)) {
			id = dmi_table[i].id;
			break;
		}
	}
	return id;
}

static void dmi_info_print(struct vastai_pci_info *pci_info)
{
	u32 reg;
	int ret;
	char *dev_state;

	VASTAI_PCI_INFO(pci_info, DUMMY_DIE_ID, "dmi info: host vendor %s:%s\n",
		host_dmi_info.sys_vendor,
		host_dmi_info.product_name);

	ret = vastai_pci_mem_read(pci_info, -1, DMI_STATE, &reg, sizeof(reg));
	if (ret) {
		VASTAI_PCI_ERR(pci_info, DUMMY_DIE_ID, "%s read reg error\n",
				__FUNCTION__);
	}


	switch (reg & 0x3) {
	case 0b00:
		dev_state = "Server need reboot to use new cfg.";
		break;
	case 0b01:
		dev_state = "Server cfg is match and be used.";
		break;
	case 0b10:
		dev_state = "Server cfg is miss. Please confirm the pcie cfg.";
		break;
	default:
		dev_state = "ERROR reg val.";
		break;
	}
	VASTAI_PCI_INFO(pci_info, DUMMY_DIE_ID, "%s\n", dev_state);
	return;
}

static int dmi_trigger_smcu_write_flash(struct vastai_pci_info *pci_info)
{
	int ret;

	ret = vastai_send_pcie_cmd(pci_info, pci_info->dies[0].die_index,
				    VASTAI_PCIE_SUB_DMI_FLASH,
				    host_dmi_info.id);
	return ret;
}

int dmi_set_device(struct vastai_pci_info *pci_info)
{
	int ret;

	ret = dmi_trigger_smcu_write_flash(pci_info);
	if (ret) {
		VASTAI_PCI_ERR(pci_info, DUMMY_DIE_ID, "%s send dmi failed\n",
				__FUNCTION__);
	}

	dmi_info_print(pci_info);
	return ret;
}

void dmi_info_init(void)
{
	host_dmi_info.sys_vendor = dmi_get_system_info(DMI_SYS_VENDOR);
	host_dmi_info.product_name = dmi_get_system_info(DMI_PRODUCT_NAME);
	if((host_dmi_info.sys_vendor != NULL) && (host_dmi_info.product_name != NULL)) {
		host_dmi_info.id = dmi_info_get_id(&host_dmi_info);
		printk("vastai host vendor:%s product:%s id:%d\n",
			host_dmi_info.sys_vendor,
			host_dmi_info.product_name,
			host_dmi_info.id);
	}
}

