#ifndef _CSRAM_PARTITION_H
#define _CSRAM_PARTITION_H

#define CSRAM_BASE_ADDR 0x08C00000

/* The tx is host to device, rx is device to host */
#define PCIE_HOST_TO_DEV_TRANSF_INFO (CSRAM_BASE_ADDR + 0x00000000) // 40K
#define PCIE_DEV_TO_HOST_TRANSF_INFO (CSRAM_BASE_ADDR + 0x0000A000) // 40K
#define PCIE_DMA_HOST_TO_DEV_DESC     (CSRAM_BASE_ADDR + 0x00014000) // 80K
#define PCIE_DMA_HOST_TO_DEV_DESC    (CSRAM_BASE_ADDR + 0x00023000)
#define PCIE_DMA_DEV_TO_HOST_CFG     (CSRAM_BASE_ADDR + 0x00028000) // 80K
#define PCIE_DMA_DEV_TO_HOST_DESC    (CSRAM_BASE_ADDR + 0x00037000)

/* For one to one MSG, between MCUS and DSPS */
#define LOCAL_MSG_BASE_ADDR (CSRAM_BASE_ADDR + 0x0003C000)

#define MSG_CMCU_TO_SMCU   (CSRAM_BASE_ADDR + 0x0003C000)
#define MSG_VDMCU0_TO_SMCU (CSRAM_BASE_ADDR + 0x0003C400)
#define MSG_VDMCU1_TO_SMCU (CSRAM_BASE_ADDR + 0x0003C800)
#define MSG_VDMCU2_TO_SMCU (CSRAM_BASE_ADDR + 0x0003CC00)
#define MSG_VEMCU0_TO_SMCU (CSRAM_BASE_ADDR + 0x0003D000)
#define MSG_VEMCU1_TO_SMCU (CSRAM_BASE_ADDR + 0x0003D400)
#define MSG_VEMCU2_TO_SMCU (CSRAM_BASE_ADDR + 0x0003D800)
#define MSG_VEMCU3_TO_SMCU (CSRAM_BASE_ADDR + 0x0003DC00)

#define MSG_SMCU_TO_VDMCU0   (CSRAM_BASE_ADDR + 0x0003E000)
#define MSG_VDSP0_TO_VDMCU0  (CSRAM_BASE_ADDR + 0x0003E400)
#define MSG_VDSP1_TO_VDMCU0  (CSRAM_BASE_ADDR + 0x0003E800)
#define MSG_VEMCU0_TO_VDMCU0 (CSRAM_BASE_ADDR + 0x0003EC00)
#define MSG_VEMCU1_TO_VDMCU0 (CSRAM_BASE_ADDR + 0x0003F000)
#define MSG_SMCU_TO_VDMCU1   (CSRAM_BASE_ADDR + 0x0003F400)
#define MSG_VDSP1_TO_VDMCU1  (CSRAM_BASE_ADDR + 0x0003F800)
#define MSG_VDSP2_TO_VDMCU1  (CSRAM_BASE_ADDR + 0x0003FC00)
#define MSG_VEMCU1_TO_VDMCU1 (CSRAM_BASE_ADDR + 0x00040000)
#define MSG_VEMCU2_TO_VDMCU1 (CSRAM_BASE_ADDR + 0x00040400)
#define MSG_SMCU_TO_VDMCU2   (CSRAM_BASE_ADDR + 0x00040800)
#define MSG_VDSP2_TO_VDMCU2  (CSRAM_BASE_ADDR + 0x00040C00)
#define MSG_VDSP3_TO_VDMCU2  (CSRAM_BASE_ADDR + 0x00041000)
#define MSG_VEMCU2_TO_VDMCU2 (CSRAM_BASE_ADDR + 0x00041400)
#define MSG_VEMCU3_TO_VDMCU2 (CSRAM_BASE_ADDR + 0x00041800)

#define MSG_SMCU_TO_VEMCU0   (CSRAM_BASE_ADDR + 0x00041C00)
#define MSG_VDSP0_TO_VEMCU0  (CSRAM_BASE_ADDR + 0x00042000)
#define MSG_VDMCU0_TO_VEMCU0 (CSRAM_BASE_ADDR + 0x00042400)
#define MSG_SMCU_TO_VEMCU1   (CSRAM_BASE_ADDR + 0x00042800)
#define MSG_VDSP1_TO_VEMCU1  (CSRAM_BASE_ADDR + 0x00042C00)
#define MSG_VDMCU0_TO_VEMCU1 (CSRAM_BASE_ADDR + 0x00043000)
#define MSG_VDMCU1_TO_VEMCU1 (CSRAM_BASE_ADDR + 0x00043400)
#define MSG_SMCU_TO_VEMCU2   (CSRAM_BASE_ADDR + 0x00043800)
#define MSG_VDSP2_TO_VEMCU2  (CSRAM_BASE_ADDR + 0x00043C00)
#define MSG_VDMCU1_TO_VEMCU2 (CSRAM_BASE_ADDR + 0x00044000)
#define MSG_VDMCU2_TO_VEMCU2 (CSRAM_BASE_ADDR + 0x00044400)
#define MSG_SMCU_TO_VEMCU3   (CSRAM_BASE_ADDR + 0x00044800)
#define MSG_VDSP3_TO_VEMCU3  (CSRAM_BASE_ADDR + 0x00044C00)
#define MSG_VDMCU2_TO_VEMCU3 (CSRAM_BASE_ADDR + 0x00045000)

#define MSG_CMCU_TO_VDSP0   (CSRAM_BASE_ADDR + 0x00045400)
#define MSG_VEMCU0_TO_VDSP0 (CSRAM_BASE_ADDR + 0x00045800)
#define MSG_VDMCU0_TO_VDSP0 (CSRAM_BASE_ADDR + 0x00045C00)
#define MSG_CMCU_TO_VDSP1   (CSRAM_BASE_ADDR + 0x00046000)
#define MSG_VEMCU1_TO_VDSP1 (CSRAM_BASE_ADDR + 0x00046400)
#define MSG_VDMCU0_TO_VDSP1 (CSRAM_BASE_ADDR + 0x00046800)
#define MSG_VDMCU1_TO_VDSP1 (CSRAM_BASE_ADDR + 0x00046C00)
#define MSG_CMCU_TO_VDSP2   (CSRAM_BASE_ADDR + 0x00047000)
#define MSG_VEMCU2_TO_VDSP2 (CSRAM_BASE_ADDR + 0x00047400)
#define MSG_VDMCU1_TO_VDSP2 (CSRAM_BASE_ADDR + 0x00047800)
#define MSG_VDMCU2_TO_VDSP2 (CSRAM_BASE_ADDR + 0x00047C00)
#define MSG_CMCU_TO_VDSP3   (CSRAM_BASE_ADDR + 0x00048000)
#define MSG_VEMCU3_TO_VDSP3 (CSRAM_BASE_ADDR + 0x00048400)
#define MSG_VDMCU2_TO_VDSP3 (CSRAM_BASE_ADDR + 0x00048800)

#define MSG_RESERGE_0	  (CSRAM_BASE_ADDR + 0x00048C00)
#define MSG_PCIE_TO_CMCU  (CSRAM_BASE_ADDR + 0x0003E400)
#define MSG_SMCU_TO_CMCU  (CSRAM_BASE_ADDR + 0x00049000)
#define MSG_VDSP0_TO_CMCU (CSRAM_BASE_ADDR + 0x00049400)
#define MSG_VDSP1_TO_CMCU (CSRAM_BASE_ADDR + 0x00049800)
#define MSG_VDSP2_TO_CMCU (CSRAM_BASE_ADDR + 0x00049C00)
#define MSG_VDSP3_TO_CMCU (CSRAM_BASE_ADDR + 0x0004A000)
#define MSG_LMCU0_TO_CMCU (CSRAM_BASE_ADDR + 0x0004A400)
#define MSG_LMCU1_TO_CMCU (CSRAM_BASE_ADDR + 0x0004A600)
#define MSG_LMCU2_TO_CMCU (CSRAM_BASE_ADDR + 0x0004A800)
#define MSG_LMCU3_TO_CMCU (CSRAM_BASE_ADDR + 0x0004AA00)
#define MSG_LMCU4_TO_CMCU (CSRAM_BASE_ADDR + 0x0004AC00)
#define MSG_LMCU5_TO_CMCU (CSRAM_BASE_ADDR + 0x0004AE00)
#define MSG_LMCU6_TO_CMCU (CSRAM_BASE_ADDR + 0x0004B000)
#define MSG_LMCU7_TO_CMCU (CSRAM_BASE_ADDR + 0x0004B200)
#define MSG_ODSP0_TO_CMCU (CSRAM_BASE_ADDR + 0x0004B400)
#define MSG_ODSP1_TO_CMCU (CSRAM_BASE_ADDR + 0x0004B600)
#define MSG_ODSP2_TO_CMCU (CSRAM_BASE_ADDR + 0x0004B800)
#define MSG_ODSP3_TO_CMCU (CSRAM_BASE_ADDR + 0x0004BA00)
#define MSG_ODSP4_TO_CMCU (CSRAM_BASE_ADDR + 0x0004BC00)
#define MSG_ODSP5_TO_CMCU (CSRAM_BASE_ADDR + 0x0004BE00)
#define MSG_ODSP6_TO_CMCU (CSRAM_BASE_ADDR + 0x0004C000)
#define MSG_ODSP7_TO_CMCU (CSRAM_BASE_ADDR + 0x0004C200)

#define MSG_RESERGE_1	  (CSRAM_BASE_ADDR + 0x0004C400)
#define MSG_CMCU_TO_PCIE  (CSRAM_BASE_ADDR + 0x0003C000)
#define MSG_CMCU_TO_LMCU0 (CSRAM_BASE_ADDR + 0x0004C800)
#define MSG_CMCU_TO_LMCU1 (CSRAM_BASE_ADDR + 0x0004CA00)
#define MSG_CMCU_TO_LMCU2 (CSRAM_BASE_ADDR + 0x0004CC00)
#define MSG_CMCU_TO_LMCU3 (CSRAM_BASE_ADDR + 0x0004CE00)
#define MSG_CMCU_TO_LMCU4 (CSRAM_BASE_ADDR + 0x0004D000)
#define MSG_CMCU_TO_LMCU5 (CSRAM_BASE_ADDR + 0x0004D200)
#define MSG_CMCU_TO_LMCU6 (CSRAM_BASE_ADDR + 0x0004D400)
#define MSG_CMCU_TO_LMCU7 (CSRAM_BASE_ADDR + 0x0004D600)

#define MSG_CMCU_TO_ODSP0 (CSRAM_BASE_ADDR + 0x0004D800)
#define MSG_CMCU_TO_ODSP1 (CSRAM_BASE_ADDR + 0x0004DA00)
#define MSG_CMCU_TO_ODSP2 (CSRAM_BASE_ADDR + 0x0004DC00)
#define MSG_CMCU_TO_ODSP3 (CSRAM_BASE_ADDR + 0x0004DE00)
#define MSG_CMCU_TO_ODSP4 (CSRAM_BASE_ADDR + 0x0004E000)
#define MSG_CMCU_TO_ODSP5 (CSRAM_BASE_ADDR + 0x0004E200)
#define MSG_CMCU_TO_ODSP6 (CSRAM_BASE_ADDR + 0x0004E400)
#define MSG_CMCU_TO_ODSP7 (CSRAM_BASE_ADDR + 0x0004E600)

#define MSG_CMCU_TO_LMCU_ODSP (CSRAM_BASE_ADDR + 0x0004E800)

/* For queue msg, between host and vdmcus, vemcus, vdsps */
#define QUEUE_MSG_BASE (CSRAM_BASE_ADDR + 0X00052000)

#define MSG_SMCUQ0_TO_PCIE (CSRAM_BASE_ADDR + 0x00051800)
#define MSG_SMCUQ1_TO_PCIE (CSRAM_BASE_ADDR + 0x00051C00)
#define MSG_VDMCU0_TO_PCIE (CSRAM_BASE_ADDR + 0x00047800)
#define MSG_VDMCU1_TO_PCIE (CSRAM_BASE_ADDR + 0x00048C00)
#define MSG_VDMCU2_TO_PCIE (CSRAM_BASE_ADDR + 0x0004A000)
#define MSG_VEMCU0_TO_PCIE (CSRAM_BASE_ADDR + 0x00052C00)
#define MSG_VEMCU1_TO_PCIE (CSRAM_BASE_ADDR + 0x00053000)
#define MSG_VEMCU2_TO_PCIE (CSRAM_BASE_ADDR + 0x00053400)
#define MSG_VEMCU3_TO_PCIE (CSRAM_BASE_ADDR + 0x00053800)
#define MSG_VDSP0_TO_PCIE  (CSRAM_BASE_ADDR + 0x0004B800)
#define MSG_VDSP1_TO_PCIE  (CSRAM_BASE_ADDR + 0x0004C400)
#define MSG_VDSP2_TO_PCIE  (CSRAM_BASE_ADDR + 0x0004D400)
#define MSG_VDSP3_TO_PCIE  (CSRAM_BASE_ADDR + 0x0004E400)

#define MSG_PCIE_TO_SMCUQ0 (CSRAM_BASE_ADDR + 0x00054C00)
#define MSG_PCIE_TO_SMCUQ1 (CSRAM_BASE_ADDR + 0x00055000)
#define MSG_PCIE_TO_VDMCU0 (CSRAM_BASE_ADDR + 0x00048800)
#define MSG_PCIE_TO_VDMCU1 (CSRAM_BASE_ADDR + 0x00049C00)
#define MSG_PCIE_TO_VDMCU2 (CSRAM_BASE_ADDR + 0x0004B000)
#define MSG_PCIE_TO_VEMCU0 (CSRAM_BASE_ADDR + 0x00056000)
#define MSG_PCIE_TO_VEMCU1 (CSRAM_BASE_ADDR + 0x00056400)
#define MSG_PCIE_TO_VEMCU2 (CSRAM_BASE_ADDR + 0x00056800)
#define MSG_PCIE_TO_VEMCU3 (CSRAM_BASE_ADDR + 0x00056C00)
#define MSG_PCIE_TO_VDSP0  (CSRAM_BASE_ADDR + 0x0004F000)
#define MSG_PCIE_TO_VDSP1  (CSRAM_BASE_ADDR + 0x0004FC00)
#define MSG_PCIE_TO_VDSP2  (CSRAM_BASE_ADDR + 0x00050C00)
#define MSG_PCIE_TO_VDSP3  (CSRAM_BASE_ADDR + 0x00051C00)

#define MODULE_INFO_BASE (CSRAM_BASE_ADDR + 0x00058000) // 510k
#define CHN_INFO_BASE	 (CSRAM_BASE_ADDR + 0x000D7800) // 100k
#define VDMCU_CSR_BASE	 (CSRAM_BASE_ADDR + 0x000F0800) // 10k

#define CORE_STATUS_SYNC (CSRAM_BASE_ADDR + 0x000F3000) // 32 bytes
#define HEART_BEAT_CNT	 (CSRAM_BASE_ADDR + 0x000F3020) // 128 bytes
#define LOG_MSG_FLAG	 (CSRAM_BASE_ADDR + 0x000F3098) // 4 bytes
#define LOG_CTRL_INFO	 (CSRAM_BASE_ADDR + 0x000F30A0) // 512 bytes
#define SYS_TIME_STRAMP	 (CSRAM_BASE_ADDR + 0x000F32A0) // Time stramp 24 bytes
#define CSRAM_RESEREVE1	 (CSRAM_BASE_ADDR + 0x000F32B8) // 328 bytes for smcu

/* Total 512 bytes for version */
#define VSERSION_INFO_BASE (CSRAM_BASE_ADDR + 0x000F3400) // total 512 bytes

#define FW_VERSION    (CSRAM_BASE_ADDR + 0x000F3400) // 64 bytes
#define SMCU_VERSION  (CSRAM_BASE_ADDR + 0x000F3440) // 64 bytes
#define VDMCU_VERSION (CSRAM_BASE_ADDR + 0x000F3480) // 64 bytes
#define VEMCU_VERSION (CSRAM_BASE_ADDR + 0x000F34C0) // 64 bytes
#define VDSP_VERSION  (CSRAM_BASE_ADDR + 0x000F3500) // 64 bytes
#define CMCU_VERSION  (CSRAM_BASE_ADDR + 0x000F3540) // 64 bytes
#define LMCU_VERSION  (CSRAM_BASE_ADDR + 0x000F3580) // 64 bytes
#define ODSP_VERSION  (CSRAM_BASE_ADDR + 0x000F35C0) // 64 bytes

#define SMI_INFO_ADDR (CSRAM_BASE_ADDR + 0x000F3600) // 2.5K

#define CSRAM_RESEREVE2 (CSRAM_BASE_ADDR + 0x000F4000) // 12K

/* msix table VF0 -- VF7, 32k bytes */
#define PCIE_MSIX_VF_TABLE (CSRAM_BASE_ADDR + 0x000F7000) // 32K

/* PF, msix table */
#define PCIE_MSIX_PF_PENDING_TABLE (CSRAM_BASE_ADDR + 0x000FF000) // 2K
#define PCIE_MSIX_PF_TABLE	   (CSRAM_BASE_ADDR + 0x000FF800) // 2K

#endif /* _CSRAM_PARTITION_H */
