/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: gang.wang
 */

#ifndef __VASTAI_PCI_H__
#define __VASTAI_PCI_H__

#include <linux/list.h>
#include <linux/pci.h>
#include <linux/cdev.h>

#include "vastai_pci_api.h"
#include "vastai_pci_dbg.h"
#include "vastai_sv100_reg.h"
#include "vastai_fifo.h"
#include "vastai_msgq.h"

#include "vastai_pci_vf.h"

#define VASTAI_PCI_DRIVER_PREFIX "vastai"
#define VASTAI_VER_COMMON_LEN	 (64)
#define VASTAI_VER_BMCU_XSPI_LEN (32)

#define PCI_VENDOR_ID_VASTAI	 (0x1EC6)

#define PCI_VENDOR_ID_DAOCLOUD	 (0x1F4F)
#define PCI_DEVICE_ID_DAOCLOUD	 (0x0100)
#define PCI_SUBDEVICE_ID_DAOCLOUD	(0x0010)

#define VASTAI_SV100_MAX_DIE_NUM (4)

#define VASTAI_VAL_DW_MASK (0xFFFFFFFF)

#define VASTAI_DIR_IN  (0)
#define VASTAI_DIR_OUT (1)

#define VASTAI_PCIE_MSI_NUM (32)
/* Support up to 128 msi-x interrupts. */
#define VASTAI_PCIE_MSIX_NUM (128)
#define VASTAI_PCIE_MSIX_NUM_PER_DIE                                           \
	(VASTAI_PCIE_MSIX_NUM / VASTAI_SV100_MAX_DIE_NUM)

#define VASTAI_PCIE_WAIT_SMCU_MSG1_TIME (3000)

#define VASTAI_PCIE_TRANS_NUM (256)

#define VASTAI_MAX_DMA_BUF SZ_4M
#define VASTAI_MAX_DMA_PAYLOAD SZ_16M
#define VASTAI_MAX_MEMSET_BUF SZ_4M

#define VASTAI_DMA_CHN 4

#define VASTAI_FILE_MODE 0666

/* s, watch dog is 4s, we need one more sec to protact noc-bus is empty */
#define VASTAI_WAITING_RESET 5

//define core num
#define		CORE_POINT_SMCU				(0)
#define		CORE_POINT_CMCU				(1)
#define		CORE_POINT_LMCU0			(2)
#define		CORE_POINT_LMCU1			(3)
#define		CORE_POINT_LMCU2			(4)
#define		CORE_POINT_LMCU3			(5)
#define		CORE_POINT_LMCU4			(6)
#define		CORE_POINT_LMCU5			(7)
#define		CORE_POINT_LMCU6			(8)
#define		CORE_POINT_LMCU7			(9)
#define		CORE_POINT_ODSP0			(10)
#define		CORE_POINT_ODSP1			(11)
#define		CORE_POINT_ODSP2			(12)
#define		CORE_POINT_ODSP3			(13)
#define		CORE_POINT_ODSP4			(14)
#define		CORE_POINT_ODSP5			(15)
#define		CORE_POINT_ODSP6			(16)
#define		CORE_POINT_ODSP7			(17)
#define		CORE_POINT_VDMCU0			(18)
#define		CORE_POINT_VDMCU1			(19)
#define		CORE_POINT_VDMCU2			(20)
#define		CORE_POINT_VEMCU0			(21)
#define		CORE_POINT_VEMCU1			(22)
#define		CORE_POINT_VEMCU2			(23)
#define		CORE_POINT_VEMCU3			(24)
#define		CORE_POINT_VDSP0			(25)
#define		CORE_POINT_VDSP1			(26)
#define		CORE_POINT_VDSP2			(27)
#define		CORE_POINT_VDSP3			(28)
#define		CORE_TOTAL_NUM				(29)

#define MSGQ_DMA 0
#define MSGQ_CMD 1

enum VASTAI_MSGQ_ID {
       MSGQ_ID_0 = 0,
       MSGQ_ID_1 = 1,
};

enum enum_core_2_core_cmd {
	VATOOLS_INTERRUPT_TOOLS_CATEGARY_CMD = 1,
	CORE_2_CORE_RESET_CMD
};

enum enum_core_2_core_reset_subcmd {
	CORE_2_CORE_RESET_AI_SYS_HANG_SUBCMD = 0,
	CORE_2_CORE_RESET_AI_ACK_SUBCMD,
	CORE_2_CORE_RESET_AI_RESET_SUBCMD,
	CORE_2_CORE_RESET_AI_FINISH_SUBCMD,
	CORE_2_CORE_RESET_CORE_SYS_HANG_SUBCMD,
	CORE_2_CORE_RESET_VDSP_HANG_SUBCMD,
	CORE_2_CORE_RESET_VDSP_RESET_SUBCMD,
	CORE_2_CORE_RESET_VDSP_ACK_SUBCMD,
	CORE_2_CORE_RESET_VDSP_FINISH_SUBCMD
};

enum VASTAI_DIE_NUM {
	VASTAI_SINGLE_DIE = 1,
	VASTAI_TWO_DIE = 2,
	VASTAI_FOUR_DIE = 4
};

enum VASTAI_DIE_ID {
	VASTAI_DIE0,
	VASTAI_DIE1,
	VASTAI_DIE2,
	VASTAI_DIE3,
	VASTAI_DIE_ID_MAX
};

enum VASTAI_PCI_BAR {
	VASTAI_PCI_BAR0,
	VASTAI_PCI_BAR1,
	VASTAI_PCI_BAR2,
	VASTAI_PCI_BAR3,
	VASTAI_PCI_BAR4,
	VASTAI_PCI_BAR5,
	VASTAI_PCI_BAR_NUM
};

enum VASTAI_PCI_GEN_SEL {
	VASTAI_PCI_GEN1 = 1,
	VASTAI_PCI_GEN2,
	VASTAI_PCI_GEN3,
	VASTAI_PCI_GEN4
};

struct vastai_pci_chn_info {
	struct chn_ops *ops[32];
	// NTD
};

// struct __attribute__((packed)) cmcu_model_info {
// 	u64 msg_seq_id;
// 	u64 model_cfg_addr;
// 	u64 input_output_addr;
// };

// struct __attribute__((packed)) vdsp_cmd_info {
// 	u32 dev;
// 	/* cmd id */
// 	u32 cmd;
// 	u32 image_id;
// 	/* dev soc entry addr */
// 	u32 stream_addr;
// 	/* dev soc addr */
// 	u64 input_addr;
// 	u32 resvd;
// };

struct __attribute__((packed)) vdmcu_msg_head {
	/* rd msg num */
	u32 rd;
	/* wr msg num */
	u32 wr;
	/* max msg num */
	u32 elem_count;
	/* every msg element size */
	u32 elem_size;
	u8 buf[0];
};

struct bar_info {
	resource_size_t mmio_start;
	resource_size_t mmio_end;
	resource_size_t mmio_len;
	unsigned long mmio_flags;
	/* inbound pcie to axi address translation */
	unsigned long at_addr;
	unsigned char *mem;
	unsigned char *vmem;
	u64 map_in_addr;
	u64 map_out_addr;
	bool minish_bar_size;
};

struct msi_x_table {
	unsigned int msg_addr;
	unsigned int msg_upper_addr;
	unsigned int msg_data;
	unsigned int vector_control;
};

typedef int (*vastai_intr_handle)(void *pci_dev, u32 die_index, int irq,
				  void *ctxt, /* for user keep private data */
				  void *out); /* for out_desc */

typedef enum
{
	RESET_START,
	RESET_END
} reset_event;

typedef int (*vastai_reset_handle)(u32 die_index, u32 core_bit,
						reset_event event);


struct vastai_time {
	unsigned char sec : 6;
	unsigned char min : 6;
	unsigned char hour : 5;
	unsigned char day : 5;
	unsigned char mon : 4;
	unsigned year : 11;
};

enum vastai_reset_type {
	VASTAI_RESET_MSG_TRIGER =
		1 << 1, /* host send msg_irq to triger fw reset */
	VASTAI_RESET_BOOT_DEV = 1 << 2 /* reboot dev, fw download and ... */
};

struct vastai_fw_bmcu_xspi_version {
	union {
		u8 bmcu_ver[VASTAI_VER_COMMON_LEN];
		struct {
			u8 bmcu_sub_ver[VASTAI_VER_BMCU_XSPI_LEN];
			u8 xspi_sub_ver[VASTAI_VER_BMCU_XSPI_LEN];
		};
	};
};

struct vastai_fw_version {
	u8 ver[VASTAI_VER_COMMON_LEN];
	u8 smcu_ver[VASTAI_VER_COMMON_LEN];
	u8 bmcu_ver[VASTAI_VER_COMMON_LEN];
	u8 vdmcu_ver[VASTAI_VER_COMMON_LEN];
	u8 vemcu_ver[VASTAI_VER_COMMON_LEN];
	u8 vdsp_ver[VASTAI_VER_COMMON_LEN];
	u8 cmcu_ver[VASTAI_VER_COMMON_LEN];
	u8 lmcu_ver[VASTAI_VER_COMMON_LEN];
	u8 odsp_ver[VASTAI_VER_COMMON_LEN];
	u8 driver_ver[VASTAI_VER_COMMON_LEN];
	u8 phy_ver[VASTAI_VER_COMMON_LEN];
	u8 bmcu_backup_ver[VASTAI_VER_COMMON_LEN];
};

struct heartbeat_die {
	unsigned long cores_status; /* bit map: 0 - alive, 1 - hang */
	u32 last_counts[HEARTBEAT_CNT_REG_NUM];
};

struct heartbeat_monitor {
	unsigned long dies_status; /* bit map: 0 - alive, 1 - hang */
	unsigned long timeout_threshold; /* unit: jiffies */
	struct delayed_work heartbeat_work;
	struct heartbeat_die dies[VASTAI_SV100_MAX_DIE_NUM];
};

union die_index_data {
	u32 val;
	struct {
		u32 die_id : 8;
		u32 dev_id : 16;
		u32 seq_num : 8;
	};
};

enum vastai_dev_type {
	VASTAI_PROCESSING_ACCELERATORS_DEVICE,
};

enum vastai_dev_cap {
	VASTAI_DEV_CAP_DEFAULT,
	VASTAI_DEV_CAP_VIDEO_ONLY,
	VASTAI_DEV_CAP_AI_ONLY,
	VASTAI_DEV_CAP_VIDEO_AI,
};

struct vastai_dev_info {
	u8 dev_type;
	u8 dev_cap;
};

struct smcu_to_host_msg1 {
	union {
		u32 val;
		struct {
			u8 type;
			u8 info[3];
		};
	};
};

struct ctrl_cmd_list {
	atomic_t ctrl_seq_id;
	spinlock_t ctrl_cmd_list_lock;
	struct ctrl_cmd_entry {
		u8 seq_id;
		struct list_head list;
		struct completion msg_comp;

		/* 4Byte */
		union completion_info_st {
			struct smcu_to_host_msg1 smcu_to_host;
			struct {
				u8 type;
				u8 seq_id;
				u16 ret_val;
			} ret_st;
		} completion_info;
	} ctrl_cmd_head;
};
struct pci_link_monitor {
	pid_t pid;
	struct timer_list timer;
	unsigned long dma_bus_addr;
	void *dma_vir;
	u32 cnt;
	struct work_struct work;
	struct workqueue_struct *wq;
	int reset_count;
	u32 saved_config_space[16];
	atomic_t enable;
	atomic_t is_reseting;
};

struct bbox_info {
	union {
		u32 val;
		struct {
			u8 card_type;//	0: VA1, 1: VA1V, 2: VA1A,3: VA10, 4: VE1
			u8 reserved1;
			u8 bbox_type;
			u8 reserved2;
		};
	};
	spinlock_t bbox_lock;
};

struct efuse_harvest_info{
        u8 type;
        u16 dlc_cores;
};
/* *
 * vastai_sv100_die have those function:
 * 1. distribute irq
 * 2. udma_chn manager
 * 3. two msgq (1. for dma, 2. for ctrl cmd)
 * 4. vastai_fifo config
 * 5. log completion
 * 6. ecc completion
 * 7. core sync
 * 8. fw_version
 * 9. work around for vdmcu_pre_addr
 * */

struct vastai_sv100_die {
	struct vastai_pci_info *pci_info;
	u32 die_index;
	u32 render_id;
	struct intrrupt_info {
		int is_out_as_stream; /* is we need malloc out buf each time ? */
		struct out_desc out;
		u32 padding[256];
		void *private_data; /* for user keep private data */
		vastai_intr_handle intr_handle;
	} ivt[32];
	spinlock_t msg_ring_buf_lock;
	struct mutex cedar_lock;
	struct vmsgq vmsgq[2];
	struct msgq msgq[2];
	u32 boot_mode;
	struct completion log_comp;
	struct completion ecc_2bit_err;
	struct completion rw_xspi;
	u32 rw_xspi_msg;
	u32 pcie_tx_vdsp_count[4];
	u32 pcie_rx_vdsp_count[4];
	u32 dev_freq;
	u32 pcie_nls; /* negotiated_link_speed */
	u32 pcie_nlw; /* negotiated_link_width */
	struct vastai_credit *credit;
	struct ctrl_cmd_list ctrl_cmd_list;
	u8 core_sync_result[VASTAI_CORE_SYNC_TOTAL_NUM * 8];
	struct vastai_fw_version fw_ver;
	u64 vdmcu_pre_addr_table[3];
	u32 smi_ack_msg[2]; /*smi ack from smcu*/
	struct completion smi_ack[2]; /*smi ack from smcu*/
	struct completion core_time_sync_ack; /*core time sync ack from smcu*/
	/* udma */
	struct vastai_udma_chn *udma_chn;
	struct mutex done_lock;
	struct work_struct dma_done;
	struct workqueue_struct *udma_wq;

	u32 core_status_bitmap;
	struct work_struct exception_work;
	struct completion reset_comp;
	struct completion ai_reset_done;
	u32 heartbeat_mask;
	bool ai_sys_reset;
	bool vf_in_use;
#ifdef CONFIG_TOOLS_V2
	struct mutex tools_die_mutex; /* tools die mutex */
#endif
	struct mutex get_dlc_mutex;
	struct completion get_dlc_core_done;
};

enum IRQ_TYPE {
	VASTAI_INVALID_IRQ = 0,
	VASTAI_LEGACY_IRQ  = 1,
	VASTAI_MSI_IRQ     = 2,
	VASTAI_MSIX_IRQ    = 3
};

/* *
 * 1. manager power
 * 2. manager bar
 * 3. manager irq
 * */
struct vastai_pci_info {
	struct list_head dev_list;
	char dev_id;
	struct pci_dev *dev;
	struct char_drv_info *p_char;
	struct vastai_version_file *version_file;
	struct vastai_ctl_file *ctl_file;
#ifdef CONFIG_VASTAI_PCI_BOOT
	struct delayed_work boot_wq;
#endif
	unsigned char irq_type;
	unsigned int irq;
	unsigned char irq_num;
	unsigned char die_num;
	unsigned char boot_stage;
	unsigned char bar_num;
	unsigned long long ddr_attr;
	spinlock_t msgq_1_lock; /* need move to die */
	spinlock_t trans_lock; /* need move to die */
	struct completion set_bar_done;
	struct msix_entry msix[VASTAI_PCIE_MSIX_NUM];
	struct msi_x_table msix_table[VASTAI_PCIE_MSIX_NUM];
	struct bar_info bar[VASTAI_PCI_BAR_NUM];
	struct vastai_dev_info vadev;
	struct vastai_sv100_die dies[VASTAI_SV100_MAX_DIE_NUM];
	struct heartbeat_monitor heartbeat; /* need move to die */
	struct list_head vacc_dev_head; // the head of vastai_cdev list
	u8 disable;
	bool ddr_bw_test;
	struct vastai_mem_pool *vastai_pool;
	struct vastai_mem_pool_ex *vastai_pool_ex;
	atomic_t pci_state;
	bool iommu_en;
	struct list_head render_head;
	struct timer_list pci_timer; /* run each 10s */
	struct bbox_info bbox_info;
	struct efuse_harvest_info harvest_info[VASTAI_SV100_MAX_DIE_NUM];

	struct timer_list vf_timer;
	u32 boot_mode;
	bool is_physfn;
	bool is_virtfn;
	struct vf_info_t vf_info;
	struct vf_fifo_info vf_fifo;
	struct completion activate_vf_done;
	struct completion deactivate_vf_done;

	struct pci_link_monitor pci_link_stat;
#ifdef CONFIG_TOOLS_V2
	struct mutex tools_dev_mutex; /* tools device mutex */
#endif
};

enum DEVICE_STATE {
	VASTAI_NO_RUN_STATE = 0,
	VASTAI_NORMAL_STATE = 1,
	VASTAI_UPDATE_STATE = 2,
	VASTAI_BOOT_STATE = 3,
	VASTAI_EXIT_STATE = 4,
	VASTAI_ERROR_STATE = 5,
	VASTAI_RESET_STATE = 6,
	VASTAI_DEBUG_STATE = 7,
};

struct inbound_reg {
	unsigned int type; /* region contril 1 ;0:mem, 2:i/o 4:cfg */
	unsigned int en; /* region contril 2 [10:8]:BAR_NUM, */
	unsigned int lower_base_addr;
	unsigned int upper_base_addr;
	unsigned int limit;
	unsigned int lower_target_addr;
	unsigned int upper_target_addr;
} __packed;

struct outbound_reg {
	unsigned int type;
	unsigned int en;
	unsigned int lower_base_addr;
	unsigned int upper_base_addr;
	unsigned int limit;
	unsigned int lower_target_addr;
	unsigned int upper_target_addr;
} __packed;

struct vastai_dma_buf {
	void *vir;
	unsigned long dma_bus_addr;
	phys_addr_t phy;
	int size;
	struct page 			**pages;
	u8 				*pages_order;
	struct sg_table 		sg_table;
	unsigned int			num_pages;
	bool is_from_pool_ex;
	struct vastai_pci_info *pci_info;
};

// command number used in FW
#define CMD_RUN_MODEL	  0x40185602
#define CMD_RUN_STREAM	  0x40185603
#define CMD_LOAD_MODEL	  0x40185604
#define CMD_DESTROY_MODEL 0x40185605
#define CMD_LOAD_OPERATOR 0x40185606

struct __attribute__((packed)) pcie_transfer_info {
	/* refer to VASTAI_TRANSFER_TYPE
	 * type: 0:ai model; 1:video
	 */
	u32 type : 8;
	/* dir: refer to VASTAI_TRANSFER_DIR, the direction of data transfer is
	 * defined based on the host side.
	 * 0: host write to device, or device read from host.
	 * 1: host read from device, or device write to host.
	 */
	u32 dir : 1;
	u32 is_vf : 1;
	u32 padding_rsvd : 6;
	/* transfer number */
	u32 num : 8;
	u32 is_rc : 8;
	/* channel id, refer to VASTAI_TX_CHN_ID and VASTAI_RX_CHN_ID */
	u32 chn;
	/* data_type: 0:normal process, 1:only copy */
	u32 pcie_config_addr;
	union {
		u64 data_struct_addr;
		struct data_struct *data_struct;
	};
	u32 vdmcu_dma_msg_dst;
	u32 vdmcu_dma_msg_len;
	/*dma profiler info*/
	u32 time_of_dma_run;
	u32 time_of_dma_finish;
};


int vastai_pci_get_sriov_numvfs(char dev_id);

struct vastai_pci_info *get_vastai_pci_device_info(char dev_id);
#ifdef CONFIG_VASTAI_TOOLS_SUITE
int get_vastai_pci_device_number(void);
int vatools_fw_config_smi(struct vastai_pci_info *priv, int die_index);
int vatools_device_event_callback(u32 die_index, u32 core_bit, u32 event);
#endif
int vastai_pci_reset_bus(struct vastai_pci_info *pcie_dev);
void vastai_pci_set_vdsp_depth(struct vastai_pci_info *priv, u32 die_id,
			       u32 elem_count, u32 flag);
#ifdef CONFIG_VASTAI_CMCU_RINGBUF
void vastai_pci_set_cmcu_depth(struct vastai_pci_info *priv, u32 die_id, u32 elem_count);
#endif
void vastai_pci_init_bar_at(struct vastai_pci_info *priv, int bar);
void vastai_pci_init_all_die(struct vastai_pci_info *priv);
int vastai_pci_config_read(struct vastai_pci_info *pcie_dev, int offset,
			   char *buf, int len);
int vastai_pci_config_write(struct vastai_pci_info *pcie_dev, int offset,
			    char *buf, int len);
int vastai_pci_mem_read(struct vastai_pci_info *pcie_dev, int die_index,
			u64 relative_addr, void *buf, unsigned int len);
int vastai_pci_mem_write(struct vastai_pci_info *pcie_dev, int die_index,
			 u64 relative_addr, const void *buf, unsigned int len);
int vastai_pci_generation_sel(struct vastai_pci_info *pcie_dev, u8 gen);
int vastai_pci_generation_sel_slave_die(struct vastai_pci_info *pcie_dev,
					u8 gen, u32 die_id);
int vastai_pci_malloc_udma_buf(struct vastai_pci_info *pcie_dev,
			       struct vastai_dma_buf *dm, u32 len);
int vastai_pci_free_udma_buf(struct vastai_pci_info *pcie_dev,
			     struct vastai_dma_buf *dm);
void vastai_clear_fifo(void *pci_info, int die_index, union core_bitmap core_id);


/* This function will be used intead of vastai_pci_transfer_reg and
 * vastai_pci_transfer_reg_vd (declarate at vastai_pci_api.h).
 * For now if you call vastai_irq_register and
 * vastai_pci_transfer_reg/vastai_pci_transfer_reg_vd at the same time,
 * vastai_pci_transfer_reg/vastai_pci_transfer_reg_vd will take effect.
 */
int vastai_irq_register(struct vastai_pci_info *pcie_dev, /* which pcie_group */
			int die_index,
			int irq, /* reference vastai_sv100_reg.h +272 */
			vastai_intr_handle handle,
			void *private_data); /* for user keep self data */
int vastai_irq_unregister(struct vastai_pci_info *pcie_dev, int die_index,
			  int irq);

int vastai_pci_get_die_id(struct vastai_pci_info *pcie_dev, u32 die_index);
int get_vastai_die_info(struct vastai_pci_info *priv, int die_index[],
			int *die_num);

/* ========================api for dev_id version ============================*/
static inline int vastai_pci_info_get_dev_id(struct vastai_pci_info *pcie_dev)
{
	return pcie_dev->dev_id;
}

static inline int vastai_pci_info_get_die_num(struct vastai_pci_info *pcie_dev)
{
	return pcie_dev->die_num;
}

/* This function returns the attributes of the DDR used by each DIE in each PCIE device:
 * e.g.
 * return 0x1_0000_0000 means ddr density is 4GB;
 * return 0x2_0000_0000 means ddr density is 8GB;
 * return 0x4_0000_0000 means ddr density is 16GB.
 */
static inline u64 vastai_pci_info_get_ddr_attr(struct vastai_pci_info *pcie_dev)
{
	return pcie_dev->ddr_attr;
}

typedef struct PCIE_cbits_s {
	/** Control bits */
	uint8_t control_bits;
} PCIE_cbits;

/** uDMA transfer size and control byte */
typedef struct PCIE_sz_ctrl_s {
	/** Number of bytes to be transferred.  For max bulk transfer size, set
	 * to zero */
	uint32_t size : 24;
	/** Control byte */
	PCIE_cbits ctrl_bits;
} PCIE_sz_ctrl;

/** uDMA status bytes */
typedef struct PCIE_sbytes_s {
	/** System (local) bus status */
	uint8_t sys_status;
	/** External (remote) bus status */
	uint8_t ext_status;
	/** uDMA channel status */
	uint8_t chnl_status;
	/** Reserved */
	uint8_t reserved_0;
} PCIE_sbytes;

/** uDMA Descriptor structure */
struct PCIE_xd_desc_s {
	/** Low 32 bits of system address */
	uint32_t sys_lo_addr;
	/** High 32 bits of system address */
	uint32_t sys_hi_addr;
	/** Access attributes for system bus */
	uint32_t sys_attr;
	/** Low 32 bits of external address */
	uint32_t ext_lo_addr;
	/** High 32 bits of external address */
	uint32_t ext_hi_addr;
	/** Access attributes for external bus */
	uint32_t ext_attr;
	/** High 32 bits of access attributes for external bus */
	uint32_t ext_attr_hi;
	/** Transfer size and control byte */
	PCIE_sz_ctrl size_and_ctrl;
	/** Transfer status.  This word is written by uDMA engine, and can be
	 * read to determine status. */
	PCIE_sbytes status;
	/** Low 32bits of pointer to next descriptor in linked list */
	uint32_t next;
	/** High 32bits of pointer to next descriptor in linked list */
	uint32_t next_hi_addr;
	uint32_t trans_info_addr;
};
typedef struct PCIE_xd_desc_s PCIE_xd_desc;

struct dma_buf *vastai_alloc_dma_buf(struct vastai_pci_info *pci_dev, int size);
struct dma_buf *vastai_alloc_dma_buf_sg(struct vastai_pci_info *pci_info, int size);
struct vastai_dma_buf *get_vastai_dma_buf(struct dma_buf *dma_buf, u32 index);
int vastai_device_reset(struct vastai_pci_info *priv, u32 flags);
int vastai_send_pcie_cmd(struct vastai_pci_info *priv, u32 die_index,
			 u32 sub_cmd, u64 paras);
bool vastai_dev_not_ready(struct vastai_pci_info *priv);
int vastai_get_seqnum(struct vastai_pci_info *pdev, unsigned int die_id);

u32 vastai_get_dlc_cores_info(struct vastai_pci_info *pcie_info, u32 die_index);

void vastai_set_desc_link(struct vastai_pci_info *priv, unsigned int die_index,
			  u64 *desc_tab, int num);
int vastai_pci_data_movement(void *pcie_dev, int die_index, u64 local_addr,
			     u64 remote_addr, u32 len, bool is_local_2_remote);
void vastai_pci_get_fw_ver(struct vastai_pci_info *priv, u32 mask);
int vastai_wait_ecc_2_bit(struct vastai_pci_info *pdev,
			  union die_index_data die_index);
int vastai_pci_dma_transfer_memset(void *pci_info,
			int die_index,
			union core_bitmap core_id,
			dma_addr_t dma_handle,
			u32 is_src_dma_addr,
			u32 is_host_to_dev,
			u32 is_src_not_user_mem,
			u64 axi_addr,
			u32 size);
int vastai_pci_dma_transfer_sg(void *pci_info, int die_index,
			union core_bitmap core_id,
			struct vastai_dma_buf *dm,
			u32 is_src_dma_addr,
			u32 is_host_to_dev,
			u32 is_src_not_user_mem,
			u64 axi_addr,
			u32 size);
int vastai_pci_dma_transfer_by_uaddr(struct vastai_pci_info *pci_info,
			int die_index,
			union core_bitmap core_id,
			u32 is_host_to_dev,
			u64 axi_addr,
			u64 user_addr,
			u32 size);
int vastai_dma_sg_set_desc(struct vastai_dmadesc* desc,
            struct vastai_sg_gather_info sg_info,
            int i,
            u64 axi_addr,
            u32 done_size,
            u32 is_host_to_dev,
            u32 is_src_not_user_mem,
            u32 is_src_dma_addr);
struct ecc_entry {
	u32 addr_1;
	u32 addr_h;
	u32 axi_id;
	u32 axi_data_l;
	u32 axi_data_h;
	u32 errbit;
};

void vastai_update_time(struct vastai_pci_info *priv);

struct vastai_file_info {
	char file_name[32];
	dev_t dev_num;
	struct cdev dev;
	struct class *class;
	struct device *device;
};

struct vastai_version_file {
	struct vastai_file_info file_info;
	struct vastai_pci_info *pci_info;
};

struct vastai_ctl_file {
	struct vastai_file_info file_info;
	struct vastai_pci_info *pci_info;
};

int vastai_file_create(struct vastai_file_info *file_info,
		       struct vastai_pci_info *pci_info,
		       const struct file_operations *fop, const char *fmt, ...);
void vastai_file_destroy(struct vastai_file_info *file_info);
#define to_vastai_file_info(ptr) container_of(ptr, struct vastai_file_info, dev)
dev_t vastai_file_get_major_minor(struct vastai_file_info *file_info);
int vastai_get_render_id(struct vastai_pci_info *pdev, u32 die_index);
int vastai_get_vacc_id(struct vastai_pci_info *pdev, u32 die_index);
int vastai_send_ctrl_cmd(struct vastai_pci_info *pci_info, u32 die_index,
			 u32 core_bit_map, u64 info);
void vastai_ctrl_cmd_compeletion(struct vastai_pci_info *pci_info,
				 u32 die_index, struct smcu_to_host_msg1 *msg);
u64 vastai_get_host_time_ns(void);
u32 vastai_pci_gen_die_index(void *pci_info, u32 die_id);
int vastai_pci_mem_read_direct(struct vastai_pci_info *priv, int die_index,
			u64 relative_addr, void *buf, unsigned int len);
int vastai_pci_mem_bar2_read(struct vastai_pci_info *priv, int die_index, u64 absolute_addr,
				 void *buf, unsigned int len);
int vastai_pci_mem_bar2_write(struct vastai_pci_info *priv, int die_index, u64 absolute_addr,
				const void *buf, unsigned int len);
int vastai_pci_mem_write_direct(struct vastai_pci_info *priv, int die_index,
			 u64 relative_addr, const void *buf, unsigned int len);
int vastai_device_send_reset(struct vastai_pci_info *priv);
void vastai_pci_reset_bar_at(struct vastai_pci_info *priv, int die_index);

int vastai_bbox_set(struct vastai_pci_info *priv, u32 bbox);
u32 vastai_bbox_get(struct vastai_pci_info *priv);

void vastai_pci_link_monitor_init(struct vastai_pci_info *pci_info);
void vastai_pci_link_monitor_uninit(struct vastai_pci_info *pci_info);
void vastai_enable_pcie_link_monitor(struct vastai_pci_info *pci_info, bool enable);

int vastai_get_msgq_id(struct vastai_pci_info *priv, int type);
int vastai_mempool_ex_init(struct vastai_pci_info *pci_info);
void vastai_mempool_ex_deinit(struct vastai_pci_info *pci_info);
void vastai_show_mempool_ex(struct vastai_pci_info *pci_info);
void vastai_reset_mempool_ex(struct vastai_pci_info *pci_info);

#endif /* end of __VASTAI_PCI_H__ */
