/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: gang.wang
 */

#ifndef __VASTAI_PCI_IOCTL_H__
#define __VASTAI_PCI_IOCTL_H__

#ifdef __KERNEL__
#include <linux/cdev.h>
#include <linux/kernel.h>

#include "vastai_pci.h"
#ifdef CONFIG_VASTAI_PCI_BOOT
#include "vastai_pci_boot.h"
#endif

#define VASTAI_CMD_READ_LEN (4096)

#define VASTAI_TEST_PASS    (0)
#define VASTAI_TEST_FAILURE (-1)

//for normal, this is 0x40_0000
#define VASTAI_PCI_TLV_BUF_LEN (VASTAI_MAX_DMA_BUF)

enum VASTAI_PCI_ARG_INDEX {
	VASTAI_PCI_ARG_ADDR,
	VASTAI_PCI_ARG_VAL,
	VASTAI_PCI_ARG_LEN,
	VASTAI_PCI_ARG_BAR,
	VASTAI_PCI_ARG_RUN,
	VASTAI_PCI_ARG_OFFSET,
	VASTAI_PCI_ARG_VIR,
	VASTAI_PCI_ARG_REGION,
	VASTAI_PCI_ARG_MODE,
	VASTAI_PCI_ARG_CASE,
	VASTAI_PCI_ARG_USER_ADDR,
	VASTAI_PCI_ARG_MAX,
};

struct vastai_pci_arg_t {
	unsigned char id;
	unsigned char *name;
	unsigned long def;
};

struct char_drv_info {
	struct vastai_file_info file_info;
	struct vastai_pci_info *pcie_dev_info;
	struct vastai_dma_buf *dm;
	u32 valid_data_length;
	struct vastai_pci_arg_t args[VASTAI_PCI_ARG_MAX];
};

int vastai_pci_ioctl_init(struct vastai_pci_info *bus);
int vastai_pci_ioctl_deinit(struct vastai_pci_info *bus);
#endif

struct user_dma_desc {
	char cmd_str[8]; //"DMAW \0"
	union {
		struct {
			u32 is_host_to_dev : 1;
			u32 mode : 2;
			u32 ch : 3;
			u32 is_ep : 1;
			u32 die_id : 2;
		};
		u32 val;
	};
	u32 num;
	struct descriptor {
		u64 dev_addr;
		u64 host_addr;
		u32 length;
	} desc_tab[0];
};

struct vastai_cedar_img_info {
	u64 *input_addr;
	u64 *output_addr;
	u32 alg_type;
	u32 blen;
};

typedef enum
{
	RESET_NOTIFY_STOP,
	RESET_STOP_COMPLETION,
	RESET_AI_SYS_DONE
} reset_msg;


struct kchar_cmd {
	union {
		struct {
			u32 size;
			int dma_buf_fd;
			u64 dma_addr_t;
		} alloc_cmd;

		struct {
			u32 is_dev_to_host;
			u32 dma_buf_fd;
			u64 axi_addr;
			u32 size;
			u32 die_index;
		} dma_start_cmd;
		struct {
			u32 is_dev_to_host;
			void *vir_addr;
			u64 axi_addr;
			u32 length;
			u32 die_index;
			int pid;
		} dma_trans_cmd;
		struct {
			u32 is_dev_to_host;
			u64 axi_addr;
			u32 length;
			u32 die_index;
			char val;
		} dma_memset_cmd;
		struct {
			u32 is_dev_to_host;
			u32 die_index;
			u64 axi_addr;
			struct vastai_channel_buf *channel_buf;
			int channel_num;
			int pid;
		} dma_trans_sg_cmd;
		struct {
			u32 die_index;
			u32 timeout_water;
			u32 is_timeout;
			void *log_ring_ctl;
		} log_poll_cmd;
		char port_name[16];
		struct {
			u32 die_index;
			char file_name[16];
		} get_render_name;
		struct {
			u32 die_index;
			u32 pcie_tx_vdsp_count[4];
			u32 pcie_rx_vdsp_count[4];
		} get_die_performance;
		struct {
			u32 pci_id;
		} get_pci_id;
		struct {
			u32 pci_sub_id;
		} get_pci_sub_id;
		struct {
			u32 major_minor;
		} get_major_minor;
		struct {
			struct vastai_cedar_img_info cedar_info;
			struct {
				u32 done_flag;
				u32 cedar_res;
				u32 hash[16];
			} cedar_res_info;
			u32 die_id;
		} cedar_ioctl;

		struct {
			reset_msg mode;
			u32 die_id;
			u32 mcu_monitor;
			u32 mcu_status;
		} reset_mode;
	};
};

#define VASTAI_PCI_IOCTL_ALLOC		 0
#define VASTAI_PCI_IOCTL_DMA_START	 1
#define VASTAI_PCI_IOCTL_DMA_TRANS	 3
#define VASTAI_PCI_IOCTL_DMA_TRANS_SG	 4
#define VASTAI_PCI_IOCTL_RESET_NOTIFY	 5
#define VASTAI_PCI_IOCTL_MEMSET      7
#define VASTAI_PCI_IOCTL_POLL_LOG	 0xa
#define VASTAI_PCI_IOCTL_RENDER		 0xB
#define VASTAI_PCI_IOCTL_PORT		 0xC
#define VASTAI_PCI_IOCTL_GET_PERFORMANCE 0xD
#define VASTAI_PCI_IOCTL_PCI_ID		 0XE
#define VASTAI_PCI_IOCTL_PCI_SUB_ID	 0XF
#define VASTAI_PCI_IOCTL_MAJOR_MINOR	 0X10
#define VASTAI_PCI_IOCTL_CEDAR_CMD	 0X20

#endif /* end of __VASTAI_PCI_IOCTL_H__ */
