/*
 * vastai_pci_vf
 * Copyright (C) 2022 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2022/10/13
 * Author: xiangming.yang
 */

#ifndef __VASTAI_PCI_VF_H__
#define __VASTAI_PCI_VF_H__

struct vastai_pci_info;

/* VF Bar size */
#define VASTAI_PCIE_VF_BAR0_SIZE		(0x100000UL)	//1MB
#define VASTAI_PCIE_VF_BAR1_SIZE		(0x100000UL)	//1MB
#define VASTAI_PCIE_VF_BAR2_SIZE		(0x2000000UL)	//32MB
#define VASTAI_PCIE_VF_BAR4_SIZE		(0x2000000UL)	//32MB

#define VASTAI_PCIE_VF_BAR_RESERVED (0x00000000ULL)

/* VF AXI address */
#define VASTAI_PCIE_VF_BAR0_AXI_ADDR (0x08b00000ULL)
#define VASTAI_PCIE_VF_BAR1_AXI_ADDR (0x08c00000ULL)
#define VASTAI_PCIE_VF_BAR2_AXI_ADDR (0x0814000000ULL)
#define VASTAI_PCIE_VF_BAR4_AXI_ADDR (0x00000000ULL)

/*
 0x08cfd800:  VF_INFO                (size 0x200)
 0x08cfda00:  VF_FIFO  (HOST -> VM)  (size 0x300)
 0x08cfdd00:  VF_FIFO  (VM -> HOST)  (size 0x300)
*/

#define VF_INFO_REGION_SIZE     (0x800)
#define VF_INFO_OFFSET			(0x000fd800UL)
#define VF_INFO_BASE            (0x08cfd800UL)
#define VF_INFO_MAX_SIZE        (0x200)
#define VF_INFO_MAP_BAR			(1)
#define MAX_SUPPORT_VF			(4)

#define VF_FIFO_OFFSET          (VF_INFO_OFFSET + VF_INFO_MAX_SIZE)
#define VF_FIFO_BASE            (VASTAI_PCIE_VF_BAR1_AXI_ADDR + VF_FIFO_OFFSET)
#define VF_FIFO_SIZE            ((VF_INFO_REGION_SIZE - VF_INFO_MAX_SIZE) / 2)

#define OS_MSG_SIZE             ()

struct vastai_os_msg {
	u32 type;
	u32 cmd;
	union {
		u32 msg_32[2];
		u8 msg_8[8];
	};
};

/*
 * Invalid mode means FLASH/FW provide none boot param
 * or it does not support compatible boot (PCIE/FW/XSPI)
 */
enum VASTAI_BOOT_MODE {
	BOOT_PF_NORMAL = 0,    // param from XSPI/FW boot_mode
	BOOT_PF_SRIOV = 1,     // according to XSPI/FW boot_mode
	BOOT_VF_SRIOV = 2,     // according to device id 0x101
	BOOT_MODE_MAX = 3,
};

enum VASTAI_PCI_VF_ID {
	VASTAI_PCI_VF_ID_1 = 1,
	VASTAI_PCI_VF_ID_2,
	VASTAI_PCI_VF_ID_3,
	VASTAI_PCI_VF_ID_4,
};

struct vf_fifo_info {
	u32 local_addr;
	u32 peer_addr;
	spinlock_t vf_msg_lock;
};

struct smcu_sriov_reg {
	u32 magic;
	u32 boot_mode;
};

struct __attribute__((packed)) vf_info_t {
	struct smcu_sriov_reg status;
	u32 vf_id;
	u32 bdf;
	u32 vf_activated;
	u32 vf_preempted;
	u32 msix_addr_l[MAX_SUPPORT_VF];
	u32 msix_addr_h[MAX_SUPPORT_VF];
	u32 update_msix_ob_flag;
};

/* make sure vf_info occupies less than 512Bytes */
_Static_assert( (sizeof(struct vf_info_t) <= VF_INFO_MAX_SIZE), "size of VF_INTO_T overflowed");

void vastai_proc_peer_msg(struct vastai_pci_info* priv, int die_id);
void vastai_send_msg_to_peer_test(struct vastai_pci_info* priv, int die_id);
void vastai_pci_config_vf_bar_at(struct vastai_pci_info *priv, int bar);
void vastai_vf_timer_init(struct vastai_pci_info *pciv);
int vastai_pci_vf_init(struct vastai_pci_info *priv);
int vastai_pci_vf_activate(struct vastai_pci_info *pci_info, int num_vfs);
int vastai_pci_vf_deactivate(struct vastai_pci_info *pci_info);
void vastai_vf_os_fifo_init(struct vastai_sv100_die *die);
int vastai_get_boot_mode(struct vastai_pci_info *priv);
int vastai_get_die_boot_mode(struct vastai_sv100_die *die);

#endif

