/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: gang.wang
 */

#ifndef __VASTAI_SV100_REG_H__
#define __VASTAI_SV100_REG_H__

#include "csram_partition_v05.h"
#include "ddr_partition_v05.h"

#define VASTAI_DIE1_BASE (0x1000000000UL)

#define PCIE_LINK_CONTROL_STATUS_REG_OFFSET  (0xD0)
#define PCIE_LINK_CONTROL_STATUS_REG2_OFFSET (0xF0)

/* address map */
#define VASTAI_PCIE_EP_IO_SPACE_BASE_ADDR (0x08000000UL)
#define VASTAI_PCIE_EP_CONFIG_BASE_ADDR	  (0x08300000UL)
#define VASTAI_AXI_CSR_BASE_ADDR	  (0x08BC0000UL)
#define VASTAI_PCIE_EP_APB_BASE_ADDR	  (0x08E00000UL)
#define VASTAI_PCIE_RC_APB_BASE_ADDR	  (0x09700000UL)

#define VASTAI_PCIE_EP_IO_SPACE_ADDR(x)                                        \
	(x * VASTAI_DIE1_BASE + VASTAI_PCIE_EP_IO_SPACE_BASE_ADDR)
#define VASTAI_CSRAM_ADDR(x)	 (x * VASTAI_DIE1_BASE + CSRAM_BASE_ADDR)
#define VASTAI_DDR_SPACE_ADDR(x) (x * VASTAI_DIE1_BASE + DDR_BASE_ADDR)

/* AXI CSR */
#define VASTAI_HOST_BOOT_STAGE_REG (VASTAI_AXI_CSR_BASE_ADDR + 0x6E0)
#define VASTAI_SMCU_BOOT_STAGE_REG (VASTAI_AXI_CSR_BASE_ADDR + 0x6E4)
#define VASTAI_MCU_DSP_STATUS_REG  (VASTAI_AXI_CSR_BASE_ADDR + 0x6E8)

/* bmcu download address: CSRAM */
#define VASTAI_BMCU_DL_ADDR (MODULE_INFO_BASE)

/* bl1 download address: CSRAM */
#define VASTAI_BL1_DL_ADDR (CSRAM_BASE_ADDR)

/* xspi download address: CSRAM */
/* same as BMCU_DL_ADDR */
#define VASTAI_XSPI_DL_ADDR (MODULE_INFO_BASE)
/* firmware download address: DDR */
#define VASTAI_FW_DL_OFFSET	 (0x40000000UL)
#define VASTAI_FW_DL_ADDR	 (DDR_BASE_ADDR + VASTAI_FW_DL_OFFSET)
#define VASTAI_DDR_FW_DL_ADDR(x) (x * VASTAI_DIE1_BASE + VASTAI_FW_DL_ADDR)

/* vastai controller wrap registers */
#define VASTAI_PCI_CONFIGURATION_IO_0 (VASTAI_PCIE_EP_APB_BASE_ADDR + 18 * 4)

/* cadence controller registers */
#define VASTAI_PCI_CDN_CONTROL_BASE_ADDR                                       \
	(VASTAI_PCIE_EP_APB_BASE_ADDR + 0x100000)
#define VASTAI_PCI_CDN_LOCAL_MANAGEMENT_REG                                    \
	(VASTAI_PCI_CDN_CONTROL_BASE_ADDR + 0x100000)
#define VASTAI_PCI_CDN_VENDOR_ID_REG                                           \
	(VASTAI_PCI_CDN_LOCAL_MANAGEMENT_REG + 0x44)
#define VASTAI_PCI_CDN_LINKWIDTH_CONTROL_REG                                   \
	(VASTAI_PCI_CDN_LOCAL_MANAGEMENT_REG + 0x50)
#define VASTAI_PCI_CDN_PHY_FUNC_BAR_CFG_REG0                                   \
	(VASTAI_PCI_CDN_LOCAL_MANAGEMENT_REG + 0x240)
#define VASTAI_PCI_CDN_PHY_FUNC_BAR_CFG_REG1                                   \
	(VASTAI_PCI_CDN_LOCAL_MANAGEMENT_REG + 0x244)

#define VASTAI_PCI_CDN_AXI_CONFIGURATION_REG                                   \
	(VASTAI_PCI_CDN_CONTROL_BASE_ADDR + 0x400000)
#define VASTAI_PCI_CDN_FUNC0_BAR0_EP_INBOUND_ADDR_TRANS_REG0                   \
	(VASTAI_PCI_CDN_AXI_CONFIGURATION_REG + 0x840)

#define VASTAI_PCI_LINK_CONTROL_STATUS_REG                                     \
	(VASTAI_PCI_CDN_CONTROL_BASE_ADDR + PCIE_LINK_CONTROL_STATUS_REG_OFFSET)
#define VASTAI_PCI_LINK_CONTROL_STATUS_REG2                                    \
	(VASTAI_PCI_CDN_CONTROL_BASE_ADDR +                                    \
	 PCIE_LINK_CONTROL_STATUS_REG2_OFFSET)

#define VASTAI_PCIE_EP_CONFIG_VID_DID	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR)
#define VASTAI_PCIE_EP_CONFIG_CMD_STATUS (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x4)
#define VASTAI_PCIE_EP_CONFIG_BAR0	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x10)
#define VASTAI_PCIE_EP_CONFIG_BAR1	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x14)
#define VASTAI_PCIE_EP_CONFIG_BAR2	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x18)
#define VASTAI_PCIE_EP_CONFIG_BAR3	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x1C)
#define VASTAI_PCIE_EP_CONFIG_BAR4	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x20)
#define VASTAI_PCIE_EP_CONFIG_BAR5	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x24)
#define VASTAI_PCIE_EP_CONFIG_MSI_CTRL	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x90)
#define VASTAI_PCIE_EP_CONFIG_MSI_MSG_L	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x94)
#define VASTAI_PCIE_EP_CONFIG_MSI_MSG_H	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x98)
#define VASTAI_PCIE_EP_CONFIG_MSIX_CTRL	 (VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0xB0)

#define VASTAI_PCIE_BAR0_PCIE_ADDR_8MB	 (0x8800000UL)
#define VASTAI_PCIE_BAR0_PCIE_ADDR_32MB	 (0x8000000UL)
#define VASTAI_PCIE_BAR0_PCIE_ADDR_256MB (0UL)
#define VASTAI_PCIE_BAR1_PCIE_ADDR_8MB	 (0x10000000UL)
#define VASTAI_PCIE_BAR1_PCIE_ADDR_32MB	 (0x10000000UL)
#define VASTAI_PCIE_BAR1_PCIE_ADDR_256MB (0x10000000UL)
#define VASTAI_PCIE_BAR2_PCIE_ADDR_32MB	 (DDR_BASE_ADDR + VASTAI_FW_DL_OFFSET)
#define VASTAI_PCIE_BAR2_PCIE_ADDR_32GB	 (DDR_BASE_ADDR)
#define VASTAI_PCIE_BAR2_PCIE_ADDR_256GB (0UL)
#define VASTAI_PCIE_BAR4_PCIE_ADDR_256GB (0UL)

#define VASTAI_PCIE_BAR0_AXI_ADDR_2MB	(0x8a00000UL)
#define VASTAI_PCIE_BAR0_AXI_ADDR_8MB	(VASTAI_PCIE_BAR0_PCIE_ADDR_8MB)
#define VASTAI_PCIE_BAR0_AXI_ADDR_32MB	(VASTAI_PCIE_BAR0_PCIE_ADDR_32MB)
#define VASTAI_PCIE_BAR0_AXI_ADDR_256MB (VASTAI_PCIE_BAR0_PCIE_ADDR_256MB)
#define VASTAI_PCIE_BAR1_AXI_ADDR_4MB	(0x8c00000UL)
#define VASTAI_PCIE_BAR1_AXI_ADDR_8MB	(0x0A000000UL + VASTAI_DIE1_BASE)
#define VASTAI_PCIE_BAR1_AXI_ADDR_32MB                                         \
	(VASTAI_PCIE_EP_IO_SPACE_BASE_ADDR + VASTAI_DIE1_BASE)
#define VASTAI_PCIE_BAR2_AXI_ADDR_32MB_1 (VASTAI_PCIE_BAR2_PCIE_ADDR_32MB)
#define VASTAI_PCIE_BAR2_AXI_ADDR_32MB_2 (DDR_BASE_ADDR + 0x14000000UL)
#define VASTAI_PCIE_BAR2_AXI_ADDR_32GB	 (VASTAI_PCIE_BAR2_PCIE_ADDR_32GB)
#define VASTAI_PCIE_BAR4_AXI_ADDR_128MB_1                                      \
	(DDR_BASE_ADDR + VASTAI_FW_DL_OFFSET + VASTAI_DIE1_BASE)
#define VASTAI_PCIE_BAR4_AXI_ADDR_128MB_2 (0x1400000000UL)
#define VASTAI_PCIE_BAR4_AXI_ADDR_32GB	  (DDR_BASE_ADDR + VASTAI_DIE1_BASE)

#define VASTAI_PCIE_BAR0_BASE0_IN_OLD (0x8000000UL)
#define VASTAI_PCIE_BAR1_BASE0_IN_OLD (0x1008000000UL)
#define VASTAI_PCIE_BAR2_BASE0_IN_OLD (0x800000000UL)
#define VASTAI_PCIE_BAR4_BASE0_IN_OLD (0x1800000000UL)

#define VASTAI_PCIE_BAR0_BASE0_IN (0x8800000UL)
#define VASTAI_PCIE_BAR1_BASEX_IN (0x8B00000UL)
#define VASTAI_PCIE_BAR1_BASE0_IN (0x1008B00000UL)
#define VASTAI_PCIE_BAR1_BASE1_IN (0x2008B00000UL)
#define VASTAI_PCIE_BAR1_BASE2_IN (0x3008B00000UL)
#define VASTAI_PCIE_BAR2_BASE0_IN (0x814000000UL)
#define VASTAI_PCIE_BAR4_BASEX_IN (0x814000000UL)
#define VASTAI_PCIE_BAR4_BASE0_IN (0x1814000000UL)
#define VASTAI_PCIE_BAR4_BASE1_IN (0x2814000000UL)
#define VASTAI_PCIE_BAR4_BASE2_IN (0x3814000000UL)

#define VASTAI_PCIE_BAR0_BASE0_OUT (0x8800000UL)
#define VASTAI_PCIE_BAR1_BASEX_OUT VASTAI_PCIE_BAR1_BASE0_OUT
#define VASTAI_PCIE_BAR1_BASE0_OUT (0x100A000000UL)
#define VASTAI_PCIE_BAR1_BASE1_OUT (0x100A200000UL)
#define VASTAI_PCIE_BAR1_BASE2_OUT (0x100A400000UL)
#define VASTAI_PCIE_BAR2_BASE0_OUT (0x814000000UL)
#define VASTAI_PCIE_BAR4_BASEX_OUT VASTAI_PCIE_BAR4_BASE0_OUT
#define VASTAI_PCIE_BAR4_BASE0_OUT (0x1400000000UL)
#define VASTAI_PCIE_BAR4_BASE1_OUT (0x1402000000UL)
#define VASTAI_PCIE_BAR4_BASE2_OUT (0x1404000000UL)

#define VASTAI_MULTI_DIE_MSI_ADDR (0x4000000000UL)

#define CMCU_GRP12_WR (96 * 4) // 0x180

#define CMCU_MSGQ_GRP0_AFULL_OFFSET  (110 * 4) // 0x1B8
#define CMCU_MSGQ_GRP0_AEMPTY_OFFSET (111 * 4) // 0x1BC
#define CMCU_MSGQ_GRP0_WR_OFFSET     (112 * 4) // 0x1C0
#define CMCU_MSGQ_GRP0_RD_OFFSET     (113 * 4) // 0x1C4
/*
 * CMCU_MSGQ_GRP0_INT_OFFSET
 * bit0:full_msk
 * bit1:afull_msk
 * bit2:empty_msk
 * bit3:nempty_msk
 * bit4:aempty_msk
 * bit5~7:reserved
 * bit8:full_int
 * bit9:afull_int
 * bit10:empty_int
 * bit11:nempty_int
 * bit12:aempty_int
 */
#define CMCU_MSGQ_GRP0_INT_OFFSET    (114 * 4) // 0x1C8
#define CMCU_MSGQ_GRP1_AFULL_OFFSET  (115 * 4) // 0x1CC
#define CMCU_MSGQ_GRP1_AEMPTY_OFFSET (116 * 4) // 0x1D0
#define CMCU_MSGQ_GRP1_WR_OFFSET     (117 * 4) // 0x1D4
#define CMCU_MSGQ_GRP1_RD_OFFSET     (118 * 4) // 0x1D8
/*
 * CMCU_MSGQ_GRP1_INT_OFFSET
 * bit0:full_msk
 * bit1:afull_msk
 * bit2:empty_msk
 * bit3:nempty_msk
 * bit4:aempty_msk
 * bit5~7:reserved
 * bit8:full_int
 * bit9:afull_int
 * bit10:empty_int
 * bit11:nempty_int
 * bit12:aempty_int
 */
#define CMCU_MSGQ_GRP1_INT_OFFSET (119 * 4) // 0x1DC
#define VDSP0_GRP5_WR		  (398 * 4) // 0x638
#define VDSP1_GRP5_WR		  (400 * 4) // 0x640
#define VDSP2_GRP5_WR		  (402 * 4) // 0x648
#define VDSP3_GRP5_WR		  (404 * 4) // 0x650
#define VDMCU0_GRP5_WR		  (414 * 4) // 0x678
#define VDMCU1_GRP4_WR		  (394 * 4) // 0x628
#define VDMCU2_GRP5_WR		  (418 * 4) // 0x688

#define CMCU_GRP12_WR_REG (VASTAI_AXI_CSR_BASE_ADDR + CMCU_GRP12_WR)
#define CMCU_MSGQ_GRP0_AFULL_REG                                               \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP0_AFULL_OFFSET)
#define CMCU_MSGQ_GRP0_AEMPTY_REG                                              \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP0_AEMPTY_OFFSET)
#define CMCU_MSGQ_GRP0_WR_REG                                                  \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP0_WR_OFFSET)
#define CMCU_MSGQ_GRP0_RD_REG                                                  \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP0_RD_OFFSET)
#define CMCU_MSGQ_GRP0_INT_REG                                                 \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP0_INT_OFFSET)
#define CMCU_MSGQ_GRP1_AFULL_REG                                               \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP1_AFULL_OFFSET)
#define CMCU_MSGQ_GRP1_AEMPTY_REG                                              \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP1_AEMPTY_OFFSET)
#define CMCU_MSGQ_GRP1_WR_REG                                                  \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP1_WR_OFFSET)
#define CMCU_MSGQ_GRP1_RD_REG                                                  \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP1_RD_OFFSET)
#define CMCU_MSGQ_GRP1_INT_REG                                                 \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP1_INT_OFFSET)
#define VDSP0_GRP5_WR_REG  (VASTAI_AXI_CSR_BASE_ADDR + VDSP0_GRP5_WR)
#define VDSP1_GRP5_WR_REG  (VASTAI_AXI_CSR_BASE_ADDR + VDSP1_GRP5_WR)
#define VDSP2_GRP5_WR_REG  (VASTAI_AXI_CSR_BASE_ADDR + VDSP2_GRP5_WR)
#define VDSP3_GRP5_WR_REG  (VASTAI_AXI_CSR_BASE_ADDR + VDSP3_GRP5_WR)
#define VDMCU0_GRP5_WR_REG (VASTAI_AXI_CSR_BASE_ADDR + VDMCU0_GRP5_WR)
#define VDMCU1_GRP4_WR_REG (VASTAI_AXI_CSR_BASE_ADDR + VDMCU1_GRP4_WR)
#define VDMCU2_GRP5_WR_REG (VASTAI_AXI_CSR_BASE_ADDR + VDMCU2_GRP5_WR)

#define VASTAI_EACH_CHN_DESC_CNT 64
#define VASTAI_PCIE_2_VDSP_MSG_DEPTH                                           \
	((MSG_PCIE_TO_VDSP1 - MSG_PCIE_TO_VDSP0 -                              \
	  sizeof(struct vastai_fifo)) /                                        \
	 sizeof(union buf_data))
#define VASTAI_PCIE_2_CMCU_MSG_DEPTH ((MSG_SMCU_TO_CMCU - MSG_PCIE_TO_CMCU -\
	sizeof(struct vastai_fifo))/ sizeof(union buf_data))
#define VASTAI_PCIE_MSGQ_0_SIZE 0x400
#define VASTAI_PCIE_MSGQ_1_SIZE 0x380
#define VASTAI_PCIE_MSGQ_1_SPE_SIZE	(0x80)

#define VASTAI_PCIE_SMCU_0_VF_OFFSET (VASTAI_PCIE_MSGQ_0_SIZE / 2)
#define VASTAI_PCIE_SMCU_1_VF_OFFSET (VASTAI_PCIE_MSGQ_1_SIZE / 2)

#define VASTAI_DMA_TRANS_INFO_FIFO_SIZE (1024 * 20)
#define VASTAI_DMA_TRANS_INFO_VF_OFFSET (VASTAI_DMA_TRANS_INFO_FIFO_SIZE / 4)

#define VASTAI_DMA_DESC_FIFO_SIZE (0xA000)
#define VASTAI_DMA_DESC_VF_OFFSET (VASTAI_DMA_DESC_FIFO_SIZE / 4)

#define SMCU_2_HOST_DMA_DONE (0)
#define SMCU_2_HOST_INT0     (1)
#define SMCU_2_HOST_INT1     (2)
#define VDSP0_2_HOST_INT0    (10)
#define VDSP0_2_HOST_INT1    (11)
#define VDSP1_2_HOST_INT0    (12)
#define VDSP1_2_HOST_INT1    (13)
#define VDSP2_2_HOST_INT0    (14)
#define VDSP2_2_HOST_INT1    (15)
#define VDSP3_2_HOST_INT0    (16)
#define VDSP3_2_HOST_INT1    (17)
#define CMCU_2_HOST_INT0     (20)
#define CMCU_2_HOST_INT1     (21)
#define VEMCU0_2_HOST_INT    (22)
#define VEMCU1_2_HOST_INT    (23)
#define VEMCU2_2_HOST_INT    (24)
#define VEMCU3_2_HOST_INT    (25)
#define VDMCU0_2_HOST_INT    (26)
#define VDMCU1_2_HOST_INT    (27)
#define VDMCU2_2_HOST_INT    (28)

#define HOST_GUEST_INT       (31)

#define HEARTBEAT_CNT_REG_NUM  30 /* reaserve0/1 are used by log_read_flag */
#define HEARTBEAT_CNT_REG_MASK 0xFFFFFFFC
enum heartbeat_cnt_reg_core_id {
	HEARTBEAT_SMCU_THREAD = 0,
	HEARTBEAT_SMCU = 1,
	HEARTBEAT_CMCU = 2,
	HEARTBEAT_LMCU0 = 3,
	HEARTBEAT_LMCU1 = 4,
	HEARTBEAT_LMCU2 = 5,
	HEARTBEAT_LMCU3 = 6,
	HEARTBEAT_LMCU4 = 7,
	HEARTBEAT_LMCU5 = 8,
	HEARTBEAT_LMCU6 = 9,
	HEARTBEAT_LMCU7 = 10,
	HEARTBEAT_ODSP0 = 11,
	HEARTBEAT_ODSP1 = 12,
	HEARTBEAT_ODSP2 = 13,
	HEARTBEAT_ODSP3 = 14,
	HEARTBEAT_ODSP4 = 15,
	HEARTBEAT_ODSP5 = 16,
	HEARTBEAT_ODSP6 = 17,
	HEARTBEAT_ODSP7 = 18,
	HEARTBEAT_VDMCU0 = 19,
	HEARTBEAT_VDMCU1 = 20,
	HEARTBEAT_VDMCU2 = 21,
	HEARTBEAT_VEMCU0 = 22,
	HEARTBEAT_VEMCU1 = 23,
	HEARTBEAT_VEMCU2 = 24,
	HEARTBEAT_VEMCU3 = 25,
	HEARTBEAT_VDSP0 = 26,
	HEARTBEAT_VDSP1 = 27,
	HEARTBEAT_VDSP2 = 28,
	HEARTBEAT_VDSP3 = 29,
	HEARTBEAT_RESERVE0 = 30,
	HEARTBEAT_RESERVE1 = 31
};

#define VASTAI_CORE_SYNC_TOTAL_NUM	(30)
#define VASTAI_FW_BOOT_OK		(0x10)
#endif /* end of __VASTAI_SV100_REG_H__ */
