/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: gang.wang
 */

#ifndef __VASTAI_UDMA_ENGINE_H__
#define __VASTAI_UDMA_ENGINE_H__

#include <linux/list.h>
#include <linux/semaphore.h>
#include <linux/spinlock_types.h>
#include "vastai_pci.h"
#include "vastai_dmabuf.h"
#include "vastai_fifo.h"

/* maximum length transfer 2^24 bytes, i.e. 16M bytes. */
#define VASTAI_UDMA_LENGTH_MAX	(0x1000000)
#define VASTAI_UDMA_LENGTH_MASK (0xFFFFFF)
#define VASTAI_UDMA_MALLOC_MAX	(0x400000)

#define CPDREF_TEST_FAILURE 1
//#define CPDREF_FUNC_TIMEDOUT     0x05

#define UDMA_DESC_SIZE (sizeof(PCIE_xd_desc))

struct vastai_pid_entry {
	struct list_head list;
	int pid;
};

struct vastai_credit {
	struct vastai_pid_entry pid_entry_head;
	spinlock_t lock;
	struct semaphore credit_sem;
};

#define VASTAI_DMA_HIGH_PRIORITY  VASTAI_MEM_POOL_MIN_NR / 2
#define VASTAI_DMA_LOW_PRIORITY	  1
#define VASTAI_DMA_DEFAULT_CREDIT VASTAI_DMA_HIGH_PRIORITY
#define VASTAI_DMA_CREDIT_LIMIT	  10

int vastai_pci_dma_transfer_sync_pid(void *pci_info, int die_index,
				     union core_bitmap core_id,
				     struct vastai_dmadesc desc[], int desc_num,
				     int pid);
int vastai_udma_init(struct vastai_sv100_die *die);
void vastai_udma_deinit(struct vastai_sv100_die *die);

#define GET_UDMA_CHN(dir, is_rc) (dir + is_rc * 2)

struct vastai_udma_chn {
	struct vastai_fifo dma_desc_fifo;
	struct vastai_fifo trans_info_fifo;
	struct list_head head;
	struct mutex trans_mutex;
	spinlock_t lock;
};

struct vastai_udma_config {
	u64 trans_info_fifo_addr;
	u64 dma_desc_fifo_addr;
};

extern const struct vastai_udma_config vastai_udma_config[VASTAI_DMA_CHN];

#endif /* end of __VASTAI_UDMA_ENGINE_H__ */
