/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: frank.bai
 */
#include "vatools.h"
#include "vatools_log.h"


LIST_HEAD(g_nodes_list);

//初始化时写入csram头部的数据
T_SMI_BLOCK_HEADER g_smi_block_header = {
	0x00000002,
	0xA1000002,
	{ SMI_CMD_BLOCK0_ADDR, SMI_CMD_BLOCK1_ADDR, SMI_CMD_BLOCK2_ADDR,
	  SMI_CMD_BLOCK3_ADDR, SMI_CMD_BLOCK4_ADDR, SMI_CMD_BLOCK5_ADDR,
	  SMI_CMD_BLOCK6_ADDR, SMI_CMD_BLOCK7_ADDR }
};

//保存block地址和使用标志
T_SMI_BLOCK g_smi_blocks[SMI_BLOCK_ID_MAX] = {
	{ SMI_CMD_BLOCK_ID_0, SMI_CMD_BLOCK0_ADDR, SMI_CMD_BLOCK0_LEN,
	  SMI_CMD_BLOCK_IDLE, SMI_CMD_LOOP_TOTAL_COUNT, 100 },
	{ SMI_CMD_BLOCK_ID_1, SMI_CMD_BLOCK1_ADDR, SMI_CMD_BLOCK1_LEN,
	  SMI_CMD_BLOCK_IDLE, 50, 1 },
	{ SMI_CMD_BLOCK_ID_2, SMI_CMD_BLOCK2_ADDR, SMI_CMD_BLOCK2_LEN,
	  SMI_CMD_BLOCK_IDLE, 0, 0 },
	{ SMI_CMD_BLOCK_ID_3, SMI_CMD_BLOCK3_ADDR, SMI_CMD_BLOCK3_LEN,
	  SMI_CMD_BLOCK_IDLE, 0, 0 },
	{ SMI_CMD_BLOCK_ID_4, SMI_CMD_BLOCK4_ADDR, SMI_CMD_BLOCK4_LEN,
	  SMI_CMD_BLOCK_IDLE, 0, 0 },
	{ SMI_CMD_BLOCK_ID_5, SMI_CMD_BLOCK5_ADDR, SMI_CMD_BLOCK5_LEN,
	  SMI_CMD_BLOCK_IDLE, 0, 0 },
	{ SMI_CMD_BLOCK_ID_6, SMI_CMD_BLOCK6_ADDR, SMI_CMD_BLOCK6_LEN,
	  SMI_CMD_BLOCK_IDLE, 0, 0 },
	{ SMI_CMD_BLOCK_ID_7, SMI_CMD_BLOCK7_ADDR, SMI_CMD_BLOCK7_LEN,
	  SMI_CMD_BLOCK_IDLE, 0, 0 }
};

//****************************************************************************
// 使用 block_id
T_SMI_BLOCK *vatools_fw_smi_get_block(unsigned int block_id)
{
	return &g_smi_blocks[block_id];
}

T_SMI_BLOCK *
vatools_fw_smi_get_block_via_app_category(unsigned int app_category)
{
	switch (app_category) {
	case VATOOLS_APP_DUMMY:
		return &g_smi_blocks[SMI_CMD_BLOCK_ID_0];
		break;
	case VATOOLS_APP_SMI:
		return &g_smi_blocks[SMI_CMD_BLOCK_ID_0];
		break;
	case VATOOLS_APP_PROFILER:
		return &g_smi_blocks[SMI_CMD_BLOCK_ID_1];
		break;
	case VATOOLS_APP_DEBUGGER:
		return &g_smi_blocks[SMI_CMD_BLOCK_ID_0];
		break;
	case VATOOLS_APP_LOGGER:
		return &g_smi_blocks[SMI_CMD_BLOCK_ID_0];
		break;
	case VATOOLS_APP_MONITOR:
		return &g_smi_blocks[SMI_CMD_BLOCK_ID_0];
		break;
	case VATOOLS_APP_SHAREDMEM:
		return &g_smi_blocks[SMI_CMD_BLOCK_ID_0];
		break;
	}

	return &g_smi_blocks[app_category];
}

// smi csram 初始化
int vatools_fw_config_smi(struct vastai_pci_info *priv, int die_index)
{
	int ret = 0;
	u8 *dummy;
	u32 size = 1024 * 4;

	dummy = (u8 *)kvmalloc(size, GFP_KERNEL);
	if (dummy == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "kvmalloc error. size=%d",
			    size);
		return -ENOMEM;
	}
	memset(dummy, 0, size);

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "smi init");
	ret = vastai_pci_mem_write_direct(priv, die_index,
					  SMI_DIE_ADDRESS_OFFSET_BASE,
					  (u8 *)dummy, 2560);
	VATOOLS_DUMP_BRIEF("vatools_fw_config_smi header init: ",
			   &g_smi_block_header, sizeof(T_SMI_BLOCK_HEADER));
	ret = vastai_pci_mem_write_direct(priv, die_index,
					  SMI_DIE_ADDRESS_OFFSET_BASE,
					  (u8 *)&g_smi_block_header,
					  sizeof(T_SMI_BLOCK_HEADER));
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "smi init done. ret=%d", ret);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "profiler init");
	ret = vastai_pci_mem_write_direct(priv, die_index, PROFILER_CSRAM_ADDR,
					  (u8 *)dummy, PROFILER_CSRAM_LEN);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "profiler init done. ret=%d", ret);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "debugger init");
	ret = vastai_pci_mem_write_direct(priv, die_index, VADBG_CSRAM_ADDR,
					  (u8 *)dummy, VADBG_CSRAM_LEN);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "debugger init done. ret=%d\n", ret);
	kvfree(dummy);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "vatools config/init done");

	return ret;
}

//封装调用pci driver 的函数
int vatools_get_vastai_pci_device_number(void)
{
	return get_vastai_pci_device_number();
}

struct vastai_pci_info *vatools_get_vastai_pci_device_info(char dev_id)
{
	return get_vastai_pci_device_info(dev_id);
}

struct vastai_pci_info *
vatools_get_vastai_pci_device_info_by_die_index(u32 die_index)
{
	union die_index_data did;
	did.val = (u32)die_index;
	return get_vastai_pci_device_info(did.dev_id);
}

/******************************************************************************
 * bar size 修改后, 封装 pcie vastai_pci_mem_read 函数,
 * 如果地址不在bar 范围内, 则<=128bytes, 使用smi访问; > 128bytes, 使用dma

Die0 BAR0: [0x00_0880_0000,0x00_0900_0000) 8M
Die0 BAR1: [0x10_0A00_0000,0x10_0A80_0000) 8M
Die0 BAR2: [0x08_4000_0000,0x08_4200_0000) 32M
Die0 BAR4: [0x18_4000_0000,0x18_4800_0000) 128M


BAR0 ----> Die0 [0x00_0880_0000,0x00_0900_0000) 8M
BAR1 ----> Die1 [0x10_08B0_0000,0x10_08D0_0000) 2M
            Die2 [0x20_08B0_0000,0x20_08D0_0000) 2M
            Die3 [0x30_08B0_0000,0x30_08D0_0000) 2M
BAR2/3 ----> Die0 [0x08_1400_0000,0x08_1600_0000) 32M
BAR4/5 ----> Die1 [0x18_1400_0000,0x18_1600_0000) 32M
            Die2 [0x28_1400_0000,0x28_1600_0000) 32M
            Die3 [0x38_1400_0000,0x38_1600_0000) 32M

 */

vatools_bar_address_t vatools_die_check[4][2] = {
	{ { 0x08800000, 0x09000000 }, { 0x0814000000, 0x0816000000 } },
	{ { 0x08b00000, 0x08d00000 }, { 0x0814000000, 0x0816000000 } },
	{ { 0x08b00000, 0x08d00000 }, { 0x0814000000, 0x0816000000 } },
	{ { 0x08b00000, 0x08d00000 }, { 0x0814000000, 0x0816000000 } }
};

//返回值: < 0 , 错误; >0 成功,读取的数据数目; =0 成功
int vatools_pci_mem_read(struct vastai_pci_info *priv, int die_index,
			 u64 relative_addr, void *buf, unsigned int len)
{
	int i = 0;
	int die_id =
		(die_index > 0) ? vastai_pci_get_die_id(priv, die_index) : 0;

	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"die_id=%d, die_index=%d relative_addr=0x%llx buf=%p len=%d\n",
		die_id, die_index, relative_addr, buf, len);

	for (i = 0; i < 2; i++) {
		if ((relative_addr >= vatools_die_check[die_id][i].begin &&
		     relative_addr < vatools_die_check[die_id][i].end)) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID, "by cpu\n");
			//在bar 可以访问的地址范围, 使用 pcie vastai_pci_mem_read
			return vastai_pci_mem_read(priv, die_index,
						   relative_addr, buf, len);
		}
	}
	if (len <= VATOOLS_BAR_SMI_MAX_BYTES) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "by smi\n");
		// len 小于等于smi可以单次访问的最大空间, 使用smi读取数据
		return vatools_fw_smcu_read(SMI_CMD_BLOCK_ID_0, priv, die_index,
					    relative_addr, buf, len);
	} else {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "by dma\n");
		// TODO:  使用 pcie dma 读取数据. 暂未实现
		return vastai_pci_mem_read(priv, die_index, relative_addr, buf,
					   len);
	}
}

//返回值: < 0 , 错误; >0 成功,读取的数据数目; =0 成功
int vatools_pci_mem_write(struct vastai_pci_info *priv, int die_index,
			  u64 relative_addr, const void *buf, unsigned int len)
{
	int i = 0;
	int die_id =
		(die_index != -1) ? vastai_pci_get_die_id(priv, die_index) : 0;

	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"die_id=%d, die_index=%d relative_addr=0x%llx buf=%p len=%d\n",
		die_id, die_index, relative_addr, buf, len);

	for (i = 0; i < 2; i++) {
		if ((relative_addr >= vatools_die_check[die_id][i].begin &&
		     relative_addr < vatools_die_check[die_id][i].end)) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID, "by cpu\n");
			//在bar 可以访问的地址范围, 使用 pcie vastai_pci_mem_read
			return vastai_pci_mem_write(priv, die_index,
						    relative_addr, buf, len);
		}
	}

	if (len <= VATOOLS_BAR_SMI_MAX_BYTES) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "by smi\n");
		// len 小于等于smi可以单次访问的最大空间, 使用smi读取数据
		return vatools_fw_smcu_write(SMI_CMD_BLOCK_ID_0, priv,
					     die_index, relative_addr, buf,
					     len);
	} else {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "by dma\n");
		// TODO:  使用 pcie dma 读取数据. 暂未实现
		return vastai_pci_mem_write(priv, die_index, relative_addr, buf,
					    len);
	}
}

// static int vatools_get_vastai_die_info( struct vastai_pci_info* priv, int die_index[], int* die_num )
// {
//     return get_vastai_die_info( priv, die_index, die_num );
// }

//函数修改u8* buf 命令里面的flag参数, 修改为0. ddr数据都写完后, 设置为1, 然后发中断到smcu
void vatools_driver_smi_set_flag(u8 *buf, u32 size, u8 flag)
{
	V_UNREFERENCE(size);
	*(buf + 6) = flag;
	VATOOLS_DUMP_BRIEF("vatools_driver_smi_set_flag: ", buf, size);
	return;
}
int vatools_get_vastai_die_info_via_device_id(char dev_id, int die_index[],
					      int *die_num)
{
	return get_vastai_die_info(vatools_get_vastai_pci_device_info(dev_id),
				   die_index, die_num);
}

int vatools_get_vastai_pci_die_index(char dev_id, int die_id)
{
	struct vastai_pci_info *priv =
		vatools_get_vastai_pci_device_info(dev_id);
	return priv->dies[die_id].die_index;
}

/// dump smi block memory
int vatools_dump_smi_block_mem(struct vastai_pci_info *priv, int die_index,
			       u32 block_id, u32 len, int from)
{
	int ret = 0;
	void *buffer;
	T_SMI_BLOCK *psmi_block;
	u64 smi_die_address_offset;
	char title[100];

	memset(title, 0, 100);

	psmi_block = vatools_fw_smi_get_block(block_id);
	smi_die_address_offset = psmi_block->block_address;

	buffer = kvmalloc(len, GFP_KERNEL);
	if (!buffer) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, " kvmalloc error. size= %d",
			    len);
		ret = -ENOMEM;
		goto out;
	}
	memset(buffer, 0x00, len);
	ret = vastai_pci_mem_read(priv, die_index, smi_die_address_offset,
				  buffer, len);
	if (ret < 0) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "vastai_pci_mem_read ret=%d",
			    ret);
		goto out_kfree;
	}
	memset(title, 0, 100);
	sprintf(title, "[die=%08x block=%x addr=0x%llx from=%d] ", die_index,
		block_id, smi_die_address_offset, from);

	print_hex_dump(KERN_INFO, title, DUMP_PREFIX_NONE, 32, 4, buffer, len,
		       false);

out_kfree:
	kvfree(buffer);
out:
	return ret;
}

int vatools_fw_send_interrupt(struct vastai_pci_info *priv, int die_index,
			      u32 type)
{
	struct pcie_transfer_info trans = { .type = type };
	u32 die_id = vastai_pci_get_die_id(priv, die_index);
	int ret;
	u32 msgq_id;

	msgq_id = vastai_get_msgq_id(priv, MSGQ_CMD);
	ret = vastai_pci_vmsgq_wr(&(priv->dies[die_id].vmsgq[msgq_id]), &trans,
				  sizeof(trans));

	if (ret)
		return -EINVAL;
	return 0;
}

int vatools_fw_send_smi_interrupt(struct vastai_pci_info *priv, int die_index)
{
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "priv=%p, die_index=%d", priv,
		    die_index);

	return vatools_fw_send_interrupt(priv, die_index, VASTAI_SMI);
}

int vatools_wait_smi_ack(struct vastai_pci_info *priv, u32 die_index,
			 u32 block_id, int from)
{
	struct vastai_sv100_die *die;
	int ret = 0;
	u32 die_id;
	union die_index_data did;
	u32 msg = 0;
	u32 timeout_ms = 1000;

	did.val = (u32)die_index;
	die_id = did.die_id;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "priv=%p, die_index=%d die_id=%d wait start from=%d", priv,
		    die_index, die_id, from);

	if (!priv || (priv->die_num <= die_id)) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "priv->die_num=%d < die_id=%d",
			    priv->die_num, die_id);
		return -EINVAL;
	}
	die = &(priv->dies[die_id]);

	/* wait completion int from smcu */
	/* 如果是用户中断, 需要在这里清除一下上次smi命令的ack应答,
	否则每次进入到这里, wait_for都会直接返回, 导致wait和ack不匹配*/
	// reinit_completion(&(die->smi_ack[block_id]));
	ret = wait_for_completion_interruptible_timeout(
		&(die->smi_ack[block_id]), msecs_to_jiffies(timeout_ms));
	msg = die->smi_ack_msg[block_id];
	reinit_completion(&(die->smi_ack[block_id]));
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"wait completion ret=%d msg=0x%x die_id=%d die_index=0x%x block_id=%d",
		ret, msg, die_id, die_index, block_id);
	if (ret == 0) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "timeout! (%d ms) ret=%d\n",
			    timeout_ms, ret);

		//dump data for debug
		vatools_dump_smi_block_mem(priv, die_index, block_id, 128,
					   from);
		return -ETIMEDOUT;
	} else if (ret == -ERESTARTSYS || ret == -EINTR) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "process be killed! ret=%d\n",
			    ret);
		return -EINTR;
	} else if (ret < 0) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "Unknow err! ret=%d\n", ret);
		return -EIO;
	} else {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "done. msg(flag)=%x", msg);
		return msg;
	}
}

//****************************************************************************
// 使用 block_id 通过 SMCU 访问DDR, CSR
int vatools_fw_smcu_read(u32 block_id, struct vastai_pci_info *priv,
			 int die_index, u64 relative_addr, void *buf,
			 unsigned int len)
{
	int ret = 0;
	int ready = 0;
	void *get_buffer;
	T_SMI_BLOCK *psmi_block;
	u64 smi_die_address_offset;
	u32 loop;
	u32 interval;
	TS_SMI_CMD_REQ cmd;
	struct vastai_sv100_die *die;
	union die_index_data did;
	u32 die_id;

	VATOOLS_DBG(priv, die_index, "block_id=%d, priv=%p, die_index=%d",
		    block_id, priv, die_index);

	if (len % 4 != 0) {
		VATOOLS_DBG(priv, die_index, "len=%d, not align by 4 bytes",
			    len);
		ret = -EFAULT;
		goto out;
	}

	psmi_block = vatools_fw_smi_get_block(block_id);
	smi_die_address_offset = psmi_block->block_address;
	loop = psmi_block->query_count;
	interval = psmi_block->query_interval;

	cmd.cmd = SMI_CMD_CODE_CTL_READ;
	cmd.flag = FLAG_APP_PREPARE;
	cmd.unit = 0xFFFFFFFF;
	cmd.data_len = (u16)len;
	cmd.rw_addr = relative_addr;

	VATOOLS_DUMP_BRIEF("vatools_fw_smcu_read write cmd: ", &cmd,
			   sizeof(cmd));

	ret = vastai_pci_mem_write(priv, die_index, smi_die_address_offset,
				   (u8 *)&cmd, sizeof(TS_SMI_CMD_REQ));
	if (ret < 0) {
		VATOOLS_DBG(priv, die_index, "ret=%d", ret);
		goto out;
	}

	// set flag to 1, write only 4 bytes , sync with fw
	cmd.flag = FLAG_APP_REQ;
	ret = vastai_pci_mem_write(priv, die_index, smi_die_address_offset + 4,
				   ((u8 *)&cmd) + 4, sizeof(u32));
	if (ret < 0) {
		VATOOLS_DBG(priv, die_index, "ret=%d", ret);
		goto out;
	}

	/* 如果是用户中断, 需要在这里清除一下上次smi命令的ack应答,
	否则每次进入到这里, wait_for都会直接返回, 导致wait和ack不匹配*/
	did.val = (u32)die_index;
	die_id = did.die_id;
	if (!priv || (priv->die_num <= die_id)) {
		VATOOLS_DBG(priv, die_index, "priv->die_num=%d < die_id=%d",
			    priv->die_num, die_id);
		ret = -EINVAL;
		goto out;
	}
	die = &(priv->dies[die_id]);
	reinit_completion(&(die->smi_ack[block_id]));

	ret = vatools_fw_send_smi_interrupt(priv, die_index);
	ret = vatools_wait_smi_ack(priv, die_index, block_id, 1);
	if (ret < 0) {
		if (ret == -EINTR) {
			VATOOLS_DBG(
				priv, die_index,
				"smi_ack: die_index=%x block_id=%d ret=%d(-EINTR)",
				die_index, block_id, ret);
			goto out;
		} else if (ret == -ETIMEDOUT) {
			VATOOLS_ERR(
				priv, die_index,
				"smiack: die_index=%x block_id=%d ret=%d(-ETIMEDOUT)",
				die_index, block_id, ret);
		} else {
			VATOOLS_ERR(priv, die_index,
				    "smi_ack: die_index=%x block_id=%d ret=%d",
				    die_index, block_id, ret);
			goto out;
		}
	} else if (ret != FLAG_FW_DATA_READY) {
		ret = -ENXIO;
		goto out;
	}

	get_buffer = kzalloc(sizeof(TS_SMI_CMD_REQ) + len, GFP_KERNEL);

	memset(get_buffer, 0x00, sizeof(TS_SMI_CMD_REQ) + len);
	ret = vastai_pci_mem_read(priv, die_index, smi_die_address_offset,
				  get_buffer, sizeof(TS_SMI_CMD_REQ) + len);
	if (ret < 0) {
		VATOOLS_DBG(priv, die_index, "ret=%d", ret);
		goto out_kfree;
	}

	VATOOLS_DUMP_BRIEF("vatools_fw_smcu_read read get_buffer: ", get_buffer,
			   sizeof(TS_SMI_CMD_REQ) + len);

	memcpy(&cmd, get_buffer, sizeof(TS_SMI_CMD_REQ));

	VATOOLS_DBG(priv, die_index, "read cmd.flag=%d", cmd.flag);

	if (cmd.flag == FLAG_FW_DATA_READY) {
		ready = 1;
		//数据准备好
		memcpy(buf, ((u8 *)get_buffer) + sizeof(TS_SMI_CMD_REQ), len);
		ret = len;
		goto out_kfree;
	} else if (cmd.flag == FALG_FW_ERROR_ABORT) {
		ret = -ENXIO;
		goto out_kfree;
	}

out_kfree:
	kvfree(get_buffer);
out:
	VATOOLS_DBG(priv, die_index, "done ret=%d", ret);

	return ret;
}

int vatools_fw_smcu_write(u32 block_id, struct vastai_pci_info *priv,
			  int die_index, u64 relative_addr, const void *buf,
			  unsigned int len)
{
	int ret = 0;
	int ready = 0;
	void *get_buffer;
	TS_SMI_CMD_REQ cmd;
	void *set_buffer;
	T_SMI_BLOCK *psmi_block;
	u64 smi_die_address_offset;
	u32 loop;
	u32 interval;
	struct vastai_sv100_die *die;
	union die_index_data did;
	u32 die_id;

	VATOOLS_DBG(priv, die_index, "block_id=%d, priv=%p, die_index=%d",
		    block_id, priv, die_index);

	if (len % 4 != 0) {
		VATOOLS_DBG(priv, die_index, "len=%d, not align by 4 bytes",
			    len);
		ret = -EFAULT;
		goto out;
	}

	psmi_block = vatools_fw_smi_get_block(block_id);
	smi_die_address_offset = psmi_block->block_address;
	loop = psmi_block->query_count;
	interval = psmi_block->query_interval;

	cmd.cmd = SMI_CMD_CODE_CTL_WRITE;
	cmd.flag = FLAG_APP_PREPARE;
	cmd.unit = 0;
	cmd.data_len = (u16)len;
	cmd.rw_addr = relative_addr;

	VATOOLS_DUMP_BRIEF("vatools_fw_smcu_write cmd: ", &cmd, sizeof(cmd));

	set_buffer = kzalloc(sizeof(TS_SMI_CMD_REQ) + cmd.data_len, GFP_KERNEL);

	memcpy(set_buffer, &cmd, sizeof(TS_SMI_CMD_REQ));
	memcpy(((u8 *)set_buffer) + sizeof(TS_SMI_CMD_REQ), buf, cmd.data_len);

	VATOOLS_DUMP_BRIEF("vatools_fw_smcu_write write_buffer: ", set_buffer,
			   sizeof(TS_SMI_CMD_REQ) + cmd.data_len);

	ret = vastai_pci_mem_write(priv, die_index, smi_die_address_offset,
				   (void *)set_buffer,
				   sizeof(TS_SMI_CMD_REQ) + cmd.data_len);

	if (ret) {
		VATOOLS_DBG(priv, die_index, "ret=%d", ret);
		goto out_kfree;
	}

	// set flag to 1, write only 4 bytes , sync with fw
	cmd.flag = FLAG_APP_REQ;
	ret = vastai_pci_mem_write(priv, die_index, smi_die_address_offset + 4,
				   ((u8 *)&cmd) + 4, sizeof(u32));
	if (ret < 0) {
		VATOOLS_DBG(priv, die_index, "ret=%d", ret);
		goto out_kfree;
	}

	/* 如果是用户中断, 需要在这里清除一下上次smi命令的ack应答,
	否则每次进入到这里, wait_for都会直接返回, 导致wait和ack不匹配*/
	did.val = (u32)die_index;
	die_id = did.die_id;
	if (!priv || (priv->die_num <= die_id)) {
		VATOOLS_DBG(priv, die_index, "priv->die_num=%d < die_id=%d",
			    priv->die_num, die_id);
		ret = -EINVAL;
		goto out;
	}
	die = &(priv->dies[die_id]);
	reinit_completion(&(die->smi_ack[block_id]));

	ret = vatools_fw_send_smi_interrupt(priv, die_index);
	ret = vatools_wait_smi_ack(priv, die_index, block_id, 2);
	if (ret < 0) {
		if (ret == -EINTR) {
			VATOOLS_DBG(
				priv, die_index,
				"smi_ack: die_index=%x block_id=%d ret=%d(-EINTR)",
				die_index, block_id, ret);
			goto out;
		} else if (ret == -ETIMEDOUT) {
			VATOOLS_ERR(
				priv, die_index,
				"smiack: die_index=%x block_id=%d ret=%d(-ETIMEDOUT)",
				die_index, block_id, ret);
		} else {
			VATOOLS_ERR(priv, die_index,
				    "smi_ack: die_index=%x block_id=%d ret=%d",
				    die_index, block_id, ret);
			goto out;
		}
	} else if (ret != FLAG_FW_DATA_READY) {
		ret = -ENXIO;
		goto out;
	}

	get_buffer = kzalloc(sizeof(TS_SMI_CMD_REQ) + len, GFP_KERNEL);

	memset(get_buffer, 0x00, sizeof(TS_SMI_CMD_REQ) + len);
	ret = vastai_pci_mem_read(priv, die_index, smi_die_address_offset,
				  get_buffer, sizeof(TS_SMI_CMD_REQ) + len);
	if (ret < 0) {
		VATOOLS_DBG(priv, die_index, "ret=%d", ret);
		goto out_kfree;
	}

	VATOOLS_DUMP_BRIEF("vatools_fw_smcu_write read_buffer: ", get_buffer,
			   sizeof(TS_SMI_CMD_REQ) + len);

	memcpy(&cmd, get_buffer, sizeof(TS_SMI_CMD_REQ));

	VATOOLS_DBG(priv, die_index, "write cmd.flag=%d", cmd.flag);

	if (cmd.flag == FLAG_FW_DATA_READY) {
		ready = 1;
		ret = 0;
		goto out_kfree_get_buffer;
	} else if (cmd.flag == FALG_FW_ERROR_ABORT) {
		ret = -ENXIO;
		goto out_kfree_get_buffer;
	}

out_kfree_get_buffer:
	kvfree(get_buffer);

out_kfree:
	kvfree(set_buffer);
out:
	VATOOLS_DBG(priv, die_index, "ret=%d", ret);

	return ret;
}

//****************************************************************************
// smi cmd block end
//****************************************************************************

struct vatools_node *vatools_file_get_node(struct file *filp)
{
	struct vatools_reader *reader =
		(struct vatools_reader *)filp->private_data;
	// VATOOLS_DBG( "filp=0x%p  reader= 0x%p  node= 0x%p  app_category=%d", filp, reader, reader->node,
	//                      reader->trans_category.app_category );
	if (!reader) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "reader NULL");
		// pr_info( "reader NULL" );
		return NULL;
	}
	return reader->node;
}

struct vatools_reader *vatools_file_get_reader(struct file *filp)
{
	struct vatools_reader *reader =
		(struct vatools_reader *)filp->private_data;
	// VATOOLS_DBG( "filp=0x%p  reader= 0x%p  node= 0x%p  app_category=%d", filp, reader, reader->node,
	//                      reader->trans_category.app_category );
	return reader;
}

//先写,然后再读, 读时检查flag
long vatools_driver_smi_fetch(struct file *filp, unsigned int cmd,
			      unsigned long arg)
{
	// struct vatools_node*   node   = vatools_file_get_node( filp );
	// struct vatools_reader* reader = vatools_file_get_reader( filp );

	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	int write_len = 0;
	int read_len = 0;

	T_SMI_IOCTL_TRANS_DATA kdata = { 0 };
	TS_SMI_CMD_REQ smi_cmd = { 0 };
	TS_SMI_CMD_REQ *smi_cmd_p = NULL;

	struct vastai_pci_info *priv = NULL;
	int die_index = DUMMY_DIE_ID;
	void *write_buf = NULL;
	T_SMI_BLOCK *smi_block = NULL;
	u64 smi_die_address_offset = 0;
	int ready = 0;
	void *read_buf = NULL;
	int re_trigger_fw_count = 3;
	int ret2 = 0;
	struct vastai_sv100_die *die;
	union die_index_data did;
	u32 die_id;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	VATOOLS_DBG(priv, die_index, "ioctl_cmd= 0x%x ", cmd);

	memset(&kdata, 0x00, sizeof(T_SMI_IOCTL_TRANS_DATA));
	ret = copy_from_user(&kdata, argp, sizeof(kdata));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

	VATOOLS_DBG(priv, die_index,
		    "block_id=%d  device_id=%d  die_id=%d  flag= 0x%x  addr= "
		    "0x%llx ",
		    kdata.block_id, kdata.device.dev_id, kdata.device.die_id,
		    kdata.flag, kdata.address);
	VATOOLS_DBG(
		priv, die_index,
		"read_len= %d  read_buf= 0x%p  write_len= %d  write_buf= 0x%p",
		kdata.output_buf.buf_size, kdata.output_buf.buf_addr,
		kdata.input_buf.buf_size, kdata.input_buf.buf_addr);

	write_len = kdata.input_buf.buf_size;
	read_len = kdata.output_buf.buf_size;
	if ((read_len % 4 != 0) || (write_len % 4 != 0)) {
		VATOOLS_ERR(priv, die_index,
			    "read_len=%d, write_len=%d, not align by 4 bytes",
			    read_len, write_len);
		ret = -EFAULT;
		goto out;
	}

	priv = vatools_get_vastai_pci_device_info((char)kdata.device.dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(priv, die_index, "priv is NULL, dev_id[%u] ",
			    kdata.device.dev_id);
		ret = -ENODEV;
		goto out;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(priv, die_index, "priv->pci_state= 0x%x\n",
			    atomic_read(&priv->pci_state));
		ret = -ENODEV;
		goto out;
	}
	die_index = vatools_get_vastai_pci_die_index((char)kdata.device.dev_id,
						     kdata.device.die_id);
	did.val = (u32)die_index;
	die_id = did.die_id;

#ifdef CONFIG_TOOLS_V2
	// 这里需要增加 v2 ioctl的 mutex 控制. 兼容 v1 ioctl, 所有使用同一资源的smi, 读需要 mutex
	domain_mutex_lock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DIE);
#endif

	write_buf = kzalloc(kdata.input_buf.buf_size, GFP_KERNEL);
	if (write_buf == NULL) {
		ret = -ENOMEM;
		VATOOLS_ERR(priv, die_index, "kzalloc write_buf ret=%ld", ret);
		goto out_free_write_buf;
	}

	ret = copy_from_user(write_buf, (void __user *)kdata.input_buf.buf_addr,
			     kdata.input_buf.buf_size);
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out_free_write_buf;
	}

	VATOOLS_DUMP_BRIEF("vatools_driver_smi_fetch write_buf: ", write_buf,
			   kdata.input_buf.buf_size);

#ifdef CONFIG_TOOLS_V2
	//TODO:  这里检查 SMI 命令, 如果是 SMI_CMD_CODE_SPI_WRITE, 则清除 dev_info 缓存
	smi_cmd_p = (TS_SMI_CMD_REQ *)write_buf;
	if (smi_cmd_p != NULL && smi_cmd_p->cmd == SMI_CMD_CODE_SPI_WRITE) {
		struct vatools_node *node_p = vatools_get_node();
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "clear dev info cache");
		kvfree(node_p->dev_info);
		node_p->dev_info = NULL;
		node_p->device_num = 0;
	}
#endif

	smi_block = vatools_fw_smi_get_block(kdata.block_id);

	smi_die_address_offset = smi_block->block_address;

	VATOOLS_DBG(priv, die_index, "smi_block->block_address=0x%x",
		    smi_block->block_address);

	vatools_driver_smi_set_flag((u8 *)write_buf, kdata.input_buf.buf_size,
				    FLAG_APP_PREPARE); // clear flag to 0

	ret = vatools_pci_mem_write(priv, die_index, smi_die_address_offset,
				    (u8 *)write_buf, kdata.input_buf.buf_size);
	if (ret < 0) {
		VATOOLS_ERR(priv, die_index,
			    "vatools_pci_mem_write flag=0 ret=%ld", ret);
		goto out_free_write_buf;
	}

	// set flag to 1, write only 4 bytes, sync with fw
	vatools_driver_smi_set_flag((u8 *)write_buf, kdata.input_buf.buf_size,
				    FLAG_APP_REQ);
	ret = vatools_pci_mem_write(priv, die_index, smi_die_address_offset + 4,
				    ((u8 *)write_buf) + 4, sizeof(u32));

	if (ret < 0) {
		VATOOLS_ERR(priv, die_index,
			    "vatools_pci_mem_write flag=1 ret=%ld", ret);
		goto out_free_write_buf;
	}

	read_buf = kzalloc(kdata.output_buf.buf_size, GFP_KERNEL);
	if (read_buf == NULL) {
		ret = -ENOMEM;
		VATOOLS_ERR(priv, die_index, "kzalloc read_buf=NULL ret=%ld",
			    ret);
		goto out_free_write_buf;
	}

	// TODO: 如果fw未响应, 则重复发送3次请求.
re_trigger_fw:

	/* 如果是用户中断, 需要在这里清除一下上次smi命令的ack应答,
	否则每次进入到这里, wait_for都会直接返回, 导致wait和ack不匹配*/
	if (!priv || (priv->die_num <= die_id)) {
		VATOOLS_DBG(priv, die_index, "priv->die_num=%d < die_id=%d",
			    priv->die_num, die_id);
		ret = -EINVAL;
		goto out_free_write_buf;
	}
	die = &(priv->dies[die_id]);
	reinit_completion(&(die->smi_ack[kdata.block_id]));

	ret = vatools_fw_send_smi_interrupt(priv, die_index);
	ret = vatools_wait_smi_ack(priv, die_index, kdata.block_id, 3);
	if (ret < 0) {
		if (ret == -EINTR) {
			VATOOLS_DBG(
				priv, die_index,
				"smi_ack: die_index=%x block_id=%d timeout ret=%ld(-EINTR) re_trigger_fw_count=%d",
				die_index, kdata.block_id, ret,
				re_trigger_fw_count);
			goto out_free_read_buf;
		} else if (ret == -ETIMEDOUT) {
			VATOOLS_ERR(
				priv, die_index,
				"smi_ack: die_index=%x block_id=%d timeout ret=%ld(-ETIMEDOUT) re_trigger_fw_count=%d",
				die_index, kdata.block_id, ret,
				re_trigger_fw_count);
		} else {
			VATOOLS_ERR(
				priv, die_index,
				"smi_ack: die_index=%x block_id=%d timeout ret=%ld re_trigger_fw_count=%d",
				die_index, kdata.block_id, ret,
				re_trigger_fw_count);
			goto out_free_read_buf;
		}
	}

	memset(read_buf, 0x00, kdata.output_buf.buf_size);
	ret = vatools_pci_mem_read(priv, die_index, smi_die_address_offset,
				   read_buf, kdata.output_buf.buf_size);
	if (ret < 0) {
		VATOOLS_ERR(priv, die_index, "ret=%ld", ret);
		goto out_free_read_buf;
	}

	VATOOLS_DUMP_BRIEF("vatools_driver_smi_fetch read_buf: ", read_buf,
			   kdata.output_buf.buf_size);

	memcpy(&smi_cmd, read_buf, sizeof(TS_SMI_CMD_REQ));

	VATOOLS_DBG(priv, die_index, "smi_cmd.flag=%d", smi_cmd.flag);
	// fw 有应答, 无论执行结果如何, 都不再重发
	if (smi_cmd.flag == FLAG_FW_DATA_READY ||
	    smi_cmd.flag == FALG_FW_ERROR_UNKNOWN_CMD ||
	    smi_cmd.flag == FALG_FW_ERROR_ABORT) {
		ready = 1;
		//数据准备好
		VATOOLS_DBG(priv, die_index,
			    "READY: kdata.output_buf.buf_addr= 0x%p  "
			    "kdata.output_buf.buf_size= %d",
			    kdata.output_buf.buf_addr,
			    kdata.output_buf.buf_size);

		ret = copy_to_user(kdata.output_buf.buf_addr, read_buf,
				   kdata.output_buf.buf_size);
		if (ret) {
			VATOOLS_ERR(priv, die_index, "copy_to_user ret=%ld",
				    ret);
			ret = -EFAULT;
			goto out_free_read_buf;
		}
		ret = smi_cmd.flag | 0x1000;
		goto out_free_read_buf;
	} else if (smi_cmd.flag == FLAG_APP_REQ) {
		// fw 没有应答, flag是请求状态, smcu未做任何处理, 重发请求 ,循环3次后结束
		VATOOLS_ERR(
			priv, die_index,
			"SMCU not responding: smi_cmd.flag == FLAG_APP_REQ(0x01): re_trigger_fw_count=%d",
			re_trigger_fw_count);
		if (re_trigger_fw_count > 0) {
			re_trigger_fw_count--;
			goto re_trigger_fw;
		}
	}

	//数据错误
	VATOOLS_DBG(priv, die_index,
		    "ABORT: kdata.output_buf.buf_addr= 0x%p  "
		    "kdata.output_buf.buf_size= %d",
		    kdata.output_buf.buf_addr, kdata.output_buf.buf_size);

	ret = copy_to_user(kdata.output_buf.buf_addr, read_buf,
			   kdata.output_buf.buf_size);
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_to_user ret=%ld", ret);
		ret = -EFAULT;
		goto out_free_read_buf;
	}
	ret = -ETIME;
	goto out_free_read_buf;

out_free_read_buf:
	kvfree(read_buf);

out_free_write_buf:
	kvfree(write_buf);

#ifdef CONFIG_TOOLS_V2
	domain_mutex_unlock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DIE);
#endif
out:
	kdata.errcode = ret;
	ret2 = copy_to_user(argp, &kdata, sizeof(kdata));
	if (ret2) {
		VATOOLS_ERR(priv, die_index, "copy_to_user ret=%ld", ret);
		ret2 = -EFAULT;
	}
	VATOOLS_DBG(priv, die_index, "out: ret=%ld   ret2=%d", ret, ret2);
	VATOOLS_FUNC_EXIT;

	return ret2;
}

struct list_head *vatools_get_vastai_head(void)
{
	return &g_nodes_list;
}

// dma分配内存
int vatools_ioctl_dma_alloc(struct file *filp, unsigned int cmd,
			    unsigned long arg)
{
	int ret = 0;
	struct vatools_cmd vcmd;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	int die_index = 0;
	struct dma_buf *dmabuf = NULL;
	union die_index_data did;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&vcmd, 0, sizeof(vcmd));
	ret = copy_from_user(&vcmd, (void *)argp, sizeof(vcmd));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user is error\n");
		return -EIO;
	}

	did.val = (u32)vcmd.cmd.alloc_cmd.die_index;
	die_index = vcmd.cmd.alloc_cmd.die_index;
	priv = vatools_get_vastai_pci_device_info(did.dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ",
			    did.dev_id);
		return -ENODEV;
	}

	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"dev_id=%d, die_id=%d, die_index=0x%x, dma_buf_fd=%d, size=%d",
		did.dev_id, did.die_id, die_index,
		vcmd.cmd.alloc_cmd.dma_buf_fd, vcmd.cmd.alloc_cmd.size);

	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv->pci_state= 0x%x\n",
			    atomic_read(&priv->pci_state));
		return -ENODEV;
	}

	if (iommu_is_enable(priv))
		dmabuf = vastai_alloc_dma_buf_sg(priv, vcmd.cmd.alloc_cmd.size);
	else
		dmabuf = vastai_alloc_dma_buf(priv, vcmd.cmd.alloc_cmd.size);
	if (dmabuf) {
		int fd = dma_buf_fd(dmabuf, O_CLOEXEC);
		vcmd.cmd.alloc_cmd.dma_buf_fd = fd;
	} else {
		vcmd.cmd.alloc_cmd.dma_buf_fd = -ENOMEM;
	}

	ret = copy_to_user((void *)arg, &vcmd, sizeof(vcmd));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user is error\n");
		return -EIO;
	}
	return ret;
}

// 开始 dma 传输
int vatools_ioctl_dma_start(struct file *filp, unsigned int cmd,
			    unsigned long arg)
{
	int ret = 0;
	struct vatools_cmd vcmd;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	int die_index = 0;
	union die_index_data did;
	struct dma_buf *dmabuf = NULL;
	struct vastai_dmadesc desc;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&vcmd, 0, sizeof(vcmd));
	ret = copy_from_user(&vcmd, (void *)argp, sizeof(vcmd));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user is error\n");
		return -EIO;
	}

	did.val = (u32)vcmd.cmd.dma_start_cmd.die_index;
	die_index = vcmd.cmd.dma_start_cmd.die_index;
	priv = vatools_get_vastai_pci_device_info(did.dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ",
			    did.dev_id);
		return -ENODEV;
	}

	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"dev_id=%d, die_id=%d, die_index=0x%x, dma_buf_fd=%d, size=%d, axi_addr=0x%llx, is_dev_to_host=%d",
		did.dev_id, did.die_id, die_index,
		vcmd.cmd.dma_start_cmd.dma_buf_fd, vcmd.cmd.dma_start_cmd.size,
		vcmd.cmd.dma_start_cmd.axi_addr,
		vcmd.cmd.dma_start_cmd.is_dev_to_host);

	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv->pci_state= 0x%x\n",
			    atomic_read(&priv->pci_state));
		return -ENODEV;
	}

	dmabuf = dma_buf_get(vcmd.cmd.dma_start_cmd.dma_buf_fd);
	if (IS_ERR(dmabuf)) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
		       "%s dma buf fd is invalid\n", __func__);
		return PTR_ERR(dmabuf);
	}
	if (iommu_is_enable(priv)) {
		union core_bitmap core_id = { .val = 0 };
		struct vastai_dma_buf *dm = get_vastai_dma_buf(dmabuf, 0);
		u32 is_src_dma_addr = 1;
		u32 is_src_not_user_mem = 1;
		if (vcmd.cmd.dma_start_cmd.size > dm->size) {
			ret = -EINVAL;
			goto free_buf;
		}
		ret = vastai_pci_dma_transfer_sg(
			priv, vcmd.cmd.dma_start_cmd.die_index, core_id, dm,
			is_src_dma_addr, !vcmd.cmd.dma_start_cmd.is_dev_to_host,
			is_src_not_user_mem, vcmd.cmd.dma_start_cmd.axi_addr,
			vcmd.cmd.dma_start_cmd.size);
	} else {
		ret = vastai_common_cut_dma_buf(
			vcmd.cmd.dma_start_cmd.size,
			(struct kchar_cmd *)&vcmd.cmd, &desc, priv, dmabuf,
			vastai_ioctl_dma_start_set_desc);
	}
free_buf:
	dma_buf_put(dmabuf);

	ret = copy_to_user((void *)arg, &vcmd, sizeof(vcmd));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user is error\n");
		return -EIO;
	}
	return ret;
}
// 用户空间内存直接传递到内核, 使用前无需 dma alloc 内存
int vatools_ioctl_dma_trans(struct file *filp, unsigned int cmd,
			    unsigned long arg)
{
	int ret = 0;
	struct vatools_cmd vcmd;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL, *pci_info = NULL;
	int die_index = 0;
	union die_index_data did;
	union core_bitmap core_id = { .val = 0 };
	u32 size = 0;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&vcmd, 0, sizeof(vcmd));
	ret = copy_from_user(&vcmd, (void *)argp, sizeof(vcmd));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user is error\n");
		return -EIO;
	}

	did.val = (u32)vcmd.cmd.alloc_cmd.die_index;
	die_index = vcmd.cmd.alloc_cmd.die_index;
	priv = vatools_get_vastai_pci_device_info(did.dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ",
			    did.dev_id);
		return -ENODEV;
	}

	pci_info = priv;
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"dev_id=%d, die_id=%d, die_index=0x%x, dma_buf_fd=%d, size=%d",
		did.dev_id, did.die_id, die_index,
		vcmd.cmd.alloc_cmd.dma_buf_fd, vcmd.cmd.alloc_cmd.size);

	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv->pci_state= 0x%x\n",
			    atomic_read(&priv->pci_state));
		return -ENODEV;
	}

	size = vcmd.cmd.dma_trans_cmd.length;

	if (iommu_is_enable(pci_info) &&
	    ((u64)vcmd.cmd.dma_trans_cmd.vir_addr % PAGE_SIZE == 0)) {
		ret = vastai_pci_dma_transfer_by_uaddr(
			pci_info, vcmd.cmd.dma_trans_cmd.die_index, core_id,
			!vcmd.cmd.dma_trans_cmd.is_dev_to_host,
			vcmd.cmd.dma_trans_cmd.axi_addr,
			(u64)vcmd.cmd.dma_trans_cmd.vir_addr, size);
	} else {
		struct vastai_dmadesc *desc;
		u32 entry_count = DIV_ROUND_UP(size, VASTAI_MAX_DMA_BUF);

		desc = kvmalloc(sizeof(struct vastai_dmadesc) * entry_count,
				GFP_KERNEL);

		vastai_common_cut_dma_buf(size, (struct kchar_cmd *)&vcmd.cmd,
					  desc, pci_info, NULL,
					  vastai_ioctl_dma_trans_set_desc);

		ret = vastai_pci_dma_transfer_sync_pid(
			pci_info, vcmd.cmd.dma_trans_cmd.die_index, core_id,
			desc, entry_count, vcmd.cmd.dma_trans_cmd.pid);

		kvfree(desc);
	}

	ret = copy_to_user((void *)arg, &vcmd, sizeof(vcmd));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user is error\n");
		return -EIO;
	}
	return 0;
}
// 读内存数据
int vatools_ioctl_read_reg_buf(struct file *filp, unsigned int cmd,
			       unsigned long arg)
{
	int ret = 0;
	struct vatools_cmd vcmd;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	int die_index = 0;
	u8 *temp_buf = NULL;
	union die_index_data did;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&vcmd, 0, sizeof(vcmd));
	ret = copy_from_user(&vcmd, (void *)argp, sizeof(vcmd));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user is error\n");
		return -EIO;
	}

	did.val = (u32)vcmd.cmd.access_mem_cmd.die_index;
	die_index = vcmd.cmd.access_mem_cmd.die_index;
	priv = vatools_get_vastai_pci_device_info(did.dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ",
			    did.dev_id);
		return -ENODEV;
	}

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "die_index=0x%x, addr=0x%llx, buf=0x%p, size=%d",
		    vcmd.cmd.access_mem_cmd.die_index,
		    vcmd.cmd.access_mem_cmd.addr, vcmd.cmd.access_mem_cmd.buf,
		    vcmd.cmd.access_mem_cmd.size);

	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv->pci_state= 0x%x\n",
			    atomic_read(&priv->pci_state));
		return -ENODEV;
	}

	temp_buf = (u8 *)kvmalloc(vcmd.cmd.access_mem_cmd.size, GFP_KERNEL);
	if (!temp_buf) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, " kvmalloc error. size= %d",
			    vcmd.cmd.access_mem_cmd.size);
		ret = -ENOMEM;
		goto out;
	}

	memset(temp_buf, 0, vcmd.cmd.access_mem_cmd.size);
	ret = vastai_pci_mem_read(priv, die_index, vcmd.cmd.access_mem_cmd.addr,
				  temp_buf, vcmd.cmd.access_mem_cmd.size);
	if (ret < 0) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "vastai_pci_mem_read is error, ret=%d\n", ret);
		goto out_kvfree;
	}

	ret = copy_to_user((void *)vcmd.cmd.access_mem_cmd.buf, temp_buf,
			   vcmd.cmd.access_mem_cmd.size);
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user is error\n");
		ret = -EIO;
		goto out_kvfree;
	}

out_kvfree:
	kvfree(temp_buf);

out:
	return ret;
}

//写内存数据
int vatools_ioctl_write_reg_buf(struct file *filp, unsigned int cmd,
				unsigned long arg)
{
	int ret = 0;
	struct vatools_cmd vcmd;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	int die_index = 0;
	u8 *temp_buf = NULL;
	union die_index_data did;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&vcmd, 0, sizeof(vcmd));
	ret = copy_from_user(&vcmd, (void *)argp, sizeof(vcmd));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user is error\n");
		return -EIO;
	}

	did.val = (u32)vcmd.cmd.access_mem_cmd.die_index;
	die_index = vcmd.cmd.access_mem_cmd.die_index;
	priv = vatools_get_vastai_pci_device_info(did.dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ",
			    did.dev_id);
		return -ENODEV;
	}

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "die_index=0x%x, addr=0x%llx, buf=0x%p, size=%d",
		    vcmd.cmd.access_mem_cmd.die_index,
		    vcmd.cmd.access_mem_cmd.addr, vcmd.cmd.access_mem_cmd.buf,
		    vcmd.cmd.access_mem_cmd.size);

	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv->pci_state= 0x%x\n",
			    atomic_read(&priv->pci_state));
		return -ENODEV;
	}

	temp_buf = (u8 *)kvmalloc(vcmd.cmd.access_mem_cmd.size, GFP_KERNEL);
	if (!temp_buf) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, " kvmalloc error. size= %d",
			    vcmd.cmd.access_mem_cmd.size);
		ret = -ENOMEM;
		goto out;
	}

	memset(temp_buf, 0, vcmd.cmd.access_mem_cmd.size);
	ret = copy_from_user(temp_buf, (void *)vcmd.cmd.access_mem_cmd.buf,
			     vcmd.cmd.access_mem_cmd.size);
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user is error\n");
		ret = -EIO;
		goto out_kvfree;
	}

	ret = vastai_pci_mem_write(priv, die_index,
				   vcmd.cmd.access_mem_cmd.addr, temp_buf,
				   vcmd.cmd.access_mem_cmd.size);
	if (ret < 0) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "vastai_pci_mem_read is error, ret=%d\n", ret);
		goto out_kvfree;
	}

out_kvfree:
	kvfree(temp_buf);

out:
	return ret;
}

/// core time sync
#ifdef ENABLE_VATOOLS_CORE_TIME_SYNC
static int vatools_wait_core_time_sync_ack(struct vastai_pci_info *priv,
					   u32 die_index)
{
	struct vastai_sv100_die *die;
	int ret = 0;
	u32 die_id;
	union die_index_data did;
	u32 timeout_ms = 100;

	did.val = (u32)die_index;
	die_id = did.die_id;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "priv=%p, die_index=%d die_id=%d wait start", priv,
		    die_index, die_id);

	if (!priv || (priv->die_num <= die_id)) {
		return -EINVAL;
	}

	die = &(priv->dies[die_id]);

	/* wait completion int from smcu */
	/* 如果是用户中断, 需要在这里清除一下上次smi命令的ack应答,
	否则每次进入到这里, wait_for都会直接返回, 导致wait和ack不匹配*/
	reinit_completion(&(die->core_time_sync_ack));
	ret = wait_for_completion_interruptible_timeout(
		&(die->core_time_sync_ack), msecs_to_jiffies(timeout_ms));
	reinit_completion(&(die->core_time_sync_ack));
	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "wait completion ret=%d die_id=%d die_index=0x%x ", ret,
		    die_id, die_index);
	if (ret == 0) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "timeout! (%d ms) ret=%d\n",
			    timeout_ms, ret);
		return -ETIMEDOUT;
	} else if (ret == -ERESTARTSYS || ret == -EINTR) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "process be killed! ret=%d\n",
			    ret);
		return -EINTR;
	} else if (ret < 0) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "Unknow err! ret=%d\n", ret);
		return -EIO;
	} else {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "done.");
		return 0;
	}
}
#endif

int vatools_ioctl_core_time_sync(struct file *filp, unsigned int cmd,
				 unsigned long arg)
{
	int ret = 0;
	struct vatools_node *node = NULL;
	struct vatools_reader *reader = NULL;
	void __user *argp = (void __user *)arg;
	T_SMI_IOCTL_TRANS_DATA kdata = { 0 };
	struct vastai_pci_info *priv = NULL;
	u32 die_index = 0;
#ifdef ENABLE_VATOOLS_CORE_TIME_SYNC
	ring_buf_elem_t ring_buf_elem;
	u32 corebitmap;
	core_time_sync_data_t core_time_sync_data;
#endif
	VATOOLS_FUNC_ENTERY;

	node = vatools_file_get_node(filp);
	reader = vatools_file_get_reader(filp);
	priv = vatools_get_vastai_pci_device_info((char)kdata.device.dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ",
			    kdata.device.dev_id);
		return -ENODEV;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv->pci_state= 0x%x\n",
			    atomic_read(&priv->pci_state));
		return -ENODEV;
	}
	die_index = vatools_get_vastai_pci_die_index((char)kdata.device.dev_id,
						     kdata.device.die_id);

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "cmd=0x%x  filp=0x%p  node=0x%p  reader=0x%p", cmd, filp,
		    node, reader);

	if (!node) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "node is NULL");
		ret = -EINVAL;
	}

	memset(&kdata, 0x00, sizeof(T_SMI_IOCTL_TRANS_DATA));
	ret = copy_from_user(&kdata, argp, sizeof(kdata));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%d", ret);
		ret = -EFAULT;
		goto out;
	}

	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"block_id=0x%x  device_id=0x%x  die_id=0x%x  die_id=0x%x  flag= 0x%x  addr= "
		"0x%llx ",
		kdata.block_id, kdata.device.dev_id, kdata.device.die_id,
		kdata.device.die_index, kdata.flag, kdata.address);

	switch (kdata.flag) {
#ifdef ENABLE_VATOOLS_CORE_TIME_SYNC
	case CORE_TIME_SYNC_MSG_STOP: {
		//发送消息到 core
		memset(&ring_buf_elem, 0, sizeof(ring_buf_elem));

		ring_buf_elem.whole = 0;
		ring_buf_elem.cmd = VATOOLS_INTERRUPT_TOOLS_CATEGARY_CMD;
		ring_buf_elem.use_ringbuf = VATOOLS_INTERRUPT_NOT_USE_RINGBUF;
		ring_buf_elem.seq_no = 0;
		ring_buf_elem.sub_cmd = VATOOLS_INTERRUPT_CORE_TIME_SYNC;
		ring_buf_elem.val = CORE_TIME_SYNC_MSG_STOP;
		corebitmap = (0x1 << (CORE_SMCU)) & ((0x1 << CORE_NUMBER) - 1);

		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"START: die_index=%d, corebitmap=0x%x kdata.flag=0x%x ring_buf_elem.whole=0x%llx",
			kdata.device.die_index, corebitmap, kdata.flag,
			ring_buf_elem.whole);

		// 发送中断到 smcu
		ret = vatools_send_ctrl_cmd(priv, kdata.device.die_index,
					    corebitmap, ring_buf_elem.whole);
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "send ctrl cmd return 0x%x\n",
			    ret);
		if (ret < 0) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "send ctrl cmd failed. ret=%d\n", ret);
			goto out;
		}
	} break;
	case CORE_TIME_SYNC_MSG_START: {
		//发送消息到 core
		memset(&ring_buf_elem, 0, sizeof(ring_buf_elem));

		ring_buf_elem.whole = 0;
		ring_buf_elem.cmd = VATOOLS_INTERRUPT_TOOLS_CATEGARY_CMD;
		ring_buf_elem.use_ringbuf = VATOOLS_INTERRUPT_NOT_USE_RINGBUF;
		ring_buf_elem.seq_no = 0;
		ring_buf_elem.sub_cmd = VATOOLS_INTERRUPT_CORE_TIME_SYNC;
		ring_buf_elem.val = CORE_TIME_SYNC_MSG_START;
		corebitmap = (0x1 << (CORE_SMCU)) & ((0x1 << CORE_NUMBER) - 1);

		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"START: die_index=%d, corebitmap=0x%x kdata.flag=0x%x ring_buf_elem.whole=0x%llx",
			kdata.device.die_index, corebitmap, kdata.flag,
			ring_buf_elem.whole);

		// 发送中断到 smcu
		ret = vatools_send_ctrl_cmd(priv, kdata.device.die_index,
					    corebitmap, ring_buf_elem.whole);
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "send ctrl cmd return 0x%x\n",
			    ret);
		if (ret < 0) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "send ctrl cmd failed. ret=%d\n", ret);
			goto out;
		}
	} break;
	case CORE_TIME_SYNC_MSG_READ: {
		u64 host_ns = 0;
		int c = 0;
		//start core time sync
		memset(&ring_buf_elem, 0, sizeof(ring_buf_elem));

		ring_buf_elem.whole = 0;
		ring_buf_elem.cmd = VATOOLS_INTERRUPT_TOOLS_CATEGARY_CMD;
		ring_buf_elem.use_ringbuf = VATOOLS_INTERRUPT_NOT_USE_RINGBUF;
		ring_buf_elem.seq_no = 0;
		ring_buf_elem.sub_cmd = VATOOLS_INTERRUPT_CORE_TIME_SYNC;
		ring_buf_elem.val = CORE_TIME_SYNC_MSG_START;
		corebitmap = (0x1 << (CORE_SMCU)) & ((0x1 << CORE_NUMBER) - 1);

		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"START: die_index=%d, corebitmap=0x%x kdata.flag=0x%x ring_buf_elem.whole=0x%llx",
			kdata.device.die_index, corebitmap, kdata.flag,
			ring_buf_elem.whole);

		// 发送中断到 smcu
		ret = vatools_send_ctrl_cmd(priv, kdata.device.die_index,
					    corebitmap, ring_buf_elem.whole);
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "send ctrl cmd return 0x%x\n",
			    ret);
		if (ret < 0) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "send ctrl cmd failed. ret=%d\n", ret);
			goto out;
		}

		// read time sync data
		memset(&core_time_sync_data, 0, sizeof(core_time_sync_data_t));
		host_ns = vastai_get_host_time_ns();
		for (c = 0; c < CORE_NUMBER; c++) {
			core_time_sync_data.host_time_ns[c] = host_ns;
		}

		ret = vatools_wait_core_time_sync_ack(priv,
						      kdata.device.die_index);
		if (ret < 0) {
			if (ret == -EINTR) {
				VATOOLS_DBG(
					NULL, DUMMY_DIE_ID,
					"vatools_wait_core_time_sync_ack timeout ret=%d die_index=%x",
					ret, kdata.device.die_index);
			} else {
				VATOOLS_ERR(
					NULL, DUMMY_DIE_ID,
					"vatools_wait_core_time_sync_ack timeout ret=%d die_index=%x",
					ret, kdata.device.die_index);
			}
			goto out_stop;
		}

		//udelay(100); //TODO: change to wait ack interrupt later

		ret = vatools_pci_mem_read(
			priv, die_index, INTER_CORE_TIME_SYNC_ADDR,
			(u8 *)core_time_sync_data.core_time,
			sizeof(core_time_sync_data.core_time));
		if (ret < 0) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ret=%d", ret);
			goto out_stop;
		}
		VATOOLS_DUMP_BRIEF(
			"core_time_sync_data read: ", &core_time_sync_data,
			sizeof(core_time_sync_data));

		//copy to user
		ret = copy_to_user(kdata.output_buf.buf_addr,
				   &core_time_sync_data,
				   sizeof(core_time_sync_data));
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%d",
				    ret);
			ret = -EFAULT;
			goto out_stop;
		}

	out_stop:
		// stop core time sync
		memset(&ring_buf_elem, 0, sizeof(ring_buf_elem));

		ring_buf_elem.whole = 0;
		ring_buf_elem.cmd = VATOOLS_INTERRUPT_TOOLS_CATEGARY_CMD;
		ring_buf_elem.use_ringbuf = VATOOLS_INTERRUPT_NOT_USE_RINGBUF;
		ring_buf_elem.seq_no = 0;
		ring_buf_elem.sub_cmd = VATOOLS_INTERRUPT_CORE_TIME_SYNC;
		ring_buf_elem.val = CORE_TIME_SYNC_MSG_STOP;
		corebitmap = (0x1 << (CORE_SMCU)) & ((0x1 << CORE_NUMBER) - 1);

		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"START: die_index=%d, corebitmap=0x%x kdata.flag=0x%x ring_buf_elem.whole=0x%llx",
			kdata.device.die_index, corebitmap, kdata.flag,
			ring_buf_elem.whole);

		// 发送中断到 smcu
		ret = vatools_send_ctrl_cmd(priv, kdata.device.die_index,
					    corebitmap, ring_buf_elem.whole);
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "send ctrl cmd return 0x%x\n",
			    ret);
		if (ret < 0) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "send ctrl cmd failed. ret=%d\n", ret);
			goto out;
		}

	} break;
#endif
	default:
		ret = -EINVAL;
	}

out:
	return ret;
}

void vatools_get_current_time(struct rtc_time *tm)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0)
	struct timex txc;
	/*GET UTC time*/
	do_gettimeofday(&(txc.time));
	/*convert UTC to localtime*/
	rtc_time_to_tm(txc.time.tv_sec, tm);
#else
	time64_t sec = 0;
	sec = ktime_to_ms(ktime_get());
	rtc_time64_to_tm(sec, tm);
#endif
}

extern int vastai_send_ctrl_cmd(struct vastai_pci_info *pci_info, u32 die_index,
				u32 core_bit_map, u64 info);
/**
 * @brief send a ctrl cmd to smcu by msgqueue1, smcu will retransmit interrupt to
 * 	other mcu / vdsp.
 *
 * @param pci_info: vastai sv100 pci base struct.
 * @param die_index: the destination die.
 * @param core_bit_map: this is a __BIT_MAP__ smcu will trigger it one by one.
 * @param info: private data. low 8bit is invalied. (for seq_id)
 * @return int: zero is success.
 */
int vatools_send_ctrl_cmd(struct vastai_pci_info *pci_info, u32 die_index,
			  u32 core_bit_map, u64 info)
{
	int ret = 0;
	ret = vastai_send_ctrl_cmd(pci_info, die_index, core_bit_map, info);
	return ret;
}

// get sriov enable and number
int vatools_ioctl_get_sriov_number(struct file *filp, unsigned int cmd,
				   unsigned long arg)
{
	int ret = 0;
	struct sriov_number_s sriov_numb = { 0 };
	ret = copy_from_user(&sriov_numb, (void __user *)arg,
			     sizeof(struct sriov_number_s));
	if (ret) {
		return -EFAULT;
	}
	sriov_numb.sriov_num = vastai_pci_get_sriov_numvfs(sriov_numb.dev_id);
	ret = copy_to_user((void __user *)arg, &sriov_numb,
			   sizeof(struct sriov_number_s));
	if (ret)
		return -EFAULT;
	return 0;
}

// get bmcu and xspi version
int vatools_ioctl_get_bmcu_xspi_ver(struct file *filp, unsigned int cmd,
				    unsigned long arg)
{
	int ret = 0;
	s32 buf_size = 0;
	struct bmcu_xspi_ver *ver = NULL;
	struct vastai_pci_info *pci_info = NULL;
	int i;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	if (get_user(buf_size, (int __user *)arg)) {
		return -EFAULT;
	}

	ver = (struct bmcu_xspi_ver *)kvmalloc(buf_size, GFP_KERNEL);
	ret = copy_from_user(ver, (void __user *)arg, buf_size);
	if (ret) {
		kvfree(ver);
		return -EFAULT;
	}

	pci_info = vatools_get_vastai_pci_device_info(ver->die_index.dev_id);
	if (!pci_info) {
		kvfree(ver);
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ",
			    ver->die_index.dev_id);
		return -ENODEV;
	}

	if ((atomic_read(&pci_info->pci_state) != VASTAI_NORMAL_STATE)) {
		kvfree(ver);
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv->pci_state= 0x%x\n",
			    atomic_read(&pci_info->pci_state));
		return -ENODEV;
	}
	memcpy(ver->bmcu_active, pci_info->dies[0].fw_ver.bmcu_ver,
	       VASTAI_VER_BMCU_XSPI_LEN);
	memcpy(ver->bmcu_backup, pci_info->dies[0].fw_ver.bmcu_backup_ver,
	       VASTAI_VER_BMCU_XSPI_LEN);
	ver->bmcu_active[VASTAI_VER_BMCU_XSPI_LEN - 1] = 0;
	ver->bmcu_backup[VASTAI_VER_BMCU_XSPI_LEN - 1] = 0;
	ver->die_num = pci_info->die_num;

	for (i = 0; i < ver->die_num; i++) {
		char *xspi_ver = (char *)pci_info->dies[i].fw_ver.bmcu_ver +
				 VASTAI_VER_BMCU_XSPI_LEN;
		memcpy(ver->dies[i].xspi_ver, xspi_ver,
		       VASTAI_VER_BMCU_XSPI_LEN);
	}

	ret = copy_to_user((void __user *)arg, ver, buf_size);
	kvfree(ver);

	if (ret)
		return -EFAULT;
	return 0;
}

// flash bmcu's version
int vatools_ioctl_flash_bmcu_fw(struct file *filp, unsigned int cmd,
				unsigned long arg)
{
	int ret = 0;
	struct flash_bmcu_xspi flash_arg;
	struct flash_bmcu_xspi *bin;
	struct vastai_pci_info *pci_info = NULL;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);

	ret = copy_from_user(&flash_arg, (void __user *)arg, sizeof(flash_arg));
	if (ret) {
		return -EFAULT;
	}

	if (flash_arg.buf_size <= 0) {
		return -EINVAL;
	}

	pci_info =
		vatools_get_vastai_pci_device_info(flash_arg.die_index.dev_id);
	if (!pci_info) {
		return -ENODEV;
	}

	/* vastai_pci_flash_bmcu_bin need smcu to response. Therefore, to check
		pcie_state is VASTAI_NORMAL_STATE. */
	if (atomic_read(&(pci_info->pci_state)) != VASTAI_NORMAL_STATE)
		return -EBUSY;

	bin = (struct flash_bmcu_xspi *)kvmalloc(
		flash_arg.buf_size + sizeof(flash_arg), GFP_KERNEL);
	if (!bin) {
		return -ENOMEM;
	}

	ret = copy_from_user(bin, (void __user *)arg,
			     flash_arg.buf_size + sizeof(flash_arg));
	if (ret) {
		kvfree(bin);
		return -EFAULT;
	}

	ret = vastai_pci_flash_bmcu_bin(pci_info, 0, NULL, bin->buf,
					flash_arg.buf_size);
	kvfree(bin);

	return ret;
}

// flash xspi's flash
int vatools_ioctl_flash_xspi_fw(struct file *filp, unsigned int cmd,
				unsigned long arg)
{
	int ret = 0;
	struct flash_bmcu_xspi flash_arg;
	struct flash_bmcu_xspi *bin;
	struct vastai_pci_info *pci_info = NULL;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);

	ret = copy_from_user(&flash_arg, (void __user *)arg, sizeof(flash_arg));
	if (ret) {
		return -EFAULT;
	}

	if (flash_arg.buf_size <= 0) {
		return -EINVAL;
	}

	pci_info =
		vatools_get_vastai_pci_device_info(flash_arg.die_index.dev_id);
	if (!pci_info) {
		return -ENODEV;
	}

	if (atomic_read(&(pci_info->pci_state)) != VASTAI_NORMAL_STATE)
		return -EBUSY;

	bin = (struct flash_bmcu_xspi *)kvmalloc(
		flash_arg.buf_size + sizeof(flash_arg), GFP_KERNEL);
	if (!bin) {
		return -ENOMEM;
	}

	ret = copy_from_user(bin, (void __user *)arg,
			     flash_arg.buf_size + sizeof(flash_arg));
	if (ret) {
		kvfree(bin);
		return -EFAULT;
	}

	ret = vastai_pci_flash_xspi_bin(pci_info, flash_arg.die_index.die_id,
					NULL, bin->buf, flash_arg.buf_size, 0);

	if (ret) {
		kvfree(bin);
		return ret;
	}

	ret = vastai_pci_update_flag_xspi(pci_info, flash_arg.die_index.die_id,
					  0);
	kvfree(bin);

	return ret;
}

// get bmcu or flash 's state
int vatools_ioctl_flash_sta(struct file *filp, unsigned int cmd,
			    unsigned long arg)
{
	struct flash_state flash_state;
	int ret = 0;
	struct vastai_pci_info *pci_info = NULL;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);

	ret = copy_from_user(&flash_state, (void __user *)arg,
			     sizeof(struct flash_state));
	if (ret) {
		return -EFAULT;
	}

	pci_info = vatools_get_vastai_pci_device_info(
		flash_state.die_index.dev_id);
	if (!pci_info) {
		return -ENODEV;
	}
	if (atomic_read(&(pci_info->pci_state)) != VASTAI_NORMAL_STATE) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "pci_info->pci_state= %d is not normal.",
			    atomic_read(&pci_info->pci_state));
		return -ENODEV;
	}

	flash_state.flash_state = 0;

	switch (flash_state.flash_type) {
	case 0:
		flash_state.flash_state = atomic_read(&(pci_info->pci_state)) ==
					  VASTAI_NORMAL_STATE;
		break;
	case 1:
		/* flash always is done */
		flash_state.flash_state = 1;
		break;
	default:
		break;
	}

	ret = copy_to_user((void __user *)arg, &flash_state,
			   sizeof(struct flash_state));
	if (ret) {
		return -EFAULT;
	}

	return ret;
}

// switch bmcu version
int vatools_ioctl_bmcu_switch(struct file *filp, unsigned int cmd,
			      unsigned long arg)
{
	struct vastai_pci_info *pci_info = NULL;
	union die_index_data die_index = { .val = arg };
	int ret;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);

	pci_info = vatools_get_vastai_pci_device_info(die_index.dev_id);
	if (!pci_info) {
		return -ENODEV;
	}
	if ((atomic_read(&pci_info->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv->pci_state= 0x%x\n",
			    atomic_read(&pci_info->pci_state));
		return -ENODEV;
	}

	ret = vastai_send_pcie_cmd(pci_info, die_index.val,
				   VASTAI_PCIE_SUB_BMCU_DEGRADE, 0);

	return ret;
}

// 遍历 fifo, 不修改rd wr指针. index范围: [0,fifo count). 返回0 为遍历结束, 需要调用者判断并退出遍历循环. 否则会有非法数据
u32 vastai_fifo_iterate(struct vastai_fifo *fifo, u32 index)
{
	u32 ret = 0;

	if ((fifo->wr) == ((fifo->rd) % (fifo->elem_count))) {
		return 0;
	}

	if ((fifo->wr) == ((fifo->rd + index) % (fifo->elem_count))) {
		return 0;
	}

	ret = (u32)offsetof(struct vastai_fifo, buf) +
	      fifo->elem_size * ((fifo->rd + index) % (fifo->elem_count));
	return ret;
}

// event 事件发生时, 采集当前系统时间; wait进程需要比较这个event发生时间和进程当前时间, 如果event时间大于当前时间, 则条件函数返回true, 唤醒进程, 标识有新的event产生
static volatile u64 _g_event_callback_timestamp_ns = 0;

int event_wait_queue_condition_func(u64 local_timestamp_ns)
{
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"_g_event_callback_timestamp_ns=%lld  local_timestamp_ns=%lld\n",
		_g_event_callback_timestamp_ns, local_timestamp_ns);
	return (_g_event_callback_timestamp_ns > local_timestamp_ns);
}

// 事件产生者调用此函数, 保存事件信息到 ringbuf
int vatools_device_event_callback(u32 die_index, u32 core_bit, u32 event)
{
	int ret = 0;
	struct vatools_node *node = NULL;
	struct vastai_fifo *fifo = NULL;
	u8 *fifo_p = NULL;
	device_event_t device_event;
	int locked;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "die_index=0x%x core_bit=%d event=%d\n",
		    die_index, core_bit, event);

	node = vatools_get_node();
	if (!node) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "node is NULL");
		return -ENXIO;
	}

	fifo = (struct vastai_fifo *)node->event_ringbuf;
	if (fifo == NULL) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "event fifo is NULL");
		return -EIO;
	}

	if (!vastai_is_fifo_valid(fifo)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "event fifo is not valid");
		return -EIO;
	}

	if (vastai_fifo_is_full(fifo)) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "event fifo is full");
	}

	locked = mutex_trylock(&node->event_mutex);
	if (!locked) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "event trylock fail=%d",
			    locked);
		return -EIO;
	}

	device_event.core_index = core_bit;
	device_event.die_index = die_index;
	device_event.event = event;
	device_event.timestamp_ns = vastai_get_host_time_ns();

	fifo_p = (u8 *)fifo + vastai_fifo_wr_next(fifo);
	memcpy(fifo_p, &device_event, fifo->elem_size);
	mutex_unlock(&node->event_mutex);

	_g_event_callback_timestamp_ns = device_event.timestamp_ns;
	wake_up_interruptible_all(&node->event_wait_queue);

	return ret;
}

// 获取 event reset 状态
// 注意: CORE_POINT_CMCU 不代表CMCU，代表整个ai-sys, 包括整个ai-sys的所有core; SMCU 没有reset; 其他代表各自的core reset
int vatools_ioctl_device_event(struct file *filp, unsigned int cmd,
			       unsigned long arg)
{
	int ret = 0;
	u32 timeout_ms = 1000;
	struct vatools_node *node = NULL;
	struct vatools_reader *reader = NULL;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	device_event_t *device_event_p = NULL;
	struct vastai_fifo *fifo = NULL;
	int i = 0;
	u32 it = 0;
	u32 event_count = 0;
	u64 local_timestamp_ns = 0;
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	node = vatools_file_get_node(filp);
	reader = vatools_file_get_reader(filp);
	priv = vatools_get_vastai_pci_device_info(0);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ", 0);
		return -ENODEV;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv->pci_state= 0x%x\n",
			    atomic_read(&priv->pci_state));
		return -ENODEV;
	}

	fifo = (struct vastai_fifo *)node->event_ringbuf;
	if (fifo == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "event fifo is NULL");
		return -EIO;
	}
	mutex_lock(&node->event_mutex);

	// 先遍历查询一下已经保存的event, 比较fd的时间戳和event的时间戳. 将event时间大于fd时间的事件都返回
	for (i = 0; i < fifo->elem_count; i++) {
		if ((it = vastai_fifo_iterate(fifo, i))) {
			device_event_p = (device_event_t *)((u8 *)fifo + it);
			if (device_event_p) {
				if (device_event_p->timestamp_ns >
					    reader->event_timestamp_ns &&
				    (device_event_p->event == EVT_RESET_START ||
				     device_event_p->event == EVT_RESET_STOP)) {
					ret = copy_to_user(
						(u8 *)argp + sizeof(u32) +
							event_count *
								sizeof(device_event_t),
						device_event_p,
						sizeof(device_event_t));
					if (ret) {
						mutex_unlock(
							&node->event_mutex);
						VATOOLS_ERR(
							NULL, DUMMY_DIE_ID,
							"copy_from_user is error\n");
						ret = -EIO;
						goto out;
					}
					reader->event_timestamp_ns =
						device_event_p->timestamp_ns;
					event_count++;
				}
			}
		} else {
			break;
		}
	}
	mutex_unlock(&node->event_mutex);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "before wait: event_count=%d\n",
		    event_count);

	// 已经有event, 直接返回
	if (event_count) {
		ret = copy_to_user((void *)argp, &event_count, sizeof(u32));
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "copy_from_user is error\n");
			ret = -EIO;
			goto out;
		}
		ret = 0;
		goto out;
	}

	//没有event, 进入等待
	local_timestamp_ns = vastai_get_host_time_ns();
	ret = wait_event_interruptible_timeout(
		node->event_wait_queue,
		event_wait_queue_condition_func(local_timestamp_ns),
		HZ * timeout_ms / 1000);

	if (ret == 0) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "timeout! (%d ms) ret=%d\n",
			    timeout_ms, ret);
		ret = -ETIMEDOUT;
		goto out;
	} else if (ret == -ERESTARTSYS || ret == -EINTR) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "process be killed! ret=%d\n",
			    ret);
		ret = -EINTR;
		goto out;
	} else if (ret < 0) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "Unknow err! ret=%d\n", ret);
		ret = -EIO;
		goto out;
	}

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "wait event done. ret=%d\n", ret);

	// 比较fd的时间戳和event的时间戳. 将event时间大于fd时间的事件都返回
	mutex_lock(&node->event_mutex);
	for (i = 0; i < fifo->elem_count; i++) {
		if ((it = vastai_fifo_iterate(fifo, i))) {
			device_event_p = (device_event_t *)((u8 *)fifo + it);
			if (device_event_p) {
				if (device_event_p->timestamp_ns >
					    reader->event_timestamp_ns &&
				    (device_event_p->event == EVT_RESET_START ||
				     device_event_p->event == EVT_RESET_STOP)) {
					ret = copy_to_user(
						(u8 *)argp + sizeof(u32) +
							event_count *
								sizeof(device_event_t),
						device_event_p,
						sizeof(device_event_t));
					if (ret) {
						mutex_unlock(
							&node->event_mutex);
						VATOOLS_ERR(
							NULL, DUMMY_DIE_ID,
							"copy_from_user is error\n");
						ret = -EIO;
						goto out;
					}
					reader->event_timestamp_ns =
						device_event_p->timestamp_ns;
					event_count++;
				}
			}
		} else {
			break;
		}
	}
	mutex_unlock(&node->event_mutex);

	if (event_count) {
		ret = copy_to_user((void *)argp, &event_count, sizeof(u32));
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "copy_from_user is error\n");
			ret = -EIO;
			goto out;
		}
		ret = 0;
		goto out;
	}

out:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ret=%d\n", ret);
	VATOOLS_FUNC_EXIT;
	return ret;
}

int vatools_ioctl_device_event_select(struct file *filp, unsigned int cmd,
				      unsigned long arg)
{
	int ret = 0;
	u32 timeout_ms = 1000;
	struct vatools_node *node = NULL;
	struct vatools_reader *reader = NULL;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	device_event_t *device_event_p = NULL;
	struct vastai_fifo *fifo = NULL;
	u32 i = 0;
	u32 it = 0;
	u32 event_count = 0;
	u64 local_timestamp_ns = 0;

	VATOOLS_FUNC_ENTERY;

	node = vatools_file_get_node(filp);
	reader = vatools_file_get_reader(filp);
	priv = vatools_get_vastai_pci_device_info(0);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ", 0);
		return -ENODEV;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv->pci_state= 0x%x\n",
			    atomic_read(&priv->pci_state));
		return -ENODEV;
	}

	fifo = (struct vastai_fifo *)node->event_ringbuf;
	if (fifo == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "event fifo is NULL");
		return -EIO;
	}
	mutex_lock(&node->event_mutex);

	// 先遍历查询一下已经保存的event, 比较fd的时间戳和event的时间戳. 将event时间大于fd时间的事件都返回
	for (i = 0; i < fifo->elem_count; i++) {
		if ((it = vastai_fifo_iterate(fifo, i))) {
			device_event_p = (device_event_t *)((u8 *)fifo + it);
			if (device_event_p) {
				if (device_event_p->timestamp_ns >
				    reader->event_timestamp_ns) {
					ret = copy_to_user(
						(u8 *)argp + sizeof(u32) +
							event_count *
								sizeof(device_event_t),
						device_event_p,
						sizeof(device_event_t));
					if (ret) {
						mutex_unlock(
							&node->event_mutex);
						VATOOLS_ERR(
							NULL, DUMMY_DIE_ID,
							"copy_from_user is error\n");
						ret = -EIO;
						goto out;
					}
					reader->event_timestamp_ns =
						device_event_p->timestamp_ns;
					event_count++;
				}
			}
		} else {
			break;
		}
	}
	mutex_unlock(&node->event_mutex);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "before wait: event_count=%d\n",
		    event_count);

	// 已经有event, 直接返回
	if (event_count) {
		ret = copy_to_user((void *)argp, &event_count, sizeof(u32));
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "copy_from_user is error\n");
			ret = -EIO;
			goto out;
		}
		ret = 0;
		goto out;
	}

	//没有event, 进入等待
	local_timestamp_ns = vastai_get_host_time_ns();
	ret = wait_event_interruptible_timeout(
		node->event_wait_queue,
		event_wait_queue_condition_func(local_timestamp_ns),
		HZ * timeout_ms / 1000);

	if (ret == 0) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "timeout! (%d ms) ret=%d\n",
			    timeout_ms, ret);
		ret = -ETIMEDOUT;
		goto out;
	} else if (ret == -ERESTARTSYS || ret == -EINTR) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "process be killed! ret=%d\n",
			    ret);
		ret = -EINTR;
		goto out;
	} else if (ret < 0) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "Unknow err! ret=%d\n", ret);
		ret = -EIO;
		goto out;
	}

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "wait event done. ret=%d\n", ret);

	// 比较fd的时间戳和event的时间戳. 将event时间大于fd时间的事件都返回
	mutex_lock(&node->event_mutex);
	for (i = 0; i < fifo->elem_count; i++) {
		if ((it = vastai_fifo_iterate(fifo, i))) {
			device_event_p = (device_event_t *)((u8 *)fifo + it);
			if (device_event_p) {
				if (device_event_p->timestamp_ns >
				    reader->event_timestamp_ns) {
					ret = copy_to_user(
						(u8 *)argp + sizeof(u32) +
							event_count *
								sizeof(device_event_t),
						device_event_p,
						sizeof(device_event_t));
					if (ret) {
						mutex_unlock(
							&node->event_mutex);
						VATOOLS_ERR(
							NULL, DUMMY_DIE_ID,
							"copy_from_user is error\n");
						ret = -EIO;
						goto out;
					}
					reader->event_timestamp_ns =
						device_event_p->timestamp_ns;
					event_count++;
				}
			}
		} else {
			break;
		}
	}
	mutex_unlock(&node->event_mutex);

	if (event_count) {
		ret = copy_to_user((void *)argp, &event_count, sizeof(u32));
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "copy_from_user is error\n");
			ret = -EIO;
			goto out;
		}
		ret = 0;
		goto out;
	}

out:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ret=%d\n", ret);
	VATOOLS_FUNC_EXIT;
	return ret;
}

#ifdef CONFIG_TOOLS_V2

int vatools_get_spi_buf(u32 die_index, u32 unit, u64 address, u32 len,
			void *spi_buf)
{
	int ret = -EINVAL;
	T_SMI_BLOCK *p_smi_block = NULL;
	struct vastai_pci_info *priv = NULL;
	u8 cmd_buf[SMI_CMD_BLOCK0_LEN] = { 0 };
	TS_SMI_CMD_REQ *pcmd = (TS_SMI_CMD_REQ *)cmd_buf;

	priv = vatools_get_vastai_pci_device_info_by_die_index(die_index);

	p_smi_block =
		vatools_fw_smi_get_block_via_app_category(VATOOLS_APP_SMI);

	pcmd->cmd = SMI_CMD_CODE_SPI_READ;
	pcmd->data_len = SMI_CMD_BLOCK0_LEN - SMI_CMD_HEADER_LEN;
	pcmd->unit = unit;
	pcmd->flag = FLAG_APP_REQ;
	pcmd->rw_addr = address;

	ret = vatools_driver_smi_cmd(p_smi_block->block_id, priv, die_index,
				     cmd_buf, SMI_CMD_BLOCK0_LEN);
	if (ret > 0) {
		memcpy(spi_buf, cmd_buf + SMI_CMD_HEADER_LEN, len);
	}
	return ret;
}
#endif
