
#ifndef __VATOOLS_COMMON_H__
#define __VATOOLS_COMMON_H__

#pragma pack(1)

extern vatools_bar_address_t vatools_die_check[4][2];

#pragma pack()

#define VATOOLS_BAR_SMI_MAX_BYTES (SMI_CMD_BLOCK0_LEN) // 128 bytes

int vatools_get_vastai_die_info_via_device_id(char dev_id, int die_index[],
					      int *die_num);
int vatools_get_vastai_pci_die_index(char dev_id, int die_id);
T_SMI_BLOCK *vatools_fw_smi_get_block(unsigned int block_id);
T_SMI_BLOCK *
vatools_fw_smi_get_block_via_app_category(unsigned int app_category);
int vatools_get_vastai_pci_device_number(void);

struct vastai_pci_info *vatools_get_vastai_pci_device_info(char dev_id);

struct list_head *vatools_get_vastai_head(void);
/******************************************************************************
 * 封装 vastai_pci_mem_read
 * 封装 vastai_pci_mem_write
 *
 */

int vatools_pci_mem_read(struct vastai_pci_info *priv, int die_index,
			 u64 relative_addr, void *buf, unsigned int len);
int vatools_pci_mem_write(struct vastai_pci_info *priv, int die_index,
			  u64 relative_addr, const void *buf, unsigned int len);

struct vatools_node *vatools_file_get_node(struct file *filp);

struct vatools_reader *vatools_file_get_reader(struct file *filp);

long vatools_driver_smi_fetch(struct file *filp, unsigned int cmd,
			      unsigned long arg);

int vatools_fw_smcu_read(u32 block_id, struct vastai_pci_info *priv,
			 int die_index, u64 relative_addr, void *buf,
			 unsigned int len);

int vatools_fw_smcu_write(u32 block_id, struct vastai_pci_info *priv,
			  int die_index, u64 relative_addr, const void *buf,
			  unsigned int len);
// 申请 dma 空间
int vatools_ioctl_dma_alloc(struct file *filp, unsigned int cmd,
			    unsigned long arg);
// 开始 dma 传输
int vatools_ioctl_dma_start(struct file *filp, unsigned int cmd,
			    unsigned long arg);
// 用户空间内存直接传递到内核, 使用前无需 dma alloc 内存
int vatools_ioctl_dma_trans(struct file *filp, unsigned int cmd,
			    unsigned long arg);

int vatools_ioctl_read_reg_buf(struct file *filp, unsigned int cmd,
			       unsigned long arg);

int vatools_ioctl_write_reg_buf(struct file *filp, unsigned int cmd,
				unsigned long arg);

int vatools_send_ctrl_cmd(struct vastai_pci_info *pci_info, u32 die_index,
			  u32 core_bit_map, u64 info);

int vatools_ioctl_core_time_sync(struct file *filp, unsigned int cmd,
				 unsigned long arg);

void vatools_get_current_time(struct rtc_time *tm);
int vatools_ioctl_get_bmcu_xspi_ver(struct file *filp, unsigned int cmd,
				    unsigned long arg);

int vatools_ioctl_get_sriov_number(struct file *filp, unsigned int cmd,
				   unsigned long arg);

int vatools_ioctl_flash_bmcu_fw(struct file *filp, unsigned int cmd,
				unsigned long arg);
int vatools_ioctl_flash_xspi_fw(struct file *filp, unsigned int cmd,
				unsigned long arg);
int vatools_ioctl_flash_sta(struct file *filp, unsigned int cmd,
			    unsigned long arg);
int vatools_ioctl_bmcu_switch(struct file *filp, unsigned int cmd,
			      unsigned long arg);
//获取 event reset 状态
int vatools_ioctl_device_event(struct file *filp, unsigned int cmd,
			       unsigned long arg);
int vatools_ioctl_device_event_select(struct file *filp, unsigned int cmd,
				      unsigned long arg);
struct vatools_node *vatools_get_node(void);

int vatools_fw_send_smi_interrupt(struct vastai_pci_info *priv, int die_index);
int vatools_wait_smi_ack(struct vastai_pci_info *priv, u32 die_index,
			 u32 block_id, int from);
int vatools_fw_send_interrupt(struct vastai_pci_info *priv, int die_index,
			      u32 type);
int vatools_dump_smi_block_mem(struct vastai_pci_info *priv, int die_index,
			       u32 block_id, u32 len, int from);
void vatools_driver_smi_set_flag(u8 *buf, u32 size, u8 flag);

#ifdef CONFIG_TOOLS_V2
struct vastai_pci_info *
vatools_get_vastai_pci_device_info_by_die_index(u32 die_index);
int vatools_get_spi_buf(u32 die_index, u32 unit, u64 address, u32 len,
			void *spi_buf);

#endif

// dma .使用的vastai 函数
typedef int (*set_desc_func)(struct vastai_dmadesc *, struct dma_buf *,
			     struct kchar_cmd *, int, u32, u32,
			     struct vastai_pci_info *);
int vastai_common_cut_dma_buf(u32 size, struct kchar_cmd *kcmd,
			      struct vastai_dmadesc *desc,
			      struct vastai_pci_info *pci_info,
			      struct dma_buf *dmabuf,
			      set_desc_func pSetDescFunc);
int vastai_ioctl_dma_start_set_desc(struct vastai_dmadesc *desc,
				    struct dma_buf *dmabuf,
				    struct kchar_cmd *kcmd, int i,
				    u32 done_size, u32 temp_size,
				    struct vastai_pci_info *pci_info);
int vastai_ioctl_dma_trans_set_desc(struct vastai_dmadesc *desc,
				    struct dma_buf *dmabuf,
				    struct kchar_cmd *kcmd, int i,
				    u32 done_size, u32 temp_size,
				    struct vastai_pci_info *pci_info);

#endif //__VATOOLS_COMMON_H__
