
#ifndef __VATOOLS_DEFINE_H__
#define __VATOOLS_DEFINE_H__

#ifndef VATOOLS_INDEPENDANCE_DRIVER
#define VATOOLS_INDEPENDANCE_DRIVER 0
#endif

#undef ENABLE_VATOOLS_CORE_TIME_SYNC
#undef ENABLE_VATOOLS_PROFILER

#define V_UNREFERENCE(param) ((void)param);

//所有和驱动接口的数据按1字节对齐
#pragma pack(1)

//****************************************************************************
//下面的定义, app, driver 需要保持一致, 拷贝到各个应用中
//****************************************************************************
//定义可以管理的最多的device数目
#ifndef MAX_DEVICE_NUM_PER_MANAGER
#define MAX_DEVICE_NUM_PER_MANAGER 128
#endif
//定义每个device里面die的最大数目
#ifndef MAX_DIE_NUM_PER_DEVICE
#define MAX_DIE_NUM_PER_DEVICE VASTAI_DIE_MAX_NUM
#endif
//定义每个die里面unit的最大数目, 如果超过, 需要各个命令自己扩充
#define MAX_UNIT_NUM_PER_DIE 32 //这个值和命令定义有关, 不能修改为更大的数目

//每个die内，保存smi数据的地址,
//#define SMI_DIE_ADDRESS_OFFSET ((u64)(0x08cf3600))

//定义vatools 文件节点文件名最大长度
#define VATOOLS_DEVICE_NAME_LEN (32)
// vatools 工具驱动文件节点
#define VATOOLS_DEV_NAME	 "vatools"
#define VATOOLS_DEV_PATH_NAME	 "/dev/vatools"
#define PCIE_DEBUG_DEV_PATH_NAME "/dev/kchar:0"
// IOCTL MAIGC CODE
#define VATOOLS_IOCTL_MAGIC 0xA1

#define VATOOLS_BUFFER_SIZE (0x1000)

//定义各个节点ioctlmingl偏移量
#define VATOOLS_GENERAL_IOCTL_BASE		0x10
#define VATOOLS_HARDWARE_PERFORMANCE_IOCTL_BASE 0x50
#define VATOOLS_SHAREDMEMORY_IOCTL_BASE		0x60
#define VATOOLS_LOGGER_IOCTL_BASE		0x70
#define VATOOLS_DEBUG_IOCTL_BASE		0x80
#define VATOOLS_SMI_IOCTL_BASE			0x90
#define VATOOLS_MONITOR_IOCTL_BASE		0xa0

// ioctl 通用命令
/// 测试命令
#define VATOOLS_GENERAL_IOCTL_TEST                                             \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x00)
/// 设置打开的文件fd的类别
#define VATOOLS_GENERAL_IOCTL_SET_APP_CATEGORY                                 \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x01)
/// 获取device 数量
#define VATOOLS_GENERAL_IOCTL_GET_DEVICE_NUM                                   \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x02)
/// 获取die info
#define VATOOLS_GENERAL_IOCTL_GET_DIE_INFO                                     \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x03)
/// 通过 smi 读数据
#define VATOOLS_GENERAL_IOCTL_READ                                             \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x04)
/// 通过 smi 写数据
#define VATOOLS_GENERAL_IOCTL_WRITE                                            \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x05)
/// 通过 dmi 读写数据, 读写通过一次调用完成
#define VATOOLS_GENERAL_IOCTL_FETCH                                            \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x06)
/// 获取 device 的 pcie bus_id
#define VATOOLS_GENERAL_IOCTL_GET_DEV_PCIE_BUS_ID                              \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x07)
/// 获取 die render
#define VATOOLS_GENERAL_IOCTL_GET_DIE_RENDER                                   \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x08)
/// 通过PCIE驱动读取数据, 不发送数据到smi
#define VATOOLS_GENERAL_IOCTL_PCIE                                             \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x09)
/// 获取pcie id
#define VATOOLS_GENERAL_IOCTL_GET_PCIE_ID                                      \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x0a)
/// 获取pcie sub id
#define VATOOLS_GENERAL_IOCTL_GET_PCIE_SUB_ID                                  \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x0b)
/// 获取驱动major minor 版本号
#define VATOOLS_GENERAL_IOCTL_GET_MAJOR_MINOR_ID                               \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x0c)
/// 获取 device 能力集
#define VATOOLS_GENERAL_IOCTL_DEVICE_CAPABILITY                                \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x0d)
/// 获取 firmware log status
#define VATOOLS_GENERAL_IOCTL_LOG_STATUS                                       \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x0e)
/// 分配 DMA 内存
#define VATOOLS_GENERAL_IOCTL_DMA_ALLOC                                        \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x0f)
/// 开始 DMA 传输数据
#define VATOOLS_GENERAL_IOCTL_DMA_START                                        \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x10)
/// 普通方式读取数据
#define VATOOLS_GENERAL_IOCTL_READ_BUF                                         \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x11)
/// 普通方式写入数据
#define VATOOLS_GENERAL_IOCTL_WRITE_BUF                                        \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x12)
/// 使用内核mutex实现进程互斥保护: 加锁
#define VATOOLS_GENERAL_IOCTL_GUARD_LOCK                                       \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x13)
/// 使用内核mutex实现进程互斥保护: 解锁
#define VATOOLS_GENERAL_IOCTL_GUARD_UNLOCK                                     \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x14)
/// 使用内核mutex实现进程互斥保护: trylock
#define VATOOLS_GENERAL_IOCTL_GUARD_TRYLOCK                                    \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x15)
/// 使用内核mutex实现进程互斥保护: is locked
#define VATOOLS_GENERAL_IOCTL_GUARD_IS_LOCKED                                  \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x16)
/// core timestamp sync
#define VATOOLS_GENERAL_IOCTL_CORE_TIME_SYNC                                   \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x17)
/// 获取bmcu xspi的版本信息
#define VATOOLS_GENERAL_IOCTL_GET_BMCU_XSPI_VERSION                            \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x18)
/// 烧入bmcu fw版本
#define VATOOLS_GENERAL_IOCTL_FLASH_BMCU_FW                                    \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x19)
/// 烧入xspi fw版本
#define VATOOLS_GENERAL_IOCTL_FLASH_XSPI_FW                                    \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x1A)
/// 获取烧写状态
#define VATOOLS_GENERAL_IOCTL_GET_FLASH_STA                                    \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x1B)
/// 切换bmcu版本
#define VATOOLS_GENERAL_IOCTL_SWITCH_BMCU                                      \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x1C)
/// 获取device info
#define VATOOLS_GENERAL_IOCTL_GET_DEV_INFO                                     \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x1D)
/// 获取reset event
#define VATOOLS_GENERAL_IOCTL_DEVICE_EVENT                                     \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x1E)

/// 获取sriov的是否使能和VF个数
#define VATOOLS_GENERAL_IOCTL_SRIOV_NUMBER                                     \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x1F)

/// 获取 event 事件
#define VATOOLS_GENERAL_IOCTL_DEVICE_EVENT_SELECT                              \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x20)

/// 释放DMA申请内存空间
#define VATOOLS_GENERAL_IOCTL_DMA_FREE                                         \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x21)
/// DMA: 用户空间内存传递到内核, 使用时无需先 alloc dma 内存
#define VATOOLS_GENERAL_IOCTL_DMA_TRANS                                        \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_GENERAL_IOCTL_BASE + 0x22)

/// 设置采集性能项目命令
#define VATOOLS_PROFILER_SET_PERFORMANCE_CSR                                   \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_HARDWARE_PERFORMANCE_IOCTL_BASE + 0x01)
/// 设置性能命令
#define VATOOLS_PROFILER_SET                                                   \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_HARDWARE_PERFORMANCE_IOCTL_BASE + 0x02)
/// 读取性能命令
#define VATOOLS_PROFILER_GET                                                   \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_HARDWARE_PERFORMANCE_IOCTL_BASE + 0x03)

/* shared memory ioctl */
// lupa modify 2021-12-15
/// 创建共享内存信息
#define VATOOLS_CREATE_SHARE_MEM                                               \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x01,     \
	      struct memory_entry)
/// 删除共享内存信息
#define VATOOLS_DELETE_SHARE_MEM                                               \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x02,     \
	      struct memory_entry)
/// 获取共享内存信息
#define VATOOLS_READ_SHARE_MEM                                                 \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x03,     \
	      struct memory_entry)
/// 写入共享内存信息
#define VATOOLS_WRITE_SHARE_MEM                                                \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x04,     \
	      struct memory_entry)
/// 等待信息开始
#define VATOOLS_POLL_SHARE_MEM                                                 \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x05,     \
	      struct memory_entry)

/// 开始video debugger信息
#define VATOOLS_SHARE_MEM_VIDEO_START_DEBUGGER                                 \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x06,     \
	      struct memory_entry)
/// 停止video  debugger信息
#define VATOOLS_SHARE_MEM_VIDEO_STOP_DEBUGGER                                  \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x07,     \
	      struct memory_entry)
/// 读取video  debugger信息
#define VATOOLS_SHARE_MEM_VIDEO_READ_DEBUGGER                                  \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x08,     \
	      struct memory_entry)
/// 写入video debugger信息
#define VATOOLS_SHARE_MEM_VIDEO_WRITE_DEBUGGER                                 \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x09,     \
	      struct memory_entry)
/// 附加回调函数
#define VATOOLS_CREATE_VIDEO_MEM_FUNCTION                                      \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x0A,     \
	      struct memory_entry)
/// 删除回调函数
#define VATOOLS_DELETE_VIDEO_MEM_FUNCTION                                      \
	_IOWR(VATOOLS_IOCTL_MAGIC, VATOOLS_SHAREDMEMORY_IOCTL_BASE + 0x0B,     \
	      struct memory_entry)
/* debug tools */
/// 配置调试工具命令
#define VATOOLS_DEBUG_CONFIG                                                   \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_DEBUG_IOCTL_BASE + 0x01)
/// 设置调试工具命令
#define VATOOLS_DEBUG_SET                                                      \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_DEBUG_IOCTL_BASE + 0x02)
/// 读取调试工具命令
#define VATOOLS_DEBUG_GET                                                      \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_DEBUG_IOCTL_BASE + 0x03)
/// 获取调试工具命令
#define VATOOLS_DEBUG_FETCH                                                    \
	_IO(VATOOLS_IOCTL_MAGIC, VATOOLS_DEBUG_IOCTL_BASE + 0x04)

#ifndef ARRAY_SIZE
#define ARRAY_SIZE(ARRAY) (sizeof(ARRAY) / sizeof(ARRAY[0]))
#endif

/// 定义core index 枚举值, core index bitmap, 每个bit代表一个core
enum core_bit_index {
	CORE_LMCU0 = 0,
	CORE_LMCU1 = 1,
	CORE_LMCU2 = 2,
	CORE_LMCU3 = 3,
	CORE_LMCU4 = 4,
	CORE_LMCU5 = 5,
	CORE_LMCU6 = 6,
	CORE_LMCU7 = 7,
	CORE_ODSP0 = 8,
	CORE_ODSP1 = 9,
	CORE_ODSP2 = 10,
	CORE_ODSP3 = 11,
	CORE_ODSP4 = 12,
	CORE_ODSP5 = 13,
	CORE_ODSP6 = 14,
	CORE_ODSP7 = 15,
	CORE_VEMCU0 = 16,
	CORE_VEMCU1 = 17,
	CORE_VEMCU2 = 18,
	CORE_VEMCU3 = 19,
	CORE_VDMCU0 = 20,
	CORE_VDMCU1 = 21,
	CORE_VDMCU2 = 22,
	CORE_SMCU = 23,
	CORE_VDSP0 = 24,
	CORE_VDSP1 = 25,
	CORE_VDSP2 = 26,
	CORE_VDSP3 = 27,
	CORE_CMCU = 28,
	CORE_NUMBER = 29,
	CORE_HOST_PCIE = 30, // TODO: 这里借用此枚举.
};

#define CORE_TYPE_CMCU	0x0
#define CORE_TYPE_LMCU	0x1
#define CORE_TYPE_ODSP	0x2
#define CORE_TYPE_VDSP	0x8
#define CORE_TYPE_VDMCU 0xA
#define CORE_TYPE_VEMCU 0xB
#define CORE_TYPE_SMCU	0xF

#define SET_INDEX_CORE(TYPE, ID) ((TYPE) << 6 | (ID))
#define GET_CORE_TYPE(TYPE)	 ((TYPE) >> 6)
#define GET_INDEX_CORE(ID)	 ((ID)&0x3F)

#define COREID_DEFINE_3(t) (t << 6) | 0, (t << 6) | 1, (t << 6) | 2
#define COREID_DEFINE_4(t)                                                     \
	(t << 6) | 0, (t << 6) | 1, (t << 6) | 2, (t << 6) | 3
#define COREID_DEFINE_8(t)                                                     \
	(t << 6) | 0, (t << 6) | 1, (t << 6) | 2, (t << 6) | 3, (t << 6) | 4,  \
		(t << 6) | 5, (t << 6) | 6, (t << 6) | 7

/// core to core 中断: cmcu ringbuf 数据结构
typedef struct __attribute__((packed)) ring_buf_elem_struct {
	union {
		u64 whole;
		struct {
			u8 seq_no : 8; ///< 低8bit, 序列号, 原样回传
			u32 val : 32; ///< 数据值
			u16 sub_cmd : 16; ///< 子命令
			u8 cmd : 7; ///< 命令族
			u8 use_ringbuf : 1; ///< 1: 使用ringbuf, 0: 不使用ringbuf
		};
	};

} __attribute__((packed)) ring_buf_elem_t;

#define VATOOLS_INTERRUPT_USE_RINGBUF	  (1)
#define VATOOLS_INTERRUPT_NOT_USE_RINGBUF (0)

#define VATOOLS_INTERRUPT_TOOLS_CATEGARY_CMD                                   \
	(0x01) ///< core2core中断ringbuf数据内,cmd=0x01 标识是debug功能的消息
#define VATOOLS_INTERRUPT_DEBUG_CMD                                            \
	(0x0001) ///< core2core中断ringbuf数据内,cmd=0x01 标识是debug功能的消息
#define VATOOLS_INTERRUPT_PROFILER_CMD                                         \
	(0x0002) ///< core2core中断ringbuf数据内,cmd=0x02 标识是profiler功能的消息
#define VATOOLS_INTERRUPT_CORE_TIME_SYNC                                       \
	(0x0003) ///< core2core中断ringbuf数据内,cmd=0x03 标识是core time sync功能的消息

enum enum_app_category {
	VATOOLS_APP_DUMMY = 0,
	VATOOLS_APP_SMI = 1,
	VATOOLS_APP_PROFILER = 2,
	VATOOLS_APP_DEBUGGER = 3,
	VATOOLS_APP_LOGGER = 4,
	VATOOLS_APP_MONITOR = 5,
	VATOOLS_APP_SHAREDMEM = 6,
	VATOOLS_APP_EVENT = 7,
	VATOOLS_APP_MAX
};

enum enum_cmd_type {
	VATOOLS_MEM_CMD_HANDLE = 0,
	VATOOLS_MEM_CMD_CHANNEL = 1,
	VATOOLS_MEM_CMD_TIMEOUT = 2,
};

// block 数量
#define SMI_BLOCK_ID_MAX (8)

enum __SMI_CMD_BLOCK_IDS {
	SMI_CMD_BLOCK_ID_0 = 0,
	SMI_CMD_BLOCK_ID_1,
	SMI_CMD_BLOCK_ID_2,
	SMI_CMD_BLOCK_ID_3,
	SMI_CMD_BLOCK_ID_4,
	SMI_CMD_BLOCK_ID_5,
	SMI_CMD_BLOCK_ID_6,
	SMI_CMD_BLOCK_ID_7,
	SMI_CMD_BLOCK_ID_MAX = SMI_BLOCK_ID_MAX,
	SMI_CMD_BLOCK_ID_IGNORE = 0x0000FFFF
};

enum __SMI_CMD_BLOCK_USED_FLAG {
	SMI_CMD_BLOCK_IDLE = 0,
	SMI_CMD_BLOCK_BUSY = 1
};

typedef struct s_device {
	u32 dev_id;
	u32 die_id;
	u32 die_index; // low die_id:8;  dev_id:16; high seq_num:8;
	// u32 core_id;    // low core_index:6; high core_type:26;
} __attribute__((packed)) T_DEVICE;

typedef struct s_vasmi_buf_array {
	u32 buf_size;
	u8 buf[0];
} __attribute__((packed)) T_SMI_BUF_ARR;

typedef struct s_vasmi_buf_ptr {
	u32 buf_size;
	void *buf_addr;
} __attribute__((packed)) T_SMI_BUF_PTR;

typedef struct s_vasmi_tlv {
	u32 address;
	u32 length;
	u8 value[0];
} __attribute__((packed)) T_SMI_TLV;

//
typedef struct SMI_IOCTL_TRANS_CATEGORY_s {
	u32 app_category;
	u32 block_id;
	//卡信息
	T_DEVICE device;
	// TODO: 为了适配arm服务器, 这里需要结构是8bytes 倍数. 调整变量时, 需要考虑是否8bytes对齐. 暂时不修改这里,
	// 在驱动结构 vatools_reader 里修改; 下次整理代码时, 需要整体考虑
} T_SMI_IOCTL_TRANS_CATEGORY;

// host 使用 ioctl 和驱动进行数据交互传输
typedef struct SMI_IOCTL_TRANS_DATA_s {
	u32 block_id;
	//卡信息
	T_DEVICE device;
	u64 address;
	u32 flag; // 0x1: use pcie interface  0x0: use smi interface
	T_SMI_BUF_PTR
	output_buf; // host侧保存 fw 返回的数据buffer. 数据: fw->host
	T_SMI_BUF_PTR input_buf; // host侧要写入 fw 的数据buffer. 数据: host->fw
	int errcode; //调用内核ioctl返回值
} T_SMI_IOCTL_TRANS_DATA;

typedef struct SMI_DEV_PCIE_BUS_ID {
	int dev_id;
	char pcie_port[16];
} TS_SMI_DEV_PCIE_BUS_ID;

typedef struct SMI_DEV_PCIE_ID {
	int dev_id;
	unsigned int pcie_id;
} TS_SMI_DEV_PCIE_ID;

typedef struct SMI_DEV_PCIE_SUB_ID {
	int dev_id;
	unsigned int pcie_sub_id;
} TS_SMI_DEV_PCIE_SUB_ID;

typedef struct SMI_DEV_MAJOR_MINOR {
	int dev_id;
	unsigned int major_minor;
} TS_SMI_DEV_MAJOR_MINOR;

typedef struct SMI_DIE_RENDER {
	int dev_id;
	int die_id;
	char render[16];
} TS_SMI_DIE_RENDER;

typedef struct SMI_DEVICE_CAPABILITY {
	int dev_id;
	int die_id;
	struct vastai_dev_info capability;
} SMI_DEVICE_CAPABILITY;

/// 保存 event 数据的ringbuf size
#define EVENT_FIFO_LENGTH (2 * 1024 * 1024)

//获取 event reset 状态
// CORE_POINT_CMCU 不代表CMCU，代表整个ai-sys, 包括整个ai-sys的所有core; SMCU 没有reset; 其他代表各自的core reset
enum device_common_event_e {
	EVT_RESET_START = RESET_START, // reset 开始
	EVT_RESET_STOP = RESET_END, // reset 完成, 恢复正常状态
	EVT_COMMON_MAX = 10 // common event 最多只能由10个
};

// VEMCU video event 子模块/core 的事件定义, 从11开始
enum vemcu_exception_event_e {
	VCENC_EXCEPTION_ERROR = 11,
	VCENC_EXCEPTION_NULL_ARGUMENT, /**ERROR Null argument**/
	VCENC_EXCEPTION_INVALID_ARGUMENT, /**ERROR Invalid argument*/
	VCENC_EXCEPTION_INVALID_STATUS, /*VCEncStrmEncode: ERROR Invalid status*/
	VCENC_EXCEPTION_INVALID_GOPSIZE, /*VCEncStrmEncode: ERROR Invalid gopSize*/
	VCENC_EXCEPTION_NOTCODED_FRAME, /*VCEncStrmEncode: ERROR Invalid coding type*/
	VCENC_EXCEPTION_STREAM_BUFFER, /*VCEncStrmEncode: ERROR Invalid output stream buffer*/
	VCENC_EXCEPTION_TOO_SMALL_OUTPUT_STREAM_BUFFER, /*VCEncStrmEncode: ERROR Too small output stream buffer*/
	VCENC_EXCEPTION_NOT_SUPPORT_MULTI_CORE, /*VCEncStrmEncode: multi-segment not support multi-core*/
	VCENC_EXCEPTION_INVALID_BASELINE_PROFILE, /*VCEncSetCodingCtrl: ERROR Invalid frame type for baseline profile*/
	VCENC_EXCEPTION_INVALID_INPUT_BUS_CHROMAV, /*VCEncStrmEncode: ERROR Invalid input busChromaV*/
	VCENC_EXCEPTION_INVALID_INPUT_BUS_CHROMAU, /*VCEncStrmEncode: ERROR Invalid input busChromaU*/
	VCENC_EXCEPTION_INVALID_INPUT_BUSLUMA, /*VCEncStrmEncode: ERROR Invalid input busLuma*/
	VCENC_EXCEPTION_INVALID_INPUT_FORMAT, /*VCEncStrmEncodeExt: ERROR Invalid input format*/
	VCENC_EXCEPTION_INVALID_INPUT_BUSLUMASTAB, /*VCEncStrmEncodeExt: ERROR Invalid input busLumaStab*/
	VCENC_EXCEPTION_MEMORY_ERROR, /**ERROR Memery error*/
	VCENC_EXCEPTION_EWL_ERROR, /**ERROR Invalid argument*/
	VCENC_EXCEPTION_EWL_MEMORY_ERROR, /**ERROR Ewl Memery error*/
	VCENC_EXCEPTION_OUTPUT_BUFFER_OVERFLOW, /**ERROR Revert FrameNum Update on output buffer overflow*/
	VCENC_EXCEPTION_HW_BUS_ERROR, /**ERROR Hardware bus error*/
	VCENC_EXCEPTION_HW_DATA_ERROR, /**ERROR Hardware data error*/
	VCENC_EXCEPTION_HW_TIMEOUT, /**ERROR hardware timeout*/
	VCENC_EXCEPTION_HW_RESERVED, /**ERROR Hardware reserved*/
	VCENC_EXCEPTION_SYSTEM_ERROR, /**ERROR System error*/
	VCENC_EXCEPTION_INSTANCE_ERROR, /**ERROR Instance error*/
	VCENC_EXCEPTION_HRD_ERROR, /**ERROR Hrd error*/
	VCENC_EXCEPTION_HW_RESET, /**ERROR Hardware reset*/
	VCENC_EXCEPTION_HASH_LEN_MISMATCH, /**ERROR Hash len mismatch*/
	VCENC_EXCEPTION_FW_RESET, /**ERROR FW reset*/
	VCENC_EXCEPTION_INSATANCE_CHECK_MISMATCH, /**ERROR Instance check mismatch*/
	VCENC_EXCEPTION_IM_RESET, /**ERROR hantrovcmd_isr failed. im reset dev->hwregs */
	VCENC_EXCEPTION_ENCODE_EXT_MCU_V2, /**ERROR  VCEncStrmEncodeExt_mcu_V2*/
	VCENC_EXCEPTION_HANDLE_ENCODER_ISR, /**ERROR  HandleEncoderIsr failed*/
	VCENC_EXCEPTION_HANG_HANTRO_RESET_ISR_FAILED, /**FAILED hang reset hantrovcmd_isr failed irq*/
	VCENC_EXCEPTION_HANTRO_VCMD_ISR_TIMEOUT, /**ERROR hantrovcmd_isr error timeout */
	VCDEC_EXCEPTION_RESERVED /**ERRORR decoder reserved*/
};

// reset event 信息
typedef struct device_event_s {
	u64 timestamp_ns; // event 发生的时间
	u32 die_index;
	u32 core_index;
	u32 event;
} device_event_t;

// TODO:
typedef struct vatools_cmd {
	union ucmd {
		// 申请 dma 空间
		struct {
			u32 size;
			int dma_buf_fd;
			u32 die_index;
		} alloc_cmd;
		// 开始 dma 传输
		struct {
			u32 is_dev_to_host;
			u32 dma_buf_fd;
			u64 axi_addr;
			u32 size;
			u32 die_index;
		} dma_start_cmd;
		// 用户空间内存直接传递到内核, 使用前无需 dma alloc 内存
		struct {
			u32 is_dev_to_host;
			void *vir_addr;
			u64 axi_addr;
			u32 length;
			u32 die_index;
			int pid;
		} dma_trans_cmd;
		struct {
			u32 is_dev_to_host;
			u32 die_index;
			u64 axi_addr;
			struct vastai_channel_buf *channel_buf;
			int channel_num;
			int pid;
		} dma_trans_sg_cmd;
		struct {
			u32 die_index;
			u32 timeout_water;
			u32 is_timeout;
			void *log_ring_ctl;
		} log_poll_cmd;
		struct {
			u32 die_index;
			u64 addr;
			u32 size;
			void *buf;
		} access_mem_cmd;
		struct {
			u32 event_count;
			device_event_t buf[0];
		} event;
	} cmd;
} __attribute__((packed)) T_VATOOLS_CMD;

/**********************************************************************************/
// PCIe 命令, 不使用smcu. 直接访问驱动
enum e_pcie_cmd_flag {
	PCIE_CMD_FLAG_NONE = 0, ///< 无
	PCIE_CMD_FLAG_BANDWIDTH = 1, ///< 获取PCI带宽
	PCIE_CMD_FLAG_RESET = 2, ///< 复位设备(device) reset
	PCIE_CMD_FLAG_REBOOT = 3, ///< 重启设备(device) reboot
	PCIE_CMD_FLAG_STRING = 4, ///< 灵活文本命令
	PCIE_CMD_FLAG_FW_VERSION = 5, ///< 获取固件版本
	PCIE_CMD_FLAG_DDR_ECC_1BIT = 6, ///< 获取DDR ECC 1bit错误数据
	PCIE_CMD_FLAG_DDR_ECC_2BIT = 7, ///< 获取DDR ECC 2bit错误数据
	PCIE_CMD_FLAG_VDSP_COUNT = 8, ///< 获取 vdsp 统计数据
	PCIE_CMD_FLAG_AI_COUNT = 9, ///< 获取ai 统计数据. 512 bytes.
	PCIE_CMD_FLAG_RING_BUF_STATISTIC = 10, ///< 获取ring buffer 统计数据
	PCIE_CMD_FLAG_SET_VIDEO_MULTI_CORE =
		11, ///< 设置video使用multicore; 0: 不使用; 1: 使用
	PCIE_CMD_FLAG_GET_VIDEO_MULTI_CORE =
		12, ///< 读取video multicore 的mode和job count
	PCIE_CMD_FLAG_FW_VERSION_V2 = 13, ///< 获取fw版本号, 读取全部die
	PCIE_CMD_FLAG_GET_BOX_TYPE = 14, ///< 获取bbox type
	PCIE_CMD_FLAG_HEART_BEAT = 15, ///< core heartbeat
	PCIE_CMD_FLAG_BOOT = 16, ///< 启动设备(device) boot
	PCIE_CMD_FLAG_BARRESET = 17, ///< bar复位
	PCIE_CMD_FLAG_BARSTATUS = 18, ///< bar status获取
	PCIE_CMD_FLAG_REBOOT_DDRBW = 19, ///< 切换至 DDR 带宽 bin
	PCIE_CMD_FLAG_REBOOT_PCIEDMA = 20, ///< 切换至测试 PCIE DMA 带宽
	PCIE_CMD_FLAG_MAX,
};

typedef struct S_HOST_ARG_VAL {
	unsigned int
		nType; // 0、 cmd , driver收到后需要发送 中断到 fw  1、  data , driver只负责读写数据, 不发中断到fw
	void *pcBuf;
} __attribute__((packed)) T_HOST_ARG_VAL;

/// 进程间互斥锁结构
struct vatools_process_mutex {
	struct list_head list_member_process_mutex;
	struct list_head list_head_filp;
	struct mutex process_mutex;
	char mutex_name[256]; ///< 互斥锁 名, 名字相同为同一个互斥锁
};

/// file* 链表
struct vatools_process_mutex_filp {
	struct list_head list_member_filp;
	struct file *filp;
};

#ifndef CONFIG_TOOLS_V2
struct vatools_node {
	struct miscdevice misc;
	wait_queue_head_t wq;
	struct mutex node_mutex;
	// lupa add 2021-12-23 begin 整个业务的资源锁。保障处理该业务时，资源独享
	struct mutex node_sharedmem_mutex;
	struct mutex node_common_mutex;
	struct mutex node_profiler_mutex;
	struct mutex node_debugger_mutex;
	struct mutex node_logger_mutex;
	struct mutex node_smi_mutex;

	// lupa add 2021-12-23 end 整个业务的资源锁。保障处理该业务时，资源独享
	struct list_head list_nodes;
	void *pci_dev;
	unsigned char *node_buffer;
	unsigned int size;
	unsigned int rsv; //适配arm服务器 需要8字节对齐
	struct list_head list_node_readers;
	struct list_head list_node_process_mutex;

	// pcie dma / msg ringbuf, for profiler timeline
	void *profiler_dma_ringbuf;
	void *profiler_msg_ringbuf;

	// core event ringbuf
	void *event_ringbuf;
	// core event mutex
	struct mutex event_mutex;
	// 等待 core 的 event; 例如 reset事件
	wait_queue_head_t event_wait_queue;
};

struct vatools_reader {
	struct vatools_node *node;

	struct list_head list_readers;
	struct list_head
		malloc_pointers; // TODO: 保存每个 open 的 fd 申请的内存指针, 如果fd关闭时, 循环释放空间

	// sharedmem
	struct list_head t_sharedmem_mem_info_head;
	// struct list_head t_sharedmem_signal_info_head;
	//添加一个信号量用于触发等待事件
	struct completion debug_comp; // 用于记录触发信号使用 lupa add 2021-12-15
	struct fasync_struct *sharedmem_pt_fasync;
	struct mutex sharedmem_mutex;
	int n_sharedmem_init_done; // 标识是否初始化过; 1-完成初始化 0-未初始化

	// reader 通用变量. 暂未使用.
	size_t r_off;
	int r_all; // 适配arm服务器. 修改boot 类型为int
	int r_ver;
	unsigned int
		profiler_checkpoint_corebitmap; ///< 记录开启profiler checkpoint 的core , 使用 corebitmap记录, 每个core 一个bit

	T_SMI_IOCTL_TRANS_CATEGORY
	trans_category; // 适配arm服务器, 和host交互的结构, 需要注意 8 byte 对齐
	int exit; //是否退出
	// 适配arm服务器, 需要8字节对齐, trans_category 暂不修改, 这里使用保留字段对齐.
	u64 event_timestamp_ns; ///< 标识此时间以及之前的event都已经发送到host线程; 每次发送event到host后, 更新此时间
};

#endif

typedef struct VASTAI_FIFO_HEADER {
	u32 rd;
	u32 wr;
	u32 elem_count;
	u32 elem_size;
} __attribute__((packed)) T_VASTAI_FIFO_HEADER;

typedef struct RING_BUF_STATITCS {
	u32 base_addr;
} __attribute__((packed)) T_RING_BUF_STATICS;

//****************************************************************************
//上面的定义, app, driver 需要保持一致, 拷贝到各个应用中
//****************************************************************************

// csram定义数据地址
#if 0 // defined in csram_partition_v05.h
#define CSRAM_BASE_ADDR 0x08C00000

#define SMI_INFO_ADDR	     (CSRAM_BASE_ADDR + 0x000F3600) // 2.5K
#define PERFORMANCE_AI_BASE  (CSRAM_BASE_ADDR + 0x000F4000) // 512 bytes
#define PERFORMANCE_AI_CMCU  (CSRAM_BASE_ADDR + 0x000F4000) // 8 bytes
#define PERFORMANCE_AI_LMCU0 (CSRAM_BASE_ADDR + 0x000F4008) // 8 bytes
#define PERFORMANCE_AI_LMCU1 (CSRAM_BASE_ADDR + 0x000F4010) // 8 bytes
#define PERFORMANCE_AI_LMCU2 (CSRAM_BASE_ADDR + 0x000F4018) // 8 bytes
#define PERFORMANCE_AI_LMCU3 (CSRAM_BASE_ADDR + 0x000F4020) // 8 bytes
#define PERFORMANCE_AI_LMCU4 (CSRAM_BASE_ADDR + 0x000F4028) // 8 bytes
#define PERFORMANCE_AI_LMCU5 (CSRAM_BASE_ADDR + 0x000F4030) // 8 bytes
#define PERFORMANCE_AI_LMCU6 (CSRAM_BASE_ADDR + 0x000F4038) // 8 bytes
#define PERFORMANCE_AI_LMCU7 (CSRAM_BASE_ADDR + 0x000F4040) // 8 bytes
#define PERFORMANCE_AI_ODSP0 (CSRAM_BASE_ADDR + 0x000F4048) // 8 bytes
#define PERFORMANCE_AI_ODSP1 (CSRAM_BASE_ADDR + 0x000F4050) // 8 bytes
#define PERFORMANCE_AI_ODSP2 (CSRAM_BASE_ADDR + 0x000F4058) // 8 bytes
#define PERFORMANCE_AI_ODSP3 (CSRAM_BASE_ADDR + 0x000F4060) // 8 bytes
#define PERFORMANCE_AI_ODSP4 (CSRAM_BASE_ADDR + 0x000F4068) // 8 bytes
#define PERFORMANCE_AI_ODSP5 (CSRAM_BASE_ADDR + 0x000F4070) // 8 bytes
#define PERFORMANCE_AI_ODSP6 (CSRAM_BASE_ADDR + 0x000F4078) // 8 bytes
#define PERFORMANCE_AI_ODSP7 (CSRAM_BASE_ADDR + 0x000F4080) // 8 bytes
#define PERFORMANCE_AI_VDSP0 (CSRAM_BASE_ADDR + 0x000F4088) // 12 bytes
#define PERFORMANCE_AI_VDSP1 (CSRAM_BASE_ADDR + 0x000F4094) // 12 bytes
#define PERFORMANCE_AI_VDSP2 (CSRAM_BASE_ADDR + 0x000F40A0) // 12 bytes
#define PERFORMANCE_AI_VDSP3 (CSRAM_BASE_ADDR + 0x000F40AC) // 12 bytes
#endif
#define PERFORMANCE_AI_VDMCU0	  (CSRAM_BASE_ADDR + 0x000F40B8) // 8 bytes
#define PERFORMANCE_AI_VDMCU1	  (CSRAM_BASE_ADDR + 0x000F40C0) // 8 bytes
#define PERFORMANCE_AI_VDMCU2	  (CSRAM_BASE_ADDR + 0x000F40C8) // 8 bytes
#define PERFORMANCE_AI_VEMCU0	  (CSRAM_BASE_ADDR + 0x000F40D0) // 8 bytes
#define PERFORMANCE_AI_VEMCU1	  (CSRAM_BASE_ADDR + 0x000F40D8) // 8 bytes
#define PERFORMANCE_AI_VEMCU2	  (CSRAM_BASE_ADDR + 0x000F40E0) // 8 bytes
#define PERFORMANCE_AI_VEMCU3	  (CSRAM_BASE_ADDR + 0x000F40E8) // 8 bytes
#define PCIE_BANDWIDTH_UPSTREAM	  (CSRAM_BASE_ADDR + 0x000F40F0) // 8 bytes
#define PCIE_BANDWIDTH_DOWNSTREAM (CSRAM_BASE_ADDR + 0x000F40F8) // 8 bytes
// fw mcu 计数时, 同步写入当前的cycle count 计数, host读取后做时间差
#define PERFORMANCE_AI_CMCU_CYCLE  (CSRAM_BASE_ADDR + 0x000F4100) // 4 bytes
#define PERFORMANCE_AI_VDSP0_CYCLE (CSRAM_BASE_ADDR + 0x000F4104) // 4 bytes
#define PERFORMANCE_AI_VDSP1_CYCLE (CSRAM_BASE_ADDR + 0x000F4108) // 4 bytes
#define PERFORMANCE_AI_VDSP2_CYCLE (CSRAM_BASE_ADDR + 0x000F410c) // 4 bytes
#define PERFORMANCE_AI_VDSP3_CYCLE (CSRAM_BASE_ADDR + 0x000F4110) // 4 bytes

#define DDR_ECC_1BIT	  0x8CF6400
#define DDR_ECC_2BIT	  0x8CF6800
#define DDR_ECC_1BIT_SIZE (1024) // use csram to store ecc , 1k bytes
#define DDR_ECC_2BIT_SIZE (1024) // use csram to store ecc , 1k bytes

// 读此地址触发bar2切换为bar2默认地址
#define DDR_PCIE_BARRESET 0x814000000

// VEMCU ring buf
#define PERFORMANCE_HOST_2_VEMCU0 (CSRAM_BASE_ADDR + 0x56000)
#define PERFORMANCE_HOST_2_VEMCU1 (CSRAM_BASE_ADDR + 0x56400)
#define PERFORMANCE_HOST_2_VEMCU2 (CSRAM_BASE_ADDR + 0x56800)
#define PERFORMANCE_HOST_2_VEMCU3 (CSRAM_BASE_ADDR + 0x56C00)
// VDMCU ring buf
#define PERFORMANCE_SMCU_2_VDMCU0   (CSRAM_BASE_ADDR + 0x3E000)
#define PERFORMANCE_VDSP0_2_VDMCU0  (CSRAM_BASE_ADDR + 0x3E400)
#define PERFORMANCE_VDSP1_2_VDMCU0  (CSRAM_BASE_ADDR + 0x3E800)
#define PERFORMANCE_VEMCU0_2_VDMCU0 (CSRAM_BASE_ADDR + 0x3EC00)
#define PERFORMANCE_VEMCU1_2_VDMCU0 (CSRAM_BASE_ADDR + 0x3F000)
#define PERFORMANCE_SMCU_2_VDMCU1   (CSRAM_BASE_ADDR + 0x3F400)
#define PERFORMANCE_VDSP1_2_VDMCU1  (CSRAM_BASE_ADDR + 0x3F800)
#define PERFORMANCE_VDSP2_2_VDMCU1  (CSRAM_BASE_ADDR + 0x3FC00)
#define PERFORMANCE_VEMCU1_2_VDMCU1 (CSRAM_BASE_ADDR + 0x40000)
#define PERFORMANCE_VEMCU2_2_VDMCU1 (CSRAM_BASE_ADDR + 0x40400)
#define PERFORMANCE_SMCU_2_VDMCU2   (CSRAM_BASE_ADDR + 0x40800)
#define PERFORMANCE_VDSP2_2_VDMCU2  (CSRAM_BASE_ADDR + 0x40C00)
#define PERFORMANCE_VDSP3_2_VDMCU2  (CSRAM_BASE_ADDR + 0x41000)
#define PERFORMANCE_VEMCU2_2_VDMCU2 (CSRAM_BASE_ADDR + 0x41400)
#define PERFORMANCE_VEMCU3_2_VDMCU2 (CSRAM_BASE_ADDR + 0x41800)
// CMCU ring buf
#define PERFORMANCE_VDSP0_2_CMCU (CSRAM_BASE_ADDR + 0x49400)
#define PERFORMANCE_VDSP1_2_CMCU (CSRAM_BASE_ADDR + 0x49800)
#define PERFORMANCE_VDSP2_2_CMCU (CSRAM_BASE_ADDR + 0x49C00)
#define PERFORMANCE_VDSP3_2_CMCU (CSRAM_BASE_ADDR + 0x4A000)
// VDSP ring buf
#define PERFORMANCE_CMCU_2_VDSP0 (CSRAM_BASE_ADDR + 0x45400)
#define PERFORMANCE_CMCU_2_VDSP1 (CSRAM_BASE_ADDR + 0x46000)
#define PERFORMANCE_CMCU_2_VDSP2 (CSRAM_BASE_ADDR + 0x47000)
#define PERFORMANCE_CMCU_2_VDSP3 (CSRAM_BASE_ADDR + 0x48000)
#define PERFORMANCE_HOST_2_VDSP0 (CSRAM_BASE_ADDR + 0x57000)
#define PERFORMANCE_HOST_2_VDSP1 (CSRAM_BASE_ADDR + 0x57400)
#define PERFORMANCE_HOST_2_VDSP2 (CSRAM_BASE_ADDR + 0x57800)
#define PERFORMANCE_HOST_2_VDSP3 (CSRAM_BASE_ADDR + 0x57C00)
// HOST(PCIE) ring buf
#define PERFORMANCE_SMCU0_2_HOST  (CSRAM_BASE_ADDR + 0x51800)
#define PERFORMANCE_SMCU1_2_HOST  (CSRAM_BASE_ADDR + 0x51C00)
#define PERFORMANCE_VDMCU0_2_HOST (CSRAM_BASE_ADDR + 0x52000)
#define PERFORMANCE_VDMCU1_2_HOST (CSRAM_BASE_ADDR + 0x52400)
#define PERFORMANCE_VDMCU2_2_HOST (CSRAM_BASE_ADDR + 0x52800)
#define PERFORMANCE_VEMCU0_2_HOST (CSRAM_BASE_ADDR + 0x52C00)
#define PERFORMANCE_VEMCU1_2_HOST (CSRAM_BASE_ADDR + 0x53000)
#define PERFORMANCE_VEMCU2_2_HOST (CSRAM_BASE_ADDR + 0x53400)
#define PERFORMANCE_VEMCU3_2_HOST (CSRAM_BASE_ADDR + 0x53800)
#define PERFORMANCE_VDSP0_2_HOST  (CSRAM_BASE_ADDR + 0x53C00)
#define PERFORMANCE_VDSP1_2_HOST  (CSRAM_BASE_ADDR + 0x54000)
#define PERFORMANCE_VDSP2_2_HOST  (CSRAM_BASE_ADDR + 0x54400)
#define PERFORMANCE_VDSP3_2_HOST  (CSRAM_BASE_ADDR + 0x54800)
// Host(DMA) ring buf
#define PERFORMANCE_DMA_HOST_2_DEV (CSRAM_BASE_ADDR + 0x00014000)
#define PERFORMANCE_DMA_DEV_2_HOST (CSRAM_BASE_ADDR + 0x00014000 + 0xA000)
#define PERFORMANCE_DMA_RC_IN	   (CSRAM_BASE_ADDR + 0x00014000 + 0xA000 * 2)
#define PERFORMANCE_DMA_RC_OUT	   (CSRAM_BASE_ADDR + 0x00014000 + 0xA000 * 3)

//****************************************************************************
// profiler host interface
//****************************************************************************
//应用层调用接口
#define MAX_HW_MONITOR_NUM 4
#define MAX_ODMA_NUM	   2
#define MAX_DLC_NUM	   8
#define MAX_MC_NUM	   4

struct T_ADDR_VAL {
	u64 nAddr;
	u32 nLen;
	// u64 pcBuf;
	void *pcBuf;
};

struct T_PERFMON_SEL {
	union {
		u32 n_whole;
		struct {
			u32 n_is_enable : 1;
			u32 n_is_start : 1;
			u32 n_is_stop : 1;
			u32 n_spare : 1;
			u32 n_index : 12;
		};
	};
	u32 n_data_sel;
	u32 n_timer;
};

typedef struct __attribute__((packed)) tar_perfmon_mc_fw {
	u32 n_val[6];
} T_PERFMON_MC_FW;

typedef struct __attribute__((packed)) tar_perfmon_fw {
	union {
		u32 n_whole;
		struct {
			u32 n_is_enable : 1;
			u32 n_is_start : 1;
			u32 n_is_stop : 1;
			u32 n_spare : 1;
			u32 n_index : 12;
		};
	};
	u8 n_data_sel[MAX_HW_MONITOR_NUM];
	u32 n_timer;
	u32 n_data[MAX_HW_MONITOR_NUM];
} T_PERFMON_FW;

struct T_PERFMON_DATA {
	T_PERFMON_FW tODMA[MAX_ODMA_NUM];
	T_PERFMON_FW tDLC[MAX_DLC_NUM];
	T_PERFMON_MC_FW tMC[MAX_MC_NUM];
};

//****************************************************************************
// smi cmd block
//****************************************************************************

#define SMI_CMD_LOOP_TOTAL_COUNT     (20) //总轮询次数
#define SMI_CMD_LOOP_ADDING_COUNT    (3) //轮询时长增加次数
#define SMI_CMD_LOOP_INTERVAL_START  (2) // 起始轮询时间间隔, 单位ms
#define SMI_CMD_LOOP_INTERVAL_ADDING (10) // 每次增加时间间隔, 单位ms

//****************************************************************************
// driver internal define
//****************************************************************************
#define VATOOLS_UNINIT	  0
#define VATOOLS_INIT_DONE 1

//****************************************************************************
// vadebugger start
//****************************************************************************
#define VADBG_CSRAM_ADDR (0x08cf4400) // debug 使用的 csram base address
#define VADBG_CSRAM_LEN	 (4096) // total size. 0x1000
//****************************************************************************
// vadebugger end
//****************************************************************************

//****************************************************************************
// profiler checkpoint start
//****************************************************************************
// profiler使用的 csram base地址
// #define PERFORMANCE_AI_BASE ( CSRAM_BASE_ADDR + 0x000F4000 )   // 512 bytes
#define PROFILER_CSRAM_ADDR (0x08CF4200)
// profiler使用的csram size
#define PROFILER_CSRAM_LEN (512)
// soc内核数目
#define PROFILER_CORE_COUNT CORE_NUMBER
// 保存profiler的 DDR的起始地址. 模拟log.
#define PROFILER_DDR_ADDR   ((u64)0x814000000LL) // v0.6分配 0x0814000000
#define PROFILER_DDR_ADDR_H 0x8
#define PROFILER_DDR_ADDR_L 0x14000000
// 每个core使用1M的地址
#define PROFILER_DDR_BUFFER_LEN 0x100000
// csram 保存的 profiler control 数据, 每个core 1 byte
#define PROFILER_CTLR_BASE_ADDR (PROFILER_CSRAM_ADDR)
// csram 保存的 rpwp 地址. rp 占低8bytes, wp 占高8bytes
#define PROFILER_RPWP_STARTADDR (PROFILER_CSRAM_ADDR + 0x20)
// csram 保存的每个core对应的rp, wp 长度. 共16bytes
#define PROFILER_RPWP_SIZE 0x10
// 读读取的 rp, wp, host 需要使用mask, 只保留低有效位
#define PROFILER_CORE_BUF_WP_MASK (PROFILER_DDR_BUFFER_LEN - 1)
//****************************************************************************
// profiler checkpoint end
//****************************************************************************

/// profiler 采集数据的命令码
enum e_profiler_checkpoint_msg_type {
	PROFILER_CHECKPOINT_MSG_STOP = 0, ///< 停止采集
	PROFILER_CHECKPOINT_MSG_START = 1, ///< 开始采集
	PROFILER_CHECKPOINT_MSG_READ_RPWP =
		2, ///< 读取读写指针. fw 不会收到此命令
	PROFILER_CHECKPOINT_MSG_WRITE_RPWP =
		3, ///< 写入读写指针. fw 不会收到此命令
	PROFILER_CHECKPOINT_MSG_READ_DATA =
		4, ///< 读取 DDR 数据. fw 不会收到此命令
	PROFILER_CHECKPOINT_MSG_READ_CTRL = 5, ///< 读控制
	PROFILER_CHECKPOINT_MSG_READ_PCIE_DMA = 6, ///< PCIE dma 数据采集
	PROFILER_CHECKPOINT_MSG_READ_PCIE_MSG = 7, ///< PCIE msg 数据采集
};

/// 定义timeline采集点数据类型 u8
enum e_profiler_checkpoint_type {
	// general
	PROFILER_CHECKPOINT_UNKNOWN = 0x00,
	PROFILER_CHECKPOINT_IDLE = 0x01,
	PROFILER_CHECKPOINT_DUTY_BEGIN = 0x02,
	PROFILER_CHECKPOINT_DUTY_END = 0x03,
	// vdsp
	PROFILER_CHECKPOINT_HOST_TO_VDSP = 0x04,
	PROFILER_CHECKPOINT_VDSP_TO_HOST = 0x05,
	PROFILER_CHECKPOINT_OPCODE_BEGIN = 0x06,
	PROFILER_CHECKPOINT_OPCODE_END = 0x07,
	PROFILER_CHECKPOINT_VDSP_TO_CMCU = 0x08,
	PROFILER_CHECKPOINT_CMCU_TO_VDSP = 0x09,
	// pcie
	PROFILER_CHECKPOINT_PCIE_DMA = 0x10, // 16
	PROFILER_CHECKPOINT_PCIE_MSG = 0x11, // 17
	PROFILER_CHECKPOINT_PCIE_DMA_HOST_TO_DEVICE = 0x12, // 18
	PROFILER_CHECKPOINT_PCIE_DMA_DEVICE_TO_HOST = 0x13, // 19
	PROFILER_CHECKPOINT_PCIE_DMA_HOST_TO_DEVICE_BANDWIDTH = 0x14, // 20
	PROFILER_CHECKPOINT_PCIE_DMA_DEVICE_TO_HOST_BANDWIDTH = 0x15, // 21

	// cmcu ping: bit7=0  pong: bit7=1
	PROFILER_CHECKPOINT_PING_MODEL_BEGIN = 0x70, // 112
	PROFILER_CHECKPOINT_PING_MODEL_END = 0x71, // 113
	PROFILER_CHECKPOINT_PING_LAYER_BEGIN = 0x72, // 114
	PROFILER_CHECKPOINT_PING_LAYER_END = 0x73, // 115
	PROFILER_CHECKPOINT_PING_TIMESTAMP = 0x74, // 116
	PROFILER_CHECKPOINT_PING_MODEL_DUTY = 0x75, // 117
	PROFILER_CHECKPOINT_PING_LAYER_DUTY = 0x76, // 118
	PROFILER_CHECKPOINT_PING_LAYER_OPCODE = 0x77, // 119

	PROFILER_CHECKPOINT_PONG_MODEL_BEGIN =
		PROFILER_CHECKPOINT_PING_MODEL_BEGIN | 0x80, // 240
	PROFILER_CHECKPOINT_PONG_MODEL_END =
		PROFILER_CHECKPOINT_PING_MODEL_END | 0x80, // 241
	PROFILER_CHECKPOINT_PONG_LAYER_BEGIN =
		PROFILER_CHECKPOINT_PING_LAYER_BEGIN | 0x80, // 242
	PROFILER_CHECKPOINT_PONG_LAYER_END =
		PROFILER_CHECKPOINT_PING_LAYER_END | 0x80, // 243
	PROFILER_CHECKPOINT_PONG_TIMESTAMP =
		PROFILER_CHECKPOINT_PING_TIMESTAMP | 0x80, // 244
	PROFILER_CHECKPOINT_PONG_MODEL_DUTY =
		PROFILER_CHECKPOINT_PING_MODEL_DUTY | 0x80, // 245
	PROFILER_CHECKPOINT_PONG_LAYER_DUTY =
		PROFILER_CHECKPOINT_PING_LAYER_DUTY | 0x80, // 246
	PROFILER_CHECKPOINT_PONG_LAYER_OPCODE =
		PROFILER_CHECKPOINT_PING_LAYER_OPCODE | 0x80, // 247

	PROFILER_CHECKPOINT_MAX,
};

#ifndef MD5_MODEL_ID_LEN
#define MD5_MODEL_ID_LEN 4 // 16 uint32_t 128bit
#endif

#ifndef PROFILER_OPCODE_MAX
#define PROFILER_OPCODE_MAX 32
#endif

/// cmcu 性能采集数据参数
typedef struct s_profiler_cmcu_section {
	uint8_t type; ///< timeline采集点数据类型 e_profiler_checkpoint_type
	uint8_t rsvd; ///< 4 bytes 对齐
	uint16_t length; ///< 数据长度
	uint32_t current_cycle_count; ///< 计时时刻; 使用core cycle count
	uint32_t last_cycle_count_after_checkpoint; ///< 检查点函数调用后采集的时间
	uint32_t sn; ///< 采集数据的序号, 自动增加1
	uint32_t process_vid; ///< 当前执行任务的进程vid
	uint32_t core_timestamp; ///< WDT clock cycle num,250us 定时器

	/// 参数联合体
	union u_param_cmcu {
		/// model 参数
		struct {
			uint32_t model_name;
			uint32_t model_id[MD5_MODEL_ID_LEN];
			uint32_t layer_count;
			uint16_t batch_size;
			uint16_t vdsp_id;
		} __attribute__((packed)) cmcu_model;
		/// layer 参数
		struct {
			uint32_t model_name;
			uint32_t model_id[MD5_MODEL_ID_LEN];
			uint32_t real_layer_id;
		} __attribute__((packed)) cmcu_layer;

		/// opcode
		struct {
			uint32_t real_layer_id;
			uint8_t opcode_num;
			uint16_t opcode_duty;
			uint8_t opcoce_data_length;
			uint8_t opcode_data[PROFILER_OPCODE_MAX];
		} __attribute__((packed)) cmcu_opcode;

	} param;
	uint32_t magic_number; ///< magic number
} __attribute__((packed)) profiler_cmcu_section_t;

/**
时间说明:
  ...  last_section_end|  ckp_occupy0  |section_begin|  ckp_occupy1  |section_end|  ckp_occupy2  |next_section_begin ...

    section_end.last_cycle_count_after_checkpoint 是section真正开始的时间; 对应图像上section的begin
    section_end.last_cycle_count_after_checkpoint - section_begin.current_cycle_count
                                                  是checkpoint 函数占用的时间; 对应ckp_occupy1
    next_section_begin.current_cycle_count - section_end.current_cycle_count 是ckp_occupy2 时间
    section_begin 和 section_end 之间, 包含了checkpoint函数时间, 图像上需要去除.
 */

/// vdsp 性能数据采集参数
typedef struct s_profiler_vdsp_section {
	uint8_t type; ///< timeline采集点数据类型 e_profiler_checkpoint_type
	uint8_t rsvd; ///< 4 bytes 对齐
	uint16_t length; ///< 数据长度
	uint32_t current_cycle_count; ///< 计时时刻; 使用core cycle count
	uint32_t last_cycle_count_after_checkpoint; ///< 检查点函数调用后采集的时间
	uint32_t sn; ///< 采集数据的序号, 自动增加1
	uint32_t process_vid; ///< 当前执行任务的进程vid
	uint32_t core_timestamp; ///< WDT clock cycle num,250us 定时器

	union u_param_vdsp {
		struct {
			uint32_t data_id;
			uint32_t stream_id;
			uint32_t msg_ringbuf_index;
		} __attribute__((packed)) host_to_vdsp;
		struct {
			uint32_t data_id;
			uint32_t stream_id;
			uint32_t context_id;
			uint32_t msg_ringbuf_index;
		} __attribute__((packed)) vdsp_to_host;
		struct {
			uint32_t data_id;
			uint32_t stream_id;
			uint32_t context_id;
			uint32_t op_id;
			uint32_t loop_count;
		} __attribute__((packed)) opcode;
		struct {
			uint32_t data_id;
			uint32_t stream_id;
			uint32_t context_id;
			uint32_t msg_ringbuf_index;
			uint32_t loop_index;
		} __attribute__((packed)) vdsp_to_cmcu;
		struct {
			uint32_t data_id;
			uint32_t stream_id;
			uint32_t context_id;
			uint32_t msg_ringbuf_index;
			uint32_t loop_index;
		} __attribute__((packed)) cmcu_to_vdsp;
	} param;
	uint32_t magic_number; ///< magic number
} profiler_vdsp_section_t;

/// Driver timeline:
/// dma:4个：Driver接收到DMA请求的时间、Driver通知用户DMA完成的时间、DMA的实际开始执行时间、DMA实际执行结束时间，对应给出.
/// 时间段
typedef struct s_profiler_dma_checkpoint {
	pid_t pid;
	u32 dev_id;
	u32 die_id;
	u32 die_index;
	u64 time_of_driver_recv_dma_req; // host. 1970.1.1 ns begin
	u64 time_of_host_trigger_dma; // host 2
	u32 time_of_dma_run; // smcu 3
	u32 time_of_dma_finish; // smcu 4
	u64 time_of_driver_notify_host_dma_done; // host 5 ; end
	u64 dma_axi_addr;
	u32 dma_dir;
	u32 dma_data_size;
} profiler_dma_checkpoint_t;

/// Driver timeline: Msg: 2个：FW收到Msg的时间，Dev发起 out_desc的时间, 时间点
typedef struct s_profiler_msg_checkpoint {
	pid_t pid;
	u32 dev_id;
	u32 die_id;
	u32 die_index;
	u64 time_of_host_send_msg; // 1970.1.1 ns begin
	u64 time_of_fw_recv_msg; // device_send_out_desc 有点困难, 暂时不做. 1970.1.1 ns end
} profiler_msg_checkpoint_t;

/// 保存pcie dma 采集数据的ringbuf
#define PROFILER_PCI_DMA_FIFO_LENGTH (100 * 1024 * 1024)
/// 保存pcie msg 采集数据的ringbuf
#define PROFILER_PCI_MSG_FIFO_LENGTH (100 * 1024 * 1024)

extern void *gp_profiler_dma_fifo;
extern void *gp_profiler_msg_fifo;

//****************************************************************************
// profiler checkpoint end
//****************************************************************************

//****************************************************************************
// core time sync start
//****************************************************************************

/// core time sync 命令码
enum e_core_time_sync_msg_type {
	CORE_TIME_SYNC_MSG_STOP = 0, ///< 停止时钟同步
	CORE_TIME_SYNC_MSG_START = 1, ///< 开始时钟同步
	CORE_TIME_SYNC_MSG_READ = 2, ///< 读取csram时钟同步数据
};

// 保存各个core的time , 各个core收到同步中断时, 写入.
typedef struct core_time_sync_s {
	unsigned int
		perfmon_data_index; // 读取oak6的perfmon PERFMON_DATA_INDEX 寄存器
	unsigned int cycle_count; // 读取当前core的cycle count: XT_RSR_CCOUNT
	unsigned int core_freq; // 由smcu写入
	unsigned int core_timestamp; // WDT clock cycle num,250us 定时器
} core_time_sync_t;

/// 保存core time sync 数据地址, 数据 core_time_sync_t . 每个core一个.
#define INTER_CORE_TIME_SYNC_ADDR (0x08cf3e00) // TODO: 临时使用, 需要申请

typedef struct core_time_sync_data_s {
	core_time_sync_t core_time[CORE_NUMBER];
	u64 host_time_ns[CORE_NUMBER];
} core_time_sync_data_t;

extern struct mutex pcie_timeline_dma_mutex;
extern struct mutex pcie_timeline_msg_mutex;

//****************************************************************************
// core time sync end
//****************************************************************************

struct bmcu_xspi_ver {
	s32 buf_size;
	union die_index_data die_index;
	char bmcu_active[32];
	char bmcu_backup[32];
	s32 die_num;
	struct {
		char xspi_ver[32];
	} dies[0];
};

struct sriov_number_s {
	int dev_id;
	int sriov_num;
};

struct flash_bmcu_xspi {
	union die_index_data
		die_index; //If you flash bmcu, pls check this die_index is master die
	s32 buf_size;
	char buf[0];
};

struct flash_state {
	union die_index_data die_index;
	u32 flash_type; // 0: bmcu; 1: xspi
	s32 flash_state; // 0: doing; 1: done; other: errno;
};

//-------------------------------------------------------
//v2 version

/// 版本号字符串长度
#define VASTAI_VER_LEN (64)

#define VASTAI_BUF_SIZE_32 (32)
#define VASTAI_DIE_MAX_NUM (4)

/// 固件版本号
typedef struct vatools_fw_version_s {
	unsigned char ver[VASTAI_VER_LEN]; ///< 产品版本号
	unsigned char fw_ver[VASTAI_VER_LEN]; ///< 固件版本号
	unsigned char smcu_ver[VASTAI_VER_LEN]; ///< SMCU版本号
	unsigned char vdmcu_ver[VASTAI_VER_LEN]; ///< VDMCU版本号
	unsigned char vemcu_ver[VASTAI_VER_LEN]; ///< VEMCU版本号
	unsigned char vdsp_ver[VASTAI_VER_LEN]; ///< VDSP版本号
	unsigned char cmcu_ver[VASTAI_VER_LEN]; ///< CMCU版本号
	unsigned char lmcu_ver[VASTAI_VER_LEN]; ///< LMCU版本号
	unsigned char odsp_ver[VASTAI_VER_LEN]; ///< ODSP版本号
	unsigned char driver_ver[VASTAI_VER_LEN]; ///< 驱动版本号
} vatools_fw_version_t;

/// die 索引结构
typedef union vatools_die_index_data_u {
	u32 val; ///< device /die 标识
	struct {
		u32 die_id : 8; ///< die ID
		u32 dev_id : 16; ///< device ID
		u32 seq_num : 8; ///< die 序列号
	};
} vatools_die_index_data_t;

typedef union major_minor_u {
	u32 val; ///< 主次版本号值
	struct {
		u16 minor; ///< 次版本号
		u16 major; ///< 主版本号
	};
} major_minor_t; ///< 主次版本号

typedef struct node_base_info_s {
	char name[VASTAI_BUF_SIZE_32];
	union major_minor_u majormin;
} node_base_info_t;

typedef struct vatools_die_info_s {
	/// die 标识
	union vatools_die_index_data_u die_index;
	/// vacc 名字
	struct node_base_info_s vacc;
	/// render 名字
	struct node_base_info_s render;
	/// video 名称
	struct node_base_info_s vastai_video;
	/// bbox type
	char bbox_type[VASTAI_BUF_SIZE_32];
} vatools_die_info_t;

struct card_info {
	int cnt;
	char buf[0];
};

union pci_id {
	u32 val; ///< PCI ID 值
	struct {
		u16 vender_id; ///< vender ID
		u16 device_id; ///< 设备ID
	};
};

union pci_sub_id {
	u32 val; ///< PCI SUB ID 值
	struct {
		u16 sub_vender_id; ///< SUB vender ID
		u16 sub_device_id; ///< SUB device ID
	};
};

struct ai_video_cap {
	int ai_cap; ///< AI 能力集
	int video_cap; ///< VIDEO 能力集
};

// bar size check
typedef struct vatools_bar_address_s {
	u64 begin;
	u64 end;
} vatools_bar_address_t;

typedef struct vatools_dev_vfpf_flag_s {
	u32 is_physfn;
	u32 is_virtfn;
} vatools_dev_vfpf_flag_t;
typedef struct vatools_dev_info_s {
	int dev_id; ///< 设备ID
	char unid[VASTAI_BUF_SIZE_32]; ///< SN号
	char pcie_slot[VASTAI_BUF_SIZE_32]; ///< PCI slot. 格式 0000:01:00.00
	char card_type[VASTAI_BUF_SIZE_32]; ///< 卡类型
	char bbox_mode[VASTAI_BUF_SIZE_32]; ///< 卡bbox类型
	/// PCI ID 信息
	union pci_id pciid; ///< PCI ID 信息
	union pci_sub_id pcisubid;
	struct node_base_info_s kchar; ///< 主次版本号
	struct node_base_info_s vatools; ///< 主次版本号
	struct node_base_info_s vastai_ctrl; ///< 主次版本号
	struct node_base_info_s vastai_version; ///< 主次版本号
	struct ai_video_cap cap; ///< AI VIDEO 能力集
	struct vatools_bar_address_s bar_address[4][2]; ///< bar size
	int die_num; ///< die 数目
	vatools_die_info_t die_info[VASTAI_DIE_MAX_NUM]; ///< die信息
} vatools_dev_info_t;

/// 设备性能数据
struct vatools_performance_info {
	/// PCI 性能数据
	struct {
		u32 pcie_tx_vdsp_count[4]; ///< PCI 发送到 VDSP 的数据数量
		u32 pcie_rx_vdsp_count[4]; ///< PCI 接收到 VDSP 的数据数量
		u64 pcie_bandwidth_upstream; ///< PCI 上行带宽. 设备到主机.
		u64 pcie_bandwidth_downstream; ///< PCI 下行带宽. 主机到设备.
	} pcie_performance;
	/// CSRAM 信息
	u8 csram_info[0];
};

/// 设备ECC信息
struct vatools_ecc_entry {
	u32 addr_1; ///< 地址低位
	u32 addr_h; ///< 地址高位
	u32 axi_id; ///< axi ID
	u32 axi_data_l; ///< axi数据低位
	u32 axi_data_h; ///< axi 数据高位
	u32 errbit; ///< err bit
};

/// 设备ECC表
struct vatools_ecc_table {
	u32 entry_count; ///< entry 数目
	u32 rsvd_0; ///< 保留
	u32 rsvd_1; ///< 保留
	u32 rsvd_2; ///< 保留
	struct vatools_ecc_entry entry[0]; ///< ecc entry 数据
};

typedef struct smi_cmd_header_s {
	u32 unit; ///< 命令操作的指定单元，从0开始，如果命令不需要解析，填0，如果是全部，填0xFFFF, 此字段也可以做命令扩展
	u16 data_len; ///< 指示发生给fw的数据或者fw返回的数据字节长度，以byte为单位
	u8 flag; ///< 指示fw是否已经准备好数据， 0x01U，数据准备好，
		///<  0x00U数据未准备好, 应用层发送命令时填写0x00U
	u8 cmd; ///< 具体命令
	u64 rw_addr; ///< 读写地址
} smi_cmd_header_t; ///< SMI命令格式, 共 64 bit

typedef struct smi_cmd_s {
	smi_cmd_header_t header; ///< smi 命令头
	u8 data[SMI_CMD_BLOCK1_LEN -
		sizeof(smi_cmd_header_t)]; ///< 数据区; 取所有block的最大值; 1K bytes
} smi_cmd_t; ///< smi cmd, 包括头部和数据区

typedef struct smi_cmd_req_s {
	u32 block_id; ///< smi block id
	u32 wait_ack_timeout; ///< smi命令等待ack超时时间, 单位: 毫秒ms; 0 不使用 wait_ack, 直接返回
	smi_cmd_t req; ///< smi req 请求命令
	smi_cmd_t rsp; ///< smi rsp 返回数据
} smi_arg_t; ///< ioctl 传递的 smi 参数

//-----------------------------------------------------------------------------
// host 使用 ioctl 和驱动进行数据交互传输
//-----------------------------------------------------------------------------
/// ioctl arg header
typedef struct vatools_ioctl_arg_header_s {
	u32 type;
	vatools_die_index_data_t die_index;
	u64 address;
	u32 length;
	int err;
} vatools_ioctl_arg_header_t;

/// ioctl arg
typedef struct vatools_ioctl_arg_s {
	vatools_ioctl_arg_header_t header;
	union {
		u8 body[0];
		vatools_dev_info_t dev_info[0];
		smi_arg_t smi;
		void *buffer;
	};
} vatools_ioctl_arg_t;

/// STORE ADDR OF HEARTBEAT COUNT
#define CORE_HEARTBEAT_ADDR 0x08cf3020
#define CORE_HEARTBEAT_LEN  (120)

typedef struct core_heartbeat_s {
	u32 smcu_thread_hb; // from smcu while(1)
	u32 smcu_timer_hb; // from smcu interrutp
	u32 cmcu_hb;
	u32 lmcu0_hb;
	u32 lmcu1_hb;
	u32 lmcu2_hb;
	u32 lmcu3_hb;
	u32 lmcu4_hb;
	u32 lmcu5_hb;
	u32 lmcu6_hb;
	u32 lmcu7_hb;
	u32 odsp0_hb;
	u32 odsp1_hb;
	u32 odsp2_hb;
	u32 odsp3_hb;
	u32 odsp4_hb;
	u32 odsp5_hb;
	u32 odsp6_hb;
	u32 odsp7_hb;
	u32 vdmcu0_hb;
	u32 vdmcu1_hb;
	u32 vdmcu2_hb;
	u32 vemcu0_hb;
	u32 vemcu1_hb;
	u32 vemcu2_hb;
	u32 vemcu3_hb;
	u32 vdsp0_hb;
	u32 vdsp1_hb;
	u32 vdsp2_hb;
	u32 vdsp3_hb;
} core_heartbeat_t;

#ifdef CONFIG_TOOLS_V2
// ================================================================
// ================================================================
// ================================================================
// ================================================================
// ================================================================

#define VATOOLS_MAGIC_CODE		0xA283419D
#define VATOOLS_DRV_VERSION_MAJOR_CODE	1
#define VATOOLS_DRV_VERSION_MIDDLE_CODE 5
#define VATOOLS_DRV_VERSION_MINOR_CODE	1

// V2 get_device_info
typedef struct vatools_v2_magic_code_s {
	// 32*u32 header
	u32 magic_number;
	u32 security[4];
	u32 tools_ver_major;
	u32 tools_ver_middle;
	u32 tools_ver_minor;
	u32 rsvd[25];
} vatools_v2_magic_code_t;

// V2 get_device_info
typedef struct vatools_v2_die_info_s {
	// 32*u32 header
	u32 rsvd[32];
	/// die 标识
	vatools_die_index_data_t die_index;
	/// vacc 名字
	struct node_base_info_s vacc;
	/// render 名字
	struct node_base_info_s render;
	/// video 名称
	struct node_base_info_s vastai_video;
	/// bbox type
	char bbox_type[VASTAI_BUF_SIZE_32];
	char spi_buf[SMI_CMD_BLOCK0_DATA_LEN]; ///< SPI 数据
	struct vastai_fw_version fw_ver;
	u32 pcie_nls; /* negotiated_link_speed */
	u32 pcie_nlw; /* negotiated_link_width */
	u32 res[510];
} vatools_v2_die_info_t;

typedef struct vatools_device_info_s {
	// 32*u32 header
	u32 rsvd[32];
	// device info
	int dev_id; ///< 设备ID
	char unid[VASTAI_BUF_SIZE_32]; ///< SN号
	char pcie_slot[VASTAI_BUF_SIZE_32]; ///< PCI slot. 格式 0000:01:00.00
	char card_type[VASTAI_BUF_SIZE_32]; ///< 卡类型
	char bbox_mode[VASTAI_BUF_SIZE_32]; ///< 卡bbox模式
	char bbox_type[VASTAI_BUF_SIZE_32]; ///< 卡bbox类型
	/// PCI ID 信息
	union pci_id pciid; ///< PCI ID 信息
	union pci_sub_id pcisubid;
	struct node_base_info_s kchar; ///< 主次版本号
	struct node_base_info_s vatools; ///< 主次版本号
	struct node_base_info_s vastai_ctrl; ///< 主次版本号
	struct node_base_info_s vastai_version; ///< 主次版本号
	struct ai_video_cap cap; ///< AI VIDEO 能力集
	struct vatools_bar_address_s bar_address[4][2]; ///< bar size
	int die_num; ///< die 数目
	vatools_v2_die_info_t die_info[VASTAI_DIE_MAX_NUM]; ///< die信息
	vatools_dev_vfpf_flag_t dev_vfpf_flag;
	u64 smcu_access_ddr_address_limit[2]; ///< smcu 可以访问的地址区间
	struct sriov_number_s sriov_number;
	u32 res[512];
} vatools_v2_device_info_t;

typedef struct vatools_devices_info_s {
	vatools_v2_magic_code_t vatools_v2_version_control;
	u32 device_number;
	vatools_v2_device_info_t *vatools_v2_device_info_p;
} vatools_devices_info_t;

struct vatools_node {
	struct miscdevice misc;
	wait_queue_head_t wq;
	struct mutex node_mutex;
	// lupa add 2021-12-23 begin 整个业务的资源锁。保障处理该业务时，资源独享
	struct mutex node_sharedmem_mutex;
	struct mutex node_common_mutex;
	struct mutex node_profiler_mutex;
	struct mutex node_debugger_mutex;
	struct mutex node_logger_mutex;
	struct mutex node_smi_mutex;
	struct mutex node_dummy_mutex;
	struct mutex node_driver_mutex;
	struct mutex node_reader_mutex;

	// lupa add 2021-12-23 end 整个业务的资源锁。保障处理该业务时，资源独享
	struct list_head list_nodes;
	void *pci_dev;
	unsigned char *node_buffer;
	unsigned int size;
	unsigned int rsv; //适配arm服务器 需要8字节对齐
	struct list_head list_node_readers;
	struct list_head list_node_process_mutex;

	// pcie dma / msg ringbuf, for profiler timeline
	void *profiler_dma_ringbuf;
	void *profiler_msg_ringbuf;

	// core event ringbuf
	void *event_ringbuf;
	// core event mutex
	struct mutex event_mutex;
	// 等待 core 的 event; 例如 reset事件
	wait_queue_head_t event_wait_queue;
	vatools_v2_device_info_t *dev_info;
	u32 device_num;
	u32 rsvd; // for arm cpu, 8 byte align
};

struct vatools_reader {
	struct vatools_node *node;
	struct list_head list_readers;
	struct list_head
		malloc_pointers; // TODO: 保存每个 open 的 fd 申请的内存指针, 如果fd关闭时, 循环释放空间
	// sharedmem
	struct list_head t_sharedmem_mem_info_head;
	// struct list_head t_sharedmem_signal_info_head;
	//添加一个信号量用于触发等待事件
	struct completion debug_comp; // 用于记录触发信号使用 lupa add 2021-12-15
	struct fasync_struct *sharedmem_pt_fasync;
	struct mutex sharedmem_mutex;
	int n_sharedmem_init_done; // 标识是否初始化过; 1-完成初始化 0-未初始化
	// reader 通用变量. 暂未使用.
	size_t r_off;
	int r_all;
	int r_ver;
	unsigned int
		profiler_checkpoint_corebitmap; ///< 记录开启profiler checkpoint 的core , 使用 corebitmap记录, 每个core 一个bit

	T_SMI_IOCTL_TRANS_CATEGORY
	trans_category; // 适配arm服务器, 和host交互的结构, 需要注意 8 byte 对齐
	int exit; //是否退出
	// 适配arm服务器, 需要8字节对齐, trans_category 暂不修改, 这里使用保留字段对齐.
	u64 event_timestamp_ns; ///< 标识此时间以及之前的event都已经发送到host线程; 每次发送event到host后, 更新此时间
	u32 ioctl_version; ///< VATOOLS_IOCTL_V2_MAGIC: v2 版本
	int rsvd; // for arm cpu, 8 byte align
};

#define SPI_BLOCK_CODE 0x1 /// <spi flash code 代码
#define SPI_BLOCK_ECC  0x2 ///< spi flash user setting 信息
#define SPI_BLOCK_PRODUCT_INFO                                                 \
	0x4 ///< spi flash productinfo Inventory information 产品信息
#define SPI_BLOCK_MFG_INFO 0x8 ///< spi flash MFG information 信息

/// ===================================================================
/// ===================================================================
/// ===================================================================
/// ===================================================================
/// ===================================================================
// ioctl 通用命令 V2 版本 vvvvvvvvvvvvvvvvvvv
/// ===================================================================
#define VATOOLS_IOCTL_V2_MAGIC 0xA2
/// 测试命令
#define VATOOLS_IOCTL_V2_TEST _IO(VATOOLS_IOCTL_V2_MAGIC, 0x00)
/// 设置打开的文件fd的类别
#define VATOOLS_IOCTL_V2_SET_APP_CATEGORY _IO(VATOOLS_IOCTL_V2_MAGIC, 0x01)
/// 获取device info
#define VATOOLS_IOCTL_V2_GET_DEVICE_INFO _IO(VATOOLS_IOCTL_V2_MAGIC, 0x02)
enum e_ioctl_get_device_info_sub_cmd {
	VATOOLS_IOCTL_DEVICE_INFO_NONE = 0,
	VATOOLS_IOCTL_DEVICE_INFO_REFRESH =
		1, // 不读取缓存, 重新从芯片读取相关数据
};

/// 普通方式读取数据 //TODO: 针对不同地址存取逻辑, 放到common里面?
#define VATOOLS_IOCTL_V2_READ_BUF _IO(VATOOLS_IOCTL_V2_MAGIC, 0x03)
/// 普通方式写入数据
#define VATOOLS_IOCTL_V2_WRITE_BUF _IO(VATOOLS_IOCTL_V2_MAGIC, 0x04)

/// 获取reset event
#define VATOOLS_IOCTL_V2_DEVICE_EVENT _IO(VATOOLS_IOCTL_V2_MAGIC, 0x05)

/// core timestamp sync
#define VATOOLS_IOCTL_V2_CORE_TIME_SYNC _IO(VATOOLS_IOCTL_V2_MAGIC, 0x06)
enum e_ioctl_core_time_sync_sub_cmd {
	VATOOLS_IOCTL_CORE_TIME_SYNC_STOP = 0, ///< 停止时钟同步
	VATOOLS_IOCTL_CORE_TIME_SYNC_START = 1, ///< 开始时钟同步
	VATOOLS_IOCTL_CORE_TIME_SYNC_READ = 2, ///< 读取csram时钟同步数据
};

/// 通过 smi 读写数据, 先写,触发中断到smcu, 后读, 读写通过一次调用完成
#define VATOOLS_IOCTL_V2_SMI_CMD _IO(VATOOLS_IOCTL_V2_MAGIC, 0x07)
/// ioctl smi sub cmd
enum e_ioctl_smi_sub_cmd {
	VATOOLS_IOCTL_SMI_NONE = 0,
	VATOOLS_IOCTL_SMI_READ = 1,
	VATOOLS_IOCTL_SMI_WRITE = 2,
	VATOOLS_IOCTL_SMI_FETCH = 3,
};

/// 通过PCIE处理的命令族
#define VATOOLS_IOCTL_V2_PCIE_CMD _IO(VATOOLS_IOCTL_V2_MAGIC, 0x08)
/// ioctl pcie type/cmd 命令, 不使用smcu. 驱动直接处理
enum e_ioctl_pcie_sub_cmd {
	VATOOLS_IOCTL_PCIE_NONE = 0,
	VATOOLS_IOCTL_PCIE_BANDWIDTH = 1,
	VATOOLS_IOCTL_PCIE_RESET = 2,
	VATOOLS_IOCTL_PCIE_REBOOT = 3,
	VATOOLS_IOCTL_PCIE_STRING = 4,
	VATOOLS_IOCTL_PCIE_FW_VERSION = 5,
	VATOOLS_IOCTL_PCIE_DDR_ECC_1BIT = 6,
	VATOOLS_IOCTL_PCIE_DDR_ECC_2BIT = 7,
	VATOOLS_IOCTL_PCIE_VDSP_COUNT = 8,
	VATOOLS_IOCTL_PCIE_AI_COUNT = 9, // 512bytes  PERFORMANCE_AI_BASE
	VATOOLS_IOCTL_PCIE_RING_BUF_STATISTIC = 10,
	VATOOLS_IOCTL_PCIE_SET_VIDEO_MULTI_CORE =
		11, ///< 设置video使用multicore; 0: 不使用; 1: 使用
	VATOOLS_IOCTL_PCIE_GET_VIDEO_MULTI_CORE =
		12, ///< 读取video multicore 的mode和job count
	VATOOLS_IOCTL_PCIE_MAX
};

#define VATOOLS_IOCTL_V2_PROFILER _IO(VATOOLS_IOCTL_V2_MAGIC, 0x0a)
enum e_ioctl_profiler_sub_cmd {
	VATOOLS_IOCTL_PROFILER_NONE = 0,
	VATOOLS_IOCTL_PROFILER_PERFMON_READ = 1, ///< 读取 perfmon 寄存器
	VATOOLS_IOCTL_PROFILER_PERFMON_WRITE = 2, ///< 写入 perfmon 寄存器
	// profiler 采集数据的命令码
	VATOOLS_IOCTL_PROFILER_TIMELINE_STOP = 3, ///< 停止采集
	VATOOLS_IOCTL_PROFILER_TIMELINE_START = 4, ///< 开始采集
	VATOOLS_IOCTL_PROFILER_TIMELINE_READ_RPWP =
		5, ///< 读取读写指针. fw 不会收到此命令
	VATOOLS_IOCTL_PROFILER_TIMELINE_WRITE_RPWP =
		6, ///< 写入读写指针. fw 不会收到此命令
	VATOOLS_IOCTL_PROFILER_TIMELINE_READ_DATA =
		7, ///< 读取 DDR 数据. fw 不会收到此命令
	VATOOLS_IOCTL_PROFILER_TIMELINE_READ_CTRL = 8, ///< 读控制
	VATOOLS_IOCTL_PROFILER_TIMELINE_READ_PCIE_DMA =
		9, ///< PCIE dma 数据采集
	VATOOLS_IOCTL_PROFILER_TIMELINE_READ_PCIE_MSG =
		10, ///< PCIE msg 数据采集
	VATOOLS_IOCTL_PROFILER_MAX
};

#define VATOOLS_IOCTL_V2_FLASH _IO(VATOOLS_IOCTL_V2_MAGIC, 0x0b)
enum e_ioctl_flash_sub_cmd {
	VATOOLS_IOCTL_FLASH_NONE = 0,
	VATOOLS_IOCTL_FLASH_BMCU_FW = 1, /// 烧入bmcu fw版本
	VATOOLS_IOCTL_FLASH_XSPI_FW = 2, /// 烧入xspi fw版本
	VATOOLS_IOCTL_FLASH_GET_FLASH_STATUS = 3, /// 获取烧写状态
	VATOOLS_IOCTL_FLASH_SWITCH_BMCU = 4, /// 切换bmcu版本
};

/// 获取 firmware log status
#define VATOOLS_IOCTL_V2_LOG_STATUS _IO(VATOOLS_IOCTL_V2_MAGIC, 0x0c)
/// DMA
#define VATOOLS_IOCTL_V2_DMA _IO(VATOOLS_IOCTL_V2_MAGIC, 0x0d)
enum e_ioctl_dma_sub_cmd {
	VATOOLS_IOCTL_DMA_NONE = 0,
	VATOOLS_IOCTL_DMA_ALLOC = 1, /// 分配 DMA 内存
	VATOOLS_IOCTL_DMA_START = 2, /// 开始 DMA 传输数据
	VATOOLS_IOCTL_DMA_FREE = 3, /// 释放 DMA 内存
};

/// 使用内核mutex实现进程互斥保护
#define VATOOLS_IOCTL_V2_GUARD _IO(VATOOLS_IOCTL_V2_MAGIC, 0x0f)
enum e_ioctl_guard_sub_cmd {
	VATOOLS_IOCTL_GUARD_NONE = 0,
	VATOOLS_IOCTL_GUARD_LOCK = 1, /// 加锁
	VATOOLS_IOCTL_GUARD_UNLOCK = 2, /// 解锁
	VATOOLS_IOCTL_GUARD_TRYLOCK = 3, /// trylock
	VATOOLS_IOCTL_GUARDU_IS_LOCKED = 4, /// is locked
};
#define VATOOLS_IOCTL_GUARD_NAME_LENGTH (256)

/// 共享内存
#define VATOOLS_IOCTL_V2_SHARE_MEM _IO(VATOOLS_IOCTL_V2_MAGIC, 0x10)
enum e_ioctl_share_mem_sub_cmd {
	VATOOLS_IOCTL_SHARE_MEM_NONE = 0,
	VATOOLS_IOCTL_SHARE_MEM_CREATE = 1, /// 创建共享内存信息
	VATOOLS_IOCTL_SHARE_MEM_DELETE = 2, /// 删除共享内存信息
	VATOOLS_IOCTL_SHARE_MEM_READ = 3, /// 获取共享内存信息
	VATOOLS_IOCTL_SHARE_MEM_WRITE = 4, /// 写入共享内存信息
	VATOOLS_IOCTL_SHARE_MEM_POLL = 5, /// 等待信息开始
};

/// video debug. 和共享内存共用子命令
#define VATOOLS_IOCTL_V2_VIDEO_DEBUGGER _IO(VATOOLS_IOCTL_V2_MAGIC, 0x11)
enum e_ioctl_video_debugger_sub_cmd {
	VATOOLS_IOCTL_VIDEO_DEBUGGER_NONE = 0,
	VATOOLS_IOCTL_VIDEO_DEBUGGER_START = 11, /// 开始video debugger信息
	VATOOLS_IOCTL_VIDEO_DEBUGGER_STOP = 12, /// 停止video  debugger信息
	VATOOLS_IOCTL_VIDEO_DEBUGGER_READ = 13, /// 读取video  debugger信息
	VATOOLS_IOCTL_VIDEO_DEBUGGER_WRITE = 14, /// 写入video debugger信息
	VATOOLS_IOCTL_VIDEO_DEBUGGER_FUNCTION_CREATE = 15, /// 附加回调函数
	VATOOLS_IOCTL_VIDEO_DEBUGGER_FUNCTION_DELETE = 16, /// 删除回调函数
};

/// ai debug
#define VATOOLS_IOCTL_V2_AI_DEBUGGER _IO(VATOOLS_IOCTL_V2_MAGIC, 0x12)
enum e_ioctl_ai_debugger_sub_cmd {
	VATOOLS_IOCTL_AI_DEBUGGER_NONE = 0,
	VATOOLS_IOCTL_AI_DEBUGGER_CONFIG = 1, /// 配置调试工具命令
	VATOOLS_IOCTL_AI_DEBUGGER_SET = 2, /// 设置调试工具命令
	VATOOLS_IOCTL_AI_DEBUGGER_GET = 3, /// 读取调试工具命令
	VATOOLS_IOCTL_AI_DEBUGGER_FETCH = 4, /// 获取调die_index试工具命令
};

/// 命令作用域
enum e_ioctl_cmd_domain {
	VATOOLS_IOCTL_DOMAIN_NONE = 0, /// 无作用域信息
	VATOOLS_IOCTL_DOMAIN_DEVICE = 1 << 0, /// 卡级命令
	VATOOLS_IOCTL_DOMAIN_DIE = 1 << 1, /// die 级命令
	VATOOLS_IOCTL_DOMAIN_DRIVER = 1 << 2, /// driver 级命令, 不下发到 SMI
	VATOOLS_IOCTL_DOMAIN_MAX = 1 << 3, ///
};

typedef struct buf_ptr_s {
	u32 buf_size;
	void *buf_addr;
} buf_ptr_t;

// host 使用 ioctl 和驱动进行数据交互传输
typedef struct ioctl_arg_s {
	u32 sub_cmd; // 根据ioctl 命令的不同, 解析为对应命令的子命令
	union die_index_data die_index; //die index信息
	u32 block_id; // SMI 命令使用的block ID, 每个blockId有自己的csram 区域
	u32 lock_domain; // 命令作用域, 用来做加锁判断
	long errcode; //调用内核ioctl返回值
	u64 address; // 通用数据地址参数
	u64 value; // 通用数据值参数
	u32 count; // 通用计数参数
	buf_ptr_t input_buf; // host侧要写入 fw 的数据buffer. 数据: host->fw
	buf_ptr_t output_buf; // host侧保存 fw 返回的数据buffer. 数据: fw->host

#ifdef WINDOWS // for win DeviceIoControl
	LPDWORD lpBytesReturned;
	LPOVERLAPPED lpOverlapped;
#else
	u64 rsvd1; // 用来和 win 驱动字节对齐
	u64 rsvd2; // 用来和 win 驱动字节对齐
#endif
	u32 rsvd[8];
} ioctl_arg_t;

/// dma 内存分配命令
typedef struct dma_alloc_cmd_s {
	HANDLE dma_buf_fd;
	UINT64 hostva;
	u32 size;
	u32 die_index;
} dma_alloc_cmd_t;

/// dma 启动命令
typedef struct dma_start_cmd_s {
	u32 is_dev_to_host;
	HANDLE dma_buf_fd;
	u64 axi_addr;
	u32 size;
	u32 die_index;
} dma_start_cmd_t;

// 用户空间内存直接传递到内核, 使用前无需 dma alloc 内存
typedef struct dma_trans_cmd_s {
	u32 is_dev_to_host;
	void *vir_addr;
	u64 axi_addr;
	u32 length;
	u32 die_index;
	int pid;
} dma_trans_cmd_t;

/// ===================================================================
// ioctl 通用命令 V2 版本 ^^^^^^^^^^^^^
/// ===================================================================
/// ===================================================================
/// ===================================================================
/// ===================================================================
/// ===================================================================
#endif

#pragma pack()

#endif //__VATOOLS_DEFINE_H__
