/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: frank.bai
 */
#include "vatools.h"
#include "vatools_log.h"

/******************************************************************************
 * dummy fops
 *
 *
 */
loff_t dummy_llseek(struct file *filp, loff_t offset, int orig)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(offset);
	V_UNREFERENCE(orig);

	return 0;
}

ssize_t dummy_read(struct file *filp, char __user *buf, size_t size,
		   loff_t *pos)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(buf);
	V_UNREFERENCE(size);
	V_UNREFERENCE(pos);

	return 0;
}

ssize_t dummy_write(struct file *filp, const char __user *buf, size_t size,
		    loff_t *pos)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(buf);
	V_UNREFERENCE(size);
	V_UNREFERENCE(pos);

	return 0;
}

ssize_t dummy_write_iter(struct kiocb *iocb, struct iov_iter *from)
{
	V_UNREFERENCE(iocb);
	V_UNREFERENCE(from);

	return 0;
}

unsigned int dummy_poll(struct file *filp, struct poll_table_struct *wait)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(wait);

	return 0;
}

long dummy_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	long ret = -EIO;
	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	V_UNREFERENCE(arg);
	return ret;
}

int dummy_mmap(struct file *filp, struct vm_area_struct *pvm)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(pvm);

	return 0;
}
int dummy_open(struct inode *inode, struct file *filp)
{
	V_UNREFERENCE(inode);
	V_UNREFERENCE(filp);

	return 0;
}

int dummy_release(struct inode *ignored, struct file *filp)
{
	V_UNREFERENCE(ignored);
	V_UNREFERENCE(filp);

	return 0;
}

int dummy_fasync(int fd, struct file *filp, int mode)
{
	V_UNREFERENCE(fd);
	V_UNREFERENCE(filp);
	V_UNREFERENCE(mode);

	return 0;
}

#ifdef CONFIG_TOOLS_V2

long vatools_ioctl_v2_get_device_info(struct file *filp, unsigned int cmd,
				      unsigned long arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	struct vatools_node *node = vatools_file_get_node(filp);
	ioctl_arg_t *ioctl_arg_p;
	vatools_v2_device_info_t *dev_info_p = NULL;
	struct vastai_cdev *vacc_dev_p = NULL;
	struct vastai_cdev *vacc_dev_temp = NULL;
	struct vastai_render_file *render_file_p = NULL;
	struct vastai_render_file *render_file_temp = NULL;
#ifdef CONFIG_DRM_DRIVER
	struct vastai_device_handle *device_handle;
#endif
	struct vastai_pci_info *pci_info = NULL;
	u32 device_num = 0;
	int dev = 0;
	int die = 0;
	void *buffer = NULL;
	int buffer_size = 0;
	char spi_buf[SMI_CMD_BLOCK0_DATA_LEN]; ///< 卡类型
	vatools_devices_info_t devices_info;

	VATOOLS_FUNC_ENTERY;

	domain_mutex_lock(NULL, 0, VATOOLS_IOCTL_DOMAIN_DRIVER);

	ioctl_arg_p = kvmalloc(sizeof(ioctl_arg_t), GFP_KERNEL);
	if (ioctl_arg_p == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "kvmalloc error. size=%ld",
			    sizeof(ioctl_arg_t));
		ret = -ENOMEM;
		goto out;
	}

	ret = copy_from_user(ioctl_arg_p, argp, sizeof(ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out_free_ioctl_arg;
	}

	memset(&devices_info, 0, sizeof(vatools_devices_info_t));
	ret = copy_from_user(&devices_info, ioctl_arg_p->output_buf.buf_addr,
			     sizeof(vatools_devices_info_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out_free_ioctl_arg;
	}

	// 获取 device 数量
	device_num = vatools_get_vastai_pci_device_number();

	// 拷贝版本号
	devices_info.vatools_v2_version_control.magic_number =
		VATOOLS_MAGIC_CODE;
	devices_info.vatools_v2_version_control.tools_ver_major =
		VATOOLS_DRV_VERSION_MAJOR_CODE;
	devices_info.vatools_v2_version_control.tools_ver_middle =
		VATOOLS_DRV_VERSION_MIDDLE_CODE;
	devices_info.vatools_v2_version_control.tools_ver_minor =
		VATOOLS_DRV_VERSION_MINOR_CODE;
	devices_info.device_number = device_num;

	ret = copy_to_user(ioctl_arg_p->output_buf.buf_addr, &devices_info,
			   sizeof(vatools_devices_info_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld", ret);
		ret = -EFAULT;
		goto out_free_ioctl_arg;
	}

	if (VATOOLS_IOCTL_DEVICE_INFO_REFRESH == ioctl_arg_p->sub_cmd) {
		// 清除缓存, 重新读取数据
		// mutex_lock(&node->node_common_mutex);
		if (node->dev_info) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "remove cache node->dev_info=%p",
				    node->dev_info);
			kvfree(node->dev_info);
			node->dev_info = NULL;
			node->device_num = 0;
		}
		// mutex_unlock(&node->node_common_mutex);
	}

	if (node->dev_info) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "copy from cache node->dev_info=%p",
			    node->dev_info);

		// mutex_lock(&node->node_common_mutex);

		buffer_size =
			node->device_num * sizeof(vatools_v2_device_info_t);
		ioctl_arg_p->errcode = 0;
		ioctl_arg_p->output_buf.buf_size = buffer_size;
		ioctl_arg_p->count = node->device_num;

		if (copy_to_user(argp, ioctl_arg_p, sizeof(ioctl_arg_t))) {
			ret = -EFAULT;
		}
		if (copy_to_user(devices_info.vatools_v2_device_info_p,
				 node->dev_info, buffer_size)) {
			ret = -EFAULT;
		}
		// mutex_unlock(&node->node_common_mutex);
		ret = 0;
		goto out_free_ioctl_arg;
	}

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "get from SoC node->dev_info=%p",
		    node->dev_info);

	if (device_num == 0 || device_num > MAX_DEVICE_NUM_PER_MANAGER) {
		ret = -ENODEV;
		ioctl_arg_p->count = device_num;
		ioctl_arg_p->errcode = ret;
		if (copy_to_user(argp, ioctl_arg_p, sizeof(ioctl_arg_t))) {
			ret = -EFAULT;
		}
		goto out_free_ioctl_arg;
	}

	buffer_size = device_num * sizeof(vatools_v2_device_info_t);

	buffer = kvmalloc(buffer_size, GFP_KERNEL);
	if (buffer == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "kvmalloc error. size=%d",
			    buffer_size);
		ret = -ENOMEM;
		goto out_free_ioctl_arg;
	}

	dev_info_p = (vatools_v2_device_info_t *)buffer;
	memset(buffer, 0, buffer_size);

	for (dev = 0; dev < device_num; dev++) {
		pci_info = get_vastai_pci_device_info(dev);
		if (pci_info == NULL) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "priv is NULL, dev_id[%u] ", dev);
			continue;
		}
		if ((atomic_read(&pci_info->pci_state) !=
		     VASTAI_NORMAL_STATE)) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "priv->pci_state= 0x%x\n",
				    atomic_read(&pci_info->pci_state));
			continue;
		}
		dev_info_p->dev_id = pci_info->dev_id;
		dev_info_p->die_num = pci_info->die_num;
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "dev_id=%d die_num=%d buffer_size=%d",
			    dev_info_p->dev_id, dev_info_p->die_num,
			    buffer_size);

		// bar size
		memcpy(dev_info_p->bar_address, vatools_die_check,
		       sizeof(vatools_die_check));

		dev_info_p->pciid.vender_id = pci_info->dev->vendor;
		dev_info_p->pciid.device_id = pci_info->dev->device;
		dev_info_p->pcisubid.sub_vender_id =
			pci_info->dev->subsystem_vendor;
		dev_info_p->pcisubid.sub_device_id =
			pci_info->dev->subsystem_device;

		memcpy(dev_info_p->pcie_slot, pci_name(pci_info->dev),
		       VASTAI_BUF_SIZE_32);

		dev_info_p->kchar.majormin.major =
			MAJOR(pci_info->p_char->file_info.dev_num);
		dev_info_p->kchar.majormin.minor =
			MINOR(pci_info->p_char->file_info.dev_num);

		memcpy(dev_info_p->kchar.name,
		       pci_info->p_char->file_info.file_name,
		       VASTAI_BUF_SIZE_32);

		dev_info_p->vastai_version.majormin.major =
			MAJOR(pci_info->version_file->file_info.dev_num);
		dev_info_p->vastai_version.majormin.minor =
			MINOR(pci_info->version_file->file_info.dev_num);
		memcpy(dev_info_p->vastai_version.name,
		       pci_info->version_file->file_info.file_name,
		       VASTAI_BUF_SIZE_32);

		dev_info_p->vastai_ctrl.majormin.major =
			MAJOR(pci_info->ctl_file->file_info.dev_num);
		dev_info_p->vastai_ctrl.majormin.minor =
			MINOR(pci_info->ctl_file->file_info.dev_num);

		memcpy(dev_info_p->vastai_ctrl.name,
		       pci_info->ctl_file->file_info.file_name,
		       VASTAI_BUF_SIZE_32);

		dev_info_p->vatools.majormin.minor = node->misc.minor;
		dev_info_p->vatools.majormin.major = 10;
		memcpy(dev_info_p->vatools.name, node->misc.name,
		       VASTAI_BUF_SIZE_32);

		dev_info_p->cap.ai_cap =
			dev_info_p->cap.ai_cap; ///< AI VIDEO 能力集
		dev_info_p->cap.video_cap =
			dev_info_p->cap.video_cap; ///< AI VIDEO 能力集

		// 虚拟化
		dev_info_p->dev_vfpf_flag.is_physfn = pci_info->is_physfn;
		dev_info_p->dev_vfpf_flag.is_virtfn = pci_info->is_virtfn;

		for (die = 0; die < dev_info_p->die_num; die++) {
			dev_info_p->die_info[die].die_index.val =
				pci_info->dies[die].die_index;
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "dies[%d] dev_info_p->die_index=%x", die,
				    dev_info_p->die_info[die].die_index.val);

			// get spi config of die
			memset(spi_buf, 0, SMI_CMD_BLOCK0_DATA_LEN);
			vatools_get_spi_buf(pci_info->dies[die].die_index,
					    SPI_BLOCK_PRODUCT_INFO, 0,
					    SMI_CMD_BLOCK0_DATA_LEN, spi_buf);
			memcpy(dev_info_p->die_info[die].spi_buf, spi_buf,
			       SMI_CMD_BLOCK0_DATA_LEN);

			// get fw version of die
			memcpy(&dev_info_p->die_info[die].fw_ver,
			       &(pci_info->dies[die].fw_ver),
			       sizeof(struct vastai_fw_version));

			// get pcie_nls, pcie_nlw
			dev_info_p->die_info[die].pcie_nls =
				pci_info->dies[die].pcie_nls;
			dev_info_p->die_info[die].pcie_nlw =
				pci_info->dies[die].pcie_nlw;

			// vastai_video
			list_for_each_entry_safe (render_file_p,
						  render_file_temp,
						  &pci_info->render_head,
						  node) {
				if (render_file_p->die_index ==
				    pci_info->dies[die].die_index) {
					dev_info_p->die_info[die]
						.vastai_video.majormin.major =
						MAJOR(render_file_p->file_info
							      .dev_num);
					dev_info_p->die_info[die]
						.vastai_video.majormin.minor =
						MINOR(render_file_p->file_info
							      .dev_num);

					snprintf(dev_info_p->die_info[die]
							 .vastai_video.name,
						 VASTAI_BUF_SIZE_32,
						 "vastai_video%d",
						 dev_info_p->die_info[die]
							 .die_index.seq_num);
					break;
				}
			}

#ifdef CONFIG_DRM_DRIVER
			// renderD
			list_for_each_entry_safe (render_file_p,
						  render_file_temp,
						  &pci_info->render_head,
						  node) {
				if (render_file_p->die_index ==
				    pci_info->dies[die].die_index) {
					device_handle =
						(struct vastai_device_handle
							 *)(render_file_p
								    ->dec_private_data);

					dev_info_p->die_info[die]
						.render.majormin.major =
						226; // drm 设备有系统指定主设备号为 226, 不变.

					dev_info_p->die_info[die]
						.render.majormin.minor =
						device_handle->drm_dev->render
							->index;

					snprintf(dev_info_p->die_info[die]
							 .render.name,
						 VASTAI_BUF_SIZE_32,
						 "renderD%d",
						 pci_info->dies[die].render_id);
					break;
				}
			}
#endif
			// vacc
			list_for_each_entry_safe (vacc_dev_p, vacc_dev_temp,
						  &pci_info->vacc_dev_head,
						  vacc_dev_node) {
				if (vacc_dev_p->die_index ==
				    pci_info->dies[die].die_index) {
					dev_info_p->die_info[die]
						.vacc.majormin.major = MAJOR(
						vacc_dev_p->file_info.dev_num);
					dev_info_p->die_info[die]
						.vacc.majormin.minor = MINOR(
						vacc_dev_p->file_info.dev_num);

					snprintf(dev_info_p->die_info[die]
							 .vacc.name,
						 VASTAI_BUF_SIZE_32, "vacc%d",
						 dev_info_p->die_info[die]
							 .die_index.seq_num);
					break;
				}
			}
		} // end of die

		// 获取虚拟化数据
		memset(&dev_info_p->sriov_number, 0,
		       sizeof(struct sriov_number_s));
		dev_info_p->sriov_number.dev_id = dev;
		dev_info_p->sriov_number.sriov_num =
			vastai_pci_get_sriov_numvfs(
				dev_info_p->sriov_number.dev_id);

		// 循环下一个device
		dev_info_p++;
	}

	node->dev_info = buffer;
	node->device_num = device_num;

	ioctl_arg_p->errcode = 0;
	ioctl_arg_p->count = device_num;
	ioctl_arg_p->output_buf.buf_size = buffer_size;

	if (copy_to_user(argp, ioctl_arg_p, sizeof(ioctl_arg_t))) {
		ret = -EFAULT;
		goto out_free_buffer;
	}

	if (copy_to_user(devices_info.vatools_v2_device_info_p, buffer,
			 buffer_size)) {
		ret = -EFAULT;
		goto out_free_buffer;
	}

	ret = 0;

out_free_buffer:
	// kvfree(buffer);

out_free_ioctl_arg:
	kvfree(ioctl_arg_p);

out:
	domain_mutex_unlock(NULL, 0, VATOOLS_IOCTL_DOMAIN_DRIVER);

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "out: ret=%ld", ret);
	VATOOLS_FUNC_EXIT;
	return ret;
}

#endif
