
#include "vatools.h"
#include "vatools_log.h"


//开启下面的宏, 不使用pcie driver调用, 自己独立生成driver module. 不调用真正的 PCIE driver 函数, 模拟实现
#if VATOOLS_INDEPENDANCE_DRIVER

#define SIMU_SMI   1
#define SIMU_DEBUG 0

#if SIMU_SMI
static TS_SMI_CMD_REQ g_smi_cmd;
#endif

#if SIMU_DEBUG
static t_break_req g_break_req;
static t_break_rsp g_break_rsp;
static t_break_cmd g_break_cmd;
static T_HOST_ARG_VAL g_host_cmd;
static u8 g_dump_body[VADBG_DUMP_BODY_LEN];
static t_config_info g_config_info;
#endif

int vastai_pci_mem_read(struct vastai_pci_info *pcie_dev, int die_index,
			u64 relative_addr, void *buf, unsigned int len)
{
	int err = 0;

#if SIMU_SMI
	int i = 0;
	u8 *p = (u8 *)buf;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "simu read: relative_addr= 0x%llx  len=%d", relative_addr,
		    len);
	g_smi_cmd.flag = FLAG_FW_DATA_READY;

	memcpy(p, &g_smi_cmd, sizeof(TS_SMI_CMD_REQ));
	p += sizeof(TS_SMI_CMD_REQ);

	// simu data
	for (i = 0; i < (len - sizeof(TS_SMI_CMD_REQ));) {
		*(p + i) = i % 256;
		i++;
		*(p + i) = i / 256;
		i++;
		*(p + i) = 0x34;
		i++;
		*(p + i) = 0x12;
		i++;
	}
#endif

#if SIMU_DEBUG

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "simu read: relative_addr= 0x%llx  len=%d", relative_addr,
		    len);
	if (relative_addr < VADBG_CSRAM_ADDR) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "simu read: NOT DEBUG ");
		return len;
	}

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "simu read: g_host_cmd.nType= 0x%x",
		    g_host_cmd.nType);

	switch (g_host_cmd.nType) {
	case HOST_CMD:
		switch (g_break_cmd.cmd) {
		case VADBG_CMD_START:
			break;
		case VADBG_CMD_CONTINUE:
			break;
		case VADBG_CMD_NEXT:
			break;
		case VADBG_CMD_STEP:
			break;
		case VADBG_CMD_DUMP: {
			memset(&g_dump_body, 0, VADBG_DUMP_BODY_LEN);
			if (g_break_cmd.break_flag.oper_index == 2) {
				memset(&g_dump_body, (u8)0x12, 128);
				g_break_cmd.break_flag.oper_flag =
					VADBG_BP_FLAG_FW_DUMP_DONE;
				g_break_cmd.sub.dump.buffer_len = 128;
				g_break_cmd.sub.dump.buffer_addr =
					(u64)&g_dump_body;
			} else {
				memset(&g_dump_body, (u8)0x98, 1024);
				g_break_cmd.break_flag.oper_flag =
					VADBG_BP_FLAG_FW_DUMP_NEXT;
				g_break_cmd.sub.dump.buffer_len = 1024;
				g_break_cmd.sub.dump.buffer_addr =
					(u64)&g_dump_body;
			}

			memcpy(buf, &g_dump_body,
			       sizeof(g_break_cmd.sub.dump.buffer_len));
			err = g_break_cmd.sub.dump.buffer_len;
		} break;
		case VADBG_CMD_WRITE:
			break;
		case VADBG_CMD_STOP:
			break;
		}
		break;
	case HOST_PREPARE_BREAKPOINTS:

		break;
	case HOST_POLL_BREAK_STATUS:

		break;
	}

#endif

	VATOOLS_DUMP_BRIEF("simu read buffer: ", buf, len);
	return err;
}
int vastai_pci_mem_write(struct vastai_pci_info *pcie_dev, int die_index,
			 u64 relative_addr, const void *buf, unsigned int len)
{
	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "simu write: relative_addr= 0x%llx  len=%d", relative_addr,
		    len);
	VATOOLS_DUMP_BRIEF("simu write buffer: ", buf, len);

#if SIMU_SMI
	memset(&g_smi_cmd, 0x00, sizeof(TS_SMI_CMD_REQ));
	memcpy(&g_smi_cmd, buf, sizeof(TS_SMI_CMD_REQ));
#endif

#if SIMU_DEBUG
	if (relative_addr < VADBG_CSRAM_ADDR) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "simu write: NOT DEBUG ");
		return len;
	}

	if (relative_addr == VADBG_CSRAM_CONFIG_ADDR) {
		memcpy(&g_config_info, buf, len);
		return len;
	}
	if (relative_addr >= VADBG_DUMP_BODY_ADDR) {
		memset(g_dump_body, 0, VADBG_DUMP_BODY_LEN);
		return len;
	}

	if (relative_addr == VADBG_CSRAM_BREAKPOINT_ADDR) {
		memset(&g_break_req, 0x00, sizeof(t_break_req));
		memcpy(&g_break_req, buf, sizeof(t_break_req));
		return len;
	}

	if (relative_addr == VADBG_CSRAM_BREAKED_ADDR) {
		memset(&g_break_rsp, 0x00, sizeof(t_break_rsp));
		memcpy(&g_break_rsp, buf, sizeof(t_break_rsp));
		return len;
	}

	if (relative_addr == VADBG_CSRAM_BREAK_CMD_ADDR) {
		memset(&g_break_cmd, 0x00, sizeof(t_break_cmd));
		memcpy(&g_break_cmd, buf, sizeof(t_break_cmd));
		return len;
	}
#endif

	return len;
}

int vastai_pci_msgq_wr(struct msgq *msgq, void *data, u32 data_size)
{
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "msg_addr=0x%x", msgq->addr);
	return 0;
}

int get_vastai_die_info(struct vastai_pci_info *priv, int die_index[],
			int *die_num)
{
	int i = 0;
	*die_num = 1;

	for (i = 0; i < *die_num; i++) {
		die_index[i] = i;
	}
	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "simu die info: *die_num= %d  die_index[0]=%d", *die_num,
		    die_index[0]);

	return 0;
}

int vastai_pci_get_die_id(struct vastai_pci_info *pcie_dev, u32 die_index)
{
	return ((union die_index_data)die_index).die_id;
}

struct vastai_pci_info *get_vastai_pci_device_info(char dev_id)
{
	static struct vastai_pci_info g_vastai_pci_info;
	g_vastai_pci_info.dies[0].die_index = 0;
	return &g_vastai_pci_info;
}

int get_vastai_pci_device_number(void)
{
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "simu device_number: 1 ");
	return 1;
}

int vastai_pci_send_msg(void *pcie_dev, int die_index,
			union core_bitmap core_id, void *msg_buf, u32 msg_len)
{
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "core_id = 0x%x ", core_id.val);
	VATOOLS_DUMP_BRIEF("vastai_pci_send_msg: ", msg_buf, msg_len);

	return 0;
}

int vastai_device_reset(struct vastai_pci_info *priv, u32 flags)
{
	return 0;
}

struct dma_buf *vastai_alloc_dma_buf(struct vastai_pci_info *pci_dev, int size)
{
	return NULL;
}
struct vastai_dma_buf *get_vastai_dma_buf(struct dma_buf *dma_buf, u32 index)
{
	return NULL;
}

int vastai_pci_dma_transfer_sync(
	void *pcie_dev, int die_id, union core_bitmap core_id,
	struct vastai_dmadesc desc[], /* a local val is ok */
	int desc_num)
{
	return -1;
}

unsigned char vastai_pci_log_level = 0xf;

int vastai_send_ctrl_cmd(struct vastai_pci_info *pci_info, u32 die_index,
			 u32 core_bit_map, u64 info)
{
	return 0;
}

u64 vastai_get_host_time_ns(void)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
	struct timespec time;
#else
	u64 time;
#endif
	u64 time_ns;

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
	/* getnstimeofday function is removed from kernel5.6 */
	getnstimeofday(&time);
#else
	time = ktime_get_ns();
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
	time_ns = timespec_to_ns(&time);
#else
	time_ns = time;
#endif

	return time_ns;
}

// test dma
static int dmatest_work(void *data)
{
	// allow_signal(SIGTERM);
	// current->state = TASK_INTERRUPTIBLE;
	u64 i = 0;
	struct s_profiler_dma_checkpoint dma_profiler;
	struct s_profiler_msg_checkpoint msg_profiler;
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "dmatest_work kernel thread run");

	while (i < 990000000) {
		i++;
		memset(&dma_profiler, 0, sizeof(dma_profiler));
		memset(&msg_profiler, 0, sizeof(msg_profiler));

		//		dma_profiler.dma_axi_addr = i;
		dma_profiler.dma_data_size = i;
		// dma_profiler.time_of_dma_finish = i;
		// dma_profiler.time_of_dma_run = i;
		// dma_profiler.time_of_driver_notify_host_dma_done = i;
		dma_profiler.time_of_driver_recv_dma_req = i;
		vatools_profiler_dma_checkpoint(&dma_profiler);

		msg_profiler.time_of_fw_recv_msg = i;
		msg_profiler.time_of_host_send_msg = i;
		vatools_profiler_msg_checkpoint(&msg_profiler);

		if (i % 10000 == 0) {
			udelay(20);
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "dmatest_work thread run %lld", i);
		}

		if (i % 100000 == 0) {
			msleep(2);
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "dmatest_work thread sleep @ %lld", i);
		}
	}

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "dmatest_work kernel thread run finish");
	return 0;
}

int dmatest_init(void)
{
	/* Schedule the test thread */
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "dmatest_init");
	kthread_run(dmatest_work, NULL, "dmatest");

	return 0;
}

int vastai_pci_flash_bmcu_bin(struct vastai_pci_info *priv, u32 die_id,
			      u8 *path_name, const void *bin_buf,
			      size_t buf_size)
{
	int ret = 0;
	return ret;
}

int vastai_send_pcie_cmd(struct vastai_pci_info *priv, u32 die_index,
			 u32 sub_cmd, u64 paras)
{
	return 0;
}
int vastai_pci_flash_xspi_bin(struct vastai_pci_info *priv, u32 die_id,
			      u8 *path_name, const void *bin_buf,
			      size_t buf_size, bool is_base_partition)
{
	int ret = 0;
	return ret;
}

int vastai_pci_update_flag_xspi(struct vastai_pci_info *priv, u32 die_id,
				bool is_base_partition)
{
	return 0;
}

/* check a die_index */
u32 vastai_pci_get_die_index(void *pci_info, u32 die_id)
{
	return 0;
}

int vastai_pci_get_sriov_numvfs(char dev_id)
{
	return 0;
}

int vastai_pci_mem_write_direct(struct vastai_pci_info *priv, int die_index,
				u64 relative_addr, const void *buf,
				unsigned int len)
{
	return 0;
}
int vastai_get_msgq_id(struct vastai_pci_info *priv, int type)
{
	return 1;
}
#endif // VATOOLS_INDEPENDANCE_DRIVER
