/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: frank.bai
 */
#include "vatools.h"
#include "vatools_log.h"


/// ===================================================================
/// ===================================================================
/// ===================================================================
/// ===================================================================
/// ===================================================================
// ioctl 通用命令 V2 版本 vvvvvvvvvvvvv
/// ===================================================================

loff_t smi_llseek(struct file *filp, loff_t offset, int orig)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(offset);
	V_UNREFERENCE(orig);

	return 0;
}

ssize_t smi_read(struct file *filp, char __user *buf, size_t size, loff_t *pos)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(buf);
	V_UNREFERENCE(size);
	V_UNREFERENCE(pos);

	return 0;
}

ssize_t smi_write(struct file *filp, const char __user *buf, size_t size,
		  loff_t *pos)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(buf);
	V_UNREFERENCE(size);
	V_UNREFERENCE(pos);

	return 0;
}

ssize_t smi_write_iter(struct kiocb *iocb, struct iov_iter *from)
{
	V_UNREFERENCE(iocb);
	V_UNREFERENCE(from);

	return 0;
}

unsigned int smi_poll(struct file *filp, struct poll_table_struct *wait)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(wait);

	return 0;
}

long smi_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	long ret = -EIO;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	V_UNREFERENCE(arg);

	return ret;
}

int smi_mmap(struct file *filp, struct vm_area_struct *pvm)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(pvm);

	return 0;
}
int smi_open(struct inode *inode, struct file *filp)
{
	VATOOLS_FUNC_ENTERY;

	V_UNREFERENCE(inode);
	V_UNREFERENCE(filp);

	VATOOLS_FUNC_EXIT;
	return 0;
}

int smi_release(struct inode *ignored, struct file *filp)
{
	V_UNREFERENCE(ignored);
	V_UNREFERENCE(filp);

	return 0;
}

int smi_fasync(int fd, struct file *filp, int mode)
{
	V_UNREFERENCE(fd);
	V_UNREFERENCE(filp);
	V_UNREFERENCE(mode);

	return 0;
}

#ifdef CONFIG_TOOLS_V2

/// ===================================================================
/// device or die mutex
int domain_mutex_lock(struct vastai_pci_info *priv, u32 die_index, u32 field)
{
	struct vatools_node *node = NULL;
	union die_index_data did;
	did.val = die_index;

	if (field & VATOOLS_IOCTL_DOMAIN_DEVICE) {
		mutex_lock(&(priv->tools_dev_mutex));
	}
	if (field & VATOOLS_IOCTL_DOMAIN_DIE) {
		mutex_lock(&(priv->dies[did.die_id].tools_die_mutex));
	}
	if (field & VATOOLS_IOCTL_DOMAIN_DRIVER) {
		node = vatools_get_node();
		mutex_lock(&(node->node_driver_mutex));
	}
	return 0;
}

int domain_mutex_unlock(struct vastai_pci_info *priv, u32 die_index, u32 field)
{
	struct vatools_node *node = NULL;
	union die_index_data did;
	did.val = die_index;

	if (field & VATOOLS_IOCTL_DOMAIN_DRIVER) {
		node = vatools_get_node();
		mutex_unlock(&(node->node_driver_mutex));
	}
	if (field & VATOOLS_IOCTL_DOMAIN_DIE) {
		mutex_unlock(&(priv->dies[did.die_id].tools_die_mutex));
	}
	if (field & VATOOLS_IOCTL_DOMAIN_DEVICE) {
		mutex_unlock(&(priv->tools_dev_mutex));
	}
	return 0;
}

/// ===================================================================
/// 访问 SMCU SMI 接口
long vatools_ioctl_smi_cmd(struct file *filp, unsigned int cmd,
			   unsigned long arg)
{
	long ret = -EINVAL;
	long ret2 = 0;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	ioctl_arg_t ioctl_arg;

	// struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;
	struct vastai_sv100_die *die;

	TS_SMI_CMD_REQ smi_cmd = { 0 };
	TS_SMI_CMD_REQ *smi_cmd_p = NULL;

	int write_len = 0;
	int read_len = 0;
	void *write_buf = NULL;
	void *read_buf = NULL;

	T_SMI_BLOCK *smi_block_p = NULL;
	u64 smi_die_address_offset = 0;
	int ready = 0;
	int re_trigger_fw_count = 3;

	VATOOLS_FUNC_ENTERY;
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ioctl cmd=0x%x ", cmd);

	memset(&ioctl_arg, 0x00, sizeof(ioctl_arg_t));
	ret = copy_from_user(&ioctl_arg, argp, sizeof(ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

	die_id = ioctl_arg.die_index.die_id;
	dev_id = ioctl_arg.die_index.dev_id;
	die_index = (u32)ioctl_arg.die_index.val;
	smi_block_p = vatools_fw_smi_get_block(ioctl_arg.block_id);

	priv = vatools_get_vastai_pci_device_info(dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ",
			    dev_id);
		ret = -ENODEV;
		goto out;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(priv, DUMMY_DIE_ID, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		ret = -ENODEV;
		goto out;
	}

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg.block_id, ioctl_arg.die_index.val, ioctl_arg.sub_cmd,
		ioctl_arg.address, ioctl_arg.value, ioctl_arg.lock_domain,
		ioctl_arg.count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg.input_buf.buf_size, ioctl_arg.input_buf.buf_addr,
		ioctl_arg.output_buf.buf_size, ioctl_arg.output_buf.buf_addr);

	domain_mutex_lock(priv, ioctl_arg.die_index.val, ioctl_arg.lock_domain);

	write_len = ioctl_arg.input_buf.buf_size;
	read_len = ioctl_arg.output_buf.buf_size;

	if (write_len > smi_block_p->block_len ||
	    read_len > smi_block_p->block_len) {
		VATOOLS_ERR(priv, die_id,
			    "read_len=%d or write_len=%d over %d of %d",
			    read_len, write_len, smi_block_p->block_len,
			    ioctl_arg.block_id);
		ret = -EFAULT;
		goto out_mutex_unlock;
	}

	if ((read_len % 4 != 0) || (write_len % 4 != 0)) {
		VATOOLS_ERR(priv, die_id,
			    "read_len=%d, write_len=%d, not align by 4 bytes",
			    read_len, write_len);
		ret = -EFAULT;
		goto out_mutex_unlock;
	}

	write_buf = kvmalloc(ioctl_arg.input_buf.buf_size, GFP_KERNEL);
	if (write_buf == NULL) {
		ret = -ENOMEM;
		VATOOLS_ERR(priv, die_id, "kvmalloc write_buf ret=%ld", ret);
		goto out_free_write_buf;
	}

	ret = copy_from_user(write_buf,
			     (void __user *)ioctl_arg.input_buf.buf_addr,
			     ioctl_arg.input_buf.buf_size);
	if (ret) {
		VATOOLS_ERR(priv, die_id, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out_free_write_buf;
	}

	VATOOLS_DUMP_BRIEF("vatools_ioctl_smi_cmd write_buf: ", write_buf,
			   ioctl_arg.input_buf.buf_size);

	// =====================================================
	//TODO:  这里检查 SMI 命令, 如果是 SMI_CMD_CODE_SPI_WRITE,
	//       则清除 dev_info 缓存, 因为spi 写操作可能修改 spi config 数据
	smi_cmd_p = (TS_SMI_CMD_REQ *)write_buf;
	if (smi_cmd_p != NULL && smi_cmd_p->cmd == SMI_CMD_CODE_SPI_WRITE) {
		struct vatools_node *node_p = vatools_get_node();
		VATOOLS_DBG(priv, die_id, "clear device info cache");
		kvfree(node_p->dev_info);
		node_p->dev_info = NULL;
		node_p->device_num = 0;
	}
	// 检查 SMI 命令结束
	// =====================================================

	smi_die_address_offset = smi_block_p->block_address;

	VATOOLS_DBG(priv, die_id, "smi_block_p->block_address=0x%x",
		    smi_block_p->block_address);

	// set flag to 0, write only 4 bytes, sync with fw
	*((u8 *)write_buf + 6) = FLAG_APP_PREPARE;
	ret = vatools_pci_mem_write(priv, die_index, smi_die_address_offset,
				    (u8 *)write_buf,
				    ioctl_arg.input_buf.buf_size);
	if (ret < 0) {
		VATOOLS_ERR(priv, die_id,
			    "vatools_pci_mem_write flag=0 ret=%ld", ret);
		goto out_free_write_buf;
	}

	// set flag to 1, write only 4 bytes, sync with fw
	*((u8 *)write_buf + 6) = FLAG_APP_REQ;
	ret = vatools_pci_mem_write(priv, die_index, smi_die_address_offset + 4,
				    ((u8 *)write_buf) + 4, sizeof(u32));
	if (ret < 0) {
		VATOOLS_ERR(priv, die_id,
			    "vatools_pci_mem_write flag=1 ret=%ld", ret);
		goto out_free_write_buf;
	}

	read_buf = kvmalloc(ioctl_arg.output_buf.buf_size, GFP_KERNEL);
	if (read_buf == NULL) {
		ret = -ENOMEM;
		VATOOLS_ERR(priv, die_id, "kvmalloc read_buf=NULL ret=%ld",
			    ret);
		goto out_free_write_buf;
	}

	// TODO: 如果fw未响应, 则重复发送3次请求.
re_trigger_fw:

	/* 如果是用户中断, 需要在这里清除一下上次smi命令的ack应答,
	否则每次进入到这里, wait_for都会直接返回, 导致wait和ack不匹配*/
	if (!priv || (priv->die_num <= die_id)) {
		VATOOLS_DBG(priv, die_id, "priv->die_num=%d < die_id=%d",
			    priv->die_num, die_id);
		ret = -EINVAL;
		goto out_mutex_unlock;
	}
	die = &(priv->dies[die_id]);
	reinit_completion(&(die->smi_ack[ioctl_arg.block_id]));

	ret = vatools_fw_send_smi_interrupt(priv, die_index);
	ret = vatools_wait_smi_ack(priv, die_index, ioctl_arg.block_id, 4);
	if (ret < 0) {
		if (ret == -EINTR) {
			VATOOLS_DBG(
				priv, die_id,
				"smi_ack: die_index=%x block_id=%d timeout ret=%ld(-EINTR) re_trigger_fw_count=%d",
				die_index, ioctl_arg.block_id, ret,
				re_trigger_fw_count);
			goto out_free_read_buf;
		} else if (ret == -ETIMEDOUT) {
			VATOOLS_ERR(
				priv, die_id,
				"smi_ack: die_index=%x block_id=%d timeout ret=%ld(-ETIMEDOUT) re_trigger_fw_count=%d",
				die_index, ioctl_arg.block_id, ret,
				re_trigger_fw_count);
		} else {
			VATOOLS_ERR(
				priv, die_id,
				"smi_ack: die_index=%x block_id=%d timeout ret=%ld re_trigger_fw_count=%d",
				die_index, ioctl_arg.block_id, ret,
				re_trigger_fw_count);
			goto out_free_read_buf;
		}
	}

	memset(read_buf, 0x00, ioctl_arg.output_buf.buf_size);
	ret = vatools_pci_mem_read(priv, die_index, smi_die_address_offset,
				   read_buf, ioctl_arg.output_buf.buf_size);
	if (ret < 0) {
		VATOOLS_ERR(priv, die_id, "ret=%ld", ret);
		goto out_free_read_buf;
	}

	VATOOLS_DUMP_BRIEF("vatools_ioctl_smi_cmd read_buf: ", read_buf,
			   ioctl_arg.output_buf.buf_size);

	memcpy(&smi_cmd, read_buf, sizeof(TS_SMI_CMD_REQ));

	VATOOLS_DBG(priv, die_id, "smi_cmd.flag=%d", smi_cmd.flag);
	// fw 有应答, 无论执行结果如何, 都不再重发
	if (smi_cmd.flag == FLAG_FW_DATA_READY ||
	    smi_cmd.flag == FALG_FW_ERROR_UNKNOWN_CMD ||
	    smi_cmd.flag == FALG_FW_ERROR_ABORT) {
		ready = 1;
		//数据准备好
		VATOOLS_DBG(priv, die_id,
			    "READY: ioctl_arg.output_buf.buf_addr=0x%p  "
			    "ioctl_arg.output_buf.buf_size=%d",
			    ioctl_arg.output_buf.buf_addr,
			    ioctl_arg.output_buf.buf_size);

		ret = copy_to_user(ioctl_arg.output_buf.buf_addr, read_buf,
				   ioctl_arg.output_buf.buf_size);
		if (ret) {
			VATOOLS_ERR(priv, die_id, "copy_to_user ret=%ld", ret);
			ret = -EFAULT;
			goto out_free_read_buf;
		}
		ret = smi_cmd.flag | 0x1000;
		goto out_free_read_buf;
	} else if (smi_cmd.flag == FLAG_APP_REQ) {
		// fw 没有应答, flag是请求状态, smcu未做任何处理, 重发请求 ,循环3次后结束
		VATOOLS_ERR(
			priv, die_id,
			"SMCU not responding: smi_cmd.flag == FLAG_APP_REQ(0x01): re_trigger_fw_count=%d",
			re_trigger_fw_count);
		if (re_trigger_fw_count > 0) {
			re_trigger_fw_count--;
			goto re_trigger_fw;
		}
	}

	//数据错误
	VATOOLS_DBG(priv, die_id,
		    "ABORT: ioctl_arg.output_buf.buf_addr=0x%p  "
		    "ioctl_arg.output_buf.buf_size=%d",
		    ioctl_arg.output_buf.buf_addr,
		    ioctl_arg.output_buf.buf_size);

	ret = copy_to_user(ioctl_arg.output_buf.buf_addr, read_buf,
			   ioctl_arg.output_buf.buf_size);
	if (ret) {
		VATOOLS_ERR(priv, die_id, "copy_to_user ret=%ld", ret);
		ret = -EFAULT;
		goto out_free_read_buf;
	}
	ret = -ETIME;
	goto out_free_read_buf;

out_free_read_buf:
	kvfree(read_buf);

out_free_write_buf:
	kvfree(write_buf);

out_mutex_unlock:
	domain_mutex_unlock(priv, ioctl_arg.die_index.val,
			    ioctl_arg.lock_domain);

out:

	ioctl_arg.errcode = ret;
	ret2 = copy_to_user(argp, &ioctl_arg, sizeof(ioctl_arg));
	if (ret2) {
		VATOOLS_ERR(priv, die_id, "copy_to_user ret=%ld", ret);
		ret2 = -EFAULT;
	}
	VATOOLS_DBG(priv, die_id, "out: ret=%ld   ret2=%ld", ret, ret2);
	VATOOLS_FUNC_EXIT;

	return ret2;
}

/// ===================================================================
/// pcie 命令接口, 直接调用pcie驱动函数. 不发送到smcu
long vatools_ioctl_pcie_cmd(struct file *filp, unsigned int cmd,
			    unsigned long arg)
{
	long ret = -EINVAL;
	long ret2 = 0;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	ioctl_arg_t ioctl_arg;

	struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;
	int i = 0;

	VATOOLS_FUNC_ENTERY;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ioctl cmd=0x%x ", cmd);

	memset(&ioctl_arg, 0x00, sizeof(ioctl_arg_t));
	ret = copy_from_user(&ioctl_arg, argp, sizeof(ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

	die_id = ioctl_arg.die_index.die_id;
	dev_id = ioctl_arg.die_index.dev_id;
	die_index = (u32)ioctl_arg.die_index.val;

	priv = vatools_get_vastai_pci_device_info(dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ",
			    dev_id);
		ret = -ENODEV;
		goto out;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(priv, DUMMY_DIE_ID, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		ret = -ENODEV;
		goto out;
	}

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg.block_id, ioctl_arg.die_index.val, ioctl_arg.sub_cmd,
		ioctl_arg.address, ioctl_arg.value, ioctl_arg.lock_domain,
		ioctl_arg.count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg.input_buf.buf_size, ioctl_arg.input_buf.buf_addr,
		ioctl_arg.output_buf.buf_size, ioctl_arg.output_buf.buf_addr);

	mutex_lock(&node->node_driver_mutex);

	switch (ioctl_arg.sub_cmd) {
	case VATOOLS_IOCTL_PCIE_BANDWIDTH: {
		u64 upstream_count = 0;
		u64 downstream_count = 0;
		ret = vatools_pci_mem_read(priv, die_index,
					   PCIE_BANDWIDTH_UPSTREAM,
					   &upstream_count, sizeof(u64));
		if (ret < 0) {
			ret = -EFAULT;
			goto out_mutex_unlock;
		}

		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "upstream_count: ret=%llx",
			    upstream_count);
		ret = copy_to_user(ioctl_arg.output_buf.buf_addr +
					   sizeof(TS_SMI_CMD_REQ),
				   &upstream_count, sizeof(u64));
		if (ret < 0) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld",
				    ret);
			ret = -EFAULT;
			goto out_mutex_unlock;
		}
		ret = vatools_pci_mem_read(priv, die_index,
					   PCIE_BANDWIDTH_DOWNSTREAM,
					   &downstream_count, sizeof(u64));
		if (ret < 0) {
			ret = -EFAULT;
			goto out_mutex_unlock;
		}
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "downstream_count: ret=%llx",
			    downstream_count);
		ret = copy_to_user(ioctl_arg.output_buf.buf_addr +
					   sizeof(TS_SMI_CMD_REQ) + sizeof(u64),
				   &downstream_count, sizeof(u64));
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld",
				    ret);
			ret = -EFAULT;
			goto out_mutex_unlock;
		}
	} break;

	case VATOOLS_IOCTL_PCIE_RESET:
		vastai_device_reset(priv, VASTAI_RESET_MSG_TRIGER);
		break;

	case VATOOLS_IOCTL_PCIE_REBOOT:
		vastai_device_reset(priv, VASTAI_RESET_MSG_TRIGER |
						  VASTAI_RESET_BOOT_DEV);
		break;

	case VATOOLS_IOCTL_PCIE_FW_VERSION: {
		struct vastai_fw_version fw_ver;
		memcpy(&fw_ver, &(priv->dies[0].fw_ver),
		       sizeof(struct vastai_fw_version));
		VATOOLS_DUMP_BRIEF("fw_ver: ", &fw_ver,
				   sizeof(struct vastai_fw_version));
		ret = copy_to_user(ioctl_arg.output_buf.buf_addr, &fw_ver,
				   sizeof(struct vastai_fw_version));
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld",
				    ret);
			ret = -EFAULT;
			goto out_mutex_unlock;
		}
	} break;

	case VATOOLS_IOCTL_PCIE_DDR_ECC_1BIT: {
		struct vastai_fifo *ecc_fifo;
		u32 table_size = ioctl_arg.output_buf.buf_size;
		u8 *buffer;

		if (table_size > DDR_ECC_1BIT_SIZE) {
			table_size = DDR_ECC_1BIT_SIZE;
		}

		ret = 0;
		buffer = kvmalloc(table_size, GFP_KERNEL);
		if (buffer == NULL) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"VATOOLS_IOCTL_PCIE_DDR_ECC_1BIT kvmalloc error. size=%d",
				table_size);
			ret = -ENOMEM;
			goto out_mutex_unlock;
		}

		ret = vatools_pci_mem_read(priv, die_index, DDR_ECC_1BIT,
					   buffer, table_size);
		if (ret < 0) {
			ret = -EFAULT;
			goto out_free_write_buf_1bit;
		}
		ecc_fifo = (struct vastai_fifo *)buffer;
		VATOOLS_DUMP_BRIEF("ecc 1bit: ", buffer, table_size);
		// ecc_fifo->rd 返回到应用层, 对应的变量是entry_count
		if (ecc_fifo->elem_count == 0) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "ecc1bit: ecc_fifo->elem_count = 0");
			ecc_fifo->rd = 0;
		} else {
			ecc_fifo->rd =
				(ecc_fifo->elem_count +
				 (((int)ecc_fifo->wr) - (int)(ecc_fifo->rd))) %
				ecc_fifo->elem_count;
		}

		ret = copy_to_user(ioctl_arg.output_buf.buf_addr, buffer,
				   table_size);
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld",
				    ret);
			ret = -EFAULT;
			goto out_free_write_buf_1bit;
		}

	out_free_write_buf_1bit:
		kvfree(buffer);
	} break;

	case VATOOLS_IOCTL_PCIE_DDR_ECC_2BIT: {
		struct vastai_fifo *ecc_fifo;
		u32 table_size = ioctl_arg.output_buf.buf_size;
		u8 *buffer;

		if (table_size > DDR_ECC_2BIT_SIZE) {
			table_size = DDR_ECC_2BIT_SIZE;
		}

		ret = 0;
		buffer = kvmalloc(table_size, GFP_KERNEL);
		if (buffer == NULL) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"VATOOLS_IOCTL_PCIE_DDR_ECC_1BIT kvmalloc error. size=%d",
				table_size);
			ret = -ENOMEM;
			goto out_mutex_unlock;
		}

		ret = vatools_pci_mem_read(priv, die_index, DDR_ECC_2BIT,
					   buffer, table_size);
		if (ret < 0) {
			ret = -EFAULT;
			goto out_free_write_buf_2bit;
		}
		ecc_fifo = (struct vastai_fifo *)buffer;
		VATOOLS_DUMP_BRIEF("ecc 2bit: ", buffer, table_size);
		// ecc_fifo->rd 返回到应用层, 对应的变量是entry_count
		if (ecc_fifo->elem_count == 0) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "ecc2bit: ecc_fifo->elem_count = 0");
			ecc_fifo->rd = 0;
		} else {
			ecc_fifo->rd =
				(ecc_fifo->elem_count +
				 (((int)ecc_fifo->wr) - (int)(ecc_fifo->rd))) %
				ecc_fifo->elem_count;
		}

		ret = copy_to_user(ioctl_arg.output_buf.buf_addr, buffer,
				   table_size);
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld",
				    ret);
			ret = -EFAULT;
			goto out_free_write_buf_2bit;
		}

	out_free_write_buf_2bit:
		kvfree(buffer);
	} break;

	case VATOOLS_IOCTL_PCIE_VDSP_COUNT: {
		ret = copy_to_user(
			ioctl_arg.output_buf.buf_addr,
			priv->dies[vastai_pci_get_die_id(priv, die_index)]
				.pcie_tx_vdsp_count,
			ioctl_arg.output_buf.buf_size);
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld",
				    ret);
			ret = -EFAULT;
			goto out_mutex_unlock;
		}

		ret = copy_to_user(
			((u8 *)ioctl_arg.output_buf.buf_addr) + sizeof(u32) * 4,
			priv->dies[vastai_pci_get_die_id(priv, die_index)]
				.pcie_rx_vdsp_count,
			ioctl_arg.output_buf.buf_size);
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld",
				    ret);
			ret = -EFAULT;
			goto out_mutex_unlock;
		}
	} break;

	case VATOOLS_IOCTL_PCIE_AI_COUNT: {
		u8 *ai_count_buffer;
		u32 ai_count_buffer_size = 512;

		if (ioctl_arg.output_buf.buf_size < ai_count_buffer_size) {
			ret = -EFAULT;
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "output buffer size < %d ret=%ld",
				    ai_count_buffer_size, ret);
			goto out_mutex_unlock;
		}

		ai_count_buffer = kvmalloc(ai_count_buffer_size, GFP_KERNEL);
		if (ai_count_buffer == NULL) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"VATOOLS_IOCTL_PCIE_AI_COUNT kvmalloc. size=%d",
				ai_count_buffer_size);
			ret = -ENOMEM;
			goto out_mutex_unlock;
		}

		ret = vatools_pci_mem_read(priv, die_index, PERFORMANCE_AI_BASE,
					   ai_count_buffer,
					   ai_count_buffer_size);
		if (ret < 0) {
			ret = -EFAULT;
			goto out_free_write_buf_ai_count;
		}

		ret = copy_to_user(ioctl_arg.output_buf.buf_addr,
				   ai_count_buffer, ai_count_buffer_size);
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld",
				    ret);
			ret = -EFAULT;
			goto out_free_write_buf_ai_count;
		}
	out_free_write_buf_ai_count:
		kvfree(ai_count_buffer);
	} break;

	case VATOOLS_IOCTL_PCIE_RING_BUF_STATISTIC: {
		T_RING_BUF_STATICS ring_buf_statics[] = {
			{ PERFORMANCE_HOST_2_VEMCU0 },
			{ PERFORMANCE_HOST_2_VEMCU1 },
			{ PERFORMANCE_HOST_2_VEMCU2 },
			{ PERFORMANCE_HOST_2_VEMCU3 },
			{ PERFORMANCE_SMCU_2_VDMCU0 },
			{ PERFORMANCE_VDSP0_2_VDMCU0 },
			{ PERFORMANCE_VDSP1_2_VDMCU0 },
			{ PERFORMANCE_VEMCU0_2_VDMCU0 },
			{ PERFORMANCE_VEMCU1_2_VDMCU0 },
			{ PERFORMANCE_SMCU_2_VDMCU1 },
			{ PERFORMANCE_VDSP1_2_VDMCU1 },
			{ PERFORMANCE_VDSP2_2_VDMCU1 },
			{ PERFORMANCE_VEMCU1_2_VDMCU1 },
			{ PERFORMANCE_VEMCU2_2_VDMCU1 },
			{ PERFORMANCE_SMCU_2_VDMCU2 },
			{ PERFORMANCE_VDSP2_2_VDMCU2 },
			{ PERFORMANCE_VDSP3_2_VDMCU2 },
			{ PERFORMANCE_VEMCU2_2_VDMCU2 },
			{ PERFORMANCE_VEMCU3_2_VDMCU2 },
			{ PERFORMANCE_VDSP0_2_CMCU },
			{ PERFORMANCE_VDSP1_2_CMCU },
			{ PERFORMANCE_VDSP2_2_CMCU },
			{ PERFORMANCE_VDSP3_2_CMCU },
			{ PERFORMANCE_CMCU_2_VDSP0 },
			{ PERFORMANCE_CMCU_2_VDSP1 },
			{ PERFORMANCE_CMCU_2_VDSP2 },
			{ PERFORMANCE_CMCU_2_VDSP3 },
			{ PERFORMANCE_HOST_2_VDSP0 },
			{ PERFORMANCE_HOST_2_VDSP1 },
			{ PERFORMANCE_HOST_2_VDSP2 },
			{ PERFORMANCE_HOST_2_VDSP3 },
			{ PERFORMANCE_SMCU0_2_HOST },
			{ PERFORMANCE_SMCU1_2_HOST },
			{ PERFORMANCE_VDMCU1_2_HOST },
			{ PERFORMANCE_VDMCU2_2_HOST },
			{ PERFORMANCE_VEMCU0_2_HOST },
			{ PERFORMANCE_VEMCU1_2_HOST },
			{ PERFORMANCE_VEMCU2_2_HOST },
			{ PERFORMANCE_VEMCU3_2_HOST },
			{ PERFORMANCE_VDSP0_2_HOST },
			{ PERFORMANCE_VDSP1_2_HOST },
			{ PERFORMANCE_VDSP2_2_HOST },
			{ PERFORMANCE_VDSP3_2_HOST },
			{ PERFORMANCE_DMA_HOST_2_DEV },
			{ PERFORMANCE_DMA_DEV_2_HOST },
			{ PERFORMANCE_DMA_RC_IN },
			{ PERFORMANCE_DMA_RC_OUT },
		};

		u32 static_buf_cnt =
			sizeof(ring_buf_statics) / sizeof(ring_buf_statics[0]);
		u32 static_buf_len =
			static_buf_cnt * sizeof(T_VASTAI_FIFO_HEADER);
		u8 *static_buf = NULL;
		if (ioctl_arg.output_buf.buf_size < static_buf_len) {
			ret = -EFAULT;
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "output buffer size < %d ret=%ld",
				    static_buf_len, ret);
			goto out_mutex_unlock;
		}
		static_buf = kvmalloc(static_buf_len, GFP_KERNEL);
		if (NULL == static_buf) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"VATOOLS_IOCTL_PCIE_HOST_COUNT kvmalloc. size =%d",
				static_buf_len);
			ret = -ENOMEM;
			goto out_mutex_unlock;
		}

		for (i = 0; i < static_buf_cnt; i++) {
			ret = vatools_pci_mem_read(
				priv, die_index, ring_buf_statics[i].base_addr,
				(void *)(static_buf +
					 i * sizeof(T_VASTAI_FIFO_HEADER)),
				sizeof(T_VASTAI_FIFO_HEADER));
			if (ret < 0) {
				VATOOLS_ERR(
					NULL, DUMMY_DIE_ID,
					"Ring buf read memory base addr %x error\n",
					ring_buf_statics[i].base_addr);
				ret = -EFAULT;
				goto out_free_static_buf;
			}
		}

		ret = copy_to_user(ioctl_arg.output_buf.buf_addr +
					   sizeof(TS_SMI_CMD_REQ),
				   static_buf, static_buf_len);
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld",
				    ret);
			ret = -EFAULT;
			goto out_free_static_buf;
		}

	out_free_static_buf:
		kvfree(static_buf);
	} break;

#ifdef CONFIG_VASTAI_VIDEO
	// 设置video使用multicore; 0: 不使用; 1: 使用
	case VATOOLS_IOCTL_PCIE_SET_VIDEO_MULTI_CORE: {
		u32 work_mode = (ioctl_arg.address & 0xFFFFFFFF);
		// call video callback function here
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "die inex=%x set video multicore work_mode to %d\n",
			    die_index, work_mode);
		ret = hantroenc_set_work_mode(die_index,
					      (encoder_workmode_t)work_mode, 0);
	} break;
	case VATOOLS_IOCTL_PCIE_GET_VIDEO_MULTI_CORE: {
		u32 work_mode = 0;
		int job_count = 0;
		// call video callback function here
		ret = hantroenc_get_work_mode(die_index, &work_mode,
					      &job_count);
		ioctl_arg.address = work_mode;
		ioctl_arg.address = ioctl_arg.address << 32 | job_count;

		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"die inex=%x get video multicore work_mode=%d, job_count=%d\n",
			die_index, work_mode, job_count);
		goto out_mutex_unlock;
	} break;
#endif

	default:
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"driver sub_cmd id %d don't support in current system\n",
			ioctl_arg.sub_cmd);
		break;
	}

out_mutex_unlock:
	mutex_unlock(&node->node_driver_mutex);

out:
	ioctl_arg.errcode = ret;
	ret2 = copy_to_user(argp, &ioctl_arg, sizeof(ioctl_arg_t));
	if (ret2) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user ret=%ld", ret);
		ret2 = -EFAULT;
	}
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "out: ret=%ld   ret2=%ld", ret, ret2);
	VATOOLS_FUNC_EXIT;
	return ret2;
}
/// ===================================================================
/// 读内存数据
long vatools_ioctl_read_buf(struct file *filp, unsigned int cmd,
			    unsigned long arg)
{
	long ret = -EINVAL;
	long ret2 = 0;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	ioctl_arg_t ioctl_arg;

	// struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;
	u8 *temp_buf = NULL;

	VATOOLS_FUNC_ENTERY;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ioctl cmd=0x%x ", cmd);

	memset(&ioctl_arg, 0x00, sizeof(ioctl_arg_t));
	ret = copy_from_user(&ioctl_arg, argp, sizeof(ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

	die_id = ioctl_arg.die_index.die_id;
	dev_id = ioctl_arg.die_index.dev_id;
	die_index = (u32)ioctl_arg.die_index.val;

	priv = vatools_get_vastai_pci_device_info(dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "priv is NULL, dev_id[%u] ",
			    dev_id);
		ret = -ENODEV;
		goto out;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(priv, die_id, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		ret = -ENODEV;
		goto out;
	}

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg.block_id, ioctl_arg.die_index.val, ioctl_arg.sub_cmd,
		ioctl_arg.address, ioctl_arg.value, ioctl_arg.lock_domain,
		ioctl_arg.count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg.input_buf.buf_size, ioctl_arg.input_buf.buf_addr,
		ioctl_arg.output_buf.buf_size, ioctl_arg.output_buf.buf_addr);

	temp_buf = (u8 *)kvmalloc(ioctl_arg.output_buf.buf_size, GFP_KERNEL);
	if (!temp_buf) {
		VATOOLS_ERR(priv, die_id, " kvmalloc error. size=%d",
			    ioctl_arg.output_buf.buf_size);
		ret = -ENOMEM;
		goto out;
	}

	memset(temp_buf, 0, ioctl_arg.output_buf.buf_size);
	ret = vastai_pci_mem_read(priv, die_index, ioctl_arg.address, temp_buf,
				  ioctl_arg.output_buf.buf_size);
	if (ret < 0) {
		VATOOLS_ERR(priv, die_id,
			    "vastai_pci_mem_read error, ret=%ld\n", ret);
		goto out_kvfree;
	}

	ret = copy_to_user((void *)ioctl_arg.output_buf.buf_addr, temp_buf,
			   ioctl_arg.output_buf.buf_size);
	if (ret) {
		VATOOLS_ERR(priv, die_id, "copy_to_user is error\n");
		ret = -EIO;
		goto out_kvfree;
	}

out_kvfree:
	kvfree(temp_buf);
out:
	ioctl_arg.errcode = ret;
	ret2 = copy_to_user(argp, &ioctl_arg, sizeof(ioctl_arg_t));
	if (ret2) {
		VATOOLS_ERR(priv, die_id, "copy_to_user ret=%ld", ret);
		ret2 = -EFAULT;
	}
	VATOOLS_DBG(priv, die_id, "out: ret=%ld   ret2=%ld", ret, ret2);
	VATOOLS_FUNC_EXIT;
	return ret2;
}

//写内存数据
long vatools_ioctl_write_buf(struct file *filp, unsigned int cmd,
			     unsigned long arg)
{
	long ret = -EINVAL;
	long ret2 = 0;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	ioctl_arg_t ioctl_arg;

	// struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;
	u8 *temp_buf = NULL;

	VATOOLS_FUNC_ENTERY;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ioctl cmd=0x%x ", cmd);

	memset(&ioctl_arg, 0x00, sizeof(ioctl_arg_t));
	ret = copy_from_user(&ioctl_arg, argp, sizeof(ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

	die_id = ioctl_arg.die_index.die_id;
	dev_id = ioctl_arg.die_index.dev_id;
	die_index = (u32)ioctl_arg.die_index.val;

	priv = vatools_get_vastai_pci_device_info(dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, die_id, "priv is NULL, dev_id[%u] ", dev_id);
		ret = -ENODEV;
		goto out;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(priv, die_id, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		ret = -ENODEV;
		goto out;
	}

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg.block_id, ioctl_arg.die_index.val, ioctl_arg.sub_cmd,
		ioctl_arg.address, ioctl_arg.value, ioctl_arg.lock_domain,
		ioctl_arg.count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg.input_buf.buf_size, ioctl_arg.input_buf.buf_addr,
		ioctl_arg.output_buf.buf_size, ioctl_arg.output_buf.buf_addr);

	temp_buf = (u8 *)kvmalloc(ioctl_arg.input_buf.buf_size, GFP_KERNEL);
	if (!temp_buf) {
		VATOOLS_ERR(priv, die_id, " kvmalloc error. size=%d",
			    ioctl_arg.input_buf.buf_size);
		ret = -ENOMEM;
		goto out;
	}

	memset(temp_buf, 0, ioctl_arg.input_buf.buf_size);
	ret = copy_from_user(temp_buf, (void *)ioctl_arg.input_buf.buf_addr,
			     ioctl_arg.input_buf.buf_size);
	if (ret) {
		VATOOLS_ERR(priv, die_id, "copy_from_user is error\n");
		ret = -EIO;
		goto out_kvfree;
	}

	ret = vastai_pci_mem_write(priv, die_index, ioctl_arg.address, temp_buf,
				   ioctl_arg.input_buf.buf_size);
	if (ret < 0) {
		VATOOLS_ERR(priv, die_id,
			    "vastai_pci_mem_read error, ret=%ld\n", ret);
		goto out_kvfree;
	}

out_kvfree:
	kvfree(temp_buf);
out:
	ioctl_arg.errcode = ret;
	ret2 = copy_to_user(argp, &ioctl_arg, sizeof(ioctl_arg_t));
	if (ret2) {
		VATOOLS_ERR(priv, die_id, "copy_to_user ret=%ld", ret);
		ret2 = -EFAULT;
	}
	VATOOLS_DBG(priv, die_id, "out: ret=%ld   ret2=%ld", ret, ret2);
	VATOOLS_FUNC_EXIT;
	return ret2;
}

/// ===================================================================
/// flash 相关
long vatools_ioctl_flash_v2_bmcu_fw(struct file *filp, unsigned int cmd,
				    unsigned long arg,
				    struct vastai_pci_info *priv,
				    ioctl_arg_t *ioctl_arg_p)
{
	long ret = -EINVAL;

	// struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;
	void *buf;

	VATOOLS_FUNC_ENTERY;

	die_id = ioctl_arg_p->die_index.die_id;
	dev_id = ioctl_arg_p->die_index.dev_id;
	die_index = (u32)ioctl_arg_p->die_index.val;

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg_p->block_id, ioctl_arg_p->die_index.val,
		ioctl_arg_p->sub_cmd, ioctl_arg_p->address, ioctl_arg_p->value,
		ioctl_arg_p->lock_domain, ioctl_arg_p->count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg_p->input_buf.buf_size,
		ioctl_arg_p->input_buf.buf_addr,
		ioctl_arg_p->output_buf.buf_size,
		ioctl_arg_p->output_buf.buf_addr);

	if (ioctl_arg_p->input_buf.buf_size <= 0) {
		ret = -EINVAL;
		goto out;
	}

	buf = kvmalloc(ioctl_arg_p->input_buf.buf_size, GFP_KERNEL);
	if (!buf) {
		ret = -ENOMEM;
		goto out_free;
	}

	ret = copy_from_user(buf, ioctl_arg_p->input_buf.buf_addr,
			     ioctl_arg_p->input_buf.buf_size);
	if (ret) {
		ret = -EFAULT;
		goto out_free;
	}

	ret = vastai_pci_flash_bmcu_bin(priv, die_id, NULL, buf,
					ioctl_arg_p->input_buf.buf_size);

out_free:
	kvfree(buf);

out:
	ioctl_arg_p->errcode = ret;
	VATOOLS_DBG(priv, die_id, "ret=%ld", ret);
	VATOOLS_FUNC_EXIT;
	return ret;
}

long vatools_ioctl_flash_v2_xspi_fw(struct file *filp, unsigned int cmd,
				    unsigned long arg,
				    struct vastai_pci_info *priv,
				    ioctl_arg_t *ioctl_arg_p)
{
	long ret = -EINVAL;

	// struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;
	void *buf;

	VATOOLS_FUNC_ENTERY;

	die_id = ioctl_arg_p->die_index.die_id;
	dev_id = ioctl_arg_p->die_index.dev_id;
	die_index = (u32)ioctl_arg_p->die_index.val;

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg_p->block_id, ioctl_arg_p->die_index.val,
		ioctl_arg_p->sub_cmd, ioctl_arg_p->address, ioctl_arg_p->value,
		ioctl_arg_p->lock_domain, ioctl_arg_p->count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg_p->input_buf.buf_size,
		ioctl_arg_p->input_buf.buf_addr,
		ioctl_arg_p->output_buf.buf_size,
		ioctl_arg_p->output_buf.buf_addr);

	if (ioctl_arg_p->input_buf.buf_size <= 0) {
		ret = -EINVAL;
		goto out;
	}

	buf = kvmalloc(ioctl_arg_p->input_buf.buf_size, GFP_KERNEL);
	if (!buf) {
		ret = -ENOMEM;
		goto out_free;
	}

	ret = copy_from_user(buf, ioctl_arg_p->input_buf.buf_addr,
			     ioctl_arg_p->input_buf.buf_size);
	if (ret) {
		ret = -EFAULT;
		goto out_free;
	}

	ret = vastai_pci_flash_xspi_bin(priv, die_id, NULL, buf,
					ioctl_arg_p->input_buf.buf_size, 0);
	// TODO: why ret>=0 return?
	if (ret) {
		goto out_free;
	}
	ret = vastai_pci_update_flag_xspi(priv, die_id, 0);

out_free:
	kvfree(buf);

out:
	ioctl_arg_p->errcode = ret;
	VATOOLS_DBG(priv, die_id, "ret=%ld", ret);
	VATOOLS_FUNC_EXIT;
	return ret;
}

long vatools_ioctl_flash_v2_get_status(struct file *filp, unsigned int cmd,
				       unsigned long arg,
				       struct vastai_pci_info *priv,
				       ioctl_arg_t *ioctl_arg_p)
{
	long ret = -EINVAL;

	// struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;
	struct flash_state flash_state;

	VATOOLS_FUNC_ENTERY;

	die_id = ioctl_arg_p->die_index.die_id;
	dev_id = ioctl_arg_p->die_index.dev_id;
	die_index = (u32)ioctl_arg_p->die_index.val;

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg_p->block_id, ioctl_arg_p->die_index.val,
		ioctl_arg_p->sub_cmd, ioctl_arg_p->address, ioctl_arg_p->value,
		ioctl_arg_p->lock_domain, ioctl_arg_p->count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg_p->input_buf.buf_size,
		ioctl_arg_p->input_buf.buf_addr,
		ioctl_arg_p->output_buf.buf_size,
		ioctl_arg_p->output_buf.buf_addr);

	ret = copy_from_user(&flash_state, ioctl_arg_p->input_buf.buf_addr,
			     sizeof(struct flash_state));
	if (ret) {
		VATOOLS_ERR(priv, die_id, "copy_to_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

	flash_state.flash_state = 0;

	switch (flash_state.flash_type) {
	case 0:
		flash_state.flash_state =
			atomic_read(&(priv->pci_state)) == VASTAI_NORMAL_STATE;
		break;
	case 1:
		/* flash always is done */
		flash_state.flash_state = 1;
		break;
	default:
		break;
	}

	ret = copy_to_user(ioctl_arg_p->input_buf.buf_addr, &flash_state,
			   sizeof(struct flash_state));
	if (ret) {
		VATOOLS_ERR(priv, die_id, "copy_to_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

out:
	ioctl_arg_p->errcode = ret;
	VATOOLS_DBG(priv, die_id, "ret=%ld", ret);
	VATOOLS_FUNC_EXIT;
	return ret;
}
long vatools_ioctl_flash_v2_switch_bmcu(struct file *filp, unsigned int cmd,
					unsigned long arg,
					struct vastai_pci_info *priv,
					ioctl_arg_t *ioctl_arg_p)
{
	long ret = -EINVAL;

	// struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;

	VATOOLS_FUNC_ENTERY;

	die_id = ioctl_arg_p->die_index.die_id;
	dev_id = ioctl_arg_p->die_index.dev_id;
	die_index = (u32)ioctl_arg_p->die_index.val;

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg_p->block_id, ioctl_arg_p->die_index.val,
		ioctl_arg_p->sub_cmd, ioctl_arg_p->address, ioctl_arg_p->value,
		ioctl_arg_p->lock_domain, ioctl_arg_p->count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg_p->input_buf.buf_size,
		ioctl_arg_p->input_buf.buf_addr,
		ioctl_arg_p->output_buf.buf_size,
		ioctl_arg_p->output_buf.buf_addr);

	ret = vastai_send_pcie_cmd(priv, die_index,
				   VASTAI_PCIE_SUB_BMCU_DEGRADE, 0);
	ioctl_arg_p->errcode = ret;
	VATOOLS_DBG(priv, die_id, "ret=%ld", ret);
	VATOOLS_FUNC_EXIT;
	return ret;
}

// flash 相关操作
long vatools_ioctl_flash_cmd(struct file *filp, unsigned int cmd,
			     unsigned long arg)
{
	long ret = -EINVAL;
	long ret2 = 0;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	ioctl_arg_t ioctl_arg;

	// struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;

	VATOOLS_FUNC_ENTERY;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ioctl cmd=0x%x ", cmd);

	memset(&ioctl_arg, 0x00, sizeof(ioctl_arg_t));
	ret = copy_from_user(&ioctl_arg, argp, sizeof(ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

	die_id = ioctl_arg.die_index.die_id;
	dev_id = ioctl_arg.die_index.dev_id;
	die_index = (u32)ioctl_arg.die_index.val;

	priv = vatools_get_vastai_pci_device_info(dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, die_id, "priv is NULL, dev_id[%u] ", dev_id);
		ret = -ENODEV;
		goto out;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(priv, die_id, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		ret = -ENODEV;
		goto out;
	}

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg.block_id, ioctl_arg.die_index.val, ioctl_arg.sub_cmd,
		ioctl_arg.address, ioctl_arg.value, ioctl_arg.lock_domain,
		ioctl_arg.count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg.input_buf.buf_size, ioctl_arg.input_buf.buf_addr,
		ioctl_arg.output_buf.buf_size, ioctl_arg.output_buf.buf_addr);

	switch (ioctl_arg.sub_cmd) {
	case VATOOLS_IOCTL_FLASH_BMCU_FW: {
		domain_mutex_lock(priv, ioctl_arg.die_index.val,
				  ioctl_arg.lock_domain);
		ret = vatools_ioctl_flash_v2_bmcu_fw(filp, cmd, arg, priv,
						     &ioctl_arg);
		domain_mutex_unlock(priv, ioctl_arg.die_index.val,
				    ioctl_arg.lock_domain);
		break;
	}
	case VATOOLS_IOCTL_FLASH_XSPI_FW: {
		domain_mutex_lock(priv, ioctl_arg.die_index.val,
				  ioctl_arg.lock_domain);
		ret = vatools_ioctl_flash_v2_xspi_fw(filp, cmd, arg, priv,
						     &ioctl_arg);
		domain_mutex_unlock(priv, ioctl_arg.die_index.val,
				    ioctl_arg.lock_domain);
		break;
	}
	case VATOOLS_IOCTL_FLASH_GET_FLASH_STATUS: {
		domain_mutex_lock(priv, ioctl_arg.die_index.val,
				  ioctl_arg.lock_domain);
		ret = vatools_ioctl_flash_v2_get_status(filp, cmd, arg, priv,
							&ioctl_arg);
		domain_mutex_unlock(priv, ioctl_arg.die_index.val,
				    ioctl_arg.lock_domain);
		break;
	}
	case VATOOLS_IOCTL_FLASH_SWITCH_BMCU: {
		domain_mutex_lock(priv, ioctl_arg.die_index.val,
				  ioctl_arg.lock_domain);
		ret = vatools_ioctl_flash_v2_switch_bmcu(filp, cmd, arg, priv,
							 &ioctl_arg);
		domain_mutex_unlock(priv, ioctl_arg.die_index.val,
				    ioctl_arg.lock_domain);
		break;
	}
	default: {
		VATOOLS_DBG(
			priv, die_id,
			"flash sub_cmd id %d don't support in current system\n",
			ioctl_arg.sub_cmd);
		break;
	}
	}

out:
	ioctl_arg.errcode = ret;
	ret2 = copy_to_user(argp, &ioctl_arg, sizeof(ioctl_arg_t));
	if (ret2) {
		VATOOLS_ERR(priv, die_id, "copy_to_user ret=%ld", ret);
		ret2 = -EFAULT;
	}
	VATOOLS_DBG(priv, die_id, "out: ret=%ld   ret2=%ld", ret, ret2);
	VATOOLS_FUNC_EXIT;
	return ret2;
}

/// ===================================================================
/// 使用内核mutex实现进程互斥保护: 加锁
int vatools_process_mutex_lock(struct file *filp, unsigned int cmd,
			       unsigned long arg, struct vastai_pci_info *priv,
			       ioctl_arg_t *ioctl_arg_p)
{
	long ret = -EIO;
	struct vatools_node *node = NULL;
	struct vatools_reader *reader = NULL;
	char mutex_name[VATOOLS_IOCTL_GUARD_NAME_LENGTH] = { 0 };
	struct vatools_process_mutex *p_process_mutex = NULL;
	struct vatools_process_mutex *p_current_process_mutex = NULL,
				     *p_next_process_mutex = NULL;
	bool found_mutex_name = false;
	struct vatools_process_mutex_filp *p_current_filp = NULL;

	VATOOLS_FUNC_ENTERY;

	node = vatools_file_get_node(filp);
	reader = vatools_file_get_reader(filp);

	memset(mutex_name, 0x00, sizeof(mutex_name));
	ret = copy_from_user(mutex_name, ioctl_arg_p->input_buf.buf_addr,
			     sizeof(mutex_name));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		return ret;
	}

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ioctl_cmd=0x%x mutex_name=%s", cmd,
		    mutex_name);

	// 如果mutex_name 不存在, 则分配新节点
	list_for_each_entry_safe (p_current_process_mutex, p_next_process_mutex,
				  &node->list_node_process_mutex,
				  list_member_process_mutex) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "p_current_process_mutex=0x%p  mutex_name=%s ",
			    p_current_process_mutex,
			    p_current_process_mutex->mutex_name);
		if (strcmp(p_current_process_mutex->mutex_name, mutex_name) ==
		    0) {
			// find mutex_name
			found_mutex_name = true;
			break;
		}
	}

	if (found_mutex_name) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "found mutex_name=%s",
			    p_current_process_mutex->mutex_name);
		p_process_mutex = p_current_process_mutex;
	} else {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "not found mutex_name=%s",
			    mutex_name);
		// 没有找到, 初始化新的 mutex 节点
		p_process_mutex = kvmalloc(sizeof(struct vatools_process_mutex),
					   GFP_KERNEL);
		if (!p_process_mutex) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "kvmalloc error. size=%ld",
				    sizeof(struct vatools_process_mutex));
			return -ENOMEM;
		}

		memset(p_process_mutex, 0,
		       sizeof(struct vatools_process_mutex));

		INIT_LIST_HEAD(&p_process_mutex->list_member_process_mutex);
		INIT_LIST_HEAD(&p_process_mutex->list_head_filp);

		strcpy(p_process_mutex->mutex_name, mutex_name);
		mutex_init(&p_process_mutex->process_mutex);

		list_add_tail(&p_process_mutex->list_member_process_mutex,
			      &node->list_node_process_mutex);
	}

	// 将当前 filp 挂载到mutex_name对应的链表上
	p_current_filp =
		kvmalloc(sizeof(struct vatools_process_mutex_filp), GFP_KERNEL);
	if (!p_current_filp) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "kvmalloc error. size=%ld",
			    sizeof(struct vatools_process_mutex_filp));
		return -ENOMEM;
	}
	p_current_filp->filp = filp;
	list_add_tail(&p_current_filp->list_member_filp,
		      &p_process_mutex->list_head_filp);

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "mutex_lock =%p ",
		    &p_process_mutex->process_mutex);
	mutex_lock(&p_process_mutex->process_mutex);

	ret = 0;
	return ret;
}

/// 使用内核mutex实现进程互斥保护: 解锁
int vatools_process_mutex_unlock(struct file *filp, unsigned int cmd,
				 unsigned long arg,
				 struct vastai_pci_info *priv,
				 ioctl_arg_t *ioctl_arg_p)
{
	long ret = -EIO;
	struct vatools_node *node = NULL;
	struct vatools_reader *reader = NULL;
	char mutex_name[VATOOLS_IOCTL_GUARD_NAME_LENGTH] = { 0 };
	struct vatools_process_mutex *p_process_mutex = NULL;
	struct vatools_process_mutex *p_current_process_mutex = NULL,
				     *p_next_process_mutex = NULL;
	bool found_mutex_name = false;
	struct vatools_process_mutex_filp *p_current_process_mutex_filp = NULL,
					  *p_next_process_mutex_filp = NULL;

	VATOOLS_FUNC_ENTERY;

	node = vatools_file_get_node(filp);
	reader = vatools_file_get_reader(filp);

	memset(mutex_name, 0x00, sizeof(mutex_name));
	ret = copy_from_user(mutex_name, ioctl_arg_p->input_buf.buf_addr,
			     sizeof(mutex_name));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ioctl_cmd=0x%x mutex_name=%s", cmd,
		    mutex_name);

	// 查找 mutex_name 匹配的节点
	list_for_each_entry_safe (p_current_process_mutex, p_next_process_mutex,
				  &node->list_node_process_mutex,
				  list_member_process_mutex) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "p_current_process_mutex=0x%p  mutex_name=%s ",
			    p_current_process_mutex,
			    p_current_process_mutex->mutex_name);
		if (strcmp(p_current_process_mutex->mutex_name, mutex_name) ==
		    0) {
			// find mutex_name
			found_mutex_name = true;
			break;
		}
	}

	if (found_mutex_name) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "found mutex_name=%s",
			    p_current_process_mutex->mutex_name);
		p_process_mutex = p_current_process_mutex;
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "mutex_unlock =%p ",
			    &p_process_mutex->process_mutex);
		mutex_unlock(&p_process_mutex->process_mutex);

		// 删除当前 filp 对应的memory
		list_for_each_entry_safe (p_current_process_mutex_filp,
					  p_next_process_mutex_filp,
					  &p_process_mutex->list_head_filp,
					  list_member_filp) {
			VATOOLS_DBG(
				NULL, DUMMY_DIE_ID,
				"p_current_process_mutex_filp=0x%p  filp=%p ",
				p_current_process_mutex_filp,
				p_current_process_mutex_filp->filp);
			if (filp == p_current_process_mutex_filp->filp) {
				// 删除 filp 节点
				list_del(&(
					p_current_process_mutex_filp
						->list_member_filp)); //删除节点，删除节点必须在删除节点内存之前
				kvfree(p_current_process_mutex_filp); //释放节点内存
			}
		}
	} else {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "not found mutex_name=%s",
			    mutex_name);
	}

out:
	return ret;
}

/// 使用内核mutex实现进程互斥保护: 是否加锁
int vatools_process_mutex_is_locked(struct file *filp, unsigned int cmd,
				    unsigned long arg,
				    struct vastai_pci_info *priv,
				    ioctl_arg_t *ioctl_arg_p)
{
	long ret = -EIO;
	struct vatools_node *node = NULL;
	struct vatools_reader *reader = NULL;
	char mutex_name[256] = { 0 };
	struct vatools_process_mutex *p_process_mutex = NULL;
	struct vatools_process_mutex *p_current_process_mutex = NULL,
				     *p_next_process_mutex = NULL;
	bool found_mutex_name = false;

	VATOOLS_FUNC_ENTERY;

	node = vatools_file_get_node(filp);
	reader = vatools_file_get_reader(filp);

	// 查找 mutex_name 匹配的节点
	list_for_each_entry_safe (p_current_process_mutex, p_next_process_mutex,
				  &node->list_node_process_mutex,
				  list_member_process_mutex) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "p_current_process_mutex=0x%p  mutex_name=%s ",
			    p_current_process_mutex,
			    p_current_process_mutex->mutex_name);
		if (strcmp(p_current_process_mutex->mutex_name, mutex_name) ==
		    0) {
			// find mutex_name
			found_mutex_name = true;
			break;
		}
	}

	if (found_mutex_name) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "found mutex_name=%s",
			    p_current_process_mutex->mutex_name);
		p_process_mutex = p_current_process_mutex;
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "mutex_is_locked =%p ",
			    &p_process_mutex->process_mutex);
		ret = mutex_is_locked(
			&p_process_mutex
				 ->process_mutex); // Returns 1 if the mutex is locked, 0 if unlocked.
	} else {
		ret = 0;
	}

	return ret;
}

/// 使用内核mutex实现进程互斥保护: 尝试加锁.
/// Try to acquire the mutex atomically.
/// Returns 1 if the mutex has been
/// acquired successfully, and 0 on contention.
int vatools_process_mutex_trylock(struct file *filp, unsigned int cmd,
				  unsigned long arg,
				  struct vastai_pci_info *priv,
				  ioctl_arg_t *ioctl_arg_p)
{
	long ret = -EIO;
	struct vatools_node *node = NULL;
	struct vatools_reader *reader = NULL;
	char mutex_name[256] = { 0 };
	struct vatools_process_mutex *p_process_mutex = NULL;
	struct vatools_process_mutex *p_current_process_mutex = NULL,
				     *p_next_process_mutex = NULL;
	bool found_mutex_name = false;

	VATOOLS_FUNC_ENTERY;

	node = vatools_file_get_node(filp);
	reader = vatools_file_get_reader(filp);

	// 查找 mutex_name 匹配的节点
	list_for_each_entry_safe (p_current_process_mutex, p_next_process_mutex,
				  &node->list_node_process_mutex,
				  list_member_process_mutex) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "p_current_process_mutex=0x%p  mutex_name=%s ",
			    p_current_process_mutex,
			    p_current_process_mutex->mutex_name);
		if (strcmp(p_current_process_mutex->mutex_name, mutex_name) ==
		    0) {
			// find mutex_name
			found_mutex_name = true;
			break;
		}
	}

	if (found_mutex_name) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "found mutex_name=%s",
			    p_current_process_mutex->mutex_name);
		p_process_mutex = p_current_process_mutex;
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "mutex_try_lock =%p ",
			    &p_process_mutex->process_mutex);
		ret = mutex_trylock(
			&p_process_mutex
				 ->process_mutex); // Try to acquire the mutex atomically. Returns 1 if the mutex has been
		// acquired successfully, and 0 on contention.
	} else {
		ret = 0;
	}

	return ret;
}

/// 使用内核mutex实现进程互斥保护: 文件关闭时, 释放所有和此文件相关的mutex
int vatools_process_release_mutex_unlock(struct file *filp, unsigned int cmd,
					 unsigned long arg,
					 struct vastai_pci_info *priv,
					 ioctl_arg_t *ioctl_arg_p)
{
	long ret = -EIO;
	struct vatools_node *node = NULL;
	struct vatools_reader *reader = NULL;
	// void __user*                       argp                    = ( void __user* )arg;
	// char                               mutex_name[ 256 ]       = { 0 };
	// struct vatools_process_mutex*      p_process_mutex         = NULL;
	struct vatools_process_mutex *p_current_process_mutex = NULL,
				     *p_next_process_mutex = NULL;
	// bool                               found_mutex_name             = false;
	struct vatools_process_mutex_filp *p_current_process_mutex_filp = NULL,
					  *p_next_process_mutex_filp = NULL;

	VATOOLS_FUNC_ENTERY;

	node = vatools_file_get_node(filp);
	reader = vatools_file_get_reader(filp);

	// 文件节点 close, unlock 所有相关的mutex

	// TODO:  如果是最后一个进程使用的, unlock后, 删除节点
	// 查找 mutex_name 匹配的节点
	list_for_each_entry_safe (p_current_process_mutex, p_next_process_mutex,
				  &node->list_node_process_mutex,
				  list_member_process_mutex) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "p_current_process_mutex=0x%p  mutex_name=%s ",
			    p_current_process_mutex,
			    p_current_process_mutex->mutex_name);

		// 删除当前 filp 对应的memory
		list_for_each_entry_safe (
			p_current_process_mutex_filp, p_next_process_mutex_filp,
			&p_current_process_mutex->list_head_filp,
			list_member_filp) {
			VATOOLS_DBG(
				NULL, DUMMY_DIE_ID,
				"p_current_process_mutex_filp=0x%p  filp=%p ",
				p_current_process_mutex_filp,
				p_current_process_mutex_filp->filp);
			if (filp == p_current_process_mutex_filp->filp) {
				VATOOLS_DBG(
					NULL, DUMMY_DIE_ID, "mutex_unlock =%p ",
					&p_current_process_mutex->process_mutex);
				mutex_unlock(
					&p_current_process_mutex->process_mutex);
				// 删除 filp 节点
				list_del(&(
					p_current_process_mutex_filp
						->list_member_filp)); //删除节点，删除节点必须在删除节点内存之前
				kvfree(p_current_process_mutex_filp); //释放节点内存
			}
		}
	}
	VATOOLS_FUNC_EXIT;
	return ret;
}

// process mutex
long vatools_ioctl_process_mutex_cmd(struct file *filp, unsigned int cmd,
				     unsigned long arg)
{
	long ret = -EINVAL;
	long ret2 = 0;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	ioctl_arg_t ioctl_arg;

	// struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;

	VATOOLS_FUNC_ENTERY;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ioctl cmd=0x%x ", cmd);

	memset(&ioctl_arg, 0x00, sizeof(ioctl_arg_t));
	ret = copy_from_user(&ioctl_arg, argp, sizeof(ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

	die_id = ioctl_arg.die_index.die_id;
	dev_id = ioctl_arg.die_index.dev_id;
	die_index = (u32)ioctl_arg.die_index.val;

	priv = vatools_get_vastai_pci_device_info(dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(NULL, die_id, "priv is NULL, dev_id[%u] ", dev_id);
		ret = -ENODEV;
		goto out;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(priv, die_id, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		ret = -ENODEV;
		goto out;
	}

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg.block_id, ioctl_arg.die_index.val, ioctl_arg.sub_cmd,
		ioctl_arg.address, ioctl_arg.value, ioctl_arg.lock_domain,
		ioctl_arg.count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg.input_buf.buf_size, ioctl_arg.input_buf.buf_addr,
		ioctl_arg.output_buf.buf_size, ioctl_arg.output_buf.buf_addr);

	switch (ioctl_arg.sub_cmd) {
	// 使用内核mutex实现进程互斥保护: 加锁
	case VATOOLS_IOCTL_GUARD_LOCK: {
		ret = vatools_process_mutex_lock(filp, cmd, arg, priv,
						 &ioctl_arg);
		return ret;
	} break;

	// 使用内核mutex实现进程互斥保护: 解锁
	case VATOOLS_IOCTL_GUARD_UNLOCK: {
		ret = vatools_process_mutex_unlock(filp, cmd, arg, priv,
						   &ioctl_arg);
		return ret;
	} break;
	//
	case VATOOLS_IOCTL_GUARD_TRYLOCK: {
		ret = vatools_process_mutex_is_locked(filp, cmd, arg, priv,
						      &ioctl_arg);
		return ret;
	} break;

	case VATOOLS_IOCTL_GUARDU_IS_LOCKED: {
		ret = vatools_process_mutex_trylock(filp, cmd, arg, priv,
						    &ioctl_arg);
		return ret;
	} break;
	default: {
		VATOOLS_DBG(
			priv, die_id,
			"process mutex sub_cmd id %d don't support in current system\n",
			ioctl_arg.sub_cmd);
		break;
	}
	}

out:
	ioctl_arg.errcode = ret;
	ret2 = copy_to_user(argp, &ioctl_arg, sizeof(ioctl_arg_t));
	if (ret2) {
		VATOOLS_ERR(priv, die_id, "copy_to_user ret=%ld", ret);
		ret2 = -EFAULT;
	}
	VATOOLS_DBG(priv, die_id, "out: ret=%ld   ret2=%ld", ret, ret2);
	VATOOLS_FUNC_EXIT;
	return ret2;
}

// 驱动内部访问 smi
int vatools_driver_smi_cmd(u32 block_id, struct vastai_pci_info *priv,
			   int die_index, void *buf, unsigned int len)
{
	int ret = 0;
	TS_SMI_CMD_REQ *cmd = (TS_SMI_CMD_REQ *)buf;
	T_SMI_BLOCK *psmi_block;
	u64 smi_die_address_offset;
	u32 loop;
	u32 interval;
	struct vastai_sv100_die *die;
	union die_index_data did;
	u32 die_id;

	did.val = (u32)die_index;
	die_id = did.die_id;

	VATOOLS_DBG(priv, die_id, "block_id=%d, priv=%p, die_index=%d",
		    block_id, priv, die_index);

	domain_mutex_lock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DIE);

	psmi_block = vatools_fw_smi_get_block(block_id);
	smi_die_address_offset = psmi_block->block_address;
	loop = psmi_block->query_count;
	interval = psmi_block->query_interval;

	VATOOLS_DUMP_BRIEF("vatools_driver_smi_cmd cmd req: ", buf, len);
	ret = vastai_pci_mem_write(priv, die_index, smi_die_address_offset,
				   (u8 *)buf, len);
	if (ret < 0) {
		VATOOLS_DBG(priv, die_id, "ret=%d", ret);
		goto out_mutex_unlock;
	}

	// set flag to 1, write only 4 bytes , sync with fw
	cmd->flag = FLAG_APP_REQ;
	ret = vastai_pci_mem_write(priv, die_index, smi_die_address_offset + 4,
				   ((u8 *)cmd) + 4, sizeof(u32));
	if (ret < 0) {
		VATOOLS_DBG(priv, die_id, "ret=%d", ret);
		goto out_mutex_unlock;
	}

	/* 如果是用户中断, 需要在这里清除一下上次smi命令的ack应答,
	否则每次进入到这里, wait_for都会直接返回, 导致wait和ack不匹配*/

	if (!priv || (priv->die_num <= die_id)) {
		VATOOLS_DBG(priv, die_id, "priv->die_num=%d < die_id=%d",
			    priv->die_num, die_id);
		ret = -EINVAL;
		goto out_mutex_unlock;
	}
	die = &(priv->dies[die_id]);
	reinit_completion(&(die->smi_ack[block_id]));

	ret = vatools_fw_send_smi_interrupt(priv, die_index);
	ret = vatools_wait_smi_ack(priv, die_index, block_id, 5);
	if (ret < 0) {
		if (ret == -EINTR) {
			VATOOLS_DBG(
				priv, die_id,
				"smi_ack: die_index=%x block_id=%d ret=%d(-EINTR)",
				die_index, block_id, ret);
			goto out_mutex_unlock;
		} else if (ret == -ETIMEDOUT) {
			VATOOLS_ERR(
				priv, die_id,
				"smiack: die_index=%x block_id=%d ret=%d(-ETIMEDOUT)",
				die_index, block_id, ret);
		} else {
			VATOOLS_ERR(priv, die_id,
				    "smi_ack: die_index=%x block_id=%d ret=%d",
				    die_index, block_id, ret);
			goto out_mutex_unlock;
		}
	} else if (ret != FLAG_FW_DATA_READY) {
		ret = -ENXIO;
		goto out_mutex_unlock;
	}

	ret = vastai_pci_mem_read(priv, die_index, smi_die_address_offset, buf,
				  len);
	if (ret < 0) {
		VATOOLS_DBG(priv, die_id, "ret=%d", ret);
		goto out_mutex_unlock;
	}
	VATOOLS_DUMP_BRIEF("vatools_driver_smi_cmd cmd rsp: ", buf, len);

	VATOOLS_DBG(priv, die_id, "read cmd.flag=%d", cmd->flag);

	if (cmd->flag == FLAG_FW_DATA_READY) {
		ret = len;
	} else {
		ret = -ENXIO;
	}

out_mutex_unlock:
	domain_mutex_unlock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DIE);

	VATOOLS_DBG(priv, die_id, "done ret=%d", ret);
	return ret;
}

/// ===================================================================
/// dma

// dma分配内存
int vatools_ioctl_dma_alloc_ex(struct file *filp, unsigned int cmd,
			       unsigned long arg, struct vastai_pci_info *priv,
			       ioctl_arg_t *ioctl_arg_p)
{
	int ret = 0;
	int die_index = 0;
	struct dma_buf *dmabuf = NULL;
	dma_alloc_cmd_t dma_alloc_cmd;
	union die_index_data did;
	u32 die_id;

	VATOOLS_FUNC_ENTERY;

	memset(&dma_alloc_cmd, 0, sizeof(dma_alloc_cmd_t));
	ret = copy_from_user(&dma_alloc_cmd,
			     (void *)ioctl_arg_p->input_buf.buf_addr,
			     sizeof(dma_alloc_cmd_t));
	if (ret) {
		VATOOLS_ERR(priv, DUMMY_DIE_ID, "copy_from_user is error\n");
		return -EIO;
	}

	die_index = dma_alloc_cmd.die_index;
	did.val = (u32)die_index;
	die_id = did.die_id;

	VATOOLS_DBG(priv, die_id, "die_index=0x%x, dma_buf_fd=%x, size=%d",
		    die_index, dma_alloc_cmd.dma_buf_fd, dma_alloc_cmd.size);

	dmabuf = vastai_alloc_dma_buf(priv, dma_alloc_cmd.size);
	if (dmabuf) {
		int fd = dma_buf_fd(dmabuf, O_CLOEXEC);
		dma_alloc_cmd.dma_buf_fd = fd;
	} else {
		dma_alloc_cmd.dma_buf_fd = -ENOMEM;
	}

	ret = copy_to_user((void *)ioctl_arg_p->output_buf.buf_addr,
			   &dma_alloc_cmd, sizeof(dma_alloc_cmd_t));
	if (ret) {
		VATOOLS_ERR(priv, die_id, "copy_from_user is error\n");
		return -EIO;
	}
	return ret;
}

//开始 dma 传输
int vatools_ioctl_dma_start_ex(struct file *filp, unsigned int cmd,
			       unsigned long arg, struct vastai_pci_info *priv,
			       ioctl_arg_t *ioctl_arg_p)
{
	int ret = 0;
	struct dma_buf *dmabuf = NULL;
	struct vastai_dmadesc desc;
	union core_bitmap core_id = { .val = 0 };
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
	struct timespec tm_begin, tm_end;
#else
	u64 tm_begin, tm_end;
#endif
	unsigned long time_total_ns;
	u32 done_size = 0;
	u32 size = 0;
	u32 i = 0;
	dma_start_cmd_t dma_start_cmd;
	union die_index_data did;
	u32 die_id;
	u32 die_index;

	VATOOLS_FUNC_ENTERY;

	memset(&dma_start_cmd, 0, sizeof(dma_start_cmd_t));
	ret = copy_from_user(&dma_start_cmd,
			     (void *)ioctl_arg_p->input_buf.buf_addr,
			     sizeof(dma_start_cmd_t));
	if (ret) {
		VATOOLS_ERR(priv, DUMMY_DIE_ID, "copy_from_user is error\n");
		return -EIO;
	}
	die_index = dma_start_cmd.die_index;
	did.val = (u32)die_index;
	die_id = did.die_id;
	VATOOLS_DBG(
		priv, die_id,
		"die_index=0x%x, dma_buf_fd=%x, size=%d, axi_addr=0x%llx, is_dev_to_host=%d",
		die_index, dma_start_cmd.dma_buf_fd, dma_start_cmd.size,
		dma_start_cmd.axi_addr, dma_start_cmd.is_dev_to_host);

	dmabuf = dma_buf_get(dma_start_cmd.dma_buf_fd);
	if (IS_ERR(dmabuf)) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
		       "%s dma buf fd is invalid\n", __func__);
		return PTR_ERR(dmabuf);
	}
	size = dma_start_cmd.size;

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
	/* getnstimeofday function is removed from kernel5.6 */
	getnstimeofday(&tm_begin);
#else
	tm_begin = ktime_get_ns();
#endif

	desc.is_host_to_dev = !dma_start_cmd.is_dev_to_host;
	desc.is_src_not_user_mem = 1;
	desc.is_src_dma_addr = 1;

	while (done_size < dma_start_cmd.size) {
		struct vastai_dma_buf *dm = get_vastai_dma_buf(dmabuf, i);
		u32 temp_size = (size - done_size) < VASTAI_MAX_DMA_BUF ?
					(size - done_size) :
					VASTAI_MAX_DMA_BUF;

		desc.dev_addr = dma_start_cmd.axi_addr + done_size;
		desc.dma_lenth = temp_size;
		desc.host_addr.dma_addr = dm->dma_bus_addr;

		ret = vastai_pci_dma_transfer_sync(
			priv, dma_start_cmd.die_index, core_id, &desc, 1);
		if (ret)
			break;

		i++;
		done_size += temp_size;
	}

	dma_buf_put(dmabuf);
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
	getnstimeofday(&tm_end);
	time_total_ns = timespec_to_ns(&tm_end) - timespec_to_ns(&tm_begin);
#else
	tm_end = ktime_get_ns();
	time_total_ns = tm_end - tm_begin;
#endif

	if (time_total_ns > 30000 && 0) {
		struct vastai_fifo fifo;
		VATOOLS_DBG(priv, die_id, "die_index 0x%x time %ld\n",
			    dma_start_cmd.die_index, time_total_ns);

		vastai_pci_mem_read(priv, dma_start_cmd.die_index, 0x8c14000,
				    &fifo, sizeof(struct vastai_fifo));

		VATOOLS_DBG(
			priv, die_id,
			"%s rd[0x%x] wr[0x%x] elem_count[0x%x] elem_size[0x%x]\n",
			"0x8c14000", fifo.rd, fifo.wr, fifo.elem_count,
			fifo.elem_size);

		vastai_pci_mem_read(priv, dma_start_cmd.die_index, 0x8c1e000,
				    &fifo, sizeof(struct vastai_fifo));

		VATOOLS_DBG(
			priv, die_id,
			"%s rd[0x%x] wr[0x%x] elem_count[0x%x] elem_size[0x%x]\n",
			"0x8c1e000", fifo.rd, fifo.wr, fifo.elem_count,
			fifo.elem_size);
	}
	return ret;
}

//开始 dma 传输
int vatools_ioctl_dma_free_ex(struct file *filp, unsigned int cmd,
			      unsigned long arg, struct vastai_pci_info *priv,
			      ioctl_arg_t *ioctl_arg_p)
{
	int ret = 0;
	// struct dma_buf *dmabuf = NULL;
	// dma_start_cmd_t dma_start_cmd;
	// union die_index_data did;
	// u32 die_id;
	// u32 die_index;

	// VATOOLS_FUNC_ENTERY;

	// memset(&dma_start_cmd, 0, sizeof(dma_start_cmd_t));
	// ret = copy_from_user(&dma_start_cmd,
	// 		     (void *)ioctl_arg_p->input_buf.buf_addr,
	// 		     sizeof(dma_start_cmd_t));
	// if (ret) {
	// 	VATOOLS_ERR(priv, DUMMY_DIE_ID, "copy_from_user is error\n");
	// 	return -EIO;
	// }
	// die_index = dma_start_cmd.die_index;
	// did.val = (u32)die_index;
	// die_id = did.die_id;
	// VATOOLS_DBG(
	// 	priv, die_id,
	// 	"die_index=0x%x, dma_buf_fd=%x, size=%d, axi_addr=0x%llx, is_dev_to_host=%d",
	// 	die_index, dma_start_cmd.dma_buf_fd, dma_start_cmd.size,
	// 	dma_start_cmd.axi_addr, dma_start_cmd.is_dev_to_host);

	return ret;
}

long vatools_ioctl_dma_cmd(struct file *filp, unsigned int cmd,
			   unsigned long arg)
{
	long ret = -EINVAL;
	long ret2 = 0;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	ioctl_arg_t ioctl_arg;

	struct vatools_node *node = vatools_file_get_node(filp);
	// struct vatools_reader *reader = vatools_file_get_reader(filp);

	u32 die_index = 0;
	u32 die_id;
	u32 dev_id;

	VATOOLS_FUNC_ENTERY;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ioctl cmd=0x%x ", cmd);

	memset(&ioctl_arg, 0x00, sizeof(ioctl_arg_t));
	ret = copy_from_user(&ioctl_arg, argp, sizeof(ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user ret=%ld", ret);
		ret = -EFAULT;
		goto out;
	}

	die_id = ioctl_arg.die_index.die_id;
	dev_id = ioctl_arg.die_index.dev_id;
	die_index = (u32)ioctl_arg.die_index.val;

	priv = vatools_get_vastai_pci_device_info(dev_id);
	if (priv == NULL) {
		VATOOLS_ERR(priv, die_id, "priv is NULL, dev_id[%u] ", dev_id);
		ret = -ENODEV;
		goto out;
	}
	if ((atomic_read(&priv->pci_state) != VASTAI_NORMAL_STATE)) {
		VATOOLS_ERR(priv, DUMMY_DIE_ID, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		ret = -ENODEV;
		goto out;
	}

	VATOOLS_DBG(
		priv, die_id,
		"block_id=%d die_index=%x sub_cmd=0x%x address=%llx value=%llx lock_domain=%d count=%d",
		ioctl_arg.block_id, ioctl_arg.die_index.val, ioctl_arg.sub_cmd,
		ioctl_arg.address, ioctl_arg.value, ioctl_arg.lock_domain,
		ioctl_arg.count);
	VATOOLS_DBG(
		priv, die_id,
		"input_buf_size=%d input_buf_addr=0x%p output_buf_size=%d output_buf_addr=0x%p",
		ioctl_arg.input_buf.buf_size, ioctl_arg.input_buf.buf_addr,
		ioctl_arg.output_buf.buf_size, ioctl_arg.output_buf.buf_addr);

	mutex_lock(&node->node_logger_mutex);

	switch (ioctl_arg.sub_cmd) {
	case VATOOLS_IOCTL_DMA_ALLOC: {
		ret = vatools_ioctl_dma_alloc_ex(filp, cmd, arg, priv,
						 &ioctl_arg);
	} break;
	case VATOOLS_IOCTL_DMA_START: {
		ret = vatools_ioctl_dma_start_ex(filp, cmd, arg, priv,
						 &ioctl_arg);
	} break;
	case VATOOLS_IOCTL_DMA_FREE: {
		ret = vatools_ioctl_dma_free_ex(filp, cmd, arg, priv,
						&ioctl_arg);
	} break;
	default:
		VATOOLS_DBG(
			priv, die_id,
			"dma sub_cmd id %d don't support in current system\n",
			ioctl_arg.sub_cmd);
		break;
	}

	mutex_unlock(&node->node_logger_mutex);

out:
	ioctl_arg.errcode = ret;
	ret2 = copy_to_user(argp, &ioctl_arg, sizeof(ioctl_arg_t));
	if (ret2) {
		VATOOLS_ERR(priv, die_id, "copy_to_user ret=%ld", ret);
		ret2 = -EFAULT;
	}
	VATOOLS_DBG(priv, die_id, "out: ret=%ld   ret2=%ld", ret, ret2);
	VATOOLS_FUNC_EXIT;
	return ret2;
}
#endif
