/* 
 * Vastai Decoder device driver (kernel module)
*
* Copyright (C) 2020 VASTAI Microelectronics Co., Ltd.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.

* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
------------------------------------------------------------------------------*/

#ifndef _VASTAIMMU_H_
#define _VASTAIMMU_H_
#ifdef __FREERTOS__
#include "basetype.h"
#include "dev_common_freertos.h"
#elif defined(__linux__)
#include <linux/fs.h>
#endif

#define REGION_IN_START          0x0
#define REGION_IN_END            0x40000000
#define REGION_OUT_START         0x40000000
#define REGION_OUT_END           0x80000000
#define REGION_PRIVATE_START     0x80000000
#define REGION_PRIVATE_END       0xc0000000

#define REGION_IN_MMU_START      0x1000
#define REGION_IN_MMU_END        0x40002000
#define REGION_OUT_MMU_START     0x40002000
#define REGION_OUT_MMU_END       0x80001000
#define REGION_PRIVATE_MMU_START 0x80001000
#define REGION_PRIVATE_MMU_END   0xc0000000

enum MMUStatus {
  MMU_STATUS_OK                          =    0,

  MMU_STATUS_FALSE                       =   -1,
  MMU_STATUS_INVALID_ARGUMENT            =   -2,
  MMU_STATUS_INVALID_OBJECT              =   -3,
  MMU_STATUS_OUT_OF_MEMORY               =   -4,
  MMU_STATUS_NOT_FOUND                   =   -19,
};

struct addr_desc {
  void                       *virtual_address;  /* buffer virtual address */
  unsigned int               bus_address;  /* buffer physical address */
  unsigned int               size;  /* physical size */
};

struct kernel_addr_desc {
  unsigned int               bus_address;  /* buffer virtual address */
  unsigned int               mmu_bus_address;  /* buffer physical address in MMU*/
  unsigned int               size;  /* physical size */
};


#define VASTAI_IOC_MMU  'm'

#define VASTAI_IOCS_MMU_MEM_MAP    _IOWR(VASTAI_IOC_MMU, 1, struct addr_desc *)
#define VASTAI_IOCS_MMU_MEM_UNMAP  _IOWR(VASTAI_IOC_MMU, 2, struct addr_desc *)
#define VASTAI_IOCS_MMU_FLUSH      _IOWR(VASTAI_IOC_MMU, 3, unsigned int *)
#define VASTAI_IOC_MMU_MAXNR 3

#endif

